<?php
require_once('fpdf/fpdf.php');
define('_SYSTEM_TTFONTS',  __DIR__ . '/fpdf/');

class PDFCert extends tFPDF
{
  function AddHeader($content)
  {
    global $uri;
    $qrcode = 'https://quickchart.io/chart?cht=qr&chld=H%7C1&chs=400x400&chl=' . urlencode("{$uri->site}verify-certificate/{$content->certificate_id}");
    $this->Image($qrcode, 230, 32, 30, 0, 'PNG');
  }

  function AddFooter()
  {
    global $uri;
    $x = 30;
    $this->SetXY($x, -40);
    $centerX = $this->GetPageWidth() / 2;
    $w = $centerX - $x;
    $this->SetFont('DejaVu', 'B', 12);
    $this->Cell($w, 5, ' _______________________', 0, 0, 'L');
    $this->Cell($w, 5, ' ___________________', 0, 1, 'R');
    $this->SetX($x);
    $this->Cell($w, 10, 'DIRECTOR OF TRAINING', 0, 0, 'L');
    $this->Cell($w, 10, str_pad('REGISTRAR', 13), 0, 1, 'R');
    $this->SetFont('Arial', 'I', 10);
    $this->SetTextColor(0, 0, 255);
    $this->Cell(0, 3, "Verify at: {$uri->site}verify-certificate", 0, 0, 'C', false, "{$uri->site}verify-certificate");
    $this->SpikedCircle($centerX, $this->GetPageHeight() - 40, 12, 20);
  }

  function AddCertificateBg()
  {
    global $uri;
    $bg = $uri->site . 'asset/picture/certBg.jpg';
    $this->Image($bg, 0, 0, $this->GetPageWidth(), $this->GetPageHeight());
  }

  function SpikedCircle($centerX, $centerY, $radius, $spikes)
  {
    $points = [];
    $spikeLength = $radius * 1.2;
    $angleStep = 360 / ($spikes * 2);
    for ($i = 0; $i < $spikes * 2; $i++) {
      $angle = deg2rad($i * $angleStep);
      $currentRadius = ($i % 2 == 0) ? $spikeLength : $radius;
      $x = $centerX + $currentRadius * cos($angle);
      $y = $centerY + $currentRadius * sin($angle);
      $points[] = $x;
      $points[] = $y;
    }
    $this->SetFillColor(236, 50, 55);
    $this->Polygon($points, 'F');
  }

  function Polygon($points, $style = 'D')
  {
    $op = 'f';
    if ($style == 'D') {
      $op = 'S';
    } elseif ($style == 'FD' || $style == 'DF') {
      $op = 'B';
    }

    $h = $this->h;
    $k = $this->k;
    $this->_out(sprintf('%.2F %.2F m', $points[0] * $k, ($h - $points[1]) * $k));
    for ($i = 2; $i < count($points); $i += 2) {
      $this->_out(sprintf('%.2F %.2F l', $points[$i] * $k, ($h - $points[$i + 1]) * $k));
    }
    $this->_out($op);
  }

  function BodyText($content)
  {
    $this->AddHeader($content);
    $this->SetFont('Urbanist', '', 20);
    $this->SetTextColor(66);
    $H = 65;
    $adj = 5;
    $dist = $H / 3;
    $y1 = ($this->GetPageHeight() - $H) / 2 + $adj;
    [$y2, $y3] = [$y1 + $dist, $y1 + 2 * $dist];
    $this->SetXY(0, $y1);
    $this->Cell(0, 8, $content->student, 0, 0, 'C');
    $this->SetXY(0, $y2);
    $this->Cell(0, 8, "Diploma in {$content->course}", 0, 0, 'C');
    $this->SetXY(0, $y3);
    $this->Cell(0, 8, $content->grade, 0, 0, 'C');
    $y4 = $y3 + $dist + $adj;
    $x = $this->GetPageWidth() / 2 - 15;
    $date = date_create($content->date);
    $this->SetXY($x, $y4);
    $this->Cell(10, 8, $date->format('jS'), 0, 0, 'C');
    $this->Cell(25, 8, '', 0, 0);
    $this->Cell(0, 8, $date->format('F, Y'), 0, 0);
  }

  function CenteredMultiCell($w, $h, $txt)
  {
    $words = explode(' ', $txt);
    $txt = '';
    $x = floor(($this->GetPageWidth() - $w) / 2);
    while (count($words) > 0) {
      $word = array_shift($words);
      if ($w < $this->GetStringWidth($txt . $word)) {
        $this->SetX($x);
        $this->Cell($w, $h, trim($txt), 0, 1, 'C');
        $txt = '';
      }
      $txt .= $word . ' ';
    }
    $txt = trim($txt);
    if ($txt != '') {
      $padTimes = intval(($w - $this->GetStringWidth($txt)) / $this->GetStringWidth(' ')) / 2;
      if ($padTimes > 0) $txt = str_repeat(' ', $padTimes) . $txt . str_repeat(' ', $padTimes);
      $this->SetX($x);
      $this->Cell($w, $h, trim($txt), 0, 1, 'C');
    }
  }

  function DecorativeBorder()
  {
    $this->SetDrawColor(184, 134, 11);
    $this->SetLineWidth(1.5);
    $this->Rect(10, 10, 277, 190, 'D');
    $this->SetLineWidth(0.5);
    $this->Rect(15, 15, 267, 180, 'D');
    $this->SetDrawColor(0);
  }

  function CreateCertificate($path, $content, $original = false)
  {
    $this->AddFont('Urbanist', '', 'Urbanist.ttf', true);
    $this->AddPage('L');
    if ($original) {
      $this->AddCertificateBg();
    }
    $this->BodyText($content);
    $this->Output('F', $path);
  }
}
