<?php
require_once('fpdf/fpdf.php');
define('_SYSTEM_TTFONTS',  __DIR__ . '/fpdf/');

class PDFReceipt extends tFPDF
{
  protected $angle = 0, $margin = 5, $all_payments, $payment, $link, $path, $callback;

  function __construct($payments, $path='', $callback=null)
  {
    parent::__construct();
    $this->all_payments = $payments;
    $this->path = $path;
    $this->callback = $callback;
    $this->AddFont('DejaVu', 'B', 'DejaVuSans-Bold.ttf', true);
  }

  function Watermark($text)
  {
    $this->SetFont('Arial', 'B', 40);
    $this->SetTextColor(212);
    $pageWidth = $this->GetPageWidth();
    $pageHeight = $this->GetPageHeight();
    $centerX = $pageWidth / 2;
    $centerY = $pageHeight / 2;
    $this->SetXY($centerX, $centerY);
    $this->Rotate(45, $centerX, $centerY);
    $this->Text($centerX - ($this->GetStringWidth($text) / 2), $centerY, $text);
    $this->Rotate(0);
    $this->SetTextColor(0);
  }

  function Rotate($angle, $x = -1, $y = -1)
  {
    if ($x == -1)
      $x = $this->x;
    if ($y == -1)
      $y = $this->y;
    if ($this->angle != 0)
      $this->_out('Q');
    $this->angle = $angle;
    if ($angle != 0) {
      $angle *= M_PI / 180;
      $c = cos($angle);
      $s = sin($angle);
      $cx = $x * $this->k;
      $cy = ($this->h - $y) * $this->k;
      $this->_out(sprintf('q %.3F %.3F %.3F %.3F %.3F %.3F cm 1 0 0 1 %.3F %.3F cm', $c, $s, -$s, $c, $cx, $cy, -$cx, -$cy));
    }
  }

  function QRCode($link)
  {
    $qrcode = 'https://quickchart.io/chart?cht=qr&chld=H%7C1&chs=400x400&chl=' . urlencode($link);
    $this->SetLineWidth(1.5);
    $boxSize = 20;
    $padding = 2;
    $adj = 5;
    $qrSize = $boxSize - 2 * $padding;
    $boxX = $this->GetPageWidth() - $this->margin - $boxSize - $adj;
    $this->Rect($boxX, $this->margin, $boxSize, $boxSize);
    $this->Image($qrcode, $boxX + $padding, $this->margin + $padding, $qrSize, $qrSize, 'PNG');
    $this->SetLineWidth(0.2);
  }

  function CenteredMultiCell($w, $h, $txt)
  {
    $words = explode(' ', $txt);
    $txt = '';
    while (count($words) > 0) {
      $word = array_shift($words);
      if ($w < $this->GetStringWidth($txt . $word)) {
        $this->Cell($w, $h, trim($txt), 0, 1, 'C');
        $txt = '';
      }
      $txt .= $word . ' ';
    }
    $txt = trim($txt);
    if ($txt != '') {
      $padTimes = intval(($w - $this->GetStringWidth($txt)) / $this->GetStringWidth(' ')) / 2;
      if ($padTimes > 0) $txt = str_repeat(' ', $padTimes) . $txt . str_repeat(' ', $padTimes);
      $this->Cell($w, $h, trim($txt), 0, 1, 'C');
    }
  }

  function Header()
  {
    global $company, $uri;
    $logo = $uri->site . 'asset/picture/dd_black_no_bg.png';
    $headquarter = strip_html_tags($company->branches[0]->desc);
    $this->link = "{$uri->site}verify-receipt/{$this->payment->trx_no}";
    $this->Watermark('Digital Dreams ICT Academy');
    $this->Image($logo, $this->margin, $this->margin, 30);
    $this->QRCode($this->link);
    $this->SetY(14);
    $this->SetX(5);
    $this->SetFont('Arial', 'B', 6);
    $this->MultiCell(35, 3, $headquarter, 0, 'L');
    $this->Ln(2);
    $this->SetX(5);
    $this->SetFont('DejaVu', 'B', 6);
    $this->Cell(0, 3, "\u{260E} Tel: {$company->phone}", 0, 1);
    $this->SetX(5);
    $this->Cell(0, 3, "\u{2709} Email: {$company->email}", 0, 1);
    $this->SetX(5);
    $this->Cell(0, 3, "\u{25EF} {$company->website}", 0, 1);
    $this->Ln(3);
  }

  function Footer()
  {
    $this->SetY(-30);
    $this->setX(0);
    $this->SetFont('Arial', 'I', 7);
    $this->SetTextColor(70);
    $this->CenteredMultiCell(70, 3, "Verify at: {$this->link}");
    $this->Ln(2);
    $this->CenteredMultiCell(70, 3, "Thank you for building your future with Digital Dreams ICT Academy!");
    $this->Ln(3);
    $this->SetFont('Arial', 'B', 6);
    $cellWidth = $this->GetPageWidth() / 3;
    $this->Cell($cellWidth, 5, date('d/m/Y'), 0, 0, 'C');
    $this->Cell($cellWidth, 5, date('H:i'), 0, 0, 'C');
    $this->Cell($cellWidth, 5, 'Pg ' . $this->PageNo() . '/{nb}', 0, 0, 'C');
  }

  function DottedLine($x1, $x2, $y, $step = 1)
  {
    $numberOfDots = floor(($x2 - $x1) / 2 / $step);
    for ($i = 0; $i < $numberOfDots; $i++) {
      $xStart = $x1 + $i * 2 * $step;
      $xEnd = $xStart + $step;
      $this->Line($xStart, $y, $xEnd, $y);
    }
  }

  function AddHeadings()
  {
    $this->SetFont('Arial', 'B', 7);
    $this->SetFillColor(0);
    $this->SetTextColor(255);
    $this->SetX(5);
    $txt = "Transaction No";
    $w = $this->GetStringWidth($txt) + 6;
    $this->Cell($w, 4, $txt, 1, 0, 'C', true);
    $this->SetFillColor(255);
    $this->SetTextColor(0);
    $this->SetFont('Arial', 'B', 6);
    $this->Cell(0, 4, $this->payment->trx_no, 0, 0);
    $this->SetFont('DejaVu', 'B', 6);
    $this->Cell(0, 4, "Invoice — {$this->payment->invoice_ref}", 0, 1, 'R');
    $this->Ln(3);
    $this->SetX(0);
    $this->SetFont('Helvetica', 'B', 10);
    $asterisk = str_repeat('* ', 4);
    $this->Cell($this->GetPageWidth(), 8, $asterisk . 'PAYMENT RECEIPT ' . $asterisk, 0, 1, 'C');
    $this->Ln(3);
  }

  function AddOrderTable()
  {
    $w = $this->GetPageWidth();
    $this->DottedLine(0, $w - $this->margin, $this->GetY());
    $this->SetFont('Arial', 'B', 7);
    $this->Cell(10, 6, 'Qty', 0, 0, 'C');
    $this->Cell(40, 6, 'Item', 0, 0);
    $this->Cell(20, 6, 'Price', 0, 1, 'R');
    $this->DottedLine(0, $w - $this->margin, $this->GetY());
    $this->Ln(2);
    $items = $this->payment->items;
    $qty = '1 ×';
    foreach ($items as $item) {
      list($desc, $price) = $item;
      $this->SetFont('DejaVu', 'B', 6);
      $this->Cell(10, 5, $qty, 0, 0, 'C');
      $x = $this->GetX();
      $y1 = $this->GetY();
      $this->SetFont('Arial', 'B', 7);
      $this->MultiCell(40, 5, $desc, 0, 0, 'L');
      $this->SetFont('DejaVu', 'B', 7);
      $y2 = $this->GetY();
      $this->SetXY($x + 40, $y1);
      $this->Cell(20, 5, '₦ ' . number_format($price), 0, 1, 'R');
      $this->setY(max($this->GetY(), $y2));
    }
    $this->Ln(5);
  }

  function AddAmtTotal()
  {
    $w = $this->GetPageWidth();
    $this->DottedLine(0, $w - $this->margin, $this->GetY());
    $this->SetFont('Arial', 'B', 7);
    $this->Cell(30, 6, 'Sub total:', 0, 0);
    $this->Cell(40, 6, 'NGN ' . number_format($this->payment->price), 0, 1, 'R');
    $this->SetFont('Arial', '', 7);
    $this->Cell(30, 4, 'Discount:', 0, 0);
    $this->Cell(40, 4, '- NGN ' . number_format($this->payment->discount), 0, 1, 'R');
    $this->Cell(30, 6, 'Previous deposits:', 0, 0);
    $this->Cell(40, 6, '- NGN ' . number_format($this->payment->deposits), 0, 1, 'R');
    $this->DottedLine(0, $w - $this->margin, $this->GetY());
    $this->SetFont('Arial', 'B', 9);
    $this->Cell(30, 6, 'Total due:', 0, 0);
    $this->Cell(40, 6, 'NGN ' . number_format($this->payment->amount_due), 0, 1, 'R');
    $this->SetFont('Arial', '', 9);
    $this->Cell(30, 6, 'Paid:', 0, 0);
    $this->Cell(40, 6, 'NGN ' . number_format($this->payment->amount), 0, 1, 'R');
    $this->DottedLine(0, $w - $this->margin, $this->GetY());
    $this->SetFont('Arial', '', 7);
    $this->Cell(30, 6, 'Debt:', 0, 0);
    $this->Cell(40, 6, 'NGN ' . number_format($this->payment->balance), 0, 1, 'R');
    $this->DottedLine(0, $w - $this->margin, $this->GetY());
  }

  function AddPayerInfo()
  {
    $this->SetFont('Arial', 'B', 9);
    $this->Cell(0, 8, 'Customer Summary', 0, 1);
    $this->SetFont('Arial', '', 7);
    $this->Cell(25, 5, 'Student Name:', 0, 0);
    $this->SetFont('Arial', 'B', 7);
    $this->Cell(0, 5, ucwords($this->payment->student), 0, 1);
    $this->SetFont('Arial', '', 7);
    $this->Cell(25, 5, 'Phone Number:', 0, 0);
    $this->SetFont('Arial', 'B', 7);
    $this->Cell(0, 5, $this->payment->phone, 0, 1);
    $this->SetFont('Arial', '', 7);
    $this->Cell(25, 5, 'Front Desk Officer:', 0, 0);
    $this->SetFont('Arial', 'B', 7);
    $this->Cell(0, 5, $this->payment->desk_officer ?? 'None', 0, 1);
    $this->SetFont('Arial', '', 7);
    $this->Cell(25, 5, 'Payment Date:', 0, 0);
    $this->SetFont('Arial', 'B', 7);
    $this->Cell(0, 5, date_format(date_create($this->payment->payment_date), 'D jS F, Y'), 0, 1);
  }

  function CreateReceipt()
  {
    date_default_timezone_set('Africa/Lagos');
    $this->AliasNbPages();
    foreach ($this->all_payments as $payment) {
      $this->payment = $payment;
      $this->AddPage('P', [80, 180]);
      $this->SetMargins(0, 0, 5);
      $this->AddHeadings();
      $this->AddOrderTable();
      $this->AddAmtTotal();
      $this->AddPayerInfo();
    }
    if (!empty($this->path)) {
      $this->Output('F', $this->path);
      if (is_callable($this->callback)) {
        ($this->callback)($this->all_payments, $this->path);
      }
    }
    $this->Output();
  }
}
