<?php
require_once(__DIR__ . "../../../functions/html_functions.php");

function company($createOnly = false)
{
	global $generic;
	$uri  = $generic->getURIdata();
	$dir = absolute_filepath($uri->site);
	$file = "{$dir}cache/company.json";
	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		$json = $generic->company();
		$json = _writeFile($file, $json, true);
	}
	return json_decode($json);
}

function get_pages($createOnly = false)
{
	global $generic;
	$uri  = $generic->getURIdata();
	$dir = absolute_filepath($uri->site);
	$file = "{$dir}cache/validpages.json";
	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		$db   = $generic->connect();
		$sql  = $db->query("SELECT DISTINCT type FROM content UNION select title as type FROM content WHERE type = 'page' UNION select url as type FROM content WHERE type = 'subPage' ") or die($db->error);
		$pages = [];

		while ($row = $sql->fetch_object()) {
			$pages[] = str_replace(' ', '-', strtolower($row->type));
		}
		$json = _writeFile($file, array_merge($pages, ["index", "home", "read", "view-course", "view-event", "verify-receipt", "courses", "view-job", "find-jobs", "view-profile", "find-talents", "view-program", "find-programs", "view-bootcamp", "coding-bootcamp"]), true);
	}
	return json_decode($json);
}

function get_nav($createOnly = false)
{
	global $generic;
	$uri  = $generic->getURIdata();
	$dir 	= absolute_filepath($uri->site);
	$file = "{$dir}cache/navbar.json";
	$file2 = "{$dir}cache/categories.json";
	$file3 = "{$dir}cache/subPages.json";
	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && file_exists($file2) && file_exists($file3) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 0)) {
		$json = _readFile($file);
	} else {
		// Get all content from database
		$all = $generic->getFromTable("content", "", 1, 0, LABEL_ASC);
		$all = array_remap($all, array_column($all, "id"));

		// Filter out the page contents
		$pages = array_filter($all, function ($value = '') {
			return $value->type === "page";
		});

		// Write categories
		$categ  = array_filter($all, function ($value = '') {
			return $value->type === "category";
		});
		$categx = array_map(function ($value) {
			$value = ucwords($value->title);
			return $value;
		}, $categ);
		_writeFile($file2, $categx);

		// Write subpages
		$subpages  = array_filter($all, function ($value = '') {
			return $value->type === "subPage";
		});
		$subpagex = array_map(function ($value) {
			$value = strToUrl($value->url);
			return $value;
		}, $subpages);
		$events = array_map(function ($val) {
			return $val->url;
		}, array_filter($pages, function ($page) {
			return $page->title == 'Events';
		}));
		$id_key = array_key_first($events);
		$subpagex[$id_key] = $events[$id_key];
		_writeFile($file3, $subpagex);

		// Loop through the pages and add their sublinks and tag labels
		$json  = array_map(function ($value) use ($all) {
			$value = (object)array_extract($value, ["id", "title", "label", "url"], true);
			$children = array_filter($all, function ($row) use ($value) {
				return $row->parent == $value->id;
			});
			usort($children, function ($a, $b) {
				return ($a->title <=> $b->title);
			});
			// pages with children
			$children = array_map(function ($row) use ($all) {
				$row->url = strToUrl($row->url);
				$row->label = !empty($all[$row->label]) ? $all[$row->label]->title : 0;
				if ($row->type !== "subPage" && $row->type !== "page") {
					$row->url = "{$row->type}/{$row->id}-{$row->url}";
				}
				return (object)array_extract($row, ["id", "title", "label", "type", "url"], true);
			}, $children);
			// Group the children into tag labels
			if (count($children)) {
				$value->child = array_group(array_values($children), "label");
			}
			return ($value);
		}, $pages);
		$json = _writeFile($file, array_values($json), true);
	}
	return (json_decode($json));
}

function get_homepage($createOnly = false)
{
	global $generic;
	$uri  = $generic->getURIdata();
	$dir = absolute_filepath($uri->site);
	$file = "{$dir}cache/home.json";
	$today = strtotime(date('Y-m-d h:i:s'));
	linkschool($uri, $createOnly);
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		$json = new stdClass;
		$nav = get_nav();
		$firstnav = is_array($nav) ? reset($nav) : null;

		$all = $generic->getFromTable("content", "type=article, type=category, type=product, type=page", 1, 0, DATE_UPDATED_DESC);
		$all = array_remap($all, array_column($all, "id"));
		$all = array_map(function ($value = '') {
			$value->body = substr(strip_tags(cleanUpTags($value->body)), 0, 200);
			$value->title = ucfirst($value->title);
			return $value;
		}, $all);

		$categ  = array_filter($all, function ($value = '') {
			return $value->type === "category";
		});
		// Insights
		$values 	= "id, title, category, url, body, type, image, last_updated, no_of_views";
		$insights = array_map(function ($value = '')  use ($values, $all, $categ, $uri) {
			if ($value->type == "article" && $value->status == 1) {

				$value->body = substr(strip_tags(cleanUpTags($value->body)), 0, 200);
				$value->image = json_decode($value->image);
				$value->image = reset($value->image);
				$value->image->src = str_replace("../", $uri->site, $value->image->src);
				if (!empty($value->category)) {
					$value->category_link = "{$uri->site}category/{$value->category}-" . strtourl($categ[$value->category]->title);
					$value->category = $categ[$value->category]->title;
				}
				return $value;
			}
		}, $all);
		$insights = array_range(array_filter($insights), 6);
		$json->insights = array_values($insights);

		// Products
		$values 	= "id, title, category, type, image, product_desc, url";
		$products = array_map(function ($value = '')  use ($values) {
			if ($value->type == "product" && $value->status == 1) {
				$value->body = substr(strip_tags(cleanUpTags($value->body)), 0, 200);
				$value->image = json_decode($value->image);
				$value->product_desc = json_decode($value->product_desc);
				$value = array_extract($value, array_map("trim", explode(",", $values)), true);
				return $value;
			}
		}, $all);
		$products = array_range(array_filter($products), 3);
		$json->products = array_values($products);

		if (!is_null($firstnav)) {
			//Page data
			$values 	= ["body", "image"];
			$homepage = $all[$firstnav->id];
			$homepage = object(array_extract($homepage, $values, true));
			$homepage->slider = json_decode($homepage->image);
			$json->page_data  = $homepage;
		}

		$json = _writeFile($file, $json, true);
	}
	return json_decode($json);
}

function contentCache($type, $id = 0, $createOnly = false)
{
	get_pages($createOnly);
	get_nav($createOnly);
	get_homepage($createOnly);

	global $generic;
	global $paramControl;
	$uri    = $generic->getURIdata();
	$params = $paramControl->get_params();
	$dir  = $dir1 = absolute_filepath($uri->site);

	if ($type == "subPage") {
		$data = $generic->getFromTable("content", "id={$id}");
		$type = strToUrl(reset($data)->url);
	} else if (empty($id)) {
		$subpages = json_decode(_readFile("{$dir}cache/subPages.json"), true);
		$id = array_search($type, $subpages);
	}

	$name = strToUrl("content-{$id}");
	$dir  = "{$dir}cache/{$type}/";
	$today = strtotime(date('Y-m-d h:i:s'));
	$file = "{$dir}{$name}.json";

	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		// Custom Params for subpages
		$param = $params->{$type};
		if (empty($data)) $data = $generic->getFromTable("content", "id={$id}");
		$data = reset($data);
		// Check if the content is published
		if (empty($data->status)) {
			if (file_exists($file)) unlink($file);
			return object(["status" => 1, "message" => "Cache cleared"]);
		} else {
			$threads =  ["article", "news", "caseStudy"];
			$data->body = cleanupHTML(cleanUpTags($data->body));
			// Format Images url
			$data->image = json_decode($data->image);
			if (json_last_error() == JSON_ERROR_NONE) {
				$data->image = array_map(function ($value = '') use ($uri) {
					$value->src = str_replace("../", $uri->site, $value->src);
					$value->src = str_replace("tbn/", "", $value->src);
					return $value;
				}, $data->image);
			}

			// category List
			get_nav();
			$allcategories = _readFile("{$dir1}cache/categories.json");
			$allcategories = json_decode($allcategories);

			if (!empty($param->related)) {
				if (in_array($type, $threads)) {
					$related = "category='{$data->category}' AND id!=$id";
				} else {
					$related = "{$param->related} AND {$data->type}='{$id}'";
				}
				$related = $generic::$mydb->query("SELECT id, title, body, category, image, url, type, date_uploaded, product_desc FROM content WHERE $related AND status='1' ORDER BY id DESC") or die($generic::$mydb->error);
				while ($item = $related->fetch_object()) {
					$item->body = substr(strip_tags(cleanUpTags($item->body)), 0, 200);
					$item->image = json_decode($item->image);
					$item->image = reset($item->image);
					$item->image->src = str_replace("../", $uri->site, $item->image->src);
					$item->title = ucfirst($item->title);
					if (!empty($item->category)) {
						$item->category_link = "{$uri->site}category/{$item->category}-" . strtourl($allcategories->{$item->category});
						$item->category = $allcategories->{$item->category};
					}
					$data->related[] = $item;
				}
			}
			if (!empty($data->category)) {
				categoryCache($data->category, $createOnly);
			}
			$listings = ["press-release", "caseStudy", "insights"];
			if (in_array($type, $listings)) {
				for ($i = 1; $i <= 10; $i++) {
					listCache($type, $i, $createOnly);
				}
			}
			$readings = ["business", "product", "industry", "training"];

			if (!empty($param->event) && $createOnly) {
				eventListCache($createOnly);
				eventCache($createOnly, $id);
			}

			if (!empty($param->job) && $createOnly) {
				jobListCache($createOnly);
				jobCache($createOnly, $id);
			}

			if (!empty($param->opportunity) && $createOnly) {
				programListCache($createOnly);
				programCache($createOnly, $id);
			}

			if (!empty($param->bootcamp) && $createOnly) {
				bootcampListCache($createOnly);
				bootcampCache($createOnly, $id);
			} elseif (!empty($param->coding_bootcamp) && $createOnly) {
				codingBootcampCache($createOnly, $id);
			}

			// ads
			if (in_array($type, $threads)) {
				$adHeight = isMobile() == true ? '250' : '90';
				$advert = '<div class="goole_ad col s12 mb-3" style="display:flex; height:auto"><script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
				<ins class="adsbygoogle"
				style="display:inline-block;width:100%;height:' . $adHeight . 'px"
				data-ad-client="ca-pub-4696224049420135"
				data-ad-slot="1880693933"></ins>
				<script>
				(adsbygoogle = window.adsbygoogle || []).push({});
					</script></div>';
				list($data->body, $wordCount) = insert($data->body, 300, $advert);
				if (!empty($data->author)) {
					$data->author = $generic->getFromTable("users", "id={$data->author}", 1, 1, null, false, ["id", "picture_ref", "first_name as name", "email", "phone", "website", "bio"]);
					$data->author = reset($data->author);
				}
			}

			$json = _writeFile($file, $data, true);
		}
	}
	return json_decode($json);
}

function categoryCache($id, $page = 1, $createOnly = false)
{
	global $generic;
	$uri  = $generic->getURIdata();
	$dir1 = absolute_filepath($uri->site);
	$page = $page || 1;
	$dir  = "{$dir1}cache/category/";
	$file = "{$dir}category-{$id}-{$page}.json";
	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		get_nav();
		$allcategories = _readFile("{$dir1}cache/categories.json");
		$allcategories = json_decode($allcategories);

		$json = $generic->getFromTable("content", "type=article, category={$id}, status=1", $page, 20, DATE_UPLOADED_DESC);
		$json = array_map(function ($item) use ($uri, $allcategories) {
			$item->body = substr(strip_tags(cleanUpTags($item->body)), 0, 200);
			$item->image = json_decode($item->image);
			$item->image = reset($item->image);
			$item->title = ucfirst($item->title);
			$item->image->src = str_replace("../", $uri->site, $item->image->src);
			if (!empty($item->category)) {
				$item->category_link = "{$uri->site}category/{$item->category}-" . strtourl($allcategories->{$item->category});
				$item->category = $allcategories->{$item->category};
			}
			return $item;
		}, $json);
		$json = _writeFile($file, $json, true);
	}
	return json_decode($json);
}

function listCache($type, $page = 1, $createOnly = false)
{
	global $generic;
	global $paramControl;
	$page = $page || 1;
	$param = $paramControl->get_params($type);
	$uri  = $generic->getURIdata();
	$dir1 = absolute_filepath($uri->site);
	$dir  = "{$dir1}cache/{$type}/";
	$file = "{$dir}{$type}-file-{$page}.json";

	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		get_nav();
		$allcategories = _readFile("{$dir1}cache/categories.json");
		$allcategories = json_decode($allcategories);

		$json = $generic->getFromTable("content", "{$param->retrieve_filter}, status=1", $page, 20, DATE_UPLOADED_DESC);
		$json = array_map(function ($item) use ($uri, $allcategories) {
			$item->body = substr(strip_tags(cleanUpTags($item->body)), 0, 200);
			$item->image = json_decode($item->image);
			$item->image = reset($item->image);
			$item->image->src = str_replace("../", $uri->site, $item->image->src);
			if (!empty($item->category)) {
				$item->category_link = "{$uri->site}category/{$item->category}-" . strtourl($allcategories->{$item->category});
				$item->category = $allcategories->{$item->category};
			}
			return $item;
		}, $json);
		$json = _writeFile($file, $json, true);
	}
	return json_decode($json);
}

function eventListCache($createOnly = false)
{
	global $generic;
	$uri  = $generic->getURIdata();
	$dir1 = absolute_filepath($uri->site);
	$dir  = "{$dir1}cache/events/";
	$file = "{$dir}all-events.json";

	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		$json = $generic->getFromTable("content", "type=event1, type=event2, status=1", 1, 0, DATE_UPDATED_ASC, false, 'id, title, image, body, date_updated, type, auto as location');
		$json = array_map(function ($item) use ($uri) {
			$item->body = truncate_txt(strip_tags(cleanUpTags($item->body)));
			$item->image = json_decode($item->image);
			$item->image = reset($item->image);
			$item->image->src = str_replace(["../", 'tbn/'], [$uri->site, ''], $item->image->src);
			$datetime = new DateTime($item->date_updated);
			$item->date_updated = ['date' => $datetime->format('d M Y'), 'time' => $datetime->format('h:i a')];
			return $item;
		}, $json);
		$json = _writeFile($file, $json, true);
	}
	return json_decode($json);
}

function eventCache($createOnly = false, $id = 0)
{
	global $generic;
	$uri  = $generic->getURIdata();
	$dir1 = absolute_filepath($uri->site);
	$dir  = "{$dir1}cache/events/";
	$file = "{$dir}event-$id.json";

	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		$json = $generic->getFromTable("content", "id=$id, status=1", 1, 1, '', false, 'id, title, image, body, date_updated, url, type, industry, business, product, auto as location')[0];
		$json->body = strip_tags(cleanUpTags($json->body));
		$json->image = array_map(function ($img) use ($uri) {
			$img['src'] = str_replace(["../", 'tbn/'], [$uri->site, ''], $img['src']);
			return $img;
		}, json_decode($json->image, true));
		$datetime = new DateTime($json->date_updated);
		$json->date_updated = ['date' => $datetime->format('d M Y'), 'time' => $datetime->format('h:i a')];
		$json->related = $generic->getFromTable('content', "id!=$id, status=1, type=event1, type=event2", 1, 5, DATE_UPDATED_ASC, false, 'id, title, image, date_updated, auto as location');
		foreach ($json->related as $related) {
			$datetime = new DateTime($related->date_updated);
			$related->date_updated = [$datetime->format('d M Y'), $datetime->format('h:i a')];
			$related->image = json_decode($related->image);
			$related->image = reset($related->image)->src;
		}
		$json = _writeFile($file, $json, true);
	}
	return json_decode($json);
}

function bootcampListCache($createOnly = false)
{
	global $generic;
	$uri  = $generic->getURIdata();
	$dir1 = absolute_filepath($uri->site);
	$dir  = "{$dir1}cache/bootcamps/";
	$file = "{$dir}all-bootcamps.json";

	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		$json = $generic->getFromTable("content", "type=bootcamp, status=1", 1, 0, '', false, "id, title, body as description, image, product as course_id, label as price, auto as start_date, parent as duration, product_desc as instructors");
		foreach ($json as $item) {
			$item->start_date = date_format(date_create($item->start_date), 'jS F, Y');
			$item->price = number_format($item->price);
			$item->description = truncate_txt(strip_tags(cleanUpTags($item->description)));
			if (!empty($item->instructors)) $item->instructors = json_decode($item->instructors);
			$item->image = json_decode($item->image);
			$item->image = reset($item->image);
			$item->image->src = str_replace(["../", 'tbn/'], [$uri->site, ''], $item->image->src);
			$sql = "SELECT name FROM category WHERE id=(SELECT category FROM academy_courses WHERE id={$item->course_id})";
			$item->category = $generic::$mydb->query($sql)->fetch_object()->name;
		}
		$json = _writeFile($file, $json, true);
	}
	return json_decode($json);
}

function bootcampCache($createOnly = false, $id = 0)
{
	global $generic;
	$uri  = $generic->getURIdata();
	$dir1 = absolute_filepath($uri->site);
	$dir  = "{$dir1}cache/bootcamps/";
	$file = "{$dir}bootcamp-$id.json";

	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		$json = $generic->getFromTable("content", "id=$id, status=1", 1, 1, '', false, "id, title, image, body as description, product as course_id, label as price, auto as start_date, parent as duration, product_desc as instructors");
		if (empty($json)) {
			if (file_exists($file)) unlink($file);
			return new stdClass;
		}
		$json = $json[0];
		$json->image = json_decode($json->image);
		if (!empty($json->instructors)) $json->instructors = json_decode($json->instructors);
		$json->image = reset($json->image);
		$json->image->src = str_replace(["../", 'tbn/'], [$uri->site, ''], $json->image->src);
		$json->description = cleanUpTags($json->description);
		$json->start_date = date_format(date_create($json->start_date), 'jS F, Y');
		$json->price = number_format($json->price);
		$json->course = $generic->getFromTable('academy_courses', "id={$json->course_id}", 1, 1, '', false, 'name')[0]->name;
		$json = _writeFile($file, $json, true);
	}
	return json_decode($json);
}

function codingBootcampCache($createOnly = false, $id = 0)
{
	global $generic;
	$uri  = $generic->getURIdata();
	$dir1 = absolute_filepath($uri->site);
	$dir  = "{$dir1}cache/bootcamps/";
	$file = "{$dir}coding-bootcamp.json";

	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true) {
		$json = _readFile($file);
	} else {
		$json = $generic->getFromTable("content", "id=$id, status=1", 1, 1, '', false, "id, title, image, body, product as genius_price, label as marvel_price, auto as genius_start, view as marvel_start, url, no_of_views as duration, product_desc as instructors")[0];
		$json->image = json_decode($json->image);
		$json->image = array_map(function ($img) use ($uri) {
			$img->src = str_replace(["../", 'tbn/'], [$uri->site, ''], $img->src);
			return $img;
		}, $json->image);
		if (!empty($json->instructors)) $json->instructors = json_decode($json->instructors);
		$json->body = cleanupHTML(cleanUpTags($json->body));
		$json->genius_start = date_format(date_create($json->genius_start), 'jS F, Y');
		$json->marvel_start = date_format(date_create($json->marvel_start), 'jS F, Y');
		$json->genius_price = number_format($json->genius_price);
		$json->marvel_price = number_format($json->marvel_price);
		$json = _writeFile($file, $json, true);
	}
	return json_decode($json);
}

function programListCache($createOnly = false, $categories = [], $search = '', $filters = [])
{
	global $generic, $paramControl;
	$uri  = $generic->getURIdata();
	$dir1 = absolute_filepath($uri->site);
	$dir  = "{$dir1}cache/opportunity/";
	$file = "{$dir}all-opportunity.json";

	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		$json = $generic->getFromTable("content", "type in ('opportunity', 'contest', 'seminar'), status=1, date is null, parse_date >= NOW()", 1, 0, LABEL_DESC . ', parse_date ASC', false, "id, title, body as description, image, business as company, auto as location, view as date, IF(TRIM(COALESCE(SUBSTRING_INDEX(view, '-', -1), '')) LIKE '%/%', STR_TO_DATE(TRIM(COALESCE(SUBSTRING_INDEX(view, '-', -1), '')), '%m/%d/%Y'), STR_TO_DATE(TRIM(COALESCE(SUBSTRING_INDEX(view, '-', -1), '')),'%Y-%m-%dT%H:%i')) as parse_date, category, type");
		$opportunity_type = $paramControl->load_sources('opportunity_type');
		$employers = $paramControl->load_data_from_param('employers');
		foreach ($json as $item) {
			if (!empty($item->date)) {
				if (strpos($item->date, ' - ') !== false) {
					$range = array_map(function ($date_str) {
						return DateTime::createFromFormat('m/d/Y', $date_str)->format('jS M Y');
					}, explode(' - ', $item->date));
					$item->date = join(' to ', $range);
				} else {
					$item->date = date_format(date_create($item->date), 'j M Y; g:i A');
				}
			}
			if ($item->type == 'opportunity' and !empty($item->category)) $item->category = $opportunity_type[$item->category];
			else $item->category = ucwords($item->type);
			$item->company = $employers[$item->company];
			$item->description = truncate_txt(strip_tags(cleanUpTags($item->description)));
			$item->image = json_decode($item->image);
			$item->image = reset($item->image);
			$item->image->src = str_replace(["../", 'tbn/'], [$uri->site, ''], $item->image->src);
		}
		$json = _writeFile($file, $json, true);
	}
	$programs = array_filter(json_decode($json), function ($program) {
		return strtotime($program->parse_date) === false || DateTime::createFromFormat('j M Y', $program->parse_date) >= new DateTime();
	});
	if (!empty($categories)) {
		$programs = array_filter($programs, function ($program) use ($categories) {
			return in_array(strtolower($program->category), array_map('strtolower', $categories));
		});
	}
	if (!empty($search)) {
		$programs = array_filter($programs, function ($program) use ($search) {
			similar_text(strtolower($program->title), strtolower($search), $percent1);
			similar_text(strtolower($program->category), strtolower($search), $percent2);
			$program->score = max($percent1, $percent2);
			return $program->score > 50;
		});
		usort($programs, function ($contest1, $contest2) {
			return $contest2->score <=> $contest1->score;
		});
	}
	if (!empty($filters)) {
		$programs = array_filter($programs, function ($program) use ($filters) {
			foreach ($filters as $key => $filter) {
				if (stripos($program->{$key}, $filter) !== FALSE) {
					return true;
				}
			}
			return false;
		});
	}
	return $programs;
}

function programCache($createOnly = false, $id = 0)
{
	global $generic, $paramControl;
	$uri  = $generic->getURIdata();
	$dir1 = absolute_filepath($uri->site);
	$dir  = "{$dir1}cache/opportunity/";
	$file = "{$dir}opportunity-$id.json";

	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		$json = $generic->getFromTable("content", "id=$id, status=1", 1, 1, '', false, "id, title, image, body as description, date_uploaded, business as company, auto as location, view as date, industry as participants, category, type, url, parent as contact_info, product as brand");
		if (empty($json)) {
			if (file_exists($file)) unlink($file);
			return new stdClass;
		}
		$json = $json[0];
		$opportunity_type = $paramControl->load_sources('opportunity_type');
		$tech_roles = $paramControl->load_sources('tech_roles');
		$employers = $paramControl->load_data_from_param('employers');
		$json->image = json_decode($json->image);
		$json->image = reset($json->image);
		$json->image->src = str_replace(["../", 'tbn/'], [$uri->site, ''], $json->image->src);
		$json->description = cleanUpTags($json->description);
		if (!empty($json->participants)) {
			$json->participants = explode(',', $json->participants);
			$json->participants = array_map(function ($p) use ($tech_roles) {
				return $tech_roles[intval($p)];
			}, $json->participants);
		}
		if (!empty($json->date)) {
			if (strpos($json->date, ' - ') !== false) {
				$range = array_map(function ($date_str) {
					return DateTime::createFromFormat('m/d/Y', $date_str)->format('jS M Y');
				}, explode(' - ', $json->date));
				$json->date = join(' to ', $range);
			} else {
				$json->date = date_format(date_create($json->date), 'j M Y; g:i A');
			}
		}
		$json->date_uploaded = date_format(date_create($json->date_uploaded), 'j M Y');
		if ($json->type == 'opportunity' and !empty($json->category)) $json->category = $opportunity_type[$json->category];
		else $json->category = ucwords($json->type);
		$json->company = $employers[$json->company];

		$json->related = $generic->getFromTable('content', "id!=$id, status=1, type={$json->category}, date is null, parse_date >= NOW()", 1, 5, LABEL_DESC . ', parse_date ASC', false, "id, title, body as description, image, business as company, auto as location, view as date, IF(TRIM(COALESCE(SUBSTRING_INDEX(view, '-', -1), '')) LIKE '%/%', STR_TO_DATE(TRIM(COALESCE(SUBSTRING_INDEX(view, '-', -1), '')), '%m/%d/%Y'), STR_TO_DATE(TRIM(COALESCE(SUBSTRING_INDEX(view, '-', -1), '')),'%Y-%m-%dT%H:%i')) as parse_date, category, type");
		foreach ($json->related as $related) {
			if (!empty($related->date)) {
				if (strpos($related->date, ' - ') !== false) {
					$range = array_map(function ($date_str) {
						return DateTime::createFromFormat('m/d/Y', $date_str)->format('jS M Y');
					}, explode(' - ', $related->date));
					$related->date = join(' to ', $range);
				} else {
					$related->date = date_format(date_create($related->date), 'j M Y; g:i A');
				}
			}
			if ($related->type == 'opportunity' and !empty($related->category)) {
				$related->category = $opportunity_type[$related->category];
			} else {
				$related->category = ucwords($related->type);
			}
			$related->company = $employers[$related->company];
			$related->description = truncate_txt(strip_tags(cleanUpTags($related->description)));
			$related->image = json_decode($related->image);
			$related->image = reset($related->image);
			$related->image->src = str_replace(["../", 'tbn/'], [$uri->site, ''], $related->image->src);
		}
		$json = _writeFile($file, $json, true);
	}
	$program = json_decode($json);
	$program->related = array_filter($program->related, function ($rel_program) {
		return strtotime($rel_program->parse_date) === false || DateTime::createFromFormat('j M Y', $rel_program->parse_date) >= new DateTime();
	});
	return $program;
}

function jobListCache($createOnly = false, $categories = [], $search = '', $filters = [])
{
	global $generic, $paramControl;
	$uri  = $generic->getURIdata();
	$dir1 = absolute_filepath($uri->site);
	$dir  = "{$dir1}cache/jobs/";
	$file = "{$dir}all-jobs.json";

	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		$json = $generic->getFromTable("content", "type=job, status=1, close_date is null, close_date >= NOW()", 1, 0, LABEL_DESC . ', close_date ASC', false, 'id, title as position, image, business as employer_name, auto as location, view as close_date, author as country, product as work_type, product_desc as salary, industry as qualification, category, date_uploaded');
		$states = $paramControl->load_sources('states');
		$countries = $paramControl->load_sources('countries');
		$work_types = $paramControl->load_sources('work_type');
		$job_cat = $paramControl->load_data_from_param('job_cat');
		$employers = $paramControl->load_data_from_param('employers');
		foreach ($json as $item) {
			if (!empty($item->close_date)) $item->close_date = date_format(date_create($item->close_date), 'j M Y');
			if (!empty($item->location)) {
				$item->location = $states->{$item->location};
				$item->country = 'Nigeria';
			} elseif (!empty($item->country)) {
				$item->location = $item->country =  $countries[$item->country];
			}
			$item->date_uploaded = date_format(date_create($item->date_uploaded), 'j M Y');
			$item->category = $job_cat[$item->category];
			$item->employer_name = $employers[$item->employer_name];
			$item->work_type = $work_types[$item->work_type];
		}
		$json = _writeFile($file, $json, true);
	}
	$jobs = array_filter(json_decode($json), function ($job) {
		return strtotime($job->close_date) === false || DateTime::createFromFormat('j M Y', $job->close_date) >= new DateTime();
	});
	if (!empty($categories)) {
		$jobs = array_filter($jobs, function ($job) use ($categories) {
			return in_array(strtolower($job->category), array_map('strtolower', $categories));
		});
	}
	if (!empty($search)) {
		$jobs = array_filter($jobs, function ($job) use ($search) {
			similar_text(strtolower($job->position), strtolower($search), $percent1);
			similar_text(strtolower($job->category), strtolower($search), $percent2);
			$job->score = max($percent1, $percent2);
			return $job->score > 50;
		});
		usort($jobs, function ($job1, $job2) {
			return $job2->score <=> $job1->score;
		});
	}
	if (!empty($filters)) {
		$jobs = array_filter($jobs, function ($job) use ($filters) {
			foreach ($filters as $key => $filter) {
				if ($key == 'salary') {
					if (salaryInRange($job->salary, $filter)) return true;
				} elseif ($filter == $job->{$key}) {
					return true;
				}
			}
			return false;
		});
	}
	return $jobs;
}

function jobCache($createOnly = false, $id = 0)
{
	global $generic, $paramControl;
	$uri  = $generic->getURIdata();
	$dir1 = absolute_filepath($uri->site);
	$dir  = "{$dir1}cache/jobs/";
	$file = "{$dir}job-$id.json";

	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 7)) {
		$json = _readFile($file);
	} else {
		$json = $generic->getFromTable("content", "id=$id, status=1", 1, 1, '', false, 'id, title as position, image, body as description, date_uploaded, url, industry as qualification, business as employer_name, auto as location, view as close_date, category, parent as employer_email, product as work_type, author as country, product_desc as salary');
		if (empty($json)) {
			if (file_exists($file)) unlink($file);
			return new stdClass;
		}
		$json = $json[0];
		$states = $paramControl->load_sources('states');
		$countries = $paramControl->load_sources('countries');
		$work_types = $paramControl->load_sources('work_type');
		$education_levels = $paramControl->load_sources('education_level');
		$job_cat = $paramControl->load_data_from_param('job_cat');
		$employers = $paramControl->load_data_from_param('employers');
		$json->description = cleanUpTags($json->description);
		if (!empty($json->close_date)) $json->close_date = date_format(date_create($json->close_date), 'j M Y');
		$json->date_uploaded = date_format(date_create($json->date_uploaded), 'j M Y');
		$json->work_type = $work_types[$json->work_type];
		if (!empty($json->qualification)) $json->qualification = $education_levels->{$json->qualification};
		if (!empty($json->location)) {
			$json->location = $states->{$json->location};
			$json->country = 'Nigeria';
		} elseif (!empty($json->country)) {
			$json->location = $json->country =  $countries[$json->country];
		}
		$json->related = $generic->getFromTable('content', "id!=$id, status=1, type=job, category={$json->category}, close_date is null, close_date >= NOW()", 1, 5, LABEL_DESC . ', close_date ASC', false, 'id, title as position, image, business as employer_name, auto as location, product as work_type, view as close_date, author as country, product_desc as salary, category, date_uploaded');
		$json->category = $job_cat[$json->category];
		$json->employer_name = $employers[$json->employer_name];
		foreach ($json->related as $related) {
			if (!empty($related->close_date)) $related->close_date = date_format(date_create($related->close_date), 'j M Y');
			if (!empty($related->location)) {
				$related->location = $states->{$related->location};
				$related->country = 'Nigeria';
			} elseif (!empty($related->country)) {
				$related->location = $related->country =  $countries[$related->country];
			}
			$related->date_uploaded = date_format(date_create($related->date_uploaded), 'j M Y');
			$related->work_type = $work_types[$related->work_type];
			$related->category = $job_cat[$related->category];
			$related->employer_name = $employers[$related->employer_name];
		}
		$json = _writeFile($file, $json, true);
	}
	$job = json_decode($json);
	$job->related = array_filter($job->related, function ($rel_job) {
		return DateTime::createFromFormat('j M Y', $rel_job->close_date) >= new DateTime();
	});
	return $job;
}

function APICourses($createOnly = false, $categories = [], $search = '')
{
	global $generic;
	$uri = $generic->getURIdata();
	$dir = absolute_filepath($uri->site);
	$dir  = "{$dir}cache/training/";
	$file = "{$dir}courses.json";

	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 0)) {
		$json = _readFile($file);
	} else {
		$courses = $generic->getFromTable('academy_courses', "available=1", 1, 0, '', false, "id, name, curriculum, bg_ref, pic_ref, students, duration, contents, category, executive_price, group_price, online_price, syllabus");
		$cat_ids = array_unique(array_map(function ($course) {
			return $course->category;
		}, $courses));
		$cat_ids = count($cat_ids) > 1 ? 'id in (' . join(',', $cat_ids) . ')' : "id={$cat_ids[0]}";
		$cats = $generic->getFromTable('category', $cat_ids, 1, 0, '', false, 'id, name');
		$cats = array_map(function ($cat) {
			return $cat[0]->name;
		}, array_group($cats, 'id'));
		foreach ($courses as $course) {
			$course->category = $cats[$course->category];
			$course->bg_ref = str_replace('tbn/', '', $course->bg_ref);
			$course->pic_ref = str_replace('tbn/', '', $course->pic_ref);
			$course->topic_count = $generic->getFromTable('academy_lessons', "course_id={$course->id}, approved=1", 1, 0, '', false, 'COUNT(*) AS count')[0]->count;
			$course->contents = substr(strip_html_tags($course->contents), 0, 80);
		}
		// $json = curl_post(["url"=>"https://academy.digitaldreamstudios.net/api/v1/?type=courses"]);
		$json = _writeFile($file, $courses, true);
	}
	$filtered_courses = json_decode($json);
	if (isset($_SESSION['student_id'])) {
		$id = $_SESSION['student_id'];
		$exempted_courses = $generic->getFromTable('enrollments', "student_id=$id", 1, 0, '', false, 'course_id');
		if (count($exempted_courses) > 0) {
			$filtered_courses = array_filter($filtered_courses, function ($course) use (&$exempted_courses) {
				$include = true;
				foreach ($exempted_courses as $exempted_course) {
					if ($exempted_course->course_id == $course->id) {
						$include = false;
						break;
					}
				}
				return $include;
			});
		}
	}
	if (!empty($categories)) {
		$filtered_courses = array_filter($filtered_courses, function ($course) use ($categories) {
			return in_array(strtolower($course->category), array_map('strtolower', $categories));
		});
	}
	if (!empty($search)) {
		$filtered_courses = array_filter($filtered_courses, function ($course) use ($search) {
			similar_text(strtolower($course->name), strtolower($search), $percent1);
			similar_text(strtolower($course->category), strtolower($search), $percent2);
			$course->score = max($percent1, $percent2);
			return $course->score > 50;
		});
		usort($filtered_courses, function ($course1, $course2) {
			return $course2->score <=> $course1->score;
		});
	}
	return $filtered_courses;
}

function APICourse($course_id = 0, $createOnly = false)
{
	global $generic;
	$uri = $generic->getURIdata();
	$dir = absolute_filepath($uri->site);
	$dir  = "{$dir}cache/training/";
	$file = "{$dir}course-{$course_id}.json";

	$today = strtotime(date('Y-m-d h:i:s'));
	if (!is_dir($dir)) {
		mkdir($dir, 0777, true);
	}
	if (file_exists($file) && $createOnly !== true && (round(($today - filemtime($file)) / (3600 * 24)) < 0)) {
		$json = _readFile($file);
	} else {
		// $json = curl_post(["url"=>"https://academy.digitaldreamstudios.net/api/v1/?type=course&course_id={$course_id}"]);
		$course = $generic->getFromTable('academy_courses', "id=$course_id", 1, 1, '', false, "id, name, curriculum, bg_ref, pic_ref, students, duration, contents, category, executive_price, group_price, online_price, syllabus")[0];
		$obj = new stdClass;
		$obj->course_id = $course_id;
		$excluded_ids = [$obj];
		$excluded_ids += isset($_SESSION['student_id']) ? $generic->getFromTable('enrollments', "student_id={$_SESSION['student_id']}", 1, 0, '', false, 'course_id') : [];
		$excluded_ids = ', id NOT IN (' . join(',', array_map(function ($obj) {
			return $obj->course_id;
		}, $excluded_ids)) . ", $course_id)";
		$course->top_courses = $generic->getFromTable('academy_courses', "available=1$excluded_ids", 1, 5, '', false, 'id AS course_id, name, bg_ref, pic_ref, contents AS description, duration, group_price, executive_price, online_price');
		$course->related_courses = $generic->getFromTable('academy_courses', "available=1, category={$course->category}$excluded_ids", 1, 5, '', false, 'id AS course_id, name, bg_ref, pic_ref, contents AS description, duration, group_price, executive_price, online_price');
		$course->category = $generic->getFromTable('category', "id={$course->category}", 1, 1, '', false, 'name')[0]->name;
		$course->bg_ref = str_replace('tbn/', '', $course->bg_ref);
		$course->pic_ref = str_replace('tbn/', '', $course->pic_ref);
		$course->duration = $course->duration . ' week' . ($course->duration > 1 ? 's' : '');
		foreach ([$course->top_courses, $course->related_courses] as $other_courses) {
			foreach ($other_courses as $other_course) {
				$other_course->bg_ref = str_replace('tbn/', '', $other_course->bg_ref);
				$other_course->pic_ref = str_replace('tbn/', '', $other_course->pic_ref);
				$other_course->topic_count = $generic->getFromTable('academy_lessons', "course_id={$other_course->course_id}, approved=1", 1, 0, '', false, 'COUNT(*) AS count')[0]->count;
				$other_course->description = substr(strip_html_tags($other_course->description), 0, 80);
			}
		}
		$course->topics = $generic->getFromTable('academy_lessons', "course_id={$course->id}, approved=1", 1, 0, SERIAL_NO_ASC, false, 'title, lessons');
		foreach ($course->topics as $topic) {
			$lessons = json_decode($topic->lessons)[0];
			$topic->lessons = $lessons->description ?? $lessons->title;
		}
		$json = _writeFile($file, $course, true);
	}
	return json_decode($json);
}

function talentProfile($id)
{
	global $generic;
	$talent = $generic->getFromTable('users', "id=$id", 1, 1, '', false, "CONCAT(first_name,' ', last_name) as name, email, phone, picture_ref, bio, other as profile")[0];
	$talent->profile = !empty($talent->profile) ? json_decode($talent->profile, true) : ['location' => '', 'workplace' => '', 'other' => '', 'title' => '', 'status' => '', 'open' => 1, 'skills' => [], 'education' => [], 'experience' => [], 'projects' => [], 'socials' => ['facebook' => '', 'x' => '', 'linkedIn' => '', 'github' => '']];
	$talent->profile['workplace'] = $generic->getFromTable('content', "id={$talent->profile['workplace']}", 1, 1, '', false, 'title as name, image')[0] ?? null;
	$experience = $talent->profile['experience'];
	foreach ($experience as $key => $exp) {
		$experience[$key]['logo'] = $generic->getFromTable('content', "title={$exp['firm']}", 1, 1, '', false, 'image')[0]?->image ?? null;
	}
	$similar_talents = $exempt_arr = [];
	$loop = true;
	$mul = 2;
	$fetch_count = 10;
	do {
		$len = count($similar_talents);
		$exempt_len = count($exempt_arr);
		$exempt_ids = $exempt_len > 1 ? ', id not in (' . join(', ', $exempt_arr) . ')' : ($exempt_len ? ", id!={$exempt_arr[0]}" : '');
		$more_talents = $generic->getFromTable('users', "id!=$id, profile is not null, academy_type=0$exempt_ids", 1, $fetch_count * $mul, 'RAND()', false, "id, CONCAT(first_name, ' ', last_name) as name, picture_ref, other as profile");
		foreach ($more_talents as $obj) {
			$obj->profile = !empty($obj->profile) ? json_decode($obj->profile) : ['location' => '', 'title' => '', 'status' => '', 'open' => 1];
		}
		$more_talents = array_filter($more_talents, function ($obj) use ($talent) {
			similar_text($talent->profile['title'], $obj->profile['title'], $percent);
			return $percent > 60;
		});
		if ($num = count($more_talents)) {
			if ($num > $fetch_count - $len) $more_talents = array_slice($more_talents, 0, $fetch_count - $len);
			else $exempt_arr = array_merge($exempt_arr, array_map(function ($obj) {
				return $obj->id;
			}, $similar_talents));
			$loop = true;
			$mul *= 2;
			$similar_talents = array_merge($similar_talents, $more_talents);
		} else {
			$loop = false;
		}
	} while (count($similar_talents) < $fetch_count && $loop);
	return [$talent, $similar_talents];
}

function talentProfiles($page = 1, $page_size = 50, $categories = [], $search = '', $filters = [])
{
	global $db, $paramControl;
	$talents = [];
	$talent_cats = $paramControl->load_sources('tech_roles');
	if (!empty($categories)) $talent_cats = array_intersect($talent_cats, $categories);
	$sql = '';
	$page = ($page - 1) * $page_size;
	if ($search != '') {
		$search =  "'%" . $db->real_escape_string($search) . "%'";
		$search = " AND (name LIKE $search OR title LIKE $search)";
	}
	foreach ($talent_cats as $cat) {
		$cat = $db->real_escape_string($cat);
		$sql .= "SELECT id, name, picture_ref, bio, other AS profile, title FROM (
    SELECT id, CONCAT(first_name, ' ', last_name) AS name, picture_ref, bio, other, JSON_UNQUOTE(JSON_EXTRACT(other, '$.title')) AS title FROM users WHERE academy_type = 0 AND other IS NOT NULL) AS subquery WHERE title = '$cat'$search ORDER BY RAND() LIMIT $page, $page_size;";
	}
	(substr_count($sql, ';') > 1 ? $db->multi_query($sql) : ($result = $db->query($sql))) or die($db->error);
	if (!isset($result)) {
		do {
			if ($result = $db->store_result()) {
				while ($row = $result->fetch_object()) {
					$row->profile = json_decode($row->profile);
					$talents[$row->profile->title][] = $row;
				}
				$result->free();
			}
		} while ($db->next_result());
	} else {
		while ($row = $result->fetch_object()) {
			$row->profile = json_decode($row->profile);
			$talents[$row->profile->title][] = $row;
		}
	}
	if (!empty($filters)) {
		$talents = array_filter($talents, function ($talent) use ($filters) {
			foreach ($filters as $key => $filter) {
				if (strtolower($filter) == strtolower($talent->profile->{$key})) {
					return true;
				}
			}
			return false;
		});
	}
	return $talents;
}
