let scrolling = false, youtubePlayer;
$(document).ready(function () {
  let mainMargin = parseInt($("#main").css("margin-left"));
  $(window).on({
    scroll: function () {
      const scrollAmt = $(window).scrollTop();

      if (!$(".classroom-banner")[0]) {
        if (scrolling) scrolling = false;
      } else {
        const docHeight = $(document).height(),
          winHeight = $(window).height(),
          footerHeight = $("footer").innerHeight(),
          threshold = docHeight - footerHeight - winHeight,
          sidebarWidth = $(window).width() > 992 ? 220 : 0,
          adjLeft = sidebarWidth - mainMargin,
          paneWidth = $(".aside-pane").prop("curWidth") ?? 350;
        // console.log('logs', sidebarWidth, adjLeft, mainMargin, paneWidth);

        if (scrollAmt > threshold) {
          if (scrolling) return;
          scrolling = true;
          const bannerTop = $(".classroom-banner").position().top + scrollAmt,
            toggleBtnTop = $(".toggle-side-pane").offset().top - 83;
          $(".classroom-banner, .aside-pane").css({
            position: "absolute",
            left: adjLeft + "px",
          });
          $(".classroom-banner")
            .removeClass("fixed-top")
            .css({
              top: `calc(${bannerTop}px - .9rem)`,
              "margin-top": "0",
              right: "0",
            });
          $(".aside-pane")
            .removeClass("fixed-bottom")
            .css({ bottom: "0", "margin-bottom": "-2.2rem" });
          $(".toggle-side-pane").css({
            left: `${(paneWidth ? 350 : 0) + adjLeft}px`,
            position: "absolute",
            top: toggleBtnTop + "px",
          });
        } else if (scrolling) {
          scrolling = false;
          $(".classroom-banner, .aside-pane").css({
            position: "fixed",
            left: adjLeft + "px",
          });
          $(".classroom-banner")
            .addClass("fixed-top")
            .css({ top: "0", "margin-top": "4rem" });
          $(".aside-pane").addClass("fixed-bottom").css("margin-bottom", "0");
          $(".toggle-side-pane").css({ top: "140px", position: "fixed" });

          if ($(window).width() > 992) {
            $(".aside-pane, .classroom-banner").css("left", "220px");
            if (paneWidth) $(".toggle-side-pane").css("left", "570px");
            else $(".toggle-side-pane").css("left", "220px");
          } else if (paneWidth) {
            $(".toggle-side-pane").css("left", "350px");
          }
        }
      }
    },
    resize: function () {
      mainMargin = parseInt($("#main").css("margin-left"));
    },
    unload: disposeYTPlayer,
  });
});

function prepareClassroom(course) {
  const coursework = course.coursework;
  const { records, completed } = coursework;
  records.progress ??= { topic: course.topics[0].id };
  records.quizes ??= [];
  records.assignments ??= [];

  let currentElem;
  $("#classroomHeader li:first-child a").click(gotoPage);
  $("#classroomHeader li:nth-child(2)")
    .text(course.name)
    .prop("course_id", course.id);
  const courseNav = $("#courseNav").empty(),
    topicsLen = course.topics.length;

  for (let i = 0; i < topicsLen; i++) {
    const topic = course.topics[i],
      current = topic.id == records.progress.topic ? " current" : "",
      ul = $("<ul>")
        .addClass("list-unstyled")
        .appendTo(
          $("<div>")
            .addClass(`accordion-collapse collapse${current ? " show" : ""}`)
            .attr({
              id: `collapseTopic${i}`,
              "aria-labelledby": `courseTopic${i}`,
              "data-parent": "#courseNav",
            })
            .css("width", "350px")
            .appendTo(
              $("<div>")
                .addClass("accordion-item")
                .appendTo(courseNav)
                .append(
                  $("<h6>")
                    .addClass("px-3 pt-3")
                    .css("width", "350px")
                    .attr({ id: `courseTopic${i}` })
                    .append(
                      $("<button>")
                        .data({ topic, type: "topic" })
                        .addClass(
                          `accordion-button text-dark font-weight-bold collapsed${current}`
                        )
                        .attr({
                          "data-toggle": "collapse",
                          "data-target": `#collapseTopic${i}`,
                          "aria-expanded": "true",
                          "aria-controls": `collapseTopic${i}`,
                        })
                        .text(topic.title)
                        .click(function () {
                          if ($(this).hasClass("current")) return;
                          $(".current", courseNav)
                            .removeClass("current")
                            .find("i")
                            .toggleClass("fas far");
                          $(this).addClass("current");
                          $("#classroomHeader li:nth-child(3)").text(
                            topic.title
                          );
                          animateMainPane(() =>
                            loadLesson(topic, "topic", coursework, course.id)
                          );
                        })
                    )
                )
            )
        );

    if (current) {
      currentElem = $(`#courseTopic${i} button`);
      $("#classroomHeader li:nth-child(3)").text(topic.title);
    }

    for (const lesson of topic.lessons) {
      const curLesson = current && records.progress.lesson == lesson.key,
        icon =
          lesson.type == "material"
            ? "fa-play-circle"
            : lesson.type == "quiz"
            ? "fa-file-alt"
            : "fa-file-code",
        li = $("<li>")
          .addClass("mb-1")
          .appendTo(ul)
          .append(
            $("<a>")
              .data({ lesson, type: "lesson" })
              .addClass(
                `list-group-item text-dark text-decoration-none bg-transparent text-wrap border-0${
                  curLesson ? " current" : ""
                }`
              )
              .attr({ href: "javascript:void(0);" })
              .append(
                $("<i>")
                  .addClass(`${curLesson ? "fas" : "far"} ${icon} mr-2`)
                  .css({ "font-size": "larger", color: "rgb(64 81 137)" })
              )
              .append(lesson.title)
              .on({
                click: function () {
                  if ($(this).hasClass("current")) return;
                  $(".current", courseNav)
                    .removeClass("current")
                    .find("i")
                    .toggleClass("fas far");
                  $(this).addClass("current").find("i").toggleClass("fas far");
                  $("#classroomHeader li:nth-child(3)").text(topic.title);
                  const ancestor = $(this).closest(".accordion-collapse");

                  if (!$(ancestor).hasClass("show")) {
                    $(ancestor).parent().children(".show").removeClass("show");
                    $(ancestor).addClass("show");
                  }
                  animateMainPane(() =>
                    loadLesson(lesson, "lesson", coursework, course.id)
                  );
                },
                "mouseenter mouseleave": function () {
                  $(this).toggleClass("bg-transparent");
                },
              })
          );

      if (curLesson) {
        currentElem = $("a", li);

        if ($(`#courseTopic${i} button`).hasClass("current"))
          $(`#courseTopic${i} button`).removeClass("current");
      }
      lesson.parent = topic;
    }
  }
  const hasMaterials = course.topics.some(checkForMaterials);

  if (hasMaterials) {
    $("<a>")
      .addClass("list-group-item list-group-item-action")
      .css({ "font-weight": "600", "font-size": "medium" })
      .attr("href", "javascript:void(0);")
      .append(
        $("<i>")
          .addClass("fas fa-cloud-download-alt mr-3")
          .css({ color: "rgb(64 81 137)", "font-size": "x-large" })
      )
      .append("Download course files")
      .click(() => downloadCourseZip(course.id, course.name))
      .insertAfter(courseNav);
  }

  if (scrolling) scrolling = false;
  $("#main-pane").css(
    "margin-left",
    `calc(${$("#main-pane").css("margin-left")} + 350px)`
  );
  $(".toggle-side-pane")
    .removeClass("disabled")
    .click(function () {
      const winWidth = $(window).width();
      $(this)
        .toggleClass("expanded")
        .find("i")
        .toggleClass("fa-arrow-left fa-arrow-right");
      if ($(this).hasClass("expanded")) {
        $(this).animate({ left: "+=350px" }, "slow", "linear");

        if (winWidth > 768)
          $("#main-pane").animate({ marginLeft: "+=300px" }, "slow", "linear");
        $(".aside-pane").prop("curWidth", 350);
      } else {
        $(this).animate({ left: "-=350px" }, "slow", "linear");

        if (winWidth > 768)
          $("#main-pane").animate({ marginLeft: "-=300px" }, "slow", "linear");
        $(".aside-pane").prop("curWidth", 0);
      }
      $(".aside-pane").animate({ width: "toggle" }, "slow");
    });

  currentElem ??= $(".accordion-button", courseNav).addClass("current").first();
  const type = $(currentElem).data("type"),
    lesson = $(currentElem).data(type);

  if (completed == 1) {
  } else {
    loadLesson(lesson, type, coursework, course.id);
  }
  activateNav(courseNav);
}

function checkForMaterials(obj) {
  if (obj.type == "topic") {
    return topic.lessons.some(checkForMaterials);
  }
  return obj.attachments?.some((mat) => mat.includes(site.domain));
}

function downloadCourseZip(course_id, courseName) {
  $.get({
    url: `${site.process}student?option=course_files_zip`,
    data: { course_id },
    xhrFields: {
      responseType: "blob",
    },
    success: function (zip) {
      $("<a>")
        .attr({
          href: URL.createObjectURL(
            new Blob([zip], { type: "application/zip" })
          ),
          download: `${courseName.replace(/\s+/, "-")}.zip`,
        })[0]
        .click();
    },
    failure: function (err) {
      console.log("zip error", err);
    },
  });
}

function loadLesson(lesson, type, coursework, course_id) {
  console.log('lesson', lesson);
  const { records } = coursework;
  const container = $("#main-pane");
  $("h3", container).text(lesson.title);
  $(container).children().not("h3").remove();
  disposeYTPlayer();

  if (type == "lesson") {
    const loader = loading();
    $(container).append(loader);
    records.progress.lesson = lesson.key;

    if (pageLoadAjax) {
      pageLoadAjax.abort();
    }
    pageLoadAjax = $.get(
      `${site.domain}portal_pages?page=${lesson.type}`,
      function (html) {
        pageLoadAjax = null;
        $(loader).replaceWith(html);

        switch (lesson.type) {
          case "material":
            $("#lesson-desc").html(lesson.description);
            prepareMaterial(lesson);
            break;

          case "assignment":
            prepareAssignment(lesson, records, course_id);
            break;

          case "quiz":
            prepareQuiz(lesson, type, coursework, course_id);
            break;

          default:
            break;
        }

        if (lesson.attachments?.length) {
          const ul = $("<ul>")
              .addClass("list-group list-group-horizontal-xl m-3")
              .appendTo(
                $("<div>")
                  .addClass("card-body pt-0")
                  .css("min-height", "unset")
                  .appendTo(
                    $("<div>")
                      .addClass("card no-scaling border-0 box-shadow")
                      .appendTo(container)
                  )
                  .append(`<svg viewBox="0 0 200 20" xmlns="http://www.w3.org/2000/svg"><style>
            .text {font-size: 5px;font-weight: 900;fill: black;}
          </style><polygon points="0,15 100,15 100,15.5 0,15.5" fill="#D23228"></polygon><text x="0" y="12" class="text">Lesson materials</text></svg>`)
              ),
            regex = new RegExp(
              site.domain.replace(/[.*+?^${}()|[\]\\]/g, "\\$&")
            );

          for (const {
            url,
            filetype,
            title,
            description,
            img,
            duration,
          } of lesson.attachments) {
            const icon =
                filetype == "image"
                  ? "fas fa-image"
                  : filetype == "video"
                  ? "fas fa-video"
                  : filetype == "youtube"
                  ? "fab fa-youtube"
                  : "fas fa-square",
              attrib = regex.test(url)
                ? { href: url, download: "" }
                : { href: url, target: "_blank" };

            $("<li>")
              .addClass(
                "list-group-item d-flex justify-content-between align-items-start"
              )
              .css("min-width", "40%")
              .appendTo(ul)
              .append(
                $("<div>")
                  .addClass("ml-2 mr-auto")
                  .append(
                    $("<div>")
                      .addClass("font-weight-bold")
                      .append($("<span>").addClass("text-wrap").text(title))
                  )
                  .append(
                    description || duration
                      ? $("<p>")
                          .addClass("text-truncate-two-lines mb-2")
                          .css("font-size", "small")
                          .html(description || `Duration &bull; ${duration}`)
                      : "<br>"
                  )
                  .append(
                    $("<a>")
                      .attr(attrib)
                      .addClass("stretched-link btn btn-sm btn-primary px-5")
                      .text(attrib.download ? "Download" : "Open")
                  )
              )
              .append(
                img
                  ? $("<img>")
                      .addClass("rounded ml-2")
                      .attr("src", img)
                      .css({ "max-width": "80px", "aspect-ratio": "16/9" })
                  : $("<span>").addClass(icon).css("font-size", "xx-large")
              );
          }
        }
      }
    );
  } else {
    records.progress.topic = lesson.id;
  }

  if (classType == 'standard') {
    $.post(
      `${site.process}student?option=update_classroom`,
      { course_id, records },
      function (jsonRes) {
        const res = JSON.parse(jsonRes);
  
        if (!res.status) {
          console.error("update err", jsonRes);
        }
      }
    ).fail((err) => console.log(err));
  }
}

function animateMainPane(callback, arg = null) {
  const video = $("#lecture-video video")[0];
  if (video && !video.paused) video.pause();

  $("#main-pane").animate(
    { opacity: 0, marginLeft: "-=50px" },
    "slow",
    function () {
      callback(arg);
      $(this).delay(400).animate({ opacity: 1, marginLeft: "+=50px" }, "slow");
    }
  );
}

function activateNav(courseNav) {
  const callback = (action) => {
    const lessonElem = $(".current", courseNav);
    console.log("lessonElem", lessonElem);

    if (!$(lessonElem)[0]) {
      $(".accordion-button", courseNav).first().click();
      return;
    }

    if (action == "previous") {
      if ($(lessonElem).hasClass("accordion-button")) {
        el = $(lessonElem).closest(".accordion-item").prev();

        if ($(el)[0]) {
          $("ul a", el).last().click();
        }
      } else if ((el = $(lessonElem).parent().prev())[0]) {
        $("a", el).click();
      } else {
        $(lessonElem)
          .closest(".accordion-item")
          .find(".accordion-button")
          .click();
      }
    } else {
      if ($(lessonElem).hasClass("accordion-button")) {
        $(lessonElem).closest(".accordion-item").find("ul a").first().click();
      } else if ((el = $(lessonElem).parent().next())[0]) {
        $("a", el).click();
      } else {
        el = $(lessonElem).closest(".accordion-item").next();

        if ($(el)[0]) {
          $(el).find(".accordion-button").click();
        } else {
          endOfCourse();
        }
      }
    }
  };
  $(".classroom-banner .pagination .page-link").click(function () {
    animateMainPane(callback, $(this).attr("action"));
  });
}

function prepareQuiz(lesson, type, coursework, course_id) {
  const {
    records: { quizes },
  } = coursework;

  if (quizes.every((item) => item.key != lesson.key)) {
    let elem;
    const { grading, questionSet, title, instruction, key } = lesson;
    const quizObj = {
      grading,
      questions: questionSet,
      title,
      instruction,
      key,
    };
    quizObj.callback = (record, score) => {
      quizes.push(record);
      score != "ungraded" ? (coursework.performance += score) : null;
      const loader = loading();
      $(elem).replaceWith(loader);
      $("#lesson-instuction").empty();

      if (classType == 'standard') {
        $.post(
          `${site.process}student?option=update_classroom`,
          { course_id, records: coursework.records },
          function (jsonRes) {
            const res = JSON.parse(jsonRes);
            $(loader).remove();
  
            if (res.status == 1) {
              prepareQuiz(lesson, type, coursework, course_id);
            }
          }
        ).catch((err) => console.error(err));
      }
    };

    if (instruction) {
      $("#lesson-instuction")
        .append($("<strong>").text("INSTRUCTION"))
        .append(`<br> ${instruction}`);
    }
    elem = $("<div>")
      .addClass("w-75 d-flex justify-content-between mx-auto")
      .appendTo($("#quiz-container"))
      .append(
        $("<div>")
          .append($("<h4>").text("Begin quiz"))
          .append(
            $("<p>")
              .addClass("text-muted")
              .append(
                $("<span>").text(grading.duration).css("font-size", "small")
              )
              .append(
                $("<i>")
                  .addClass("fa fa-circle mx-1 align-middle")
                  .css({ "font-size": "5px" })
              )
              .append(
                $("<span>")
                  .text(
                    grading.totalPoints == "Ungraded"
                      ? "Ungraded"
                      : `${grading.totalPoints} points`
                  )
                  .css("font-size", "small")
              )
          )
      )
      .append(
        $("<button>")
          .addClass("btn btn-primary my-auto")
          .attr({
            "data-target": "#generalDialog",
            "data-toggle": "modal",
            "data-action": "take_quiz",
          })
          .prop("quiz", quizObj)
          .text("Start quiz")
      );
  } else {
    const record = quizes.find((item) => item.key == lesson.key),
      rows = {
        duration: "Quiz duration",
        qcount: "Total questions",
        points: "Quiz points",
        time_taken: "Time taken",
        answer_count: "Questions answered",
        score: "My score",
      };
    displayResult(record, rows);
  }
}

function displayResult(result, rows, isQuiz = true) {
  const idFix = result.key.replace(/\..+$/, "");

  $("<div>")
    .addClass("mx-auto w-75 rounded box-shadow card-body")
    .css("min-width", "400px")
    .appendTo($("#quiz-container"))
    .append(
      $("<a>")
        .addClass("text-decoration-none text-dark")
        .attr({
          "data-toggle": "collapse",
          href: `#quiz-result${idFix}`,
          role: "button",
          "aria-expanded": "false",
          "aria-controls": `quiz-result${idFix}`,
        })
        .append(
          $("<div>")
            .addClass("d-flex justify-content-between align-items-center")
            .append(
              $("<span>").append(
                `<svg class="align-middle rounded-circle svg-bg mr-2" focusable="false" width="24" height="24"><use xlink:href="#quiz-icon"></use></svg>Hide result`
              )
            )
            .append(
              $("<span>")
                .addClass("mr-3")
                .css("font-size", "small")
                .text(result.date_taken || result.date_submitted)
            )
        )
        .click(function () {
          const span = $("span", this).first();

          if ($(this).next("div").hasClass("show")) {
            $(span).html($(span).html().replace("Hide", "Show"));
          } else {
            $(span).html($(span).html().replace("Show", "Hide"));
          }
        })
    )
    .append(
      $("<div>")
        .addClass("collapse show")
        .attr({ id: `quiz-result${idFix}` })
        .append(
          $("<ul>").addClass("list-group list-group-flush border-top mt-3")
        )
        .append(
          isQuiz
            ? $("<a>")
                .addClass("btn btn-sm btn-primary ml-3")
                .attr({
                  "data-toggle": "modal",
                  href: "#generalDialog",
                  "data-action": "quiz_result",
                })
                .prop("result", result)
                .text(`Open quiz report`)
            : ""
        )
    );
  const ul = $(`#quiz-result${idFix} ul`);

  for (const key in rows) {
    $(ul).append(
      $("<li>")
        .addClass("list-group-item list-group-item-action border-0 d-flex")
        .append($("<span>").addClass("col-6").text(rows[key]))
        .append(
          $("<span>")
            .addClass("col-6 text-center")
            .text(result[key] ?? "None")
        )
    );
  }
}

function prepareAssignment(lesson, records, course_id) {
  const { dueDate, closeSubmitAfterDue, totalPoints } = lesson.grading;
  const assignments = records.assignments;
  const comparison = GenericDialog.compareWithDate(dueDate);

  if (
    (comparison == "after" && closeSubmitAfterDue) ||
    (assignments.some((item) => item.key == lesson.key) &&
      assignments.score != null)
  ) {
    if (comparison == "after") {
      $("#assignment-container")
        .empty()
        .append(
          $("<h6>")
            .addClass("mx-auto font-weight-normal")
            .css({
              "text-transform": "none",
              "font-size": "medium",
              "line-height": "1.5",
            })
            .append(
              $("<i>")
                .addClass("fas fa-exclamation-triangle mr-2")
                .css("color", "rgb(64, 81, 137)")
            )
            .append(
              "You can't submit the assignment because the scheduled date for Submission has passed. If you think this message is in error, please report the issue to your tutor."
            )
        );
    } else {
      const record = quizes.find((item) => item.key == lesson.key),
        rows = {
          repo: "Assignment link / repository",
          points: "Assignment points",
          date_submitted: "Date submitted",
          score: "My score",
        };
      displayResult(record, rows);
    }
  } else {
    const trustedDomains = [
      "github.com",
      "docs.google.com",
      "docs.microsoft.com",
      "codepen.io",
      "codesandbox.io",
      "dartpad.dev",
      "play.kotlinlang.org",
      "drive.google.com",
      "onedrive.live.com",
      "mega.nz",
      "jupyter.org",
      "www.office.com",
    ];
    const spanElems = $("#assignment-container .form-text span");
    $(spanElems).first().text(dueDate);
    $(spanElems)
      .last()
      .text(totalPoints == "Ungraded" ? "Ungraded" : `${totalPoints} points`);
    $("<p>")
      .css("font-size", "small")
      .append($("<strong>").text("NB"))
      .append("The valid links for assignments are the following:")
      .appendTo($("#assignment-container"))
      .after(
        $("<ul>")
          .addClass("list-group list-group-flush")
          .css("font-size", "small")
          .attr("id", "valid-domains")
      );

    for (const domain of trustedDomains) {
      $("<li>")
        .addClass("list-group-item text-muted")
        .appendTo($("#valid-domains"))
        .append(
          $("<i>")
            .addClass("fa fa-circle align-middle text-warning mr-2")
            .css({ "font-size": "xx-small" })
        )
        .append(`https://${domain}`);
    }
    const editing = assignments.some((item) => item.key == lesson.key);

    if (editing) $("#submit-assignment").text("Edit submission");
    $("#submit-assignment").click(function () {
      let repo = $("#assignmentRepo").val().trim();
      repo = isSafeURL(repo, trustedDomains);
      $(this).addClass("disabled");

      if (!repo) {
        if (!$(".invalid-feedback").hasClass("d-block"))
          $(".invalid-feedback").removeClass("d-block");
        $(this).removeClass("disabled");
        return;
      } else if ($(".invalid-feedback").hasClass("d-block")) {
        $(".invalid-feedback").removeClass("d-block");
      }
      let submission;
      const date_submitted = GenericDialog.getCurrentDateTime();

      if (editing) {
        submission = assignments.find((item) => item.key == lesson.key);
        submission = { ...submission, date_submitted, repo };
      } else {
        submission = { repo, points: totalPoints, date_submitted };
        assignments.push(submission);
      }

      if (classType == 'standard') {
        $.post(
          `${site.process}student?option=update_classroom`,
          { course_id, records },
          function (jsonRes) {
            const res = JSON.parse(jsonRes);
  
            if (res.status) {
              toast(editing ? "Submission edited!" : "Assignment submitted!");
              $("#assignmentRepo").val("");
            }
          }
        );
      }
    });
  }
}

function isSafeURL(url, trustedDomains) {
  const urlRegex = /^(https):\/\/([^:/?#]+)(?:[/?#]|$)/i;

  if (!urlRegex.test(url)) {
    return false;
  }
  let domain = new URL(url).hostname;

  if (!trustedDomains.includes(domain)) {
    return false;
  }
  // Blacklist known malicious patterns (optional)
  let blacklist = [
    "SELECT",
    "INSERT",
    "UPDATE",
    "DELETE",
    "<script>",
    "javascript:",
  ];

  if (blacklist.some((keyword) => url.includes(keyword))) {
    return false;
  }
  return sanitizeURLParams(url);
}

function sanitizeURLParams(url) {
  // Split the URL into base URL and query parameters
  let [baseUrl, queryStr] = url.split("?");

  if (!queryStr) {
    return url;
  }

  // Split query parameters into key-value pairs
  let params = queryStr.split("&");

  // Sanitize each parameter
  let sanitizedParams = params.map((param) => {
    let [key, val] = param.split("=");
    val = encodeURIComponent(val);
    return key + "=" + val;
  });
  let sanitizedQueryString = sanitizedParams.join("&");

  // Reconstruct the sanitized URL
  let sanitizedURL = baseUrl + "?" + sanitizedQueryString;

  return sanitizedURL;
}

function prepareMaterial(lesson) {
  const videos =
      lesson.attachments?.filter((obj) =>
        obj.filetype.match(/video|youtube/)
      ) ?? [],
    vidContainer = $("#lecture-video"),
    videoElem = $("video", vidContainer)[0];

  if (videos.length) {
    addVideoControls();
    prepareVideoPlayer(videos[0], true);

    if (videos.length > 1) {
      const changeVideo = (action) => {
          let curIndex = $(videoElem).data("vidIndex");
          const lastVidIndex = $(videoElem).data("lastVidIndex");

          if (action == "prev") {
            if (curIndex <= 0) return;
            $(videoElem).data("vidIndex", --curIndex);
          } else {
            if (curIndex >= lastVidIndex) return;
            $(videoElem).data("vidIndex", ++curIndex);
          }

          if (youtubePlayer) {
            if (youtubePlayer.getPlayerState() == YT.PlayerState.PLAYING) {
              youtubePlayer.pauseVideo();
            }
          } else if (!videoElem.paused) {
            videoElem.pause();
          }
          $(".page-link", nav).addClass("disabled");
          prepareVideoPlayer(videos[curIndex]);
        },
        nav = $("<nav>")
          .insertAfter(vidContainer)
          .append(
            $("<ul>")
              .addClass("pagination mx-auto")
              .css("width", "fit-content")
              .append(
                $("<li>")
                  .addClass("page-item")
                  .append(
                    $("<a>")
                      .addClass("page-link disabled")
                      .attr({
                        href: "javascript:void(0);",
                        "aria-label": "Previous",
                      })
                      .append(
                        $("<span>")
                          .addClass("fa fa-chevron-left")
                          .attr({ "aria-hidden": "true" })
                      )
                      .append($("<span>").addClass("sr-only").text("Previous"))
                      .click(() => changeVideo("prev"))
                  )
              )
              .append(
                $("<li>")
                  .addClass("page-item")
                  .append(
                    $("<a>")
                      .addClass("page-link disabled")
                      .attr({
                        href: "javascript:void(0);",
                        "aria-label": "Next",
                      })
                      .append(
                        $("<span>")
                          .addClass("fa fa-chevron-right")
                          .attr({ "aria-hidden": "true" })
                      )
                      .append($("<span>").addClass("sr-only").text("Next"))
                      .click(() => changeVideo("next"))
                  )
              )
          );
      $(videoElem).data({
        vidIndex: 0,
        lastVidIndex: videos.length - 1,
        videos,
      });
    }
  } else {
    $("#video-spinner").add(vidContainer).remove();
  }
}

function onYouTubeIframeAPIReady() {
  youtubePlayer = new YT.Player("youtubePlayer", {
    videoId: $("#youtubePlayer").prop("vidID"),
    playerVars: {
      autoplay: 0,
      controls: 0,
      disablekb: 1,
      enablejsapi: 1,
      fs: 0,
      modestbranding: 1,
      rel: 0,
      showinfo: 0,
    },
    events: {
      onReady: setVideoLength,
      onStateChange: onPlayerStateChange,
    },
  });
  youtubePlayer.started = false;
}

function onPlayerStateChange(event) {
  const btn = $("#main-pane .btn-toolbar button").get(0);
  setPlaybackTime();

  if (event.data == YT.PlayerState.PLAYING) {
    if (!youtubePlayer.started) {
      if (youtubePlayer.getCurrentTime() != 0) youtubePlayer.seekTo(0);
      youtubePlayer.started = true;
    }
    youtubePlayer.intervalId = setInterval(setPlaybackTime, 1000);
    $("i", btn).removeClass("fas fa-play").addClass("fas fa-pause");
  } else {
    if (youtubePlayer.intervalId) {
      clearInterval(youtubePlayer.intervalId);
      youtubePlayer.intervalId = undefined;
    }
    $("i", btn).removeClass("fas fa-pause").addClass("fas fa-play");

    if (event.data == YT.PlayerState.ENDED) {
      playNextVideo();
    }
  }
}

function prepareVideoPlayer(video, firstTime = false) {
  const vidContainer = $("#lecture-video"),
    videoElem = $("video", vidContainer);

  if (!firstTime) {
    $(vidContainer)
      .addClass("d-none")
      .next("nav")
      .find(".page-link")
      .addClass("disabled");
    $("#video-spinner").toggleClass("d-none d-flex");
  }

  if ($("#youtubePlayer").prop("tagName") == "IFRAME") {
    $("#youtubePlayer").replaceWith(
      $("<div>")
        .addClass("embed-responsive-item bg-secondary d-none")
        .attr("id", "youtubePlayer")
    );
  }
  const youtubeElem = $("#youtubePlayer");

  if (video.filetype == "video") {
    disposeYTPlayer();
    $(youtubeElem).addClass("d-none");
    $(videoElem).removeClass("d-none").attr("src", video.url);
  } else {
    $(videoElem).addClass("d-none");
    $(youtubeElem)
      .removeClass("d-none")
      .prop("vidID", video.url.match(/[a-zA-Z0-9_-]{11}$/)[0]);

    if (
      !document.querySelector(
        'script[src="https://www.youtube.com/iframe_api"]'
      )
    ) {
      const ytScript = document.createElement("script");
      ytScript.src = "https://www.youtube.com/iframe_api";
      document.body.appendChild(ytScript);
    } else {
      onYouTubeIframeAPIReady();
    }
  }
}

function addVideoControls() {
  const vidContainer = $("#lecture-video"),
    videoElem = $("video", vidContainer)[0];
  $(vidContainer).on({
    mouseover: function () {
      const width = $("#video-seeker .progress-bar").css("width");
      $("#video-seeker .progress-bar").css("width", `max(${width}, 1.5%)`);
    },
    mouseout: function () {
      const currentTime = youtubePlayer
        ? youtubePlayer.getCurrentTime()
        : videoElem.currentTime;

      if (currentTime == 0) {
        $("#video-seeker .progress-bar").css("width", "1%");
      }
    },
    mouseup: function () {
      $("#video-seeker .progress-bar").removeAttr("dragon");
      if ($("#video-seeker .progress-bar").attr("paused")) {
        youtubePlayer ? youtubePlayer.playVideo() : videoElem.play();
        $("#video-seeker .progress-bar").removeAttr("paused");
      }
    },
  });

  $(videoElem).on({
    canplay: function () {
      setVideoLength();
    },
    timeupdate: setPlaybackTime,
    ended: function () {
      const btn = $("#main-pane .btn-toolbar button").get(0);
      $("i", btn).removeClass("fas fa-pause").addClass("fas fa-play");
      $("#video-seeker .progress-bar").css("width", "100%");
      this.pause();
      playNextVideo();
    },
    pause: function () {
      const btn = $("#main-pane .btn-toolbar button").get(0);
      $("i", btn).removeClass("fas fa-pause").addClass("fas fa-play");
    },
    play: function () {
      const btn = $("#main-pane .btn-toolbar button").get(0);
      $("i", btn).removeClass("fas fa-play").addClass("fas fa-pause");
    },
  });

  $("#main-pane .btn-toolbar button").each(function (i, btn) {
    $(btn).click(function () {
      switch (i) {
        case 0: {
          if (youtubePlayer) {
            if (youtubePlayer.getPlayerState() == YT.PlayerState.PLAYING) {
              youtubePlayer.pauseVideo();
            } else {
              youtubePlayer.playVideo();
            }
          } else {
            if (videoElem.paused || videoElem.ended) {
              videoElem.play();
            } else {
              videoElem.pause();
            }
          }
          break;
        }

        case 1: {
          const currentTime = youtubePlayer
            ? youtubePlayer.getCurrentTime()
            : videoElem.currentTime;

          if (currentTime > 5) {
            youtubePlayer
              ? youtubePlayer.seekTo(currentTime - 5, true)
              : (videoElem.currentTime -= 5);
          } else {
            youtubePlayer
              ? youtubePlayer.seekTo(0, true)
              : (videoElem.currentTime = 0);
          }
          break;
        }

        case 2: {
          const currentTime = youtubePlayer
              ? youtubePlayer.getCurrentTime()
              : videoElem.currentTime,
            duration = youtubePlayer
              ? youtubePlayer.getDuration()
              : videoElem.duration;

          if (currentTime < duration - 5) {
            youtubePlayer
              ? youtubePlayer.seekTo(currentTime + 5, true)
              : (videoElem.currentTime += 5);
          } else {
            youtubePlayer
              ? youtubePlayer.seekTo(duration, true)
              : (videoElem.currentTime = duration);
          }
          break;
        }

        case 5: {
          if (!document.fullscreenElement) {
            const elem = youtubePlayer ? $("#youtubePlayer")[0] : videoElem;
            elem.requestFullscreen().catch((err) => {
              console.log("fullscreen err", err);
              alert("Unable to enable fullscreen mode");
            });
          } else {
            document.exitFullscreen();
          }
          break;
        }
      }
    });
  });

  $("#volume-bar").change(function () {
    const volBtn = $(this).closest(".dropup").find("button.btn-dark"),
      vol = $(this).val() / 100;
    if (vol == 0) {
      $(volBtn).html(
        `<svg width="18px" height="16px" fill="currentColor" viewBox="0 0 16 16" xmlns="http://www.w3.org/2000/svg"><path d="M4 5h-4v6h4l5 4v-14z"/></svg>`
      );
    } else if (
      youtubePlayer ? youtubePlayer.isMuted() : videoElem.volume == 0
    ) {
      $(volBtn).html(
        `<svg width="16px" height="16px" viewBox="0 0 16 16" xmlns="http://www.w3.org/2000/svg" fill="currentColor"><path d="M11.8 2.4l-0.5 1c1.1 1.4 1.7 3.2 1.7 5.1 0 1.7-0.5 3.2-1.3 4.6l0.7 0.8c1.1-1.5 1.7-3.4 1.7-5.4-0.1-2.3-0.9-4.4-2.3-6.1z"/><path d="M10.8 4.4l-0.5 1.1c0.5 0.9 0.8 1.9 0.8 3 0 1-0.3 2-0.7 2.9l0.7 0.9c0.6-1.1 1-2.4 1-3.7-0.1-1.6-0.5-3-1.3-4.2z"/><path d="M4 5h-4v6h4l5 4v-14z"/></svg>`
      );
    }
    youtubePlayer ? youtubePlayer.setVolume(vol) : (videoElem.volume = vol);
  });

  $("#main-pane .btn-toolbar .btn-group .dropup a").each(function (i, option) {
    $(option).click(function () {
      if ($(this).hasClass("active")) return;
      const playbackRateBtn = $(this)
        .closest(".dropup")
        .find("button.btn-dark");
      console.log("playbackRateBtn", $(playbackRateBtn)[0]);
      $(this).closest("ul").find(".active").removeClass("active");
      $(this).addClass("active");
      switch (i) {
        case 0:
          youtubePlayer
            ? youtubePlayer.setPlaybackRate(0.5)
            : (videoElem.playbackRate = 0.5);
          $("small", playbackRateBtn).text("0.5");
          break;
        case 1:
          youtubePlayer
            ? youtubePlayer.setPlaybackRate(0.75)
            : (videoElem.playbackRate = 0.75);
          $("small", playbackRateBtn).text("0.75");
          break;
        case 2:
          youtubePlayer
            ? youtubePlayer.setPlaybackRate(1)
            : (videoElem.playbackRate = 1);
          $("small", playbackRateBtn).text("1.0");
          break;
        case 3:
          youtubePlayer
            ? youtubePlayer.setPlaybackRate(1.5)
            : (videoElem.playbackRate = 1.5);
          $("small", playbackRateBtn).text("1.5");
          break;
        case 4:
          youtubePlayer
            ? youtubePlayer.setPlaybackRate(2)
            : (videoElem.playbackRate = 2);
          $("small", playbackRateBtn).text("2.0");
          break;
      }
    });
  });

  $("#video-seeker")
    .on({
      click: function (e) {
        const duration = youtubePlayer
          ? youtubePlayer.getDuration()
          : videoElem.duration;
        const xmax = $(this).prop("clientWidth"),
          xmin = xmax * 0.015,
          x = e.offsetX;
        const currentTime = Math.round(((x - xmin) / (xmax - xmin)) * duration);
        youtubePlayer
          ? youtubePlayer.seekTo(currentTime, true)
          : (videoElem.currentTime = currentTime);
        $(".progress-bar", this).css("width", x + "px");
      },
      mousemove: function (e) {
        if ($(".progress-bar", this).attr("dragon") == "true") {
          const duration = youtubePlayer
            ? youtubePlayer.getDuration()
            : videoElem.duration;
          const xmax = $(this).prop("clientWidth"),
            xmin = xmax * 0.01,
            x = e.offsetX;
          console.log("parent mousemove", x, xmin);
          if (x < xmin) return;
          const currentTime = Math.round(
            ((x - xmin) / (xmax - xmin)) * duration
          );
          youtubePlayer
            ? youtubePlayer.seekTo(currentTime, true)
            : (videoElem.currentTime = currentTime);
          $(".progress-bar", this).css("width", `calc(${x}px + 1%)`);
        }
      },
    })
    .find(".progress-bar")
    .on({
      mousedown: function (e) {
        e.stopPropagation();
        const duration = youtubePlayer
          ? youtubePlayer.getDuration()
          : videoElem.duration;
        const xmax = $(this).parent().prop("clientWidth"),
          xmin = xmax * 0.01,
          x = e.offsetX;

        if (x < xmin) return;

        if (youtubePlayer) {
          if (youtubePlayer.getPlayerState() == YT.PlayerState.PLAYING)
            youtubePlayer.pauseVideo();
        } else if (!videoElem.paused) {
          videoElem.pause();
        }
        const currentTime = Math.round(((x - xmin) / (xmax - xmin)) * duration);
        youtubePlayer
          ? youtubePlayer.seekTo(currentTime, true)
          : (videoElem.currentTime = currentTime);
        $(this)
          .css("width", `calc(${x}px + 1%)`)
          .attr({ dragon: "true", paused: "true" });
      },
      mousemove: function (e) {
        e.stopPropagation();

        if ($(this).attr("dragon") == "true") {
          const duration = youtubePlayer
            ? youtubePlayer.getDuration()
            : videoElem.duration;
          const xmax = $(this).parent().prop("clientWidth"),
            xmin = xmax * 0.01,
            x = e.offsetX;
          if (x < xmin) return;
          const currentTime = Math.round(
            ((x - xmin) / (xmax - xmin)) * duration
          );
          youtubePlayer
            ? youtubePlayer.seekTo(currentTime, true)
            : (videoElem.currentTime = currentTime);
          $(this).css("width", `calc(${x}px + 1%)`);
        }
      },
      mouseup: function (e) {
        e.stopPropagation();
        $(this).removeAttr("dragon");

        if ($(this).attr("paused") == "true") {
          youtubePlayer ? youtubePlayer.playVideo() : videoElem.play();
          $(this).removeAttr("paused");
        }
      },
    });
}

function disposeYTPlayer() {
  if (youtubePlayer) {
    youtubePlayer.destroy();
    youtubePlayer = null;
  }
}

function playNextVideo() {
  const vidContainer = $("#lecture-video"),
    videoElem = $("video", vidContainer)[0],
    videos = $(videoElem).data("videos");
  const vidIndex = $(videoElem).data("vidIndex"),
    lastVidIndex = $(videoElem).data("lastVidIndex");

  if (lastVidIndex && vidIndex < lastVidIndex) {
    $(videoElem).data("vidIndex", vidIndex + 1);
    prepareVideoPlayer(videos[vidIndex + 1]);
  }
}

function setVideoLength() {
  const video = $("#lecture-video video")[0];
  let sec = Math.round(
      youtubePlayer ? youtubePlayer.getDuration() : video.duration
    ),
    duration = formatTime(sec);
  $("#main-pane .btn-toolbar span:only-child").text(
    $("#main-pane .btn-toolbar span:only-child")
      .text()
      .replace(/(?<=\/).+/, duration)
  );
  $("#video-spinner").toggleClass("d-none d-flex");
  $("#lecture-video").removeClass("d-none");

  const vidIndex = $(video).data("vidIndex"),
    lastVidIndex = $(video).data("lastVidIndex");

  if (lastVidIndex) {
    const vidNav = $("#lecture-video").next("nav"),
      pageLinks = $(".page-link", vidNav);

    if (vidIndex > 0) {
      $(pageLinks).first().removeClass("disabled");
    }

    if (vidIndex < lastVidIndex) {
      $(pageLinks).last().removeClass("disabled");
    }
  }
}

function formatTime(sec) {
  sec = Math.round(sec);
  let duration = "";
  const hr = Math.floor(sec / 3600);

  if (hr > 0) {
    duration += `${hr}:`;
    sec -= hr * 3600;
  }
  const min = Math.floor(sec / 60);

  if (min > 0) {
    sec -= min * 60;
  }
  duration +=
    (min < 10 && hr > 0 ? "0" : "") +
    `${min}:` +
    sec.toString().padStart(2, "0");
  return duration;
}

function setPlaybackTime() {
  const video = $("#lecture-video video")[0];
  const currentTime = youtubePlayer
      ? youtubePlayer.getCurrentTime()
      : video.currentTime,
    duration = youtubePlayer ? youtubePlayer.getDuration() : video.duration;
  const mediaTime = formatTime(currentTime);
  $(".btn-toolbar span:only-child").text(
    $(".btn-toolbar span:only-child")
      .text()
      .replace(/.+(?=\/)/, mediaTime)
  );
  const progress = Math.round((currentTime * 99) / duration) + 1;
  if ($("#lecture-video").is(":hover")) {
    $("#video-seeker .progress-bar").css("wdith", `max(${progress}%, 1.5%)`);
  } else $("#video-seeker .progress-bar").css("wdith", progress + "%");
}

function endOfCourse() {
  // $("<div>")
  //   .addClass("fs-4 mt-5")
  //   .attr({ id: "completion-msg" })
  //   .append($("<p>"))
  //   .append(
  //     $("<svg>")
  //       .attr({
  //         xmlns: "http://www.w3.org/2000/svg",
  //         viewBox: "0 0 512 512",
  //         height: "20rem",
  //         width: "20rem",
  //       })
  //       .append(
  //         $("<g>")
  //           .attr({ transform: "translate(0,-200)" })
  //           .append(
  //             $("<path>")
  //               .attr({
  //                 d: "M404.504,404.803l-4.582-3.326l-4.57,3.326h-0.012l-21.736,15.815v-15.815v-67.047v-22.514 c0.969,0.263,1.89,0.598,2.727,1.029c5.156,2.632,13.741,15.737,14.654,10.144c9.264-56.773,11.424-4.235,17.071-5.132 c4.725-0.742,10.515,1.376,15.312,0.467h2.883v16.006v67.047v15.803L404.504,404.803z",
  //               })
  //               .css({ fill: "#F95428" })
  //           )
  //       )
  //       .append(
  //         $("<g>")
  //           .attr({ transform: "translate(0,-200)" })
  //           .append(
  //             $("<path>")
  //               .attr({
  //                 d: "M373.604,352.383c-4.749-1.376-10.647-1.412-14.092-4.869c-4.163-4.163-3.361-11.843-5.898-16.819 c-2.632-5.156-9.271-9.056-10.156-14.654c-0.885-5.658,4.247-11.424,5.144-17.083c0.873-5.587-2.249-12.668,0.383-17.824 c2.536-4.976,10.037-6.627,14.2-10.778c4.163-4.163,5.814-11.676,10.79-14.212c5.156-2.62,12.226,0.502,17.824-0.383 c5.646-0.885,11.412-6.017,17.071-5.132c5.599,0.885,9.498,7.524,14.654,10.156c4.976,2.536,12.668,1.723,16.819,5.886 c4.163,4.163,3.362,11.855,5.898,16.831c2.632,5.156,9.271,9.056,10.156,14.654c0.885,5.646-4.247,11.424-5.144,17.071 c-0.873,5.599,2.249,12.668-0.383,17.824c-2.536,4.976-10.037,6.627-14.2,10.79c-4.163,4.163-5.814,11.664-10.778,14.2 c-0.802,0.407-1.651,0.682-2.524,0.849c-4.797,0.909-10.587-1.208-15.312-0.467c-5.646,0.897-11.412,6.029-17.071,5.132 c-5.598-0.873-9.498-7.513-14.654-10.144C375.495,352.981,374.573,352.646,373.604,352.383z",
  //               })
  //               .css({ fill: "#F7B239" })
  //           )
  //       )
  //       .append(
  //         $("<g>")
  //           .attr({ transform: "translate(0,-200)" })
  //           .append(
  //             $("<path>")
  //               .attr({
  //                 d: "M417.495,289.531c9.702,9.702,9.702,25.433,0,35.122c-9.702,9.702-25.433,9.702-35.134,0 c-9.702-9.69-9.702-25.421,0-35.122C392.063,279.829,407.794,279.829,417.495,289.531z",
  //               })
  //               .css({ fill: "#F95428" })
  //           )
  //       )
  //   );
}
