class DateRangePicker {
  #callback = (daterange) => alert(JSON.stringify(daterange));
  #today;
  #latterMonth;
  #formerMonth;
  #latterYear;
  #formerYear;
  #months;
  #triggerElem;
  #preventHide;
  #lowerElem;
  #upperElem;
  #ignoreSlash

  constructor() {
    this.#months = [
      "January",
      "February",
      "March",
      "April",
      "May",
      "June",
      "July",
      "August",
      "September",
      "October",
      "November",
      "December",
    ];
    this.#today = new Date();
    this.#preventHide = this.#ignoreSlash = false;

    if (!$("#daterangepicker")[0]) {
      var day = this.#today.getDate();
      var month = this.#today.getMonth();
      var year = this.#today.getFullYear();
      var monthNumber = month + 1;
      monthNumber = monthNumber < 10 ? "0" + monthNumber : monthNumber;
      var monthShortName = this.#months[month].slice(0, 3);
      var dateFormat1 = day + "/" + monthNumber + "/" + year;
      var dateFormat2 = day + "-" + monthNumber + "-" + year;
      var dateFormat3 =
        (day < 10 ? "0" + day : day) + " " + monthShortName + " " + year;

      $("<div>")
        .addClass("modal fade")
        .attr({
          tabindex: "-1",
          role: "dialog",
          "aria-labelledby": "daterangepickerLabel",
          id: "daterangepicker",
          "aria-hidden": "true",
        })
        .appendTo($("body"))
        .append(
          $("<div>")
            .addClass("modal-dialog modal-lg modal-dialog-centered")
            .append(
              $("<div>")
                .addClass("modal-content")
                .append(
                  $("<div>")
                    .addClass("modal-header")
                    .append(
                      $("<h5>")
                        .addClass("modal-title")
                        .attr({ id: "daterangepickerLabel" })
                        .text("Date Range Picker")
                    )
                    .append(
                      $("<button>")
                        .addClass("close")
                        .attr({
                          type: "button",
                          "data-dismiss": "modal",
                          "aria-label": "Close",
                        })
                        .append(
                          $("<span>").attr({ "aria-hidden": "true" }).text("×")
                        )
                    )
                )
                .append(
                  $("<div>")
                    .addClass("modal-body px-4")
                    .append(
                      $("<div>")
                        .addClass("rangepicker-container")
                        .append(
                          $("<div>")
                            .addClass("rangepicker-header d-flex flex-column")
                            .append($("<span>").text("Select date range"))
                            .append(
                              $("<div>")
                                .addClass("rangepicker-range d-flex")
                                .append(
                                  $("<div>")
                                    .addClass("form-group mb-0")
                                    .append(
                                      $("<input>")
                                        .addClass("border-0")
                                        .attr({
                                          id: "startDate",
                                          type: "text",
                                          placeholder: "dd/mm/yyyy",
                                        })
                                        .on({
                                          input: this.#onInputDate.bind(this),
                                          focusout:
                                            this.#parseDateStr.bind(this),
                                          keydown: ({ key, target }) => {
                                            if (key == "Enter")
                                              $(target).blur();
                                            else if (['Backspace', 'Delete'].includes(key)) this.#ignoreSlash = true;
                                          },
                                        })
                                    )
                                    .append(
                                      $("<label>")
                                        .attr({
                                          for: "startDate",
                                          "data-toggle": "tooltip",
                                          "data-placement": "left",
                                          title: `Formats: <ul class="list-unstyled font-weight-bold"><li>${dateFormat1}</li><li>${dateFormat2}</li><li>${dateFormat3}</li></ul>`,
                                          "data-html": "true",
                                        })
                                        .text("Start date*")
                                    )
                                )
                                .append(
                                  $("<h5>").addClass("mx-1").html("&mdash;")
                                )
                                .append(
                                  $("<div>")
                                    .addClass("form-group mb-0")
                                    .append(
                                      $("<input>")
                                        .addClass("border-0")
                                        .attr({
                                          id: "endDate",
                                          type: "text",
                                          placeholder: "dd/mm/yyyy",
                                        })
                                        .on({
                                          input: this.#onInputDate.bind(this),
                                          focusout: (evt) => this.#parseDateStr(evt, false),
                                          keydown: ({ key, target }) => {
                                            if (key == "Enter")
                                              $(target).blur();
                                            else if (['Backspace', 'Delete'].includes(key)) this.#ignoreSlash = true;
                                          },
                                        })
                                    )
                                    .append(
                                      $("<label>")
                                        .attr({
                                          for: "endDate",
                                          "data-toggle": "tooltip",
                                          "data-placement": "right",
                                          title: `Formats: <ul class="list-unstyled font-weight-bold"><li>${dateFormat1}</li><li>${dateFormat2}</li><li>${dateFormat3}</li></ul>`,
                                          "data-html": "true",
                                        })
                                        .text("End date*")
                                    )
                                )
                            )
                        )
                        .append(
                          $("<ul>")
                            .addClass("rangepicker-nav nav flex-column mr-4")
                            .attr("role", "tablist")
                        )
                        .append(
                          $("<div>")
                            .addClass("rangepicker-calendar")
                            .append(
                              $("<div>")
                                .addClass("d-lg-flex")
                                .append(
                                  $("<div>")
                                    .addClass("pr-2")
                                    .attr("id", "former-calendar")
                                )
                                .append(
                                  $("<div>").addClass("d-none d-lg-block vr")
                                )
                                .append(
                                  $("<div>")
                                    .addClass("pr-4 pl-2")
                                    .attr("id", "latter-calendar")
                                )
                            )
                        )
                    )
                )
                .append(
                  $("<div>")
                    .addClass("modal-footer py-1")
                    .append(
                      $("<button>")
                        .addClass("btn btn-primary px-4")
                        .attr({ type: "button" })
                        .text("OK")
                        .click(this.#addDateRange.bind(this))
                    )
                )
            )
        )
        .on("show.bs.modal", this.#onShowModal.bind(this));
      this.#addSideNav();
      this.#styleElements();
      this.#createCalendar($("#former-calendar, #latter-calendar"));
      $('#daterangepicker [data-toggle="tooltip"]').tooltip();
    }
  }

  #onShowModal(evt) {
    this.#triggerElem = evt.relatedTarget;
    this.#callback = $(this.#triggerElem).prop("callback") ?? this.#callback;
    $("#daterangepicker ul.rangepicker-nav li").removeClass('active');
    $('#endDate, #startDate').removeClass('invalid-date');
    this.#showCurrentDate(true);
  }

  #addDateRange() {
    if (!this.#lowerElem) {
      $('#startDate').addClass('invalid-date');
      toast('Start date is empty!', 'red');
      return;
    }

    if (!this.#upperElem) {
      $('#endDate').addClass('invalid-date');
      toast('End date is empty!', 'red');
      return;
    }
    const dateRange = {start: $(this.#lowerElem).prop('date').toDateString(), end: $(this.#upperElem).prop('date').toDateString()};
    $('#daterangepicker').modal('hide');
    this.#callback(dateRange);
  }

  #onInputDate(evt) {
    const input = evt.target;
    let dateStr = input.value
    $(input).prop("changed", true);

    if (dateStr.includes('/')) {
      dateStr = dateStr.replace(/[^\d/]/g, '');
    }

    if (!this.#ignoreSlash && ((dateStr.length == 2 && !dateStr.includes('/')) || (dateStr.length == 5 && dateStr.includes('/') && dateStr.match(/\//g).length == 1))) {
      dateStr += '/'
    }
    input.value = dateStr;
  }

  #addSideNav() {
    const sideNav = $("#daterangepicker ul.rangepicker-nav"),
      items = [
        "Last 7 days",
        "Last month",
        "This month",
        "Last year",
        "This year",
        "Reset",
      ];
    for (let i = 0; i < 6; i++) {
      const item = items[i];
      $("<li>")
        .addClass(`btn bg-light mt-2`)
        .text(item)
        .appendTo(sideNav)
        .click((evt) => this.#setTabPane(evt, i));
    }
  }

  #setTabPane(evt, navPos) {
    const tab = evt.target;
    if ($(tab).hasClass("active")) return;
    $(tab).parent("ul").find("li.active").removeClass("active");
    if (navPos != 5) $(tab).addClass("active");
    let date1, date2;

    switch (navPos) {
      case 0:
        date2 = new Date(this.#today.getFullYear(), this.#today.getMonth(), this.#today.getDate());
        date1 = new Date(date2);
        date1.setDate(date1.getDate() - 7);
      case 1:
        if (!date1 || !date2) {
          const yr = this.#today.getMonth() > 0 ? this.#today.getFullYear() : this.#today.getFullYear() - 1, month = this.#today.getMonth() > 0 ? this.#today.getMonth() - 1 : 11;
          date1 = new Date(yr, month);
          date2 = new Date(date1);
          this.#getMaxDaysInMonth(date2);
        }
      case 2:
        if (!date1 || !date2) {
          date1 = new Date(this.#today.getFullYear(), this.#today.getMonth(), 1);
          date2 = new Date(date1);
          this.#getMaxDaysInMonth(date2);
        }
        this.#lowerElem = $("<td>").prop("date", date1)[0];
        this.#upperElem = $("<td>").prop("date", date2)[0];
        this.#showCurrentDate();
        break;
      case 3:
        date1 = new Date(this.#today.getFullYear()-1, 0);
        date2 = new Date(date1);
        this.#getLastDayInYear(date2);
        this.#lowerElem = $("<td>").prop("date", date1)[0];
        this.#upperElem = $("<td>").prop("date", date2)[0];
        this.#formerMonth = date1.getMonth();
        this.#formerYear = date1.getFullYear();
        this.#latterMonth = date2.getMonth();
        this.#latterYear = date2.getFullYear();
        this.#jumpToDate();
        break;
      case 4:
        date1 = new Date(this.#today.getFullYear(), 0);
        date2 = new Date(date1);
        this.#getLastDayInYear(date2);
        this.#lowerElem = $("<td>").prop("date", date1)[0];
        this.#upperElem = $("<td>").prop("date", date2)[0];
        this.#formerMonth = date1.getMonth();
        this.#formerYear = date1.getFullYear();
        this.#latterMonth = date2.getMonth();
        this.#latterYear = date2.getFullYear();
        this.#jumpToDate();
        break;
      case 5:
        this.#lowerElem = this.#upperElem = null;
        this.#showCurrentDate();
        break;
    }

    if (navPos != 5) {
      $("#startDate").val(
        date1.toLocaleDateString("en-GB", {
          day: "2-digit",
          month: "short",
          year: "numeric",
        })
      );
      $("#endDate").val(
        date2.toLocaleDateString("en-GB", {
          day: "2-digit",
          month: "short",
          year: "numeric",
        })
      );
    } else {
      $("#startDate").val('');
      $("#endDate").val('');
    }
  }

  #createCalendar(containers) {
    $(containers).each((i, container) => {
      const table = $("<table>")
        .appendTo(container)
        .addClass("calendar w-100")
        .attr({
          role: "presentation",
          cellspacing: "0",
          cellpadding: "0",
        })
        .append(
          $("<thead>").append(
            $("<tr>")
              .attr("role", "row")
              .append(
                $("<th>").append(
                  $("<button>")
                    .addClass("btn bg-transparent rounded-circle p-0")
                    .attr({
                      type: "button",
                      "aria-hidden": "true",
                      tabindex: "-1",
                    })
                    .html(
                      `<svg width="24" height="24" class="align-middle" focusable="false" aria-hidden="true" viewBox="0 0 24 24"><path d="M15.41 16.59L10.83 12l4.58-4.59L14 6l-6 6 6 6 1.41-1.41z"></path>`
                    )
                    .click((evt) => this.#showPrevMonth(evt, i > 0))
                )
              )
              .append(
                $("<th>")
                  .addClass("text-center pt-1 dropdown")
                  .attr("colspan", "7")
                  .append(
                    $("<button>")
                      .addClass("btn bg-white dropdown-toggle calendar-header")
                      .attr({ type: "button", "data-toggle": "dropdown" })
                      .click(() => this.#resetDateSelector(i))
                  )
                  .append(
                    $("<div>")
                      .addClass('dropdown-menu')
                      .append(
                        $("<div>")
                          .addClass("d-flex")
                          .css("width", "15rem")
                          .append(
                            $("<div>")
                              .addClass("col-7 form-group pr-0")
                              .append(
                                $("<label>")
                                  .addClass("font-weight-normal")
                                  .attr("for", "selectCalendarMonth" + i)
                                  .text("Month")
                              )
                              .append(
                                $("<select>")
                                  .addClass("form-control")
                                  .attr("id", "selectCalendarMonth" + i)
                                  .change((evt) =>
                                    this.#selectDate(evt.target, "month", i)
                                  )
                              )
                          )
                          .append(
                            $("<div>")
                              .addClass("col form-group pl-1")
                              .append(
                                $("<label>")
                                  .addClass("font-weight-normal")
                                  .attr("for", "selectCalendarYear" + i)
                                  .text("Year")
                              )
                              .append(
                                $("<select>")
                                  .addClass("form-control")
                                  .attr("id", "selectCalendarYear" + i)
                                  .change((evt) =>
                                    this.#selectDate(evt.target, "year", i)
                                  )
                              )
                          )
                      )
                      .click(() => (this.#preventHide = true))
                  )
                  .on("hide.bs.dropdown", (evt) => {
                    if (this.#preventHide) {
                      evt.preventDefault();
                      this.#preventHide = false;
                    }
                  })
              )
              .append(
                $("<th>").addClass('text-right').append(
                  $("<button>")
                    .addClass("btn bg-transparent rounded-circle p-0")
                    .attr({
                      type: "button",
                      "aria-hidden": "true",
                      tabindex: "-1",
                    })
                    .html(
                      `<svg class="align-middle" focusable="false" aria-hidden="true" viewBox="0 0 24 24" width="24" height="24"><path d="M8.59 16.59L13.17 12 8.59 7.41 10 6l6 6-6 6-1.41-1.41z"></path></svg>`
                    )
                    .click((evt) => this.#showNextMonth(evt, i > 0))
                )
              )
          )
        )
        .append($("<tbody>"));
      this.#createDays($("tbody", table));
    });
  }

  #resetDateSelector(index) {
    const monthSelect = $(`#selectCalendarMonth${index}`).empty(),
      yearSelect = $(`#selectCalendarYear${index}`).empty(),
      stopYear = this.#today.getFullYear();
    for (let yr = 2000; yr <= stopYear; yr++) {
      const disabled =
          index == 0
            ? yr > this.#latterYear ||
              (yr == this.#latterYear && this.#formerMonth >= this.#latterMonth)
            : yr < this.#formerYear ||
              (yr == this.#formerYear &&
                this.#latterMonth <= this.#formerMonth),
        selected = index == 0 ? yr == this.#formerYear : yr == this.#latterYear;
      $("<option>")
        .appendTo(yearSelect)
        .prop({ selected, disabled })
        .text(yr)
        .val(yr);
    }

    for (let j = 0; j < 12; j++) {
      const month = this.#months[j],
        disabled =
          this.#formerYear == this.#latterYear &&
          (index == 0 ? j >= this.#latterMonth : j <= this.#formerMonth),
        selected = index == 0 ? j == this.#formerMonth : j == this.#latterMonth;
      $("<option>")
        .appendTo(monthSelect)
        .prop({ selected, disabled })
        .text(month)
        .val(j);
    }
    setTimeout(() => {
      $(yearSelect).closest('.dropdown-menu').css('transform', 'translate3d(0px, 44px, 0px)');
    }, 100);
  }

  #selectDate(selectEl, type, index) {
    const dateBtn = $(selectEl).closest(".dropdown-menu").prev("button"),
      selection = $(selectEl).val();

    if (type == "year") {
      if (index == 0) {
        this.#formerYear = selection;
        this.#showCalendar(this.#formerMonth, this.#formerYear, false);
        $(dateBtn).text(
          `${this.#months[this.#formerMonth]} ${this.#formerYear}`
        );
      } else {
        this.#latterYear = selection;
        this.#showCalendar(this.#latterMonth, this.#latterYear);
        $(dateBtn).text(
          `${this.#months[this.#latterMonth]} ${this.#latterYear}`
        );
      }
    } else {
      if (index == 0) {
        this.#formerMonth = selection;
        this.#showCalendar(this.#formerMonth, this.#formerYear, false);
        $(dateBtn).text(
          `${this.#months[this.#formerMonth]} ${this.#formerYear}`
        );
      } else {
        this.#latterMonth = selection;
        this.#showCalendar(this.#latterMonth, this.#latterYear);
        $(dateBtn).text(
          `${this.#months[this.#latterMonth]} ${this.#latterYear}`
        );
      }
    }
    this.#resetDateSelector(index);
  }

  #showCurrentDate(reset = false) {
    this.#today = new Date();
    this.#latterMonth = this.#today.getMonth();
    this.#latterYear = this.#today.getFullYear();
    this.#formerMonth = this.#latterMonth > 0 ? this.#latterMonth - 1 : 11;
    this.#formerYear = this.#latterMonth > 0 ? this.#latterYear : this.#latterYear - 1;

    if (reset) {
      this.#lowerElem = this.#upperElem = null;
      $("#startDate, #endDate").val("");
    }
    this.#showCalendar(this.#latterMonth, this.#latterYear);
    this.#showCalendar(this.#formerMonth, this.#formerYear, false);
  }

  #createDays(parent) {
    const tr = $("<tr>")
      .addClass("text-muted")
      .css("font-size", "small")
      .appendTo(parent);
    const weekdays = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"];

    for (let i = 0; i < 7; i++) {
      const day = weekdays[i];
      $("<td>")
        .addClass(i == 0 ? "text-danger" : "")
        .attr("role", "columnheader")
        .text(day)
        .appendTo(tr);
    }
  }

  #showNextMonth(evt, isLatter) {
    let nxtMonth, year;
    evt.stopPropagation();

    if (isLatter) {
      year = this.#latterYear =
        this.#latterMonth === 11 ? this.#latterYear + 1 : this.#latterYear;
      nxtMonth = this.#latterMonth = (this.#latterMonth + 1) % 12;
    } else {
      year = this.#formerMonth === 11 ? this.#formerYear + 1 : this.#formerYear;
      nxtMonth = (this.#formerMonth + 1) % 12;

      if (nxtMonth < this.#latterMonth || year < this.#latterYear) {
        this.#formerMonth = nxtMonth;
        this.#formerYear = year;
      } else {
        nxtMonth = this.#formerMonth;
        year = this.#formerYear;
      }
    }
    this.#showCalendar(nxtMonth, year, isLatter);
  }

  #showPrevMonth(evt, isLatter) {
    let prevMonth, year;
    evt.stopPropagation();
    if (!isLatter) {
      year = this.#formerYear =
        this.#formerMonth === 0 ? this.#formerYear - 1 : this.#formerYear;
      prevMonth = this.#formerMonth =
        this.#formerMonth === 0 ? 11 : this.#formerMonth - 1;
    } else {
      year = this.#latterMonth === 0 ? this.#latterYear - 1 : this.#latterYear;
      prevMonth = this.#latterMonth === 0 ? 11 : this.#latterMonth - 1;

      if (prevMonth > this.#formerMonth || year > this.#formerYear) {
        this.#latterYear = year;
        this.#latterMonth = prevMonth;
      } else {
        prevMonth = this.#latterMonth;
        year = this.#latterYear;
      }
    }
    this.#showCalendar(prevMonth, year, isLatter);
  }

  #jumpToDate(month = null, yr = null, isLowerRange = false) {
    if (month && yr) {
      if (isLowerRange) {
        this.#formerYear = yr;
        this.#formerMonth = month;
  
        if (month >= this.#latterMonth && yr >= this.#latterYear) {
          this.#latterYear = month < 11 ? yr : yr + 1;
          this.#latterMonth = month < 11 ? month + 1 : 0;
        }
      } else {
        this.#latterYear = yr;
        this.#latterMonth = month;
  
        if (month <= this.#formerMonth && yr <= this.#formerYear) {
          this.#formerYear = month > 0 ? yr : yr - 1;
          this.#formerMonth = month > 0 ? month - 1 : 11;
        }
      }
    } 
    this.#showCalendar(this.#latterMonth, this.#latterYear);
    this.#showCalendar(this.#formerMonth, this.#formerYear, false);
  }

  #showCalendar(month, yr, latter = true) {
    const parent = latter
      ? $("#latter-calendar table.calendar")
      : $("#former-calendar table.calendar");
    const tbody = $("tbody", parent);
    $(".calendar-header", parent).text(this.#months[month] + " " + yr);
    const dateAt = new Date(yr, month),
      firstDay = dateAt.getDay(),
      maxDays = this.#getMaxDaysInMonth(dateAt);
    let day = 1;
    $(tbody).children(":nth-child(n+2)").remove();
    for (let i = 0; i < 6; i++) {
      let row = $("<tr>");
      for (let j = 0; j < 7; j++) {
        if (i === 0 && j < firstDay) {
          $(row).append($("<td>").addClass("mb-1"));
        } else if (day > maxDays) {
          break;
        } else {
          const date = new Date(yr, month, day),
            cell = $("<td>")
              .appendTo(row)
              .addClass("mb-1")
              .prop("date", date)
              .css("cursor", "pointer")
              .append($("<div>").attr("href", "javascript:void(0);").text(day))
              .on({
                click: this.#setDateRange.bind(this),
                mouseover: this.#selectRange.bind(this),
                mouseout: this.#deselectRange.bind(this),
              });
          if (
            day === this.#today.getDate() &&
            month === this.#today.getMonth() &&
            yr === this.#today.getFullYear()
          ) {
            $(cell).addClass("current");
          }
          
          if (
            this.#lowerElem &&
            date.getTime() == $(this.#lowerElem).prop("date").getTime()
          ) {
            this.#lowerElem = $(cell).addClass("active")[0];
          }
          if (
            this.#upperElem &&
            date.getTime() == $(this.#upperElem).prop("date").getTime()
          ) {
            this.#upperElem = $(cell).addClass("active")[0];
          }
          if (
            this.#lowerElem &&
            this.#upperElem &&
            date >= $(this.#lowerElem).prop("date") &&
            date <= $(this.#upperElem).prop("date")
          ) {
            const mod = j % 7,
              pos = i * 7 + j,
              isLowerDate =
                date.getTime() == $(this.#lowerElem).prop("date").getTime(),
              isUpperDate =
                date.getTime() == $(this.#upperElem).prop("date").getTime();
            if (
              mod == 0 ||
              pos == firstDay ||
              isLowerDate ||
              mod == 6 ||
              day == maxDays ||
              isUpperDate
            ) {
              if (mod == 0 || pos == firstDay || isLowerDate) {
                $(cell).addClass("selected left-end");
              }
              if (mod == 6 || day == maxDays || isUpperDate) {
                $(cell).addClass("selected right-end");
              }
            } else {
              $(cell).addClass("selected inner");
            }
          }
          if (j == 0) $(cell).addClass("text-danger");
          day++;
        }
      }
      tbody.append(row);
    }
  }

  #deselectRange() {
    $(".rangepicker-calendar .select").removeClass(
      "select left-end right-end inner"
    );
  }

  #selectRange(evt) {
    const td = $(evt.target).closest("td")[0],
      date = $(td).prop("date");

    if (this.#lowerElem && !this.#upperElem) {
      $(".rangepicker-calendar .select").removeClass(
        "select left-end right-end inner"
      );

      if (date > $(this.#lowerElem).prop("date")) {
        for (let i = 0; i < 2; i++) {
          const tbody = $(".rangepicker-calendar table.calendar tbody")[i];
          const cells = $("td", tbody).slice(7);
          const firstCell = $(cells).not(":empty")[0],
            firstDate = $(firstCell).prop("date"),
            dateAt = new Date(firstDate.getFullYear(), firstDate.getMonth()),
            firstDay = dateAt.getDay(),
            maxDays = this.#getMaxDaysInMonth(dateAt);
          const len = $(cells).length;
          let day = 1;

          for (let j = 0; j < len; j++) {
            const cell = $(cells)[j],
              cellDate = $(cell).prop("date") ?? new Date(0);

            if (
              $(cell).children("div").length == 0 ||
              cellDate < $(this.#lowerElem).prop("date") ||
              cellDate > date
            )
              continue;
            const mod = $(cell).closest("tr").find("td").index(cell) % 7,
              isLowerDate =
                cellDate.getTime() == $(this.#lowerElem).prop("date").getTime(),
              isUpperDate = cellDate.getTime() == $(td).prop("date").getTime();

            if (
              mod == 0 ||
              j == firstDay ||
              isLowerDate ||
              mod == 6 ||
              day == maxDays ||
              isUpperDate
            ) {
              if (mod == 0 || j == firstDay || isLowerDate) {
                $(cell).addClass("select left-end");
              }

              if (mod == 6 || day == maxDays || isUpperDate) {
                $(cell).addClass("select right-end");
              }
            } else {
              $(cell).addClass("select inner");
            }
            day++;
          }
        }
      }
    }
  }

  #getMaxDaysInMonth(date) {
    date.setMonth(date.getMonth() + 1);
    date.setDate(date.getDate() - 1);
    return date.getDate();
  }
  
  #getLastDayInYear(date) {
    date.setFullYear(date.getFullYear() + 1);
    date.setDate(date.getDate() - 1);
    return date;
  }

  #parseDateStr(evt, isStart = true) {
    const input = evt.target,
      dateStr = $(input).val().trim();
    $(input).val(dateStr);
    let day, month, year;
    this.#ignoreSlash = false;

    if ($(input).hasClass('invalid-date')) $(input).removeClass('invalid-date');

    if (!dateStr) {
      if (isStart) {
        if (!this.#lowerElem) return;
        if (this.#upperElem) {
          this.#lowerElem = this.#upperElem;
          $(input).val($("#endDate").val()).data("value", $("#endDate").val());
        } else {
          this.#lowerElem = null;
          $(input).removeData("value");
        }
      } else {
        if (!this.#upperElem) return;
        this.#lowerElem = null;
        $(input).removeData("value");
      }
    } else {
      const dateFormat1 = /^(\d{1,2})[\/-](\d{1,2})[\/-](\d{2}|\d{4})$/;
      const dateFormat2 = /^(\d{1,2}) (\w{3,}) (\d{4})$/;

      let isInvalid = false;
      if (dateFormat1.test(dateStr)) {
        const parts = dateStr.match(dateFormat1);
        day = parseInt(parts[1]);
        month = parseInt(parts[2]) - 1;
        year = parseInt(parts[3]);
      } else if (dateFormat2.test(dateStr)) {
        const parts = dateStr.match(dateFormat2);
        day = parseInt(parts[1]);
        month = this.#months.findIndex((m) => m.includes(parts[2]));
        year = parseInt(parts[3]);
      } else {
        isInvalid = true;
      }

      if (!isInvalid && (month < 0 || month > 11)) {
        isInvalid = true;
        $(input).focus();
      }

      if (isInvalid) {
        $(input).val($(input).data("value") ?? "").addClass('invalid-date');
        return;
      }

      if (year < 100) year += 2000;
      $(input).data("value", $(input).val()).prop("changed", false);

      if (isStart) {
        const lowerDate = new Date(year, month, day);
        this.#lowerElem = $("<td>").prop("date", lowerDate)[0];

        if (this.#upperElem && $(this.#upperElem).prop("date") < lowerDate) {
          this.#upperElem = null;
          $("#endDate").val("").removeData("value").focus();
        }
      } else {
        const upperDate = new Date(year, month, day);
        this.#upperElem = $("<td>").prop("date", upperDate)[0];

        if (!this.#lowerElem || $(this.#lowerElem).prop("date") > upperDate) {
          this.#lowerElem = this.#upperElem;
          $("#startDate").val(dateStr).data("value", dateStr);
        }
      }
    }
    this.#jumpToDate(month, year, isStart);
  }

  #setDateRange(evt) {
    const td = $(evt.target).closest("td").addClass("active")[0],
      startDateEl = $(".rangepicker-range #startDate"),
      endDateEl = $(".rangepicker-range #endDate"),
      otherDate = $(td).prop("date"),
      dateStr = otherDate.toLocaleDateString("en-GB", {
        day: "2-digit",
        month: "short",
        year: "numeric",
      });

    if (this.#lowerElem) {
      if (this.#upperElem && this.#upperElem != this.#lowerElem) {
        $(this.#upperElem).removeClass("active");
      }
      const lowerDate = $(this.#lowerElem).prop("date"),
        upperDate = this.#upperElem
          ? $(this.#upperElem).prop("date")
          : new Date(8640000000000000);

      if (otherDate >= lowerDate && otherDate <= upperDate) {
        this.#upperElem = $(td).removeClass("inner").addClass("right-end");
        $(endDateEl).val(dateStr).data("value", dateStr);
        $(".rangepicker-calendar table.calendar .select").toggleClass(
          "select selected"
        );
        $(".rangepicker-calendar table.calendar .selected")
          .filter((_, td) => {
            return $(td).prop("date") > otherDate;
          })
          .removeClass("selected left-end right-end inner");
      } else {
        $(this.#lowerElem).removeClass("active");
        this.#lowerElem = td;
        this.#upperElem = null;
        $(startDateEl).val(dateStr).data("value", dateStr);
        $(endDateEl).val("").removeData("value");
        $(
          ".rangepicker-calendar table.calendar .select, .rangepicker-calendar table.calendar .selected"
        ).removeClass("selected select left-end right-end inner");
      }
    } else {
      this.#lowerElem = td;
      $(startDateEl).val(dateStr).data("value", dateStr);
    }
    // const elem = evt.target;
    // let day = $(elem).attr("data-day"),
    //   month = $(elem).attr("data-month_name").substr(0, 3),
    //   year = $(elem).attr("data-year");
    // const len = day.length,
    //   lastChar = parseInt(day.charAt(len - 1)),
    //   selectedDate = FormDialog.#selectedDate;
    // if (selectedDate) $(selectedDate).removeClass("active");
    // $(elem).addClass("active");
    // if (lastChar == 0 || lastChar > 3 || (len > 1 && day.charAt(0) == "1")) {
    //   day += "th";
    // } else if (lastChar == 3) {
    //   day += "rd";
    // } else if (lastChar == 2) {
    //   day += "nd";
    // } else {
    //   day += "st";
    // }
    // const dateTxt = `${day} ${month}, ${year}`,
    //   toggler = $("#lessonDueDate")[0],
    //   timeElem = $("#lessonDueTime"),
    //   closeSubmitcheckbox = $(timeElem).parent("ul").next("div"),
    //   txtNode = $(toggler)
    //     .data("date", dateTxt)
    //     .contents()
    //     .filter(function () {
    //       return this.nodeType === Node.TEXT_NODE;
    //     })[0];
    // txtNode.textContent = dateTxt;
    // if ($(timeElem).hasClass("d-none")) {
    //   $(timeElem).removeClass("d-none");
    // }
    // if ($(closeSubmitcheckbox).hasClass("d-none")) {
    //   $(closeSubmitcheckbox).removeClass("d-none");
    // }
    // FormDialog.#selectedDate = elem;
  }

  #styleElements() {
    const styleElem = $("style")[0]
      ? $("style").last()[0]
      : $("<style>").appendTo($("body"))[0];
    styleElem.textContent = `#daterangepicker .modal-body{height:460px;}\n@media(max-width:992px){\n#daterangepicker .modal-body{overflow:auto;}\n}\n#daterangepicker button{outline:none;cursor:pointer;}\n.rangepicker-container{display:grid;grid-auto-columns:max-content auto max-content;grid-auto-rows:max-content auto max-content;overflow:visible;min-width:320px;}\n.rangepicker-header{align-items:flex-start;grid-column:2/4;grid-row:1;}\n.rangepicker-header>span:first-of-type{font-weight:400;font-size:0.75rem;line-height:2.66;letter-spacing:0.08333em;text-transform:uppercase;color:rgba(0, 0, 0, 0.6);margin-bottom:.5rem;}\n.rangepicker-range{flex:1;}\n.rangepicker-range .form-group{position:relative;}\n.rangepicker-range label{position:absolute;left:0.3rem;top:0;translate:0 -65%;background:white;padding:0 .2rem;cursor:text;font-weight:400;font-size:.75rem;letter-spacing:0em;color:grey;}\n.rangepicker-range input{height:2.2rem;width:7rem;padding-left:.5rem;outline:1px solid grey;outline-offset:2px;border-radius:4px;}\n.rangepicker-range input::placeholder{color:grey;}\n.rangepicker-range input.invalid-date{outline-color:red;}\n.rangepicker-range input.invalid-date+label,.rangepicker-range input.invalid-date::placeholder{color:red;}\n.rangepicker-range h5{font-weight:400;font-size:1.5rem;line-height:1.3;letter-spacing:0em}\nul.rangepicker-nav{grid-column:1;grid-row:2/3;list-style:none;}\nul.rangepicker-nav li{border-radius:50rem;cursor:pointer;font-size:0.9rem;color:rgba(0, 0, 0, 0.87);}\nul.rangepicker-nav li.active{background-color:skyblue!important;}\n.rangepicker-calendar{grid-column:6/2;grid-row:2;line-height:2.5;position:relative;}\n.rangepicker-calendar table.calendar thead tr th:first-child button:hover,.rangepicker-calendar table.calendar thead tr th:last-child button:hover{background:#f8f9fa!important;cursor:pointer;}\n.rangepicker-calendar .tab-content{max-width:100%;}\ntable.calendar thead tr{display:grid;grid-template-columns:1fr 5fr 1fr;}\ntable.calendar tbody tr{display:grid;grid-template-columns:repeat(7, 1fr);}\ntable.calendar tbody tr *{text-align:center;}\ntable.calendar tbody tr td{aspect-ratio:1;}\ntable.calendar tbody tr:nth-child(n+1) td{border:1px solid transparent;}\ntable.calendar tbody tr td.current div{border:1px solid lightgrey;}\ntable.calendar tbody tr td.select{border:1px dashed lightgrey;}\ntable.calendar tbody tr td.selected{background-color:aliceblue;}\ntable.calendar tbody tr td.select.left-end,table.calendar tbody tr td.selected.left-end{border-right:0;border-top-left-radius:50%;border-bottom-left-radius:50%;}\ntable.calendar tbody tr td.select.right-end,table.calendar tbody tr td.selected.right-end{border-left:0;border-top-right-radius:50%;border-bottom-right-radius:50%;}\ntable.calendar tbody tr td.select.left-end.right-end{border-right:1px dashed lightgrey;border-left:1px dashed lightgrey;}\ntable.calendar tbody tr td.select.inner,table.calendar tbody tr td.selected.inner{border-right:0;border-left:0;}\ntable.calendar tbody tr td div{border-radius:50%;width:100%;height:100%;}\ntable.calendar tbody tr td.active div{background-color:#004085;color:white;}\ntable.calendar tbody tr td:not(.active):not(:empty):hover div{background-color:#e1ebfd;}\n.vr{display:inline-block;align-self:stretch;width:1px;min-height:1em;background-color:currentcolor;opacity:.25;}\n#former-calendar,#latter-calendar{flex:1;}`;
  }
}

if (typeof toast !== "function") {
  function toast(msg, color = "green", time) {
    time = time || 2000;
    var ts = $("<div>")
      .appendTo("body")
      .css({
        overflow: "hidden",
        display: "block",
        position: "fixed",
        "z-index": "8001",
        top: "20%",
        left: "4%",
        "background-color": color,
        "border-radius": "2px",
        "text-align": "center",
      })
      .attr({ id: "toast-container_" })
      .append(
        $("<div>")
          .css({
            "background-color": color,
            opacity: "1",
            float: "left",
            padding: "5px 10px",
            "text-align": "center",
            "line-height": "30px",
            "margin-top": "0px",
            "font-size": "15px",
            color: "white",
            "font-family": "'Raleway', sans-serif",
          })
          .addClass("toast_")
          .text(msg)
      );
    $(ts).animate({ top: 0 }, 2000);
    setTimeout(function () {
      $(ts).remove();
    }, time);
  }
}
