class FormDialog {
  #topicList = [];
  static #today;
  static #currentMonth;
  static #currentYear;
  static #months;
  static #selectedDate;
  clickedInside = false;
  #selectionRange = document.getSelection();
  #current;
  #relElem;
  #draggedElem;
  static surveyForm;

  static {
    this.#today = new Date();
    this.#currentMonth = FormDialog.#today.getMonth();
    this.#currentYear = FormDialog.#today.getFullYear();
    this.#months = [
      "January",
      "February",
      "March",
      "April",
      "May",
      "June",
      "July",
      "August",
      "September",
      "October",
      "November",
      "December",
    ];
    this.surveyForm = false;
  }

  constructor() {
    this.#createDialog();
  }

  #createDialog() {
    if (!$("#lessonDialog")[0]) {
      const lessonDialog = $("<div>")
        .addClass("modal fade")
        .attr({
          id: "lessonDialog",
          tabindex: "-1",
          role: "dialog",
          "data-keyboard": false,
          "aria-labelledby": "lessonDialogLabel",
          "aria-hidden": "true",
        })
        .append(
          $("<div>")
            .addClass("modal-dialog m-0")
            .attr("role", "document")
            .append(
              $("<div>")
                .addClass("modal-content")
                .css({ width: "100vw", "min-width": "100vw", height: "100vh" })
                .append(
                  $("<div>")
                    .addClass("modal-header border-bottom py-2")
                    .append(
                      $("<h5>")
                        .addClass("modal-title")
                        .attr("id", "lessonDialogLabel")
                    )
                    .append(
                      $("<button>")
                        .addClass("close")
                        .attr({
                          type: "button",
                          "data-dismiss": "modal",
                          "aria-label": "Close",
                        })
                        .css("top", "20px")
                        .append(
                          $("<span>").attr("aria-hidden", "true").text("×")
                        )
                    )
                )
                .append(
                  $("<div>")
                    .addClass("modal-body bg-light custom-scrollbar p-0")
                    .css({ "overflow-y": "auto", "overflow-x": "hidden" })
                    .append(
                      $("<div>")
                        .addClass("container-fluid")
                        .append(
                          $("<div>")
                            .addClass("row flex-md-nowrap")
                            .append(
                              $("<div>").addClass(
                                "col-12 col-md-8 col-lg-9 col-xl d-flex flex-column align-items-center mx-3 mb-auto left-pane"
                              )
                            )
                            .append(
                              $("<div>").addClass(
                                "col-12 col-md-4 col-lg-3 bg-white ml-auto right-pane"
                              )
                            )
                        )
                    )
                )
                .append(
                  $("<div>")
                    .addClass("modal-footer py-2 border-top")
                    .append(
                      $("<button>")
                        .addClass("btn text-dark bg-light mr-3 px-4")
                        .attr({ type: "button", "data-dismiss": "modal" })
                        .text("Cancel")
                    )
                    .append(
                      $("<button>")
                        .addClass("btn btn-primary disabled px-4")
                        .attr({ type: "button", id: "addLesson" })
                        .text("Add")
                        .click(() => this.#addLesson())
                    )
                )
            )
        )
        .on({
          "show.bs.modal": this.#onShowModal.bind(this),
          "shown.bs.modal": () => {
            if (FormDialog.surveyForm) {
              $.get(`${site.process}api?option=course_list`, (jsonRes) => {
                const courses = JSON.parse(jsonRes);
                this.#putCourses(courses);
              }).catch((err) => console.error(err));
            }
            setTimeout(() => {
              $("#lessonTitle").focus();
            }, 500);
          },
          "hidden.bs.modal": this.#onModalHidden.bind(this),
        });
      $("#uploadDialog")[0]
        ? $(lessonDialog).insertBefore($("#uploadDialog"))
        : $(lessonDialog).insertBefore($("body").find("script").first());
      this.#reset(
        $("#lessonDialog .modal-body .left-pane"),
        $("#lessonDialog .modal-body .right-pane")
      );
      $("#lessonDialog .modal-content").timePicker(this);
      $('[data-toggle="tooltip"]', lessonDialog).tooltip();
      this.#createStyles();
    }
  }

  #reset(leftPane, rightPane) {
    const removeObserver = $("#lessonDialog .editable-container").prop(
      "observer"
    );
    if (removeObserver) removeObserver();
    $(leftPane)
      .empty()
      .append(
        $("<div>")
          .addClass("card no-scaling shadow-0 mb-3 mt-2 w-100")
          .css("max-width", "936px")
          .append(
            $("<div>")
              .addClass("card-body")
              .css("min-height", "unset")
              .append(
                $("<div>")
                  .addClass("form-group create-lesson-title py-2 mb-3")
                  .append(
                    $("<input>")
                      .addClass("form-control bg-transparent w-100")
                      .attr({
                        type: "text",
                        id: "lessonTitle",
                        placeholder: "Title",
                      })
                      .css({ border: "none", "font-size": "medium" })
                      .on("input", this.#checkToEnableBtn.bind(this))
                  )
                  .append($("<label>").attr("for", "lessonTitle").text("Title"))
              )
              .append(
                $("<div>")
                  .addClass("editable-container")
                  .append(
                    $("<p>")
                      .addClass("px-3 pt-4 mb-0")
                      .css("min-height", "10rem")
                      .attr("contenteditable", "true")
                  )
                  .append($("<span>").text("Description (optional)"))
              )
              .append(
                $("<div>")
                  .addClass("btn-toolbar mb-2 ml-2")
                  .attr("role", "toolbar")
                  .append(
                    $("<div>").addClass("btn-group format-btns").attr({
                      role: "group",
                      "aria-label": "Text formatting buttons",
                    })
                  )
              )
              .append(
                $("<div>")
                  .addClass("d-flex my-3")
                  .append(
                    $("<div>")
                      .addClass(
                        "btn-toolbar flex-grow-1 justify-content-center"
                      )
                      .attr({
                        role: "toolbar",
                        "aria-label": "Toolbar with button groups",
                      })
                  )
              )
              .append($("<div>").addClass("attachments"))
          )
      );
    this.#addFormatBtns($("#lessonDialog .btn-toolbar .btn-group"));
    this.#addAttachBtns($("#lessonDialog .btn-toolbar").last());
    $(".editable-container", leftPane).richEditor(this.#selectionRange);

    $(rightPane)
      .empty()
      .append(
        $("<div>")
          .addClass("dropdown card-body w-100")
          .css("min-width", "170px")
          .append($("<label>").text("Topic *"))
          .append(
            $("<button>")
              .addClass(
                "btn btn-lg bg-light dropdown-toggle customSelect text-dark shadow-0 w-100 text-start px-3 rounded-0 text-truncate"
              )
              .attr({
                type: "button",
                id: "topicSelect",
                "data-toggle": "dropdown",
                "aria-haspopup": "true",
                "aria-expanded": "false",
              })
              .text("No topic")
          )
          .append(
            $("<div>")
              .addClass("dropdown-menu topicList")
              .attr("aria-labelledby", "topicSelect")
              .css("width", "86%")
          )
      );
    this.#refreshTopicList();
  }

  #addFormatBtns(container) {
    const btns = [
      ["fas fa-bold", "Bold", "strong"],
      ["fa fa-italic", "Italic", "em"],
      ["fa fa-underline", "Underline", "line"],
      ["fa fa-list", "List", "list"],
      [
        `<svg width="24" height="24" viewBox="0 0 640 512" fill="currentColor"><path d="M336 416h-11.2l9.3-27.8L267 336.4 240.5 416H208a16 16 0 0 0 -16 16v32a16 16 0 0 0 16 16h128a16 16 0 0 0 16-16v-32a16 16 0 0 0 -16-16zm297.8 42.1L377 259.6 426.2 112H544v32a16 16 0 0 0 16 16h32a16 16 0 0 0 16-16V48a16 16 0 0 0 -16-16H176a16 16 0 0 0 -16 16v43.9L45.5 3.4A16 16 0 0 0 23 6.2L3.4 31.5a16 16 0 0 0 2.8 22.5l588.4 454.7a16 16 0 0 0 22.5-2.8l19.6-25.3a16 16 0 0 0 -2.8-22.5zM309.9 207.8L224 141.4V112h117.8z" /></svg>`,
        "Remove formatting",
        "format",
      ],
    ];
    let i = 0;

    for (const [icon, title, tag] of btns) {
      $(container).append(
        $("<button>")
          .addClass("btn bg-transparent text-dark edit-btn rounded-0")
          .attr({
            type: "button",
            "data-toggle": "tooltip",
            "data-placement": "top",
            title,
            tag,
          })
          .prop("tag", tag)
          .append(i < 4 ? $("<i>").addClass(icon) : icon)
      );
      i++;
    }
  }

  #addAttachBtns(container) {
    const btns = [
      [`<img src="${site.domain}asset/picture/google-drive.svg">`, "Drive"],
      [`<img src="${site.domain}asset/picture/youtube.svg">`, "YouTube"],
      [
        `<svg viewBox="0 0 24 24" class="align-bottom"><path d="M4 15h2v3h12v-3h2v3c0 1.1-.9 2-2 2H6c-1.1 0-2-.9-2-2m4.41-7.59L11 7.83V16h2V7.83l2.59 2.59L17 9l-5-5-5 5 1.41 1.41z"></path></svg>`,
        "Upload",
      ],
      [
        `<svg viewBox="0 0 24 24" class="align-bottom"><path d="M17 7h-4v2h4c1.65 0 3 1.35 3 3s-1.35 3-3 3h-4v2h4c2.76 0 5-2.24 5-5s-2.24-5-5-5zm-6 8H7c-1.65 0-3-1.35-3-3s1.35-3 3-3h4V7H7c-2.76 0-5 2.24-5 5s2.24 5 5 5h4v-2z" /><path d="M8 11h8v2H8z" /></svg>`,
        "Link",
      ],
    ];

    for (const [elem, title] of btns) {
      const action = title.toLowerCase();
      $("<div>")
        .addClass("btn-group-vertical mr-4 align-items-center")
        .appendTo(container)
        .append(
          $("<button>")
            .addClass("btn bg-transparent edit-btn border")
            .attr({
              type: "button",
              "data-toggle": "modal",
              "data-target": "#uploadDialog",
              "data-action": action,
            })
            .append($(elem).attr({ width: "24", height: "24" }))
            .click(
              () => (FileDialog.addAttachment = this.#addAttachment.bind(this))
            )
        )
        .append($("<h6>").addClass("text-capitalize mt-2").text(title));
    }
  }

  updateTopics(list) {
    this.#topicList = list.map(({ topic, index }) => ({ topic, index }));
    this.#refreshTopicList();
  }

  #refreshTopicList() {
    const topicListElem = $("#lessonDialog .topicList");

    if ($(topicListElem)[0]) {
      $(topicListElem)
        .empty()
        .append(
          $("<a>")
            .addClass(
              `dropdown-item${this.#topicList.length ? " text-muted" : ""}`
            )
            .attr({
              href: "#generalDialog",
              "data-toggle": "modal",
              "data-action": "lesson",
            })
            .text("Create a topic")
        );

      for (const { topic, index } of this.#topicList) {
        const topicElem = $("<a>")
          .appendTo(topicListElem)
          .addClass("dropdown-item text-truncate")
          .attr("href", "javascript:void(0);")
          .text(topic)
          .data({ topic, index })
          .click((evt) => {
            const topicElem = evt.target;
            $("#topicSelect")
              .text($(topicElem).data("topic"))
              .data("index", $(topicElem).data("index"));
            this.#checkToEnableBtn();
          });

        if (this.#topicList.length == 1) {
          $(topicElem).click();
        }
      }

      if (!this.#topicList.length) {
        $(topicListElem).prepend(
          $("<h6>")
            .addClass("dropdown-header text-muted")
            .css({ "text-transform": "none", "font-size": "12px" })
            .text("No topic created")
        );
      }
    }
  }

  #setMaterial() {
    $("h5#lessonDialogLabel").html(
      `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" focusable="false" width="1em" height="1em" class="align-middle rounded-circle svg-bg mr-2" fill="currentColor"><path d="M18 2H6c-1.1 0-2 .9-2 2v16c0 1.1.9 2 2 2h12c1.1 0 2-.9 2-2V4c0-1.1-.9-2-2-2zm0 18H6V4h2v8l2.5-1.5L13 12V4h5v16z" /></svg> Material`
    );
  }

  #setAssignment() {
    $("h5#lessonDialogLabel").html(
      `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" focusable="false" width="1em" height="1em" class="align-middle rounded-circle svg-bg mr-2" fill="currentColor"><path d="M19 2H5a2 2 0 0 0-2 2v14a2 2 0 0 0 2 2h4l3 3 3-3h4c1.1 0 2-.9 2-2V4c0-1.1-.9-2-2-2zm0 16H5V4h14v14z" /><path d="M11 15h2v2h-2zm1-8c1.1 0 2 .9 2 2 0 2-3 1.75-3 5h2c0-2.25 3-2.5 3-5 0-2.21-1.79-4-4-4S8 6.79 8 9h2c0-1.1.9-2 2-2z" /></svg> Assignment`
    );
    this.#addGradeForm();
  }

  #setQuiz(edit) {
    $("h5#lessonDialogLabel").html(
      `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" focusable="false" width="1em" height="1em" class="align-middle rounded-circle svg-bg mr-2" fill="currentColor"><path d="M7 15h7v2H7zm0-4h10v2H7zm0-4h10v2H7z"></path><path d="M19 3h-4.18C14.4 1.84 13.3 1 12 1c-1.3 0-2.4.84-2.82 2H5c-.14 0-.27.01-.4.04a2.008 2.008 0 0 0-1.44 1.19c-.1.23-.16.49-.16.77v14c0 .27.06.54.16.78s.25.45.43.64c.27.27.62.47 1.01.55.13.02.26.03.4.03h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm-7-.25c.41 0 .75.34.75.75s-.34.75-.75.75-.75-.34-.75-.75.34-.75.75-.75zM19 19H5V5h14v14z" /></svg> Quiz`
    );
    this.#addGradeForm(true);
    if (!edit)
      this.richForm($("#lessonDialog .modal-body .left-pane"), null, true);
  }

  #setSurvey(edit) {
    $("h5#lessonDialogLabel").html(
      `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="align-middle rounded-circle svg-bg mr-2">
      <line x1="18" y1="20" x2="18" y2="10"></line>
      <line x1="12" y1="20" x2="12" y2="4"></line>
      <line x1="6" y1="20" x2="6" y2="14"></line>
    </svg> Survey`
    );
    const attachmentElem = $("#lessonDialog .attachments");
    $(attachmentElem).prev("div.d-flex").remove();
    $(attachmentElem).remove();
    this.#addGradeForm();

    if (!edit)
      this.richForm($("#lessonDialog .modal-body .left-pane"), null, true);
  }

  #addGradeForm(isQuiz = false) {
    const container = $("#lessonDialog .modal-body .right-pane"),
      instance = this;

    if (FormDialog.surveyForm) $(container).empty();
    else
      $("#lessonDialog .editable-container span").text(
        "Instruction (optional)"
      );

    $("<div>")
      .addClass("dropdown card-body pb-0 w-100")
      .prependTo(container)
      .append($("<label>").text("Due date & time"))
      .append(
        $("<button>")
          .addClass(
            "btn btn-lg bg-light dropdown-toggle customSelect text-dark shadow-0 w-100 text-start px-3 rounded-0"
          )
          .attr({
            type: "button",
            id: "lessonDueDatetime",
            "data-toggle": "dropdown",
            "aria-haspopup": "true",
            "aria-expanded": "false",
          })
          .text("No due date ")
      )
      .append(
        $("<ul>")
          .addClass("dropdown-menu")
          .attr("aria-labelledby", "lessonDueDatetime")
          .css("width", "86%")
          .append(
            $("<li>")
              .addClass("dropdown-header text-muted text-capitalize py-0")
              .css({ "font-size": "inherit", "font-weight": "400" })
              .text("Due date & time")
          )
          .append($("<div>").addClass("dropdown-divider"))
          .append(
            $("<li>")
              .addClass("dropdown w-100 px-3 mb-3 mt-2")
              .append(
                $("<a>")
                  .addClass("dropdown-item bg-light dropdown-toggle")
                  .attr({
                    href: "javascript:void(0);",
                    role: "button",
                    id: "lessonDueDate",
                    "data-toggle": "dropdown",
                    "aria-haspopup": "true",
                    "aria-expanded": "false",
                  })
                  .text("No due date ")
                  .append(
                    $("<button>")
                      .attr({ type: "button", "aria-label": "Close" })
                      .addClass("close d-none")
                      .css("outline", "none")
                      .append($("<span>").attr("aria-hidden", "true").text("×"))
                      .click(function (evt) {
                        instance.#clearDateTime(this, evt, true);
                      })
                  )
                  .click(this.#setInitDate.bind(this))
              )
              .append(
                $("<div>")
                  .addClass("dropdown-menu w-100 shadow-0 p-0 d-none")
                  .attr("aria-labelledby", "lessonDueDate")
                  .append(
                    $("<div>")
                      .addClass("p-2 border")
                      .append(
                        $("<table>")
                          .addClass("calendar w-100")
                          .attr({
                            role: "presentation",
                            cellspacing: "0",
                            cellpadding: "0",
                          })
                          .append(
                            $("<thead>").append(
                              $("<tr>")
                                .attr("role", "row")
                                .append(
                                  $("<th>").append(
                                    $("<button>")
                                      .addClass(
                                        "btn bg-transparent p-0 shadow-0"
                                      )
                                      .attr({
                                        "aria-hidden": "true",
                                        tabindex: "-1",
                                      })
                                      .html(
                                        `<svg width="24" height="24" viewBox="0 0 24 24" focusable="false"><path d="M14 7l-5 5 5 5V7z"></path></svg>`
                                      )
                                      .click((evt) =>
                                        FormDialog.#showPrevMonth(evt)
                                      )
                                  )
                                )
                                .append(
                                  $("<th>")
                                    .addClass(
                                      "text-center pt-1 calendar-header"
                                    )
                                    .css("font-size", "small")
                                    .attr("colspan", "5")
                                )
                                .append(
                                  $("<th>")
                                    .addClass("text-end")
                                    .append(
                                      $("<button>")
                                        .addClass(
                                          "btn bg-transparent p-0 shadow-0"
                                        )
                                        .attr({
                                          "aria-hidden": "true",
                                          tabindex: "-1",
                                        })
                                        .html(
                                          `<svg width="24" height="24" viewBox="0 0 24 24" focusable="false"><path d="M10 17l5-5-5-5v10z"/></svg>`
                                        )
                                        .click((evt) =>
                                          FormDialog.#showNextMonth(evt)
                                        )
                                    )
                                )
                            )
                          )
                          .append($("<tbody>"))
                      )
                  )
              )
          )
          .append(
            $("<a>")
              .addClass("dropdown-item mx-3 bg-light mb-2 d-none")
              .attr({
                href: "#timePicker",
                role: "button",
                id: "lessonDueTime",
                "data-toggle": "modal",
              })
              .css("width", "87%")
              .text("Time (optional) ")
              .append(
                $("<button>")
                  .addClass("close d-none")
                  .attr({ type: "button", "aria-label": "Close" })
                  .css("outline", "none")
                  .append($("<span>").attr("aria-hidden", "true").text("×"))
                  .click(function (evt) {
                    instance.#clearDateTime(this, evt);
                  })
              )
          )
      )
      .append(
        $("<div>")
          .addClass("custom-control custom-checkbox d-none")
          .append(
            $("<input>")
              .addClass("custom-control-input")
              .attr({ type: "checkbox", id: "submitAfterDue" })
          )
          .append(
            $("<label>")
              .addClass("custom-control-label")
              .attr("for", "submitAfterDue")
              .css("cursor", "pointer")
              .text("Close submissions after due date")
          )
      )
      .on({
        "hide.bs.dropdown": this.#onHideDropdown.bind(this),
        click: (evt) => {
          const dropdown = $("#lessonDueDatetime").next(".dropdown-menu")[0];

          if (dropdown == evt.target || $(dropdown).has(evt.target)[0])
            this.clickedInside = true;
        },
      });
    this.#createDays();
    const focusin = function () {
      $(this)
        .attr("placeholder", "")
        .parent()
        .next("div")
        .removeClass("d-none");
    };

    if (FormDialog.surveyForm) {
      $(container)
        .prepend(
          $("<div>")
            .addClass("form-group card-body mb-0")
            .append($("<span>").text("Response status"))
            .append(
              $("<div>")
                .addClass("switch mt-2")
                .append(
                  $("<label>")
                    .addClass("text-muted")
                    .append("Closed")
                    .append(
                      $("<input>").attr({
                        type: "checkbox",
                        id: "responseStatus",
                      })
                    )
                    .append($("<span>").addClass("lever"))
                    .append("Accepting")
                )
            )
        )
        .append(
          $("<div>")
            .addClass("dropdown card-body w-100 pt-4")
            .prependTo(container)
            .append($("<label>").text("Course(s) to apply form"))
            .append(
              $("<button>")
                .addClass(
                  "btn btn-lg bg-light dropdown-toggle customSelect text-dark shadow-0 w-100 text-start px-3 rounded-0 text-truncate disabled"
                )
                .attr({ id: "selectSurveyCourse", "data-toggle": "dropdown" })
                .append(
                  $("<div>")
                    .addClass("spinner-border spinner-border-sm")
                    .attr("role", "status")
                )
                .click(function () {
                  $(this).next(".dropdown-menu").toggleClass("d-none");
                })
            )
            .append(
              $("<div>")
                .addClass("dropdown-menu d-none")
                .attr("id", "surveyCourseList")
                .css({
                  position: "absolute",
                  transform: "translate3d(21px, 108px, 0px)",
                  top: "0px",
                  left: "0px",
                  "will-change": "transform",
                  visibility: "visible",
                  opacity: "1",
                })
            )
        );
      return;
    } else if (isQuiz) {
      $("<div>")
        .addClass("dropdown card-body col-6 pr-0 pb-0")
        .css("min-width", "170px")
        .prependTo(container)
        .append($("<label>").text("Duration (minutes)"))
        .append(
          $("<button>")
            .addClass(
              "btn btn-lg bg-light dropdown-toggle customSelect text-dark shadow-0 w-100 text-start px-3 rounded-0 d-flex"
            )
            .append(
              $("<input>")
                .addClass(
                  "bg-transparent border-0 p-0 form-control flex-grow-1"
                )
                .attr({
                  type: "text",
                  value: "0",
                  placeholder: "0",
                  id: "quizDuration",
                })
                .css({ outline: "none", "font-size": "medium" })
                .data("duration", "Not timed")
                .on({
                  focusin,
                  focusout: (evt) => this.#setGradeOrDuration(evt, true),
                })
            )
            .click(function () {
              $("input", this).trigger("focusin");
            })
        )
        .append(
          $("<div>")
            .addClass("dropdown-menu d-none")
            .css({
              position: "absolute",
              transform: "translate3d(21px, 108px, 0px)",
              top: "0px",
              left: "0px",
              "will-change": "transform",
              visibility: "visible",
              opacity: "1",
            })
            .append(
              $("<a>")
                .addClass("dropdown-item")
                .attr("href", "javascript:void(0);")
                .text("Not timed")
                .click(() =>
                  $("#quizDuration").val("0").data("duration", "Not timed")
                )
            )
        );
    }

    $("<div>")
      .addClass("dropdown card-body col-6 pr-0 pb-0")
      .css("min-width", "160px")
      .prependTo(container)
      .append($("<label>").text("Total points (%)"))
      .append(
        $("<button>")
          .addClass(
            "btn btn-lg bg-light dropdown-toggle customSelect text-dark shadow-0 w-100 text-start px-3 rounded-0"
          )
          .append(
            $("<input>")
              .addClass("bg-transparent border-0 p-0 w-75")
              .attr({
                type: "text",
                value: "0",
                placeholder: "0",
                id: "lessonPoints",
              })
              .css({ outline: "none", "font-size": "medium" })
              .data("points", "Ungraded")
              .on({
                focusin,
                focusout: (evt) => this.#setGradeOrDuration(evt),
              })
          )
          .click(function () {
            $("input", this).trigger("focusin");
          })
      )
      .append(
        $("<div>")
          .addClass("dropdown-menu d-none")
          .css({
            position: "absolute",
            transform: "translate3d(21px, 108px, 0px)",
            top: "0px",
            left: "0px",
            "will-change": "transform",
            visibility: "visible",
            opacity: "1",
          })
          .append(
            $("<a>")
              .addClass("dropdown-item")
              .attr("href", "javascript:void(0);")
              .text("Ungraded")
              .click(() =>
                $("#lessonPoints").val("0").data("points", "Ungraded")
              )
          )
      );
  }

  #addOption(index, initial = false) {
    let optionsLen = $(`#multichoice${index} .option-item`).length,
      start = optionsLen + 1,
      end = optionsLen + (initial ? 2 : 1);
    const focusout = (evt) => {
      const label = $(evt.target).closest("label");

      if (!$(label).prop("changed")) {
        const index = $(label).prop("index");
        label[0].childNodes[0].textContent = `Option ${index}`;
      } else if (this.#checkForDuplicate(evt)) {
        $(label).addClass("border-danger");
      }
    };

    for (let i = start; i <= end; i++) {
      const optionElem = $("<div>")
        .insertBefore($(`#addOption${index}`))
        .addClass("option-item d-flex justify-content-between mb-3 drag-option")
        .append(
          $(
            `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24" height="24" class="text-muted mr-2" style="cursor:move;fill:currentColor;"><circle cy="6.5" cx="9.5" r="1.5"></circle><circle cy="6.5" cx="14.5" r="1.5"></circle><circle cy="12.5" cx="9.5" r="1.5"></circle><circle cy="12.5" cx="14.5" r="1.5"></circle><circle cy="18.5" cx="9.5" r="1.5"></circle><circle cy="18.5" cx="14.5" r="1.5"></circle></svg>`
          ).on("mousedown", this.#startDragDrop.bind(this))
        )
        .append(
          $("<div>")
            .addClass("custom-control custom-radio flex-grow-1 mr-0 pr-3 mb-1")
            .append(
              $("<input>")
                .attr({
                  id: `question${index}-option${i}`,
                  name: `question${index}-option`,
                  type: "radio",
                })
                .prop("disabled", FormDialog.surveyForm)
                .addClass("custom-control-input")
            )
            .append(
              $("<label>")
                .addClass("custom-control-label w-100 mr-3")
                .attr("for", `question${index}-option${i}`)
                .css("cursor", "text")
                .prop({ index: i, changed: false })
                .click((evt) => this.#highlightOption(evt, index))
            )
        )
        .append(
          !FormDialog.surveyForm
            ? $(
                `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 20 20" width="1.5em" height="1.5em" fill="currentColor" data-toggle="modal" data-target="#uploadDialog" data-action="image" data-multiple="false" class="optImage" style="cursor:pointer;"><path d="M19 2.5A1.5 1.5 0 0017.5 1h-15A1.5 1.5 0 001 2.5v15A1.5 1.5 0 002.5 19H10v-2H3.497c-.41 0-.64-.46-.4-.79l3.553-4.051c.19-.21.52-.21.72-.01L9 14l3.06-4.781a.5.5 0 01.84.02l.72 1.251A5.98 5.98 0 0116 10h3V2.5zm-11.5 2c1.1 0 2 .9 2 2s-.9 2-2 2-2-.9-2-2 .9-2 2-2zm12.207 10.793A1 1 0 0019 15h-2v-2a1 1 0 00-2 0v2h-2a1 1 0 000 2h2v2a1 1 0 102 0v-2h2a1 1 0 00.707-1.707z" /></svg>`
              ).click(
                (evt) =>
                  (FileDialog.addAttachment = (data) =>
                    this.#attachImage(data, evt.target))
              )
            : ""
        );
      this.#addListeners(optionElem);

      if (initial) {
        !FormDialog.surveyForm &&
          i == 1 &&
          $(`#question${index}-option1`).prop("checked", true);

        $(`#question${index}-option${i}`)
          .next("label")
          .attr("contenteditable", "")
          .css("outline", "none")
          .text(`Option ${i}`)
          .on({ input: (evt) => this.#checkForDuplicate(evt), focusout });
      } else {
        const span = $(`label[for="question${index}-option${i}"]`)
          .append(
            $("<span>")
              .text(`Option ${i}`)
              .addClass("w-100")
              .css("outline", "none")
              .attr("contenteditable", "")
          )
          .append(
            $("<button>")
              .addClass("close float-right")
              .css("outline", "none")
              .append($("<span>").html("×"))
              .click(function () {
                const disOption = $(this).closest(".option-item"),
                  sibs = $(disOption).parent().children(),
                  j = $(sibs).index(disOption[0]) + 1;
                $(sibs)
                  .not(disOption)
                  .filter(`:nth-child(n+${j})`)
                  .each(function (i, div) {
                    const newIndex = j + i;
                    const span = $("input", div)
                      .attr("id", `question${index}-option${newIndex}`)
                      .next("label")
                      .attr("for", `question${index}-option${newIndex}`)
                      .prop("index", newIndex)
                      .find("span")
                      .first();

                    if (
                      $(span)
                        .text()
                        .match(/option\s\d+/i)
                    ) {
                      $(span).text(`Option ${newIndex}`);
                    }
                  });
                $(disOption).remove();
              })
          )
          .find("span")
          .first()
          .on({ input: (evt) => this.#checkForDuplicate(evt), focusout })
          .focus()[0];

        if (this.#selectionRange.rangeCount == 0)
          this.#selectionRange.addRange(new Range());
        const range = this.#selectionRange.getRangeAt(0);
        range.selectNodeContents(span);
      }
    }
  }

  #checkForDuplicate(evt) {
    const label = $(evt.target).closest("label"),
      curOption = label[0].childNodes[0]?.textContent?.trim() ?? "",
      labels = $(label)
        .closest(".tab-pane")
        .find(".option-item label")
        .not(label);
    let hasDuplicate = false;

    if (curOption == "") {
      if ($(label).prop("changed")) $(label).prop("changed", false);
      hasDuplicate = true;
    } else {
      if (!$(label).prop("changed")) $(label).prop("changed", true);

      for (let i = 0; i < labels.length; i++) {
        const elem = labels[i],
          elemOption = elem.childNodes[0].textContent.trim();

        if (elemOption == curOption) {
          hasDuplicate = true;
          break;
        }
      }
    }

    if (hasDuplicate) {
      const dupElem = $(".duplicate", label)[0]
        ? $(".duplicate", label)
        : $("<small>")
            .addClass("text-danger duplicate")
            .prop("contenteditable", false)
            .css({
              position: "absolute",
              top: "100%",
              left: "0",
              "font-size": "x-small",
            })
            .appendTo(
              $(label).addClass("text-danger").css("position", "relative")
            );
      $(dupElem).text(
        curOption == "" ? "Option cannot be empty" : "Option already exists"
      );
    } else if ($(label).find(".duplicate")) {
      $(label).removeClass("text-danger").find(".duplicate").remove();
    }
    return hasDuplicate;
  }

  #findProblemsInQuiz(container = null) {
    container ??= $("#lessonDialog");
    let hasProblems = false;
    const qcards = $(container).find(".qcard");

    for (let i = 0; i < qcards.length; i++) {
      const qcard = qcards[i];
      const qtype = $('[id^="questionType"]', qcard).data("qtype");
      const qtxt = $('[id^="quizQuestion"]', qcard).val().trim();

      if (!qtxt.match(/\b\w+\b/)) return true;

      switch (qtype) {
        case "multiple":
          const labels = $(".option-item label", qcard);

          for (let j = 0; j < labels.length; j++) {
            const label = labels[j];
            if ($(label).find(".duplicate")[0] && !hasProblems) return true;
          }

          if (
            !FormDialog.surveyForm &&
            !$(".option-item input:checked", qcard)[0]
          )
            return true;
          break;

        case "short":
          if (FormDialog.surveyForm) break;
          const ansTxt = $('[id^="shortAnswer"]', qcard).val().trim();

          if (ansTxt == "") return true;
      }
    }
    return false;
  }

  #editMaterial(lessonObj) {
    $("#lessonTitle").val(lessonObj.title);
    $("#lessonDialog p[contenteditable]").html(
      lessonObj.description || lessonObj.instruction
    );
    $("#addLesson").removeClass("disabled").text("Edit");

    if (FormDialog.surveyForm) return;
    const topicData = this.#topicList.find(
      (obj) => obj.topic == lessonObj.parent.topic
    );
    $("#topicSelect")
      .text(lessonObj.parent.topic)
      .data("index", topicData.index);
    this.#addAttachment(lessonObj.attachments ?? []);
  }

  #editAssigment(lessonObj) {
    const { dueDate, closeSubmitAfterDue, totalPoints } = FormDialog.surveyForm
      ? lessonObj
      : lessonObj.grading;
    this.#editMaterial(lessonObj);
    const datetime = dueDate.match(
      /(\d{1,2}(?:st|nd|th) [A-Za-z]{3}, \d{4})\s?(\d{1,2}:\d{2} [AP]M)?/
    );

    if (datetime) {
      $("#lessonDueDatetime").text(dueDate).data("datetime", dueDate);
      $("#lessonDueDate").data(
        "date",
        datetime[1]
      )[0].childNodes[0].textContent = datetime[1];

      if (datetime[2]) this.setTime(datetime[2]);
      $("#submitAfterDue")
        .prop("checked", closeSubmitAfterDue)
        .removeClass("d-none");
    }

    if (FormDialog.surveyForm) return;
    totalPoints != "Ungraded" &&
      $("#lessonPoints").val(totalPoints).data("points", totalPoints);
  }

  #editQuiz(lessonObj) {
    const duration = lessonObj.grading.duration,
      parentElem = $("#lessonDialog .modal-body .left-pane");
    this.#editAssigment(lessonObj);
    duration != "Not timed" &&
      $("#quizDuration").val(duration).data("duration", duration);
    let index = parseInt(Date.now().toString().slice(-10));

    for (const qObj of lessonObj.questionSet) {
      this.richForm(parentElem, null, true, qObj, index);
      index++;
    }
  }

  #editSurvey(surveyObj) {
    const parentElem = $("#lessonDialog .modal-body .left-pane"),
      course = surveyObj.course;
    this.#editAssigment(surveyObj);
    $("#responseStatus").prop("checked", surveyObj.available);

    if (course)
      $("#selectSurveyCourse").text(course.name).data("course", course);
    let index = parseInt(Date.now().toString().slice(-10));

    for (const qObj of surveyObj.questions) {
      this.richForm(parentElem, null, true, qObj, index);
      index++;
    }
  }

  #addLesson() {
    if (FormDialog.surveyForm) return this.#addSurvey();
    const topic = $("#topicSelect").text(),
      index = $("#topicSelect").data("index"),
      editMode = this.#relElem ? $(this.#relElem).prop("edit") : null,
      oldLessonObj = editMode
        ? $(this.#relElem).closest(".card").data("lesson")
        : null,
      attachments = $("#lessonDialog .attachments").children(),
      desc = $("#lessonDialog p[contenteditable]").html(),
      type = this.#current;

    if (!editMode) {
      if (!courseObj.topics.some((obj, i) => obj.topic == topic && i == index)) {
        toast("Select a valid topic", "black");
        return;
      }
    }
    const parent = courseObj.topics[index],
      otherParent = oldLessonObj?.parent,
      sameTopic = otherParent == parent;

    const suffix = Date.now().toString().slice(-10),
      lessonObj = { type, attachments: [] };
    lessonObj.parent = parent;

    if (editMode) {
      lessonObj.key = oldLessonObj.key;

      if (sameTopic) {
        const index = parent.lessons.findIndex(
          (lesson) => lesson == oldLessonObj
        );
        parent.lessons.splice(index, 1, lessonObj);
      } else {
        const index = otherParent.lessons.findIndex(
          (lesson) => lesson == oldLessonObj
        );
        otherParent.lessons.splice(index, 1);
        parent.lessons.push(lessonObj);
      }
    } else {
      parent.lessons.push(lessonObj);
    }
    lessonObj.title = $("#lessonTitle").val();
    type == "material"
      ? (lessonObj.description = desc)
      : (lessonObj.instruction = desc);

    const lessonElem = this.returnLessonCard(lessonObj, suffix);

    if (editMode && sameTopic) {
      $(this.#relElem).closest(".card").replaceWith(lessonElem);
    } else {
      $(lessonElem).appendTo($(`#${parent.id}`));
      if (!sameTopic) $(this.#relElem).closest(".card").remove();
    }
    const lessonCount = parent.lessons.length;
    $(`#${parent.id}`)
      .addClass("show")
      .prev()
      .text(`● ${lessonCount} lesson${lessonCount > 1 ? "s" : ""} added`);

    const contentContainer = $("<ul>")
      .addClass("list-group list-group-horizontal-xl mt-2")
      .appendTo($(`#lesson-${suffix}`, lessonElem));

    $(attachments).each(function (_, el) {
      const data = $(el).data("data"),
        url = data.url,
        title = data.title,
        filetype = data.filetype,
        description = data.description,
        iconClass =
          filetype == "youtube" || filetype == "video"
            ? "fa fa-file-video"
            : filetype == "image"
            ? "fa fa-file-image"
            : filetype == "file"
            ? "fa fa-file"
            : "fa fa-link";
      const obj = data;
      lessonObj.attachments.push(obj);

      $("<li>")
        .data(obj)
        .addClass(
          "list-group-item d-flex justify-content-between align-items-start"
        )
        .append(
          $("<div>")
            .addClass("ml-2 mr-auto")
            .append($("<div>").addClass("font-weight-bold").text(title))
            .append(description || url)
        )
        .append($("<span>").addClass(iconClass).css("font-size", "x-large"))
        .appendTo(contentContainer);
    });

    if (type == "assignment" || type == "quiz") {
      let dueDate = $("#lessonDueDatetime").data("datetime") ?? "No due date",
        totalPoints = $("#lessonPoints").data("points"),
        closeSubmitAfterDue = $("#submitAfterDue").prop("checked"),
        duration;

      if (type == "quiz") {
        duration = $("#quizDuration").data("duration");
        lessonObj.questionSet = [];
        this.#putFormData(lessonObj.questionSet);
        const showQuizBtn = $("<div>").append(
          $("<button>")
            .addClass("btn show-question-set")
            .attr({
              "data-toggle": "modal",
              "data-target": "#generalDialog",
              "data-action": "questionSet",
            })
            .css("background", "#17a2b8")
            .data({
              questions: lessonObj.questionSet,
              title: lessonObj.title,
            })
            .text("View questions")
        );
        $(contentContainer).is(":empty")
          ? $(contentContainer).replaceWith(showQuizBtn)
          : $(contentContainer).before(showQuizBtn);
      }
      const itemData = { dueDate, totalPoints, closeSubmitAfterDue };
      lessonObj.grading = itemData;

      const itemDataElem = $("<div>")
        .data(itemData)
        .appendTo(
          $("<div>")
            .addClass("d-block mt-3 float-right")
            .prependTo($(`#lesson-${suffix}`, lessonElem))
        )
        .addClass("card-text d-flex")
        .css("height", "80px")
        .append(
          $("<div>")
            .addClass("btn-group-vertical")
            .append($("<span>").addClass("mb-2").text(totalPoints))
            .append(
              $("<span>").addClass("font-weight-bold").text("Total points")
            )
        )
        .append($("<div>").addClass("vr mx-3"))
        .append(
          $("<div>")
            .addClass("btn-group-vertical")
            .append($("<span>").addClass("mb-2").text(dueDate))
            .append($("<span>").addClass("font-weight-bold").text("Due date"))
        );

      if (duration) {
        itemData.duration = duration;
        $(itemDataElem)
          .append($("<div>").addClass("vr mx-3"))
          .append(
            $("<div>")
              .addClass("btn-group-vertical")
              .append($("<span>").addClass("mb-2").text(duration))
              .append($("<span>").addClass("font-weight-bold").text("Duration"))
          );
      }

      if (closeSubmitAfterDue) {
        $(itemDataElem).after(
          $("<span>")
            .append(
              $("<i>")
                .addClass("mr-2 text-dark fas fa-check-square")
                .css("font-size", "large")
            )
            .append(
              $("<span>")
                .addClass("text-muted")
                .text("Close submissions after due date")
            )
        );
      }
    }

    if (!$(`#${parent.id}`).hasClass("show"))
      $(`#${parent.id}`).addClass("show");
    $(lessonElem)[0].scrollIntoView({ behavior: "smooth", block: "center" });
    $("#lessonDialog").modal("hide");

    if (typeof window.saveCourse == "function") {
      saveCourse(false, true);
    }
  }

  returnLessonCard(lessonObj, suffix) {
    const lessonIcon = `<svg width="1.5em" height="1.5em" class="align-middle rounded-circle svg-bg mr-2"><use xlink:href="#${lessonObj.type}-icon" /></svg>`,
      desc = lessonObj.instruction || lessonObj.description;
    const lessonElem = $("<div>")
      .addClass("card no-scaling shadow-0 drag-lesson")
      .css("overflow", "visible")
      .data({ lesson: lessonObj, type: "lesson" })
      .prop("type", lessonObj.type)
      .append(
        $("<div>")
          .addClass(
            "card-header bg-white d-flex align-items-center px-0 dropdown w-100"
          )
          .append(
            $(
              '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24" height="24" class="text-muted mr-2 anchor" style="cursor:move;fill:currentColor;position:absolute;top:.5rem;left:50%;"><g transform-origin="center" transform="rotate(90)"><circle cy="6.5" cx="9.5" r="1.5"></circle><circle cy="6.5" cx="14.5" r="1.5"></circle><circle cy="12.5" cx="9.5" r="1.5"></circle><circle cy="12.5" cx="14.5" r="1.5"></circle><circle cy="18.5" cx="9.5" r="1.5"></circle><circle cy="18.5" cx="14.5" r="1.5"></circle><g></svg>'
            ).on("mousedown", (evt) => {
              const contentElem = $(evt.target)
                .closest(".card")
                .find(".collapse");

              if ($(contentElem).hasClass("show")) {
                $(contentElem).removeClass("show");
              }
              this.#startDragDrop(evt);
            })
          )
          .append(lessonIcon)
          .append(
            $("<button>")
              .addClass("btn btn-link flex-grow-1 text-start text-dark")
              .attr({
                "data-toggle": "collapse",
                "data-target": `#lesson-${suffix}`,
                "aria-expanded": "true",
                "aria-controls": `lesson-${suffix}`,
              })
              .css("font-size", "medium")
              .text(lessonObj.title)
              .click(function () {
                const dropdown = $(this).next().next();
                if ($(dropdown).hasClass("show")) {
                  setTimeout(() => {
                    $(dropdown).removeClass("show");
                  }, 500);
                }
              })
          )
          .append(
            lessonObj.parent.editable
              ? $("<i>")
                  .addClass("fa fa-ellipsis-v")
                  .attr({ "data-toggle": "dropdown", "aria-expanded": "false" })
                  .css("cursor", "pointer")
              : ""
          )
          .append(
            lessonObj.parent.editable
              ? $("<div>")
                  .addClass("dropdown-menu")
                  .append(
                    $("<a>")
                      .addClass("dropdown-item")
                      .attr({
                        "data-toggle": "modal",
                        "data-target": "#lessonDialog",
                        "data-action": lessonObj.type,
                      })
                      .prop("edit", true)
                      .text("Edit")
                  )
                  .append(
                    $("<a>")
                      .addClass("dropdown-item")
                      .attr({
                        "data-toggle": "modal",
                        "data-target": "#generalDialog",
                        "data-action": "confirm",
                      })
                      .text("Delete")
                      .prop("callback", () => (confirm) => {
                        if (confirm) {
                          const lessonObj = $(lessonElem).data("lesson");
                          const parentObj = lessonObj.parent;
                          const index = parentObj.lessons.findIndex(
                            (obj) => obj == lessonObj
                          );
                          parentObj.lessons.splice(index, 1);
                          const lessonCount = parentObj.lessons.length,
                            txt = lessonCount
                              ? `● ${lessonCount} lesson${
                                  lessonCount > 1 ? "s" : ""
                                } added`
                              : "● No lessons added";
                          $(`#${parentObj.id}`).prev("p").text(txt);
                          $(lessonElem).remove();

                          if (typeof window.saveCourse == "function")
                            saveCourse(false, true);
                        }
                      })
                  )
              : ""
          )
      )
      .append(
        $("<div>")
          .addClass("collapse")
          .attr({
            id: `lesson-${suffix}`,
            "data-parent": `#${lessonObj.parent.id}`,
          })
          .append(
            $("<div>")
              .addClass("card-body mt-5")
              .css("min-height", "unset")
              .append(
                desc
                  ? $("<h6>")
                      .addClass("text-muted mt-4")
                      .text(
                        lessonObj.type == "material"
                          ? "Description"
                          : "Instruction"
                      )
                      .add($("<p>").addClass("card-text").html(desc))
                  : ""
              )
          )
      );
    this.#addListeners(lessonElem);
    return lessonElem;
  }

  #putFormData(list = []) {
    const container = $("#lessonDialog");
    $(".qcard .stretched-link", container).each((_, stretchedElem) => {
      const index = $(stretchedElem).attr("id").match(/\d+$/)[0];
      if (!$(stretchedElem).parent().hasClass("show"))
        this.#displayQuestion(index);
      const el = $(stretchedElem).children("div:only-child"),
        qtype = $(el).data("qtype"),
        questionObj = {},
        question = $("a, h6", el).text();
      questionObj.qtype = qtype;
      qtype == "section"
        ? (questionObj.section = question)
        : (questionObj.question = question);

      if ($("a img, h6 img", el)[0]) {
        questionObj.qImage = $("a img, h6 img", el).first().attr("src");
      }

      if (qtype == "short") {
        questionObj.answer = $("p", el).text().replace("Answer: ", "");
      } else if (qtype == "section") {
        questionObj.instruct = $("p", el).text().replace("Instruction: ", "");
      } else if (qtype == "multiple") {
        questionObj.options = [];
        $("div.d-block div.d-flex", el).each(function (_, div) {
          const correct = $("span > i", div).data("correct"),
            optionTxt = $("span > span", div).text(),
            img = $("img", div).attr("src");
          const option = { correct, optionTxt };

          if (img) option.optImage = img;
          questionObj.options.push(option);
        });
      }
      list.push(questionObj);
    });
    return list;
  }

  #addQuestion(evt, index, currentType) {
    this.#displayQuestion(index);
    const card = $(evt.target).closest(".qcard");
    $(card).toggleClass("shadow-0 active");
    this.richForm(card, currentType);
  }

  #displayQuestion(index, qObj = null, currentType = null) {
    const qtype = qObj
        ? qObj.qtype
        : currentType
        ? currentType
        : $(`#questionType${index}`).data("qtype"),
      sectionInstruct = qObj
        ? qObj.instruct
        : $(`#section-instruct${index}`).val().trim(),
      qImageElem = $(`#quizQuestion${index}`).parent().find(`.qImage`),
      qImage = qObj ? qObj.qImage : $(qImageElem).data("src");
    let qtxt = qObj
        ? qObj.question || qObj.section
        : $(`#quizQuestion${index}`).val().trim(),
      shortAnswer = FormDialog.surveyForm
        ? ""
        : qObj
        ? qObj.answer
        : $(`#shortAnswer${index}`).val().trim();
    qtxt == ""
      ? (qtxt = qtype == "section" ? "No section set" : "No question set")
      : null;
    shortAnswer == "" && !FormDialog.surveyForm
      ? (shortAnswer = "No answer set")
      : null;

    const qParent = $(`#qq-${index}`).empty();

    if ($(qParent)[0]) {
      const headerElem =
          qtype == "section"
            ? $("<h6>").addClass("mt-3 mx-4 text-dark")
            : $("<a>")
                .addClass(
                  "btn text-start bg-white w-100 shadow-0 text-dark align-top qHeader"
                )
                .attr("href", "javascript:void(0);"),
        suffix = Date.now().toString().slice(-10);
      const questionElem = $("<div>")
        .addClass("m-0")
        .css("min-height", "unset")
        .data("qtype", qtype)
        .appendTo(qParent)
        .append(
          $(headerElem)
            .css({
              "font-size": "medium",
              "text-transform": "none",
              cursor: "pointer",
            })
            .text(qtxt)
            .append(
              qImage
                ? $("<img>")
                    .addClass("float-right mr-4")
                    .css({
                      "max-width": "60px",
                      "max-height": "60px",
                    })
                    .attr("src", qImage)
                : ""
            )
        )
        .append($("<div>").addClass(`mx-4 question${suffix}`));

      const qBody = $(`.question${suffix}`, questionElem);

      if (qObj) {
        if (qImage)
          this.#attachImage([{ url: qImage, filetype: "image" }], qImageElem);
        $(`#quizQuestion${index}`).val(qObj.question || qObj.section);
      }

      switch (qtype) {
        case "multiple":
          const blockElem = $("<div>").addClass("d-block").appendTo(qBody);
          const arrLike = qObj
            ? qObj.options
            : $(`#multichoice${index} .option-item`);

          if (qObj || currentType) {
            $(`#multiple-choice${index}`).click();
          }
          $.each(arrLike, (i, item) => {
            const option = qObj
                ? item.optionTxt
                : $("label", item)[0].childNodes[0].textContent,
              correct = qObj ? item.correct : $("input", item).is(":checked"),
              optImage = qObj
                ? item.optImage
                : $(".optImage", item).data("src");
            // console.log("correct", qObj, item.correct, typeof item.correct);

            $("<div>")
              .appendTo(blockElem)
              .addClass(
                "d-flex justify-content-between align-items-center mb-2"
              )
              .append(
                $("<span>")
                  .append(
                    $("<i>")
                      .addClass(
                        `mr-2 ${
                          String(correct) == "true"
                            ? "text-success"
                            : "text-muted"
                        }`
                      )
                      .css("font-size", "large")
                      .text(String(correct) == "true" ? "🞉" : "🞅")
                      .data("correct", correct)
                  )
                  .append($("<span>").addClass("text-muted").text(option))
              )
              .append(
                optImage
                  ? $("<img>")
                      .css({
                        "max-width": "60px",
                        "max-height": "60px",
                      })
                      .attr("src", optImage)
                  : ""
              );

            if (qObj) {
              if (!$(`#multichoice${index} .option-item`)[i]) {
                $(`#addOption${index}`).click();
              }
              const optItem = $(`#multichoice${index} .option-item`)[i];
              $("label", optItem).prop(
                "changed",
                true
              )[0].childNodes[0].textContent = option;
              $("input", optItem).prop("checked", correct);

              if (optImage)
                this.#attachImage(
                  [{ url: optImage, filetype: "image" }],
                  $(".optImage", optItem)
                );
            }
          });
          break;
        case "short":
          if (qObj || currentType) $(`#short-answer${index}`).click();
          if (FormDialog.surveyForm) break;
          if (qObj) $(`#shortAnswer${index}`).val(shortAnswer);
        case "section":
          if ((qObj || currentType) && qtype == "section") {
            $(`#new-section${index}`).click();
            if (!currentType)
              $(`#section-instruct${index}`).val(sectionInstruct);
          }

          (qtype != "section" || sectionInstruct) &&
            $("<p>")
              .appendTo(qBody)
              .addClass("card-text text-muted")
              .text(
                qtype == "short"
                  ? `Answer: ${shortAnswer}`
                  : `Instruction: ${sectionInstruct}`
              );
          break;
      }
    }

    if (!qObj) {
      this.#checkToEnableBtn();
    }
  }

  #setQuestionType(elem, qType, index) {
    if ($(`#questionType${index}`).data("qtype") == qType) return;
    $(`#questionType${index}`).html($(elem).html()).data("qtype", qType);
    $(elem).parent().find(".active").removeClass("active");

    if (qType == "section") {
      $(`#quizQuestion${index}`)
        .attr("placeholder", "Section")
        .next("label")
        .text("Section");
    } else {
      $(`#quizQuestion${index}`)
        .attr("placeholder", "Question")
        .next("label")
        .text("Question");
    }
    this.#clearQuizFields(index);
    $(`#questionContainer${index}`).click();
  }

  #createDays() {
    const tr = $("<tr>")
      .addClass("text-muted")
      .css("font-size", "small")
      .appendTo($("#lessonDialog table.calendar tbody"));
    const weekdays = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"];

    for (let i = 0; i < 7; i++) {
      const day = weekdays[i],
        color = i == 0 ? "text-danger" : "";
      $("<td>")
        .addClass(color)
        .attr("role", "columnheader")
        .text(day)
        .appendTo(tr);
    }
  }

  #clearDateTime(elem, evt, isDate = false) {
    evt.stopPropagation();
    const txt = isDate ? "No due date" : "Time (optional)";
    const nodeTxt = $(elem)
      .parent()
      .data("date", txt)
      .contents()
      .filter(function () {
        return this.nodeType === Node.TEXT_NODE;
      })[0];
    nodeTxt.textContent = txt;
    $(elem).addClass("d-none");

    if (isDate) {
      const timeToggler = $(elem)
        .parent()
        .addClass("dropdown-toggle")
        .parent("li")
        .next("a");
      $(timeToggler)
        .addClass("d-none")
        .data("date", "Time (optional)")
        .find("button")
        .addClass("d-none");
      $(timeToggler).parent("ul").next("div").addClass("d-none");
    }
  }

  #setGradeOrDuration(evt, isDuration = false) {
    const elem = $(evt.target).attr("placeholder", "0");

    if (isDuration) {
      let duration = $(elem).val().trim();

      if (duration == "") {
        $(elem).val("0").data("duration", "Not timed");
      } else {
        duration = this.#parseDurationInput(duration) || "0";
        $(elem)
          .val(duration)
          .data("duration", duration == "0" ? "Not timed" : duration);
      }
    } else {
      let totalPoints = parseInt($(elem).val());

      if (!Number.isNaN(totalPoints) && totalPoints > 0) {
        totalPoints = Math.min(totalPoints, 100);
        $(elem).val(totalPoints).data("points", totalPoints);
      } else {
        $(elem).val("0").data("points", "Ungraded");
      }
    }
    setTimeout(() => {
      $(elem).parent().next(".dropdown-menu").addClass("d-none");
    }, 200);
  }

  #onShowModal(evt) {
    const relElem = evt.relatedTarget;
    const action = (this.#current = $(relElem).attr("data-action"));
    let lessonObj;
    FormDialog.surveyForm = this.#current == "survey";

    if ($(relElem).prop("edit")) {
      this.#relElem = relElem;
      lessonObj = $(relElem)
        .closest(".card")
        .data(FormDialog.surveyForm ? "survey" : "lesson");
    } else if ($(relElem).closest(".card")[0]) {
      const topicObj = $(relElem).closest(".card").data("topic"),
        index = courseObj.topics.findIndex((item) => item == topicObj);
      $("#topicSelect").text(topicObj.topic).data("index", index);
    }

    if (action == "material") {
      this.#setMaterial();
      if ($(relElem).prop("edit")) this.#editMaterial(lessonObj);
    } else if (action == "assignment") {
      this.#setAssignment();
      if ($(relElem).prop("edit")) this.#editAssigment(lessonObj);
    } else if (action == "quiz") {
      this.#setQuiz($(relElem).prop("edit"));
      if ($(relElem).prop("edit")) this.#editQuiz(lessonObj);
    } else if (action == "survey") {
      this.#setSurvey($(relElem).prop("edit"));
      if ($(relElem).prop("edit")) this.#editSurvey(lessonObj);
    } else {
      evt.preventDefault();
      this.#current = null;
      toast(
        "'data-action' attribute is not set correctly on the caller",
        "black"
      );
      return;
    }
  }

  #onModalHidden() {
    this.#reset(
      $("#lessonDialog .modal-body .left-pane"),
      $("#lessonDialog .modal-body .right-pane")
    );
    $("#addLesson").text("Add");
    this.#relElem = this.#current = null;
  }

  #attachImage([data], elem) {
    if (data.filetype != "image") {
      toast("The attach file is not an image!", "black");
      return;
    }
    elem = $(elem).closest(".qImage, .optImage");

    if ($(elem).prop("tagName").toLowerCase() != "span") {
      const className = $(elem).prop("className").baseVal;
      const onclick = (evt) =>
        (FileDialog.addAttachment = (data) =>
          this.#attachImage(data, $(evt.target)));

      $(elem).replaceWith(
        $("<span>")
          .addClass(`text-nowrap ${className}`)
          .attr({
            "data-toggle": "modal",
            "data-target": "#uploadDialog",
            "data-action": "image",
            "data-multiple": "false",
          })
          .data("src", data.url)
          .append(
            $("<img>")
              .attr("src", data.img || data.url)
              .css({ width: "40px", height: "40px" })
          )
          .append(
            $("<sup>")
              .addClass("ml-1")
              .css({ cursor: "pointer", "font-size": "x-large" })
              .text("×")
              .click(function (evt) {
                evt.stopPropagation();
                const iElem = $(this).parent(),
                  className = $(iElem).hasClass("qImage")
                    ? "qImage"
                    : "optImage";
                $(iElem).replaceWith(
                  $(
                    `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 20 20" width="1.5em" height="1.5em" fill="currentColor" data-toggle="modal" data-target="#uploadDialog" data-action="image" data-multiple="false" class="${className}" style="cursor:pointer;"><path d="M19 2.5A1.5 1.5 0 0017.5 1h-15A1.5 1.5 0 001 2.5v15A1.5 1.5 0 002.5 19H10v-2H3.497c-.41 0-.64-.46-.4-.79l3.553-4.051c.19-.21.52-.21.72-.01L9 14l3.06-4.781a.5.5 0 01.84.02l.72 1.251A5.98 5.98 0 0116 10h3V2.5zm-11.5 2c1.1 0 2 .9 2 2s-.9 2-2 2-2-.9-2-2 .9-2 2-2zm12.207 10.793A1 1 0 0019 15h-2v-2a1 1 0 00-2 0v2h-2a1 1 0 000 2h2v2a1 1 0 102 0v-2h2a1 1 0 00.707-1.707z" /></svg>`
                  ).click(onclick)
                );
              })
          )
          .click(onclick)
      );
    } else {
      $(elem)
        .data("src", data.url)
        .find("img")
        .attr("src", data.img || data.url);
    }
  }

  #checkToEnableBtn(evt = null) {
    const selectedTopic = FormDialog.surveyForm ? "" : $("#topicSelect").text(), selectedTopicIndex = $("#topicSelect").data('index'),
      title = $("#lessonTitle").val().trim(),
      isQuiz = this.#current == "quiz";

    if (
      (FormDialog.surveyForm ||
        this.#topicList.some(({ topic, index }) => topic == selectedTopic && index == selectedTopicIndex)) &&
      title &&
      (!(isQuiz || FormDialog.surveyForm) ||
        ($("#lessonDialog .qcard").length && !this.#findProblemsInQuiz()))
    ) {
      if ($("#addLesson").hasClass("disabled"))
        $("#addLesson").removeClass("disabled");
    } else if (!$("#addLesson").hasClass("disabled")) {
      $("#addLesson").addClass("disabled");
    }

    if (
      evt &&
      $(evt.target)
        .prop("tagName")
        .match(/input|textarea/i)
    ) {
      this.#resize(evt);
    }
  }

  #parseDurationInput(duration) {
    const match = duration.match(/\b\d+\s*(?:min(?:ute)?s?)?\b/i) || [];
    let result = "";
    console.log("matches", match);
    const parts = match[0].split(/(?<=\d)\s*(?=[a-zA-Z])/);
    const number = parseInt(parts[0]);

    if (!Number.isNaN(number) && number > 0) {
      result += number + " min";
    }
    return result;
  }

  #clearQuizFields(index) {
    $(`#shortAnswer${index}, #section-instruct${index}`).val("");
    $(`#section-instruct${index}, #quizQuestion${index}`).each(function (
      _,
      txtarea
    ) {
      if ($(txtarea).prop("minHeight")) {
        const minHeight = $(txtarea).prop("minHeight");
        $(txtarea).css("height", minHeight + "px");
      }
    });
    $(`#multichoice${index} .option-item:nth-child(n+3)`).remove();
    $(`#multichoice${index} .option-item`).each(function (i, elem) {
      const label = $("label", elem).text(`Option ${i + 1}`);
      $(".optImage", elem).find("sup").click();

      if ($(label).find(".duplicate")[0]) {
        $(label)
          .removeClass("text-danger border-danger")
          .find(".duplicate")
          .remove();
      }

      if (i == 0 && !FormDialog.surveyForm) {
        $("input", elem).prop("checked", true);
      }
    });
  }

  #resize(evt) {
    const input = evt.target;
    const cssHeight = parseFloat($(input).css("height").replace("px", "")),
      scrollHeight = $(input).prop("scrollHeight");
    if (!$(input).prop("minHeight")) $(input).prop("minHeight", cssHeight);
    const minHeight = $(input).prop("minHeight");

    if (cssHeight < scrollHeight) {
      $(input).css("height", scrollHeight + "px");
    } else if (cssHeight > scrollHeight && cssHeight > minHeight) {
      const newHeight = Math.max(scrollHeight, minHeight);
      $(input).css("height", newHeight + "px");
    }
  }

  setTime(time) {
    if ($("#lessonDueTime").hasClass("d-none"))
      $("#lessonDueTime").removeClass("d-none");
    $("#lessonDueTime").data("time", time).find("button").removeClass("d-none");
    $("#lessonDueTime")[0].childNodes[0].textContent = time;
  }

  #highlightOption(evt, index) {
    const label = $(evt.target).closest("label")[0];

    if ($(`#multichoice${index} .option-item label`).find(".duplicate")[0]) {
      $(`#multichoice${index} .option-item label`)
        .removeClass("text-danger border-danger")
        .find(".duplicate")
        .remove();
    }

    if (!$(label).prev("input").is(":checked")) {
      const before = getComputedStyle(label, ":before"),
        { offsetX, offsetY } = evt;

      if (before) {
        const top = parseFloat(
          before.getPropertyValue("top").replace("px", "")
        );
        const bottom =
          parseFloat(before.getPropertyValue("height").replace("px", "")) + top;
        const left = parseFloat(
          before.getPropertyValue("left").replace("px", "")
        );
        const right =
          parseFloat(before.getPropertyValue("width").replace("px", "")) + left;

        if (
          offsetX >= left &&
          offsetX <= right &&
          offsetY >= top &&
          offsetY <= bottom
        ) {
          evt.preventDefault();
          !FormDialog.surveyForm &&
            $(label).prev("input").prop("checked", true);
          return;
        }
      }
    }

    if (!$(label).prop("changed")) {
      label.childNodes[0].textContent = "";
      $("span", label)[0] ? $("span", label).focus() : $(label).focus();
      return false;
    }

    if (this.#selectionRange.rangeCount == 0)
      this.#selectionRange.addRange(new Range());
    const range = this.#selectionRange.getRangeAt(0);
    const elem = $("span", label)[0]
      ? $("span", label).focus()[0]
      : $(label).focus()[0];
    range.selectNodeContents(elem.childNodes[0]);
  }

  #onHideDropdown(evt) {
    if (this.clickedInside) {
      evt.preventDefault();
      this.clickedInside = false;
      return;
    }
    let dateTime = $("#lessonDueDate").data("date") ?? "";

    if (
      !dateTime.includes("No due date") &&
      /\d{1,2}(?:st|nd|rd|th)\s\w{3},\s\d{4}/.test(dateTime)
    ) {
      const time = $("#lessonDueTime").data("time");

      if (/\d{1,2}:\d{2}\s[AP]M/.test(time))
        dateTime = dateTime.concat(" ", time);
      $("#lessonDueDatetime").text(dateTime).data("datetime", dateTime);
    } else {
      $("#lessonDueDatetime").text("No due date");
      if ($("#lessonDueDatetime").data("datetime"))
        $("#lessonDueDatetime").removeData("datetime");
    }
    let dropdown = $("#lessonDueDate").next(".dropdown-menu");

    if (!$(dropdown).hasClass("d-none")) {
      $(dropdown).addClass("d-none");
    }
  }

  #setInitDate() {
    const dateElem = $("#lessonDueDate"),
      dropdown = $(dateElem).next(".dropdown-menu").toggleClass("d-none");

    if (!$(dropdown).hasClass("d-none")) {
      const txt = $(dateElem).data("date") ?? "";

      if (!/\d{1,2}(?:st|nd|rd|th)\s\w{3},\s\d{4}/.test(txt)) {
        FormDialog.#showDatetime(true);
        $("#lessonDialog table.calendar td.active").click();
        $(dateElem)
          .removeClass("dropdown-toggle")
          .find("button")
          .removeClass("d-none");
      } else {
        const yr = txt.slice(-4),
          month = txt.slice(-9, -6),
          day = txt.match(/^\d+/)[0];
        FormDialog.#jumpToDate(month, yr);

        const tds = $("#lessonDialog table.calendar td:not(:empty)"),
          activeTd = $(tds).filter(".active")[0],
          td = $(tds).filter((_, td) => $(td).text() == day)[0];

        if (activeTd !== td) {
          $(td).click();
        }
      }
    }

    if (!this.clickedInside) this.clickedInside = true;
  }

  #addAttachment(dataArr) {
    const attachment = $("#lessonDialog .attachments");

    for (const data of dataArr) {
      const { img, url, duration, description, title, filetype } = data;

      if (
        Array.prototype.some(
          (elem) => $(elem).attr("src") == url || $(elem).attr("src") == img,
          $(attachment).find("embed, video, img")
        )
      ) {
        toast("Content has been added before", "#0a58ca");
        continue;
      }
      const icon =
        filetype == "image"
          ? "far fa-image"
          : filetype == "video"
          ? "fas fa-video"
          : "far fa-file";

      $("<div>")
        .addClass("card mb-3 no-scaling border shadow-0")
        .data("data", Object.assign({}, data))
        .appendTo(attachment)
        .append(
          $("<div>")
            .addClass("row")
            .append(
              $("<div>")
                .addClass("d-none d-lg-block col-auto m-auto")
                .css({
                  "aspect-ratio": "16/9",
                  width: "120px",
                  height: "min-content",
                })
                .append(
                  img
                    ? $("<img>").attr("src", img).addClass("img-fluid mt-2")
                    : $("<i>")
                        .addClass(`mt-3 ml-4 ${icon}`)
                        .css("font-size", "xx-large")
                )
            )
            .append(
              $("<div>")
                .addClass(
                  "col-lg-10 pl-0 d-flex justify-content-between align-items-start"
                )
                .append(
                  $("<div>")
                    .addClass("card-body pl-4 pl-lg-0 pr-0 py-3 col-11")
                    .css("min-height", "unset")
                    .append(
                      $("<a>")
                        .attr({ href: url, target: "_blank" })
                        .addClass(
                          "font-weight-bold text-truncate text-capitalize d-inline-block"
                        )
                        .css("max-width", "100%")
                        .html(title)
                    )
                    .append(
                      $("<p>")
                        .addClass(
                          "card-text text-truncate-two-lines max-line-one"
                        )
                        .html(
                          duration
                            ? `YouTube <i class="fa fa-circle mx-1 align-middle" style="font-size:4px;"></i> ${duration}`
                            : description || url
                        )
                    )
                )
                .append(
                  $("<button>")
                    .addClass("close mx-auto mt-3")
                    .css("outline", "none")
                    .attr({ type: "button", "aria-label": "Close" })
                    .click(function () {
                      $(this).closest(".card").remove();
                    })
                    .append($("<span>").attr("aria-hidden", "true").text("×"))
                )
            )
        );
    }
  }

  #startDragDrop(evt) {
    const qcard = $(evt.target).closest(".qcard, .drag-lesson, .drag-option");
    let data = $(qcard).clone();
    $(data)
      .find(".tab-content")
      .empty()
      .append(
        $("<h4>")
          .addClass("my-3 text-center")
          .append($("<i>").addClass("fas fa-ellipsis-h text-muted"))
      );
    data = $(data).prop("outerHTML");
    $(qcard)
      .prop("draggable", true)
      .on({
        dragstart: (evt) => {
          evt.stopPropagation();
          this.#draggedElem = $(qcard).addClass("draggable")[0];
          const dataTransfer = evt.originalEvent.dataTransfer;
          dataTransfer.effectAllowed = "move";
          dataTransfer.setData("text/html", data);
        },
        dragend: () => {
          $(qcard)
            .removeClass("draggable")
            .prop("draggable", false)
            .off("dragstart dragend");

          if ($(qcard).hasClass("drag-lesson")) {
            const parent = $(qcard)
                .parent()
                .children()
                .not(this.#draggedElem)
                .first()
                .data("lesson").parent,
              draggedLesson = $(qcard).data("lesson"),
              draggedParent = draggedLesson.parent,
              lessonIndex = $(qcard)
                .parent()
                .children()
                .index(this.#draggedElem),
              draggedIndex = draggedParent.lessons.findIndex(
                (item) => item == draggedLesson
              );
            draggedParent.lessons.splice(draggedIndex, 1);
            parent.lessons = [
              ...parent.lessons.slice(0, lessonIndex),
              draggedLesson,
              ...parent.lessons.slice(lessonIndex),
            ];
            draggedLesson.parent = parent;
          }
          this.#draggedElem = null;
          console.log("running3");
        },
      });
  }

  #addListeners(qcard) {
    $(qcard).on({
      dragover: (evt) => {
        evt.preventDefault();
        evt.stopPropagation();

        if (this.#cancelDrop($(qcard)[0], this.#draggedElem)) return;
        const receeds =
          $(qcard)[0].compareDocumentPosition(this.#draggedElem) &
          Node.DOCUMENT_POSITION_PRECEDING;

        if (receeds) {
          $(qcard).insertBefore(this.#draggedElem);
        } else {
          $(qcard).insertAfter(this.#draggedElem);
        }
      },
      drop: (evt) => {
        console.log("drop");
        evt.preventDefault();
      },
    });
  }

  #cancelDrop(elem1, elem2) {
    if (elem1 == elem2) return true;

    if (
      !($(elem1).hasClass("qcard") && $(elem2).hasClass("qcard")) &&
      !($(elem1).hasClass("drag-lesson") && $(elem2).hasClass("drag-lesson")) &&
      !($(elem1).hasClass("drag-option") && $(elem2).hasClass("drag-option"))
    ) {
      return true;
    }
    return false;
  }

  static #showDatetime(reset = false) {
    this.#today = new Date();
    this.#currentMonth = this.#today.getMonth();
    this.#currentYear = this.#today.getFullYear();
    if (reset) this.#selectedDate = undefined;
    this.#showCalendar();
  }

  static #showNextMonth(evt) {
    evt.stopPropagation();
    this.#currentYear =
      this.#currentMonth === 11 ? this.#currentYear + 1 : this.#currentYear;
    this.#currentMonth = (this.#currentMonth + 1) % 12;
    this.#showCalendar();
  }

  static #showPrevMonth(evt) {
    evt.stopPropagation();
    this.#currentYear =
      this.#currentMonth === 0 ? this.#currentYear - 1 : this.#currentYear;
    this.#currentMonth = this.#currentMonth === 0 ? 11 : this.#currentMonth - 1;
    this.#showCalendar();
  }

  static #jumpToDate(month, yr) {
    this.#currentYear = parseInt(yr);
    this.#currentMonth = this.#months.findIndex((m) => m.includes(month));
    this.#showCalendar();
  }

  static #showCalendar() {
    const parent = $("#lessonDialog table.calendar");
    const tbody = $("tbody", parent);
    $(".calendar-header", parent).text(
      this.#months[this.#currentMonth] + " " + this.#currentYear
    );
    let dateAt = new Date(this.#currentYear, this.#currentMonth),
      firstDay = dateAt.getDay(),
      maxDays = this.#getMaxDaysInMonth(dateAt);
    let day = 1;
    $(tbody).children(":nth-child(n+2)").remove();

    for (let i = 0; i < 6; i++) {
      let row = $("<tr>");

      for (let j = 0; j < 7; j++) {
        if (i === 0 && j < firstDay) {
          $(row).append("<td>");
        } else if (day > maxDays) {
          break;
        } else {
          let cell = $("<td>")
            .appendTo(row)
            .attr({
              "data-day": day,
              "data-month": this.#currentMonth + 1,
              "data-year": this.#currentYear,
              "data-month_name": this.#months[this.#currentMonth],
            })
            .css("cursor", "pointer")
            .text(day)
            .click(this.#setDate.bind(this));

          if (
            day === this.#today.getDate() &&
            this.#currentYear === this.#today.getFullYear() &&
            this.#currentMonth === this.#today.getMonth()
          ) {
            $(cell).addClass("active");
            this.#selectedDate = cell;
          }

          if (j == 0) $(cell).addClass("text-danger");
          day++;
        }
      }
      tbody.append(row);
    }
  }

  static #getMaxDaysInMonth(date) {
    date.setMonth(date.getMonth() + 1);
    date.setDate(date.getDate() - 1);
    return date.getDate();
  }

  static #setDate(evt) {
    const elem = evt.target;
    let day = $(elem).attr("data-day"),
      month = $(elem).attr("data-month_name").substr(0, 3),
      year = $(elem).attr("data-year");
    const len = day.length,
      lastChar = parseInt(day.charAt(len - 1)),
      selectedDate = FormDialog.#selectedDate;
    if (selectedDate) $(selectedDate).removeClass("active");
    $(elem).addClass("active");

    if (lastChar == 0 || lastChar > 3 || (len > 1 && day.charAt(0) == "1")) {
      day += "th";
    } else if (lastChar == 3) {
      day += "rd";
    } else if (lastChar == 2) {
      day += "nd";
    } else {
      day += "st";
    }
    const dateTxt = `${day} ${month}, ${year}`,
      toggler = $("#lessonDueDate")[0],
      timeElem = $("#lessonDueTime"),
      closeSubmitcheckbox = $(timeElem).parent("ul").next("div"),
      txtNode = $(toggler)
        .data("date", dateTxt)
        .contents()
        .filter(function () {
          return this.nodeType === Node.TEXT_NODE;
        })[0];
    txtNode.textContent = dateTxt;

    if ($(timeElem).hasClass("d-none")) {
      $(timeElem).removeClass("d-none");
    }

    if ($(closeSubmitcheckbox).hasClass("d-none")) {
      $(closeSubmitcheckbox).removeClass("d-none");
    }
    FormDialog.#selectedDate = elem;
  }

  #createStyles() {
    const styleElem = $("style")[0]
      ? $("style")[0]
      : $("<style>").appendTo($("head"))[0];
    const initialStyles = styleElem.textContent;
    styleElem.textContent = `${initialStyles}\n#lessonDialog .modal.fade.show{background-color:#000000b3;}\n#lessonDialog .form-group:focus-within label{color:#1f88c0;}\n.custom-control{z-index:1;display:block;-webkit-print-color-adjust:exact;}\i.invalid::before{border-color:#dc3545;}\n.custom-control-input{left:0;width:1rem;height:1.25rem;}\n.custom-control-label{position:relative;margin-bottom:0;vertical-align:top;}\n.custom-control-input:checked~.custom-control-label::before{color:#fff;border-color:#18ce0f;background-color:#18ce0f;}\n.custom-radio .custom-control-label::before {border-radius:50% !important;}\n.custom-control-label::before{transition:background-color .15s ease-in-out, border-color .15s ease-in-out, box-shadow .15s ease-in-out;}\n.custom-control-label::before{position:absolute;top:0.25rem;left:-1.5rem;display:block;width:1rem;height:1rem;pointer-events:none;content:"";background-color:#fff;border:#adb5bd solid 1px;}\n.custom-control-label::after{position:absolute;top:0.25rem;left:-1.5rem;display:block;width:1rem;height:1rem;content:"";background:no-repeat 50% / 50% 50%;}\n.custom-control-label::before, .custom-control-label::after{cursor:default;}\n.custom-radio .custom-control-input:checked~.custom-control-label::after{background-image:url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' width='12' height='12' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='%23fff'/%3e%3c/svg%3e");}\n.custom-checkbox .custom-control-label::before{border-radius:0.25rem;}\n.custom-checkbox .custom-control-input:checked~.custom-control-label::after{background-image:url("data:image/svg+xml;charset=utf8,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3E%3Cpath fill='%23fff' d='M6.564.75l-3.59 3.612-1.538-1.55L0 4.26 2.974 7.25 8 2.193z'/%3E%3C/svg%3E");}\n@media only screen and (min-width: 768px){\n#lessonDialog .right-pane {min-height: 84vh;}\n}\n#lessonDialog .form-group .form-control,#lessonDialog .input-group .form-control{padding:10px 18px 10px 18px;}\ntextarea.form-control{max-width:100%;padding:10px 10px 0 0;resize:none;border:none;border-bottom:1px solid #E3E3E3;border-radius:0;line-height:2;}\n.stretched-link::after{position:absolute;top:0;right:0;bottom:0;left:0;z-index:1;content:"";}\ntextarea.form-control:focus,textarea.form-control:active{border-left:none;border-top:none;border-right:none;}\n.btn{border-width:2px;font-weight:400;font-size:0.8571em;line-height:1.35em;margin:5px 1px;border:none;border-radius:0.1875rem;padding:11px 22px;cursor:pointer;}\n.qcard.active{box-shadow:rgba(0, 0, 0, 0.14) 0px 2px 2px 0px, rgba(0, 0, 0, 0.12) 0px 3px 1px -2px, rgba(0, 0, 0, 0.2) 0px 1px 5px 0px;}\n.qcard>svg,.anchor{display:none;}\n.qcard:hover>svg,.card-header:hover>.anchor{display:block;}\n.qcard .dropdown-item.active{background-color:#f8f9fa;}\n.txtbox:focus{border-color:#1f88c0!important;}\n.txtbox::-webkit-scrollbar{display:none;}\n#lessonDialog .btn-primary{background-color:#f96332;color:#FFFFFF;}\n#lessonDialog .btn-sm{font-size:14px;border-radius:0.1875rem;padding:5px 15px;}\n#lessonDialog .modal-header .close{color:#FF3636;text-shadow:none;outline:none;cursor:pointer;}\nbutton.close{cursor:pointer;}\n.text-start{text-align:start;}\n.custom-scrollbar::-webkit-scrollbar{width:10px;}\n.custom-scrollbar::-webkit-scrollbar-track{background: #f1f1f1;}\n.custom-scrollbar::-webkit-scrollbar-thumb{background:#888;}\n.custom-scrollbar::-webkit-scrollbar-thumb:hover{background: #555;}\n.card{border-radius:10px;}\n.box-shadow{box-shadow:0 .25rem .75rem rgba(0, 0, 0, .05);}\n.shadow-0{box-shadow:none !important;}\n.text-truncate-two-lines{overflow:hidden;text-overflow:ellipsis;-webkit-line-clamp:2;display: -webkit-box;-webkit-box-orient:vertical;white-space:normal;}\n.max-line-one{-webkit-line-clamp:1;}\n.flex-grow-1{flex-grow:1;}\n.modal-dialog-centered{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;min-height:calc(100% - (.5rem * 2));justify-content:center;}\n@media (min-width: 576px){\n.modal-dialog-centered{min-height:calc(100% - (1.75rem * 2));}\n}\n.rounded-0{border-radius:0 !important;}\n.editable-container,.create-lesson-title{position:relative;border-bottom:1px solid #343a40;}\n.create-lesson-title label{position:absolute;translate:0% -50%;left:1rem;top:50%;scale:1.1;cursor:text;transition:.2s cubic-bezier(.4,0,.2,1);}\n.create-lesson-title textarea + label{top:1.5rem;}\n.create-lesson-title:focus-within,.editable-container:focus-within,.customSelect:focus-visible{border-bottom:1px solid #1f88c0;border-width:2px;}\n.create-lesson-title input::placeholder,.create-lesson-title textarea::placeholder{color:transparent !important;}\n.create-lesson-title:focus-within label,.create-lesson-title input:not(:placeholder-shown) + label,.create-lesson-title textarea:not(:placeholder-shown) + label{color:#1f88c0;scale:.9;top:20%;cursor:default;}\n.create-lesson-title:focus-within label.qlabel,.create-lesson-title input:not(:placeholder-shown) + label.qlabel{top:0;}\n.create-lesson-title:hover,.editable-container:hover,.customSelect:hover{background-color:rgb(233 236 239)!important;}\n.editable-container span:first-of-type{position:absolute;translate:0% -50%;left:1rem;top:1.5rem;scale:1.1;cursor:text;transition:.2s cubic-bezier(.4,0,.2,1);}\n.editable-container:focus-within span:first-of-type,.editable-container p:first-of-type:not(:empty) + span{color:#1f88c0;scale:.9;top:0.8rem;cursor:default;}\n.editable-container p[contenteditable]{outline:none;}\n.svg-bg{background:#dff0fa;fill:#1f88c0;color:#1f88c0;width:2.8rem;height:2.8rem;padding:.5rem;}\n.edit-btn{padding:.5rem .75rem;}\n.edit-btn i,.edit-btn svg{font-size:large;}\n.edit-btn:hover{background-color:#f8f9fa !important;box-shadow:none;}\n.edit-btn.active{color:#004085 !important;background-color:#dff0fa!important;}\n.edit-btn.border{border-radius:50% !important;padding:.75rem .75rem;width:fit-content;}\n.edit-btn.focus{outline:none;}\n.customSelect::after{float:right;margin-top:0.5rem;}\n.question-type::after{margin-left:1.3em;}\n.attachments a{color:#343a40;}\n.attachments a:hover{color:#1f88c0;text-decoration:underline;}\ntable.calendar thead tr{display:grid;grid-template-columns:1fr 5fr 1fr;}\ntable.calendar tbody tr{display:grid;grid-template-columns:repeat(7, 1fr);}\ntable.calendar tbody tr *{text-align:center;}\ntable.calendar tbody tr td.active{background-color:#004085;color:white;}\ntable.calendar tbody tr td:not(.active):not(:empty):hover{background-color:#e1ebfd;}\n.timepicker-ui-wrapper{position:fixed;z-index:5001;width:20.5rem;height:31.25rem;top:50%;left:50%;transform:translate(-50%, -50%);background-color:#fff;border-radius:0.25rem;box-shadow:0px 3px 5px -1px rgba(0, 0, 0, 0.2), 0px 5px 8px 0px rgba(0, 0, 0, 0.14), 0px 1px 14px 0px rgba(0, 0, 0, 0.12);display:flex;flex-direction:column;}\n.timepicker-ui-normalize{box-sizing:content-box !important;}\n.timepicker-ui-header{padding-top:3.25rem;padding-bottom:2.25rem;padding-right:1.5rem;padding-left:1.5rem;height:6.5rem;display:flex;flex-direction:row;justify-content:center;align-items:center;position:relative;}\n.timepicker-ui-select-time{text-transform:uppercase;position:absolute;top:calc(1.75rem - 0.75rem);left:1.5rem;font-size:0.75rem;color:#a9a9a9;}\n.timepicker-ui-wrapper-time{display:flex;margin-right:0.625rem;}\n.timepicker-ui-wrapper-type-time{display:flex;flex-direction:column;height:5rem;justify-content:center;align-items:center;font-size:1rem;font-weight:500;color:#787878;align-self:start;}\n.timepicker-ui-hour:hover,.timepicker-ui-hour.active,.timepicker-ui-minutes:hover,.timepicker-ui-minutes.active{color:#6200ee;background-color:#ece0fd;}\n.timepicker-ui-hour,.timepicker-ui-minutes{width:calc(6rem - 1.5rem);height:calc(5rem - 1.5rem);display:flex;justify-content:center;align-items:center;font-size:3.2rem;background-color:#e4e4e4;border-radius:0.4375rem;cursor:pointer;transition:all 0.3s ease;outline:none;border:0.125rem solid transparent;padding:0.625rem;}\n.timepicker-ui-dots{padding-left:0.3125rem;padding-right:0.3125rem;display:flex;justify-content:center;align-items:center;font-size:3.6rem;user-select:none;touch-action:none;}\n.timepicker-ui-am:hover,.timepicker-ui-am.active,.timepicker-ui-pm:hover,.timepicker-ui-pm.active{color:#6200ee;background-color:#ece0fd;}\n.timepicker-ui-am{border-top-left-radius:0.4375rem;border-top-right-radius:0.4375rem;border-bottom-width: calc(0.0469rem / 2)!important;}\n.timepicker-ui-pm{border-bottom-left-radius:0.4375rem;border-bottom-right-radius:0.4375rem;border-top-width:calc(0.0469rem / 2)!important;}\n.timepicker-ui-am,.timepicker-ui-pm{height:calc(2.5rem - 0.125rem);width:calc(3.25rem - 0.125rem);display:flex;justify-content:center;align-items:center;border:0.125rem solid #d6d6d6;transition:all 0.3s ease;cursor:pointer;}\n.timepicker-ui-body{height:16rem;padding-right:2.25rem;padding-left:2.25rem;}\n.timepicker-ui-clock-face{background-color:#e0e0e0;height:100%;width:100%;border-radius:100%;position:relative;}\n.timepicker-ui-dot{position:absolute;top:50%;left:50%;user-select:none;touch-action:none;transform:translate(-50%, -50%);background-color:#6200ee;height:0.5rem;width:0.5rem;border-radius:100%;}\n.timepicker-ui-clock-hand{position:absolute;background-color:#6200ee;bottom:50%;height:40.5%;left:calc(50% - 0.0625rem);transform-origin:center bottom 0;width:0.125rem;transition:rotate .2s cubic-bezier(0.4,0,0.2,1);}\n.timepicker-ui-circle-hand{position:absolute;top:-1.3125rem;left:-1.3125rem;width:0.25rem;border:1.25rem solid #6200ee;height:0.25rem;box-sizing:content-box;border-radius:100%;transition:all 0.2s ease;}\n.timepicker-ui-hour-time-12,.timepicker-ui-minutes-time{position:absolute;width:2rem;height:2rem;text-align:center;cursor:pointer;font-size:1.1rem;display:flex;justify-content:center;align-items:center;touch-action:none;user-select:none;}\n.timepicker-ui-hour-time-12 span,.timepicker-ui-minutes-time span{touch-action:none;user-select:none;}\n.timepicker-ui-value-tips{outline:none;}\n.timepicker-ui-value-tips.active{color:#fff;}\n.timepicker-ui-circle-hand.small-circle{top:-0.8125rem;left:-0.8125rem;border-width:0.75rem;}\n.timepicker-ui-footer{height:4.75rem;display:flex;justify-content:end;align-items:start;margin-bottom:0.25rem;}\n.timepicker-ui-wrapper-btn{display:flex;justify-content:center;align-items:start;outline:none;margin-right:0.5rem;position:relative;bottom:-0.875rem;}\n.timepicker-ui-cancel-btn{width:4.5rem;margin-right:0.25rem;}\n.timepicker-ui-ok-btn{width:4rem;margin-left:0.25rem;}\n.timepicker-ui-cancel-btn, .timepicker-ui-ok-btn{color:#6200ee;text-transform:uppercase;border-radius:0.4375rem;background-color:transparent;text-align:center;ont-size:0.95rem;padding-top:0.5625rem;padding-bottom:0.5625rem;font-weight:500;transition:all 0.3s ease;cursor:pointer;outline:none;}\n.timepicker-ui-cancel-btn:hover,.timepicker-ui-ok-btn:hover{background-color:#d6d6d6;}\n.dropdown-item.dropdown-toggle::after{float:right; margin-top:.6rem;}\n#questionType::after{margin-left:2rem;}\n.option-item svg:first-child{visibility:hidden;}\n.option-item:hover svg:first-child{visibility:visible;}\n.option-item label{border-bottom:1px solid transparent;}\n.option-item .custom-control::after{content:'';position:absolute;top:calc(100% - 1px);left:50%;transform:translateX(-49.5%)!important;height:1px;width:94.5%;background-color:transparent;}\n.option-item:hover .custom-control::after{background-color:#adb5bd;}\n.option-item:focus-within .custom-control::after{animation:border-anim .3s ease-in-out forwards;background-color:#1f88c0;}\n@keyframes border-anim{\n0%{width:0;}\n100%{width:94.5%;}\n}\n.disabled{pointer-events:none;}\n.qHeader:hover{background-color:#fcfcfd !important;}\n.vr{align-self:stretch;width:1px;min-height:1em;background-color:currentcolor;opacity:.25;}\n#lessonDialog .left-pane{counter-reset:qnum;}\n.qcard .stretched-link a.qHeader::before,.qcard .stretched-link.collapsed a.qHeader::before{counter-increment:qnum;content:counter(qnum) ".";margin-right:1rem;}\n.qcard.active + .qcard .stretched-link a.qHeader::before{counter-increment:qnum 2;}\n.draggable{max-height:190px;opacity:.1;}\n.switch, .switch *{user-select:none;}\n.switch label{cursor:pointer;font-size:.8rem;}\n.switch label input[type=checkbox]{opacity:0;width:0;height:0;position:absolute;pointer-events:none;}\n.switch label input[type=checkbox]:checked+.lever{background-color:#dff0fa;}\n.switch label .lever{content:"";display:inline-block;position:relative;width:36px;height:14px;background-color:rgba(0, 0, 0, 0.38);border-radius:15px;margin-right:10px;transition:background 0.3s ease;vertical-align:middle;margin:0 16px;}\n.switch label .lever:before, .switch label .lever:after{content:"";position:absolute;display:inline-block;width:20px;height:20px;border-radius:50%;left:0;top:-3px;transition:left 0.3s ease, background .3s ease, box-shadow 0.1s ease, transform .1s ease;}\n.switch label .lever:after{background-color:#F1F1F1;box-shadow:0px 3px 1px -2px rgba(0, 0, 0, 0.2), 0px 2px 2px 0px rgba(0, 0, 0, 0.14), 0px 1px 5px 0px rgba(0, 0, 0, 0.12);}\n.switch label .lever:before{background-color:rgba(38, 166, 154, 0.15);}\n.switch label input[type=checkbox]:checked+.lever:before, .switch label input[type=checkbox]:checked+.lever:after{left:18px;}\n.switch label input[type=checkbox]:checked+.lever:after{background-color:#1f88c0;}\n@media only screen and (min-width: 1400px){\n.col-xxl-3{width:25%;}\n.col-xxl-4{width:33%;}\n}`;
  }

  richForm(
    elem,
    currentType = null,
    firstTime = false,
    qObj = null,
    index = null
  ) {
    const instance = this;
    index ??= Date.now().toString().slice(-10);
    const qcard = $("<div>")
      .addClass("card no-scaling shadow-0 mb-3 mt-2 w-100 qcard")
      .css({ position: "relative", "max-width": "936px" })
      .append(
        $(
          '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24" height="24" class="text-muted mr-2" style="cursor:move;fill:currentColor;position:absolute;top:.5rem;left:50%;"><g transform-origin="center" transform="rotate(90)"><circle cy="6.5" cx="9.5" r="1.5"></circle><circle cy="6.5" cx="14.5" r="1.5"></circle><circle cy="12.5" cx="9.5" r="1.5"></circle><circle cy="12.5" cx="14.5" r="1.5"></circle><circle cy="18.5" cx="9.5" r="1.5"></circle><circle cy="18.5" cx="14.5" r="1.5"></circle><g></svg>'
        ).on({
          mousedown: this.#startDragDrop.bind(this),
        })
      )
      .append(
        $("<div>")
          .addClass(`card-body collapse qcollapse${index} show`)
          .css({ "min-height": "unset", "max-width": "936px" })
          .append(
            $("<a>")
              .addClass("stretched-link text-decoration-none")
              .attr({
                "data-toggle": "collapse",
                href: `.qcollapse${index}`,
                id: `qq-${index}`,
              })
              .css("cursor", "text")
          )
          .click(function () {
            const card = $(this).closest(".qcard")[0],
              curActive = $(".qcard.active")[0];

            if (curActive != card) $(curActive).find("[id^=qq-]").click();
            else instance.#displayQuestion(index);
            $(card).toggleClass("shadow-0 active");
            setTimeout(() => {
              $(`#questionContainer${index}`).click();
            }, 500);

            if ($("#undoDel")[0]) $("#undoDel").remove();
          })
      )
      .append(
        $("<div>")
          .addClass(`pt-2 m-0 collapse qcollapse${index}`)
          .append(
            $("<div>")
              .addClass("card-body pb-0")
              .css("min-height", "unset")
              .append(
                $("<div>")
                  .append(
                    $("<div>")
                      .addClass("form-inline align-items-stretch")
                      .append(
                        $("<div>")
                          .addClass("form-group create-lesson-title mr-3")
                          .css("flex", "1")
                          .append(
                            $("<div>")
                              .addClass("input-group mb-0 w-100 pb-0")
                              .css("cursor", "text")
                              .attr("id", `questionContainer${index}`)
                              .append(
                                $("<textarea>")
                                  .addClass(
                                    "form-control bg-transparent w-100 py-0 mt-4"
                                  )
                                  .attr({
                                    type: "text",
                                    id: `quizQuestion${index}`,
                                    placeholder: "Question",
                                  })
                                  .css({
                                    border: "none",
                                    "font-size": "small",
                                    height: "1.5rem",
                                    "line-height": "1.1",
                                  })
                                  .on({
                                    click: (evt) => evt.stopPropagation(),
                                    input: this.#checkToEnableBtn.bind(this),
                                  })
                              )
                              .append(
                                $("<label>")
                                  .attr("for", `quizQuestion${index}`)
                                  .text("Question")
                              )
                              .append(
                                $("<div>")
                                  .addClass(
                                    "input-group-append bg-transparent pt-2 mr-2"
                                  )
                                  .css("cursor", "pointer")
                                  .append(
                                    !FormDialog.surveyForm
                                      ? $(
                                          `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 20 20" width="1.5em" height="1.5em" fill="currentColor" data-toggle="modal" data-target="#uploadDialog" data-action="image" data-multiple="false" class="qImage" style="cursor:pointer;"><path d="M19 2.5A1.5 1.5 0 0017.5 1h-15A1.5 1.5 0 001 2.5v15A1.5 1.5 0 002.5 19H10v-2H3.497c-.41 0-.64-.46-.4-.79l3.553-4.051c.19-.21.52-.21.72-.01L9 14l3.06-4.781a.5.5 0 01.84.02l.72 1.251A5.98 5.98 0 0116 10h3V2.5zm-11.5 2c1.1 0 2 .9 2 2s-.9 2-2 2-2-.9-2-2 .9-2 2-2zm12.207 10.793A1 1 0 0019 15h-2v-2a1 1 0 00-2 0v2h-2a1 1 0 000 2h2v2a1 1 0 102 0v-2h2a1 1 0 00.707-1.707z" /></svg>`
                                        ).click((evt) => {
                                          FileDialog.addAttachment = (data) =>
                                            this.#attachImage(data, evt.target);
                                        })
                                      : ""
                                  )
                              )
                              .click(function () {
                                const txtarea = $("textarea", this).focus()[0],
                                  txtLen = txtarea.value.length;
                                txtarea.setSelectionRange(txtLen, txtLen);
                              })
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("dropdown")
                          .append(
                            $("<a>")
                              .addClass(
                                "btn btn-lg bg-light dropdown-toggle text-dark shadow-0 my-0 rounded-0 px-3 py-2 mt-2 question-type"
                              )
                              .attr({
                                href: "javascript:void(0);",
                                role: "button",
                                id: `questionType${index}`,
                                "data-toggle": "dropdown",
                                "aria-haspopup": "true",
                                "aria-expanded": "false",
                              })
                              .data("qtype", "short")
                              .html(
                                `<svg focusable="false" width="24" height="24" viewBox="0 0 24 24" class="align-bottom mr-2"><path d="M4 9h16v2H4V9zm0 4h10v2H4v-2z" /></svg> Short answer`
                              )
                              .click(function () {
                                setTimeout(() => {
                                  $(this)
                                    .next(".dropdown-menu")
                                    .css(
                                      "transform",
                                      "translate3d(1px, -2px, 0px)"
                                    );
                                }, 100);
                              })
                          )
                          .append(
                            $("<div>")
                              .addClass("dropdown-menu w-100")
                              .attr("aria-labelledby", `questionType${index}`)
                              .append(
                                $("<a>")
                                  .addClass("dropdown-item text-dark")
                                  .attr({
                                    href: `#shortContainer${index}`,
                                    id: `short-answer${index}`,
                                    role: "tab",
                                    "data-toggle": "tab",
                                    "aria-controls": `shortContainer${index}`,
                                    "aria-selected": "true",
                                  })
                                  .prop("type", "short")
                                  .html(
                                    `<svg focusable="false" width="24" height="24" viewBox="0 0 24 24" class="align-bottom mr-2 pe-none"><path d="M4 9h16v2H4V9zm0 4h10v2H4v-2z" /></svg> Short answer`
                                  )
                                  .click(() =>
                                    this.#setQuestionType(
                                      $(`#short-answer${index}`),
                                      "short",
                                      index
                                    )
                                  )
                              )
                              .append(
                                $("<a>")
                                  .addClass("dropdown-item text-dark")
                                  .attr({
                                    href: `#multichoice${index}`,
                                    id: `multiple-choice${index}`,
                                    role: "tab",
                                    "data-toggle": "tab",
                                    "aria-controls": `multichoice${index}`,
                                    "aria-selected": "true",
                                  })
                                  .prop("type", "multiple")
                                  .html(
                                    `<svg focusable="false" width="24" height="24" viewBox="0 0 24 24" class="align-bottom mr-2 pe-none"><path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z" /><circle cx="12" cy="12" r="5"></circle></svg> Multiple choice`
                                  )
                                  .click(() =>
                                    this.#setQuestionType(
                                      $(`#multiple-choice${index}`),
                                      "multiple",
                                      index
                                    )
                                  )
                              )
                              .append(
                                $("<a>")
                                  .addClass("dropdown-item text-dark")
                                  .attr({
                                    href: `#qSection${index}`,
                                    id: `new-section${index}`,
                                    role: "tab",
                                    "data-toggle": "tab",
                                    "aria-controls": `qSection${index}`,
                                    "aria-selected": "true",
                                  })
                                  .prop("type", "section")
                                  .html(
                                    `<svg focusable="false" width="24" height="24" viewBox="0 0 24 24" class="align-bottom mr-2 pe-none"><path xmlns="http://www.w3.org/2000/svg" d="M7 20V7H2V4h13v3h-5v13Zm9 0v-8h-3V9h9v3h-3v8Z" /></svg> New section`
                                  )
                                  .click(() =>
                                    this.#setQuestionType(
                                      $(`#new-section${index}`),
                                      "section",
                                      index
                                    )
                                  )
                              )
                          )
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("tab-content")
                      .append(
                        $("<div>")
                          .addClass("tab-pane fade show active")
                          .attr({
                            id: `shortContainer${index}`,
                            "aria-labelledby": `short-answer${index}`,
                          })
                          .append(
                            !FormDialog.surveyForm
                              ? $("<div>")
                                  .addClass("form-group mt-3")
                                  .append(
                                    $("<label>")
                                      .addClass("w-100 mb-0")
                                      .attr("for", `shortAnswer${index}`)
                                      .text("Short asnwer")
                                  )
                                  .append(
                                    $("<textarea>")
                                      .addClass(
                                        "form-control txtbox p-0 border-dark"
                                      )
                                      .attr("id", `shortAnswer${index}`)
                                      .css({
                                        "font-size": "small",
                                        height: "2rem",
                                      })
                                      .on("input", (evt) =>
                                        this.#checkToEnableBtn(evt)
                                      )
                                  )
                              : ""
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("tab-pane fade")
                          .attr({
                            id: `multichoice${index}`,
                            "aria-labelledby": `multiple-choice${index}`,
                          })
                          .append(
                            $("<div>")
                              .addClass("my-3")
                              .append(
                                $("<a>")
                                  .attr({
                                    href: "javascript:void(0);",
                                    id: `addOption${index}`,
                                  })
                                  .addClass("list-group-item-action ml-3")
                                  .text("Add option")
                                  .prepend(
                                    $("<i>")
                                      .addClass(
                                        "far fa-circle mr-2 text-secondary"
                                      )
                                      .css("font-size", "smaller")
                                  )
                                  .click(() => this.#addOption(index))
                              )
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("tab-pane fade")
                          .attr({
                            id: `qSection${index}`,
                            "aria-labelledby": `new-section${index}`,
                          })
                          .append(
                            $("<div>")
                              .addClass("form-group mt-3")
                              .append(
                                $("<label>")
                                  .addClass("w-100 mb-0")
                                  .attr("for", `section-instruct${index}`)
                                  .text("Instruction (optional)")
                              )
                              .append(
                                $("<textarea>")
                                  .addClass(
                                    "form-control p-0 txtbox border-dark"
                                  )
                                  .attr("id", `section-instruct${index}`)
                                  .css({ "font-size": "small", height: "2rem" })
                                  .on("input", this.#resize.bind(this))
                              )
                          )
                      )
                  )
              )
          )
          .append(
            $("<div>")
              .addClass("card-footer bg-white mt-3 py-2")
              .append(
                $("<div>")
                  .addClass("btn-toolbar d-flex justify-content-between")
                  .append(
                    $("<div>")
                      .addClass("btn-group")
                      .append(
                        $("<button>")
                          .addClass(
                            "btn bg-transparent text-dark edit-btn mt-0"
                          )
                          .attr({
                            type: "button",
                            "data-toggle": "tooltip",
                            "data-placement": "top",
                            title: "Delete",
                          })
                          .css({ "border-radius": "50%", "aspect-ratio": "1" })
                          .append($("<i>").addClass("far fa-trash-alt"))
                          .click(function () {
                            const card = $(this).closest(".qcard");
                            $(this)
                              .tooltip("hide")
                              .parent()
                              .find('[data-toggle="tooltip"]')
                              .tooltip("dispose");

                            if ($(card).parent().children().length < 3) {
                              toast("Question set cannot be empty", "black");
                              return;
                            }
                            const clonedCard = $(card).clone(true),
                              cardUnder = $(card).next(".qcard")[0],
                              parent = $(card).parent(),
                              undoBtn = $("<a>")
                                .appendTo($("#lessonDialog .modal-body"))
                                .attr("id", "undoDel")
                                .addClass(
                                  "btn bg-dark btn-sm py-3 text-light rounded-0"
                                )
                                .css({
                                  position: "fixed",
                                  left: "1rem",
                                  bottom: "5rem",
                                  "z-index": "1",
                                })
                                .text("Item deleted")
                                .append(
                                  $("<strong>")
                                    .addClass("text-warning ml-5")
                                    .text("Undo")
                                )
                                .click(function () {
                                  cardUnder
                                    ? $(clonedCard).insertBefore(cardUnder)
                                    : $(clonedCard).appendTo(parent);
                                  $(`#questionContainer${index}`).click();
                                  const actionBtns = $(
                                    '[data-toggle="tooltip"]',
                                    clonedCard
                                  );
                                  $(actionBtns).each((_, btn) =>
                                    $(btn).attr(
                                      "title",
                                      $(btn).attr("data-original-title")
                                    )
                                  );
                                  $(actionBtns).tooltip();
                                  clearTimeout(timeout);
                                  $(this).remove();
                                }),
                              timeout = setTimeout(() => {
                                if ($(undoBtn).parent()[0]) {
                                  $(undoBtn).remove();
                                  $(this).tooltip("dispose");
                                  instance.#checkToEnableBtn();
                                }
                              }, 4000);
                            $(card).remove();
                          })
                      )
                      .append(
                        $("<button>")
                          .addClass(
                            "btn bg-transparent text-dark edit-btn mt-0"
                          )
                          .attr({
                            type: "button",
                            "data-toggle": "tooltip",
                            "data-placement": "top",
                            title: "Duplicate",
                          })
                          .css({ "border-radius": "50%", "aspect-ratio": "1" })
                          .append($("<i>").addClass("far fa-clone"))
                          .click(function () {
                            let card = $(this).closest(".qcard");
                            const togglerB4 = $(".stretched-link", card);
                            card = $(card).clone(true).insertAfter(card);
                            const index = Date.now().toString().slice(-10);
                            const attrs = [
                              "id",
                              "aria-labelledby",
                              "href",
                              "aria-controls",
                            ];
                            const elems = $(
                              "[id],[aria-labelledby],[href],[aria-controls]",
                              card
                            );
                            $(elems)
                              .filter((i, elem) => $(elems).index(elem) === i)
                              .each(function (_, elem) {
                                for (const attr of attrs) {
                                  if ($(elem).attr(attr)) {
                                    $(elem).attr(
                                      attr,
                                      $(elem).attr(attr).replace(/\d+/, index)
                                    );
                                  }
                                }
                                if ($(elem).hasClass("dropdown-item")) {
                                  $(elem).replaceWith(
                                    $(elem)
                                      .clone()
                                      .click(() =>
                                        instance.#setQuestionType(
                                          elem,
                                          $(elem).prop("type"),
                                          index
                                        )
                                      )
                                  );
                                }

                                if ($(elem).attr("aria-labelledby")) {
                                  $(elem).attr(
                                    "aria-labelledby",
                                    $(elem)
                                      .attr("aria-labelledby")
                                      .replace(/\d+$/, index)
                                  );
                                }
                              });
                            $(togglerB4).click();
                            $(card)[0].scrollIntoView({
                              behavior: "smooth",
                              block: "center",
                            });
                            $('[data-toggle="tooltip"]', card).tooltip();
                            $(`#questionContainer${index}`).click();
                          })
                      )
                  )
                  .append(
                    $("<button>")
                      .addClass("btn btn-sm text-light")
                      .css("background-color", "#17a2b8")
                      .attr({
                        id: `addQuestion${index}`,
                        "data-toggle": "collapse",
                        href: `.qcollapse${index}`,
                      })
                      .html(`<i class="fa fa-plus mr-2"></i>New question`)
                      .click((evt) =>
                        this.#addQuestion(
                          evt,
                          index,
                          $(`#questionType${index}`).data("qtype") ?? "short"
                        )
                      )
                  )
              )
          )
      );
    firstTime ? $(qcard).appendTo(elem) : $(qcard).insertAfter(elem);
    this.#addListeners(qcard);
    this.#addOption(index, true);
    this.#displayQuestion(index, qObj, currentType);
    $('[data-toggle="tooltip"]', qcard).tooltip();

    if (!firstTime || !qObj) {
      $(`#qq-${index}`).click();
      $(qcard)[0].scrollIntoView({ behavior: "smooth", block: "center" });
    }
  }

  #addSurvey() {
    const editMode = this.#relElem ? $(this.#relElem).prop("edit") : null,
      oldSurveyObj = editMode
        ? $(this.#relElem).closest(".card").data("survey")
        : null,
      title = $("#lessonTitle").val(),
      description = $("#lessonDialog p[contenteditable]").html(),
      dueDate = $("#lessonDueDatetime").data("datetime") ?? "No due date",
      available = $("#responseStatus").prop("checked"),
      closeSubmitAfterDue = $("#submitAfterDue").prop("checked"),
      course = $("#selectSurveyCourse").data("course"),
      surveyObj = {
        title,
        description,
        available,
        dueDate,
        closeSubmitAfterDue,
        editable: true,
        responseCount: 0,
        questions: [],
      },
      params = {
        option: "survey",
        ...surveyObj,
        action: "new",
        course_id: course?.id,
        course: course?.name ?? "All courses",
      };
    this.#putFormData(surveyObj.questions);

    if (editMode) {
      params.action = "edit";
      params._id = surveyObj._id = oldSurveyObj._id;
    }
    const spinner = $("<div>")
      .addClass("spinner-border spinner-border-sm ml-2")
      .attr("role", "status")
      .appendTo($("#addLesson").addClass("disabled"));
    $.post(`${site.process}api`, params, (jsonRes) => {
      const res = JSON.parse(jsonRes);
      console.log("survey res", res);
      $(spinner).remove();
      $("#addLesson").removeClass("disabled");

      if (res.status) {
        const parentElem =
            surveyList.length == 0
              ? $("#survey_list").empty()
              : $("#survey_list"),
          surveyElem = this.returnSurveyCard(surveyObj);
        editMode
          ? $(this.#relElem).closest(".card").parent().replaceWith(surveyElem)
          : $(parentElem).append(surveyElem);
        surveyObj.course = course;

        if (editMode) {
          const index = surveyList.findIndex(
            (otherSurveyObj) => otherSurveyObj == oldSurveyObj
          );
          surveyList.splice(index, 1, surveyObj);
        } else {
          surveyObj._id = res.id;
          surveyList.push(surveyObj);
        }
        $("#lessonDialog").modal("hide");
        $(surveyElem)[0].scrollIntoView({
          behavior: "smooth",
          block: "center",
        });
      }
    });
  }

  returnSurveyCard(survey) {
    const surveyCard = $("<div>")
      .addClass("col-12 col-lg-6 col-xxl-4")
      .css("min-width", "370px")
      .append(
        $("<div>")
          .addClass("card no-scaling box-shadow")
          .css({ "max-width": "500px", overflow: "visible" })
          .data({ survey: survey, type: "survey" })
          .append(
            $("<div>")
              .addClass("card-body")
              .append(
                $("<div>")
                  .addClass("d-flex mb-2")
                  .append(
                    $("<div>")
                      .addClass("flex-grow-1")
                      .append(
                        $("<h6>")
                          .addClass("mb-0 text-truncate")
                          .append(
                            $("<a>")
                              .addClass("d-block")
                              .attr("href", "javascript:void(0);")
                              .text(survey.title)
                              .click(function () {
                                $(this)
                                  .closest(".d-flex")
                                  .find('[data-action="questionSet"]')
                                  .click();
                              })
                          )
                      )
                  )
                  .append(
                    survey.available
                      ? $("<a>")
                          .addClass("text-muted mr-3")
                          .attr({
                            href: "javascript:void(0);",
                            "data-toggle": "tooltip",
                            "data-placement": "top",
                            title: "Available",
                          })
                          .append($("<i>").addClass("fas fa-check-double"))
                      : ""
                  )
                  .append(
                    $("<div>")
                      .addClass("flex-shrink-0 dropdown")
                      .append(
                        $("<a>")
                          .addClass("text-muted")
                          .attr({
                            href: "javascript:void(0);",
                            "data-toggle": "dropdown",
                            "aria-expanded": "false",
                          })
                          .append($("<i>").addClass("fas fa-ellipsis-h"))
                      )
                      .append(
                        $("<div>")
                          .addClass("dropdown-menu")
                          .append(
                            $("<a>")
                              .addClass("dropdown-item")
                              .text("View")
                              .attr({
                                "data-toggle": "modal",
                                "data-target": "#generalDialog",
                                "data-action": "questionSet",
                              })
                              .data({
                                questions: survey.questions,
                                title: survey.title,
                              })
                          )
                          .append(
                            survey.editable
                              ? $("<a>")
                                  .addClass("dropdown-item")
                                  .text("edit")
                                  .attr({
                                    "data-toggle": "modal",
                                    "data-target": "#lessonDialog",
                                    "data-action": "survey",
                                  })
                                  .prop("edit", true)
                              : ""
                          )
                          .append(
                            (survey.editable && survey.responseCount == 0) ||
                              survey.closed
                              ? $("<a>")
                                  .addClass("dropdown-item")
                                  .text("Delete")
                                  .attr({
                                    "data-toggle": "modal",
                                    "data-target": "#generalDialog",
                                    "data-action": "confirm",
                                  })
                                  .prop("callback", () => (confirm) => {
                                    if (confirm) {
                                      const index = surveyList.findindex(
                                        (otherSurvey) => otherSurvey == survey
                                      );
                                      if (index != -1)
                                        surveyList.splice(index, 1);
                                      surveyCard.remove();
                                    }
                                  })
                              : ""
                          )
                      )
                  )
              )
              .append(
                $("<p>")
                  .addClass("text-muted text-truncate-two-lines max-lines")
                  .css("height", "5em")
                  .html(survey.description)
              )
              .append(
                $("<div>")
                  .addClass("row align-items-center")
                  .append(
                    $("<div>")
                      .addClass("col-sm pr-0")
                      .append(
                        $("<span>")
                          .addClass("badge alert-custom border-0 text-start")
                          .text(survey.course?.name ?? "All courses")
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("col-sm pl-2 text-end")
                      .append(
                        $("<span>")
                          .addClass("font-weight-bold")
                          .text(survey.creator)
                      )
                  )
              )
              .append(
                survey.closeSubmitAfterDue
                  ? $("<span>").append(
                      $("<i>")
                        .addClass("mr-2 text-dark fas fa-check-square")
                        .css("font-size", "large")
                    )
                        .append(
                          $("<span>")
                            .addClass("text-muted")
                            .text("Close response after due date")
                        )
                  : ""
              )
          )
          .append(
            $("<div>")
              .addClass("card-footer bg-transparent")
              .css("border-top-style", "dashed")
              .append(
                $("<div>")
                  .addClass("d-flex")
                  .append(
                    $("<div>")
                      .addClass("flex-grow-1")
                      .append(
                        $("<span>")
                          .addClass("text-muted mb-0")
                          .html(
                            `<i class="far fa-clock"></i> ${
                              survey.dueDate || "No due date"
                            }`
                          )
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("flex-shrink-0")
                      .append(
                        $("<a>")
                          .attr({
                            href: "javascript:void(0);",
                            "data-toggle": "tooltip",
                            "data-placement": "top",
                            title: `${
                              survey.responseCount > 0 ? "View" : "No"
                            } response${survey.responseCount > 1 ? "s" : ""}`,
                          })
                          .addClass("text-muted")
                          .html(
                            `<svg width="20" height="20" fill="none" xmlns="http://www.w3.org/2000/svg" style="vertical-align: bottom;" viewBox="0 0 16 16"><path opacity="0.4" d="M2 6h8v5H7l-3 2v-2H2V6Z" fill="#6E7687" stroke="#6E7687" stroke-width="1.5" stroke-linecap="round" stroke-linejoin="round"></path><path d="M14 3H6v5h3l3 2V8h2V3Z" fill="#6E7687" stroke="#6E7687" stroke-width="1.5" stroke-linecap="round" stroke-linejoin="round"></path></svg> ${
                              ~~survey.responseCount || "No"
                            } response${survey.responseCount > 1 ? "s" : ""}`
                          )
                      )
                  )
              )
          )
      );

    return surveyCard;
  }

  #putCourses(courses) {
    $("#surveyCourseList").append(
      $("<a>")
        .addClass("dropdown-item")
        .attr("href", "javascript:void(0);")
        .text("All courses")
        .click(function () {
          $("#selectSurveyCourse").text("All courses").removeData("course");
        })
    );

    for (const { name, id } of courses) {
      $("#surveyCourseList").append(
        $("<a>")
          .addClass("dropdown-item")
          .attr("href", "javascript:void(0);")
          .text(name)
          .data("id", id)
          .click(function () {
            $("#selectSurveyCourse").text(name).data("course", { id, name });
            $(this).parent().addClass("d-none");
          })
      );
    }
    $("#selectSurveyCourse").text("All courses").removeClass("disabled");
  }
}

$.fn.extend({
  timePicker: function (fileDialog) {
    if (!$("#timePicker")[0]) {
      $("<div>")
        .addClass("modal fade")
        .attr({
          id: "timePicker",
          tabindex: "-1",
          role: "dialog",
          "aria-hidden": "true",
        })
        .appendTo($("body"))
        .append(
          $("<div>")
            .addClass("modal-dialog modal-dialog-centered")
            .attr("role", "document")
            .append(
              $("<div>")
                .addClass("timepicker-ui-wrapper timepicker-ui-normalize")
                .append(
                  $("<div>")
                    .addClass("timepicker-ui-header timepicker-ui-normalize")
                    .append(
                      $("<div>")
                        .addClass(
                          "timepicker-ui-select-time timepicker-ui-normalize"
                        )
                        .text("Select Time")
                    )
                    .append(
                      $("<div>")
                        .addClass(
                          "timepicker-ui-wrapper-time timepicker-ui-normalize"
                        )
                        .append(
                          $("<div>")
                            .addClass(
                              "timepicker-ui-hour timepicker-ui-normalize active"
                            )
                            .attr("role", "button")
                            .text("12")
                            .click(function () {
                              if ($(this).hasClass("active")) return;
                              $(".timepicker-ui-minutes").removeClass("active");
                              $(this).addClass("active");
                              const spans = $(".timepicker-ui-tips-wrapper")
                                .attr("hand", "hour")
                                .find("span > span")
                                .each(function (i, span) {
                                  if (i == 0) $(span).text("12");
                                  else $(span).text(i);
                                });
                              const txt = $(this).text();
                              const pos = parseInt(txt) % 12,
                                activeSpan = $(spans).filter(".active")[0],
                                ativePos = $(spans).index(activeSpan);
                              const theta = pos * 30;

                              if (pos != ativePos) {
                                $(activeSpan).removeClass("active");
                                $(spans[pos]).addClass("active");
                              }
                              $(".timepicker-ui-clock-hand").css(
                                "rotate",
                                `${theta}deg`
                              );
                            })
                        )
                        .append(
                          $("<div>")
                            .addClass(
                              "timepicker-ui-dots timepicker-ui-normalize"
                            )
                            .text(":")
                        )
                        .append(
                          $("<div>")
                            .addClass(
                              "timepicker-ui-minutes timepicker-ui-normalize"
                            )
                            .attr("role", "button")
                            .text("00")
                            .click(function () {
                              if ($(this).hasClass("active")) return;
                              $(".timepicker-ui-hour").removeClass("active");
                              $(this).addClass("active");
                              const spans = $(".timepicker-ui-tips-wrapper")
                                .attr("hand", "minute")
                                .find("span > span")
                                .each(function (i, span) {
                                  const txt = (i * 5)
                                    .toString()
                                    .padStart(2, "0");
                                  $(span).text(txt);
                                });
                              const txt = parseInt($(this).text()),
                                activeSpan = $(spans).filter(".active")[0];

                              if (txt % 5 == 0) {
                                const pos = txt / 5,
                                  ativePos = $(spans).index(activeSpan);
                                if (pos != ativePos) {
                                  $(activeSpan).removeClass("active");
                                  $(spans[pos]).addClass("active");
                                }
                              } else {
                                $(activeSpan).removeClass("active");
                              }
                              const theta = txt * 6;
                              $(".timepicker-ui-clock-hand").css(
                                "rotate",
                                `${theta}deg`
                              );
                            })
                        )
                    )
                    .append(
                      $("<div>")
                        .addClass(
                          "timepicker-ui-wrapper-type-time timepicker-ui-normalize"
                        )
                        .click(function (evt) {
                          const elem = evt.target;
                          if ($(elem).hasClass("active")) return;
                          $("div.active", this).removeClass("active");
                          $(elem).addClass("active");
                        })
                        .append(
                          $("<div>")
                            .addClass(
                              "timepicker-ui-am timepicker-ui-normalize active"
                            )
                            .attr({ role: "button", "data-type": "AM" })
                            .text("AM")
                        )
                        .append(
                          $("<div>")
                            .addClass(
                              "timepicker-ui-pm timepicker-ui-normalize"
                            )
                            .attr({ role: "button", "data-type": "PM" })
                            .text("PM")
                        )
                    )
                )
                .append(
                  $("<div>")
                    .addClass("timepicker-ui-normalize")
                    .append(
                      $("<div>")
                        .addClass("timepicker-ui-body timepicker-ui-normalize")
                        .append(
                          $("<div>")
                            .addClass(
                              "timepicker-ui-clock-face timepicker-ui-normalize"
                            )
                            .click(function ({ offsetX, offsetY, target }) {
                              let theta;
                              const handAttr =
                                $(".timepicker-ui-tips-wrapper").attr("hand") ??
                                "hour";

                              if (
                                $(target).prop("tagName").toLowerCase() ==
                                "span"
                              ) {
                                const resolvedTarget =
                                  $(target).children().length > 0
                                    ? $(target).find("span")
                                    : target;
                                const pos = $(
                                  ".timepicker-ui-tips-wrapper > span > span"
                                ).index(resolvedTarget);
                                theta = pos * 30;
                              } else {
                                const centerX =
                                    parseInt($(this).css("width")) / 2,
                                  centerY = parseInt($(this).css("height")) / 2;
                                theta =
                                  (Math.atan2(
                                    offsetY - centerY,
                                    offsetX - centerX
                                  ) *
                                    180) /
                                    Math.PI +
                                  90;
                                theta =
                                  handAttr == "hour"
                                    ? Math.round(theta / 30) * 30
                                    : Math.round(theta / 6) * 6;
                              }
                              const clockHand = $(
                                ".timepicker-ui-clock-hand"
                              ).css("rotate", `${theta}deg`);

                              if (handAttr == "hour" || theta % 30 == 0) {
                                $(
                                  ".timepicker-ui-circle-hand",
                                  clockHand
                                ).removeClass("small-circle");
                                const spans = $(
                                    ".timepicker-ui-tips-wrapper > span > span"
                                  ),
                                  len = $(spans).length;
                                const pos = (theta / 30) % len,
                                  spanAt = $(spans).get(pos);
                                const activeSpan =
                                  $(spans).filter(".active")[0];
                                if (activeSpan == spanAt) return;
                                $(activeSpan).removeClass("active");
                                const txt = $(spanAt)
                                  .addClass("active")
                                  .text()
                                  .padStart(2, "0");
                                handAttr == "hour"
                                  ? $(".timepicker-ui-hour").text(txt)
                                  : $(".timepicker-ui-minutes").text(txt);
                              } else {
                                $(
                                  ".timepicker-ui-circle-hand",
                                  clockHand
                                ).addClass("small-circle");
                                const txt = theta / 6;
                                handAttr == "hour"
                                  ? $(".timepicker-ui-hour").text(txt)
                                  : $(".timepicker-ui-minutes").text(txt);
                              }
                            })
                            .append(
                              $("<div>").addClass(
                                "timepicker-ui-dot timepicker-ui-normalize"
                              )
                            )
                            .append(
                              $("<div>")
                                .addClass(
                                  "timepicker-ui-clock-hand timepicker-ui-normalize"
                                )
                                .css("rotate", "360deg")
                                .append(
                                  $("<div>").addClass(
                                    "timepicker-ui-circle-hand timepicker-ui-normalize"
                                  )
                                )
                            )
                            .append(
                              $("<div>")
                                .addClass(
                                  "timepicker-ui-tips-wrapper timepicker-ui-normalize"
                                )
                                .attr("hand", "hour")
                            )
                        )
                    )
                    .append(
                      $("<div>")
                        .addClass(
                          "timepicker-ui-footer timepicker-ui-normalize"
                        )
                        .append(
                          $("<div>")
                            .addClass(
                              "timepicker-ui-wrapper-btn timepicker-ui-normalize"
                            )
                            .append(
                              $("<div>")
                                .addClass(
                                  "timepicker-ui-cancel-btn timepicker-ui-normalize"
                                )
                                .attr({
                                  role: "button",
                                  "aria-pressed": "false",
                                  "data-dismiss": "modal",
                                })
                                .text("CANCEL")
                            )
                            .append(
                              $("<div>")
                                .addClass(
                                  "timepicker-ui-ok-btn timepicker-ui-normalize"
                                )
                                .attr({
                                  role: "button",
                                  "aria-pressed": "false",
                                  "data-dismiss": "modal",
                                  id: "showSelectedTime",
                                })
                                .text("OK")
                                .click(function () {
                                  fileDialog.clickedInside = true;
                                  const hr = parseInt(
                                    $(".timepicker-ui-hour").text()
                                  );
                                  const minute = $(
                                    ".timepicker-ui-minutes"
                                  ).text();
                                  const am = $(".timepicker-ui-am").hasClass(
                                    "active"
                                  )
                                    ? "AM"
                                    : "PM";

                                  if (!isNaN(hr) && /^\d{2}$/.test(minute)) {
                                    fileDialog.setTime(`${hr}:${minute} ${am}`);
                                  }
                                })
                            )
                        )
                    )
                )
            )
        )
        .on({
          "show.bs.modal": function (evt) {
            let match, timeElem;

            if ($("#assignmentModal").find(evt.relatedTarget)[0])
              timeElem = $("#assignTime");
            else timeElem = $("#quizTime");
            $(this).prop("relElem", timeElem);

            if (
              (match = $(timeElem)
                .text()
                .match(/\d{1,2}:\d{2}\s[AP]M/))
            ) {
              const time = match[0].split(/(?::|\b(?=[AP]))/);
              const hr = Math.min(Math.max(parseInt(time[0]), 0), 12),
                minute = Math.min(Math.max(parseInt(time[1]), 0), 60);
              if (time[2] == "AM") {
                $(".timepicker-ui-am").addClass("active");
                $(".timepicker-ui-pm").removeClass("active");
              } else {
                $(".timepicker-ui-pm").addClass("active");
                $(".timepicker-ui-am").removeClass("active");
              }
              $(".timepicker-ui-hour")
                .text(hr.toString().padStart(2, "0"))
                .click();
              $(".timepicker-ui-minutes").text(
                minute.toString().padStart(2, "0")
              );
            }
            if (!this.clickedInside) this.clickedInside = true;
          },
          "hidden.bs.modal": function () {
            $(".timepicker-ui-hour").text("12");
            $(".timepicker-ui-minutes").text("00");
            $(".timepicker-ui-am").addClass("active");
            $(".timepicker-ui-pm").removeClass("active");
            $(".timepicker-ui-clock-hand").css("rotate", "0deg");
            if (!$(".timepicker-ui-value-tips").first().hasClass("active")) {
              $(".timepicker-ui-value-tips.active").removeClass("active");
              $(".timepicker-ui-value-tips").first().addClass("active");
            }
          },
          click: () => (fileDialog.clickedInside = true),
        });

      (function () {
        const clockFace = $("#timePicker .timepicker-ui-tips-wrapper"),
          clockArr = [
            { left: "112px", bottom: "215px", num: "12" },
            { left: "163.5px", bottom: "201.2px", num: "1" },
            { left: "201.2px", bottom: "163.5px", num: "2" },
            { left: "215px", bottom: "112px", num: "3" },
            { left: "201.2px", bottom: "60.5px", num: "4" },
            { left: "163.5px", bottom: "22.8px", num: "5" },
            { left: "112px", bottom: "9px", num: "6" },
            { left: "60.5px", bottom: "22.8px", num: "7" },
            { left: "22.8px", bottom: "60.5px", num: "8" },
            { left: "9px", bottom: "112px", num: "9" },
            { left: "22.8px", bottom: "163.5px", num: "10" },
            { left: "60.5px", bottom: "201.2px", num: "11" },
          ];

        for (let i = 0; i < 12; i++) {
          const { left, bottom, num } = clockArr[i],
            active = i == 0 ? " active" : "";
          $("<span>")
            .addClass("timepicker-ui-hour-time-12")
            .css({ left, bottom })
            .appendTo(clockFace)
            .append(
              $("<span>")
                .addClass(`timepicker-ui-value-tips${active}`)
                .text(num)
            );
        }
      })();
    }
  },
  richEditor: function (selection) {
    const toolbar = $(this).next(".btn-toolbar"),
      editPad = $(this).find("p"),
      formatStack = [];
    let focusElem = editPad;
    const resizeObserver = new ResizeObserver((_) => {
      $(editPad).css("width", $(this).width() + "px");
    });
    const focusOnElem = (elem, node = null) => {
      elem = elem[0] ?? elem;
      $(elem).focus();
      node ??= elem;

      if (selection.rangeCount == 0) selection.addRange(new Range());
      const range = selection.getRangeAt(0);
      range.selectNodeContents(node);
      range.collapse(false);
    };
    $(this).prop("observer", () => () => resizeObserver.disconnect());
    resizeObserver.observe(this[0]);
    $(editPad).on({
      focusin: function () {},
      focusout: function () {},
      beforeinput: function (evt) {
        console.log("key", evt.key);
      },
    });

    $("button", toolbar).click(function () {
      const tag = $(this).prop("tag");
      if (tag != "format") $(this).toggleClass("active");
      const isActive = $(this).hasClass("active");

      switch (tag) {
        case "strong":
        case "em":
          // let txtNode;

          // if (isActive) {
          //   formatStack.push(tag);
          // } else {
          //   pos = formatStack.findIndex((addedTag) => addedTag == tag);
          //   formatStack.splice(pos, 1);
          // }
          // focusOnElem(focusElem, txtNode);

          if (isActive) {
            focusElem = $(`<${tag}>`)
              .appendTo($(focusElem).removeAttr("contenteditable"))
              .attr("contenteditable", true)
              .css("outline", "none");
            formatStack.push(tag);
          } else {
            const formatElem = $(focusElem).closest(`${tag}`),
              pos = formatStack.findIndex((addedTag) => addedTag == tag),
              addedTags = formatStack.slice(pos + 1);
            let isNextTo = false;
            focusElem = $(formatElem).parent();

            if ($(formatElem).isEmpty()) {
              $(formatElem).remove();
            } else {
              $(formatElem).removeAttr("contenteditable");
              isNextTo = true;
            }

            for (const addedTag of addedTags) {
              if (isNextTo) {
                focusElem = $(`<${addedTag}>`).insertAfter(focusElem);
                isNextTo = false;
              } else {
                focusElem = $(`<${addedTag}>`).appendTo(focusElem);
              }
            }

            if (addedTags.length == 0) {
              txtNode = document.createTextNode("");
              isNextTo
                ? $(formatElem).after(txtNode)
                : $(focusElem).append(txtNode);
            }
            $(focusElem).attr("contenteditable", true).css("outline", "none");
            formatStack.splice(pos, 1);
          }
          break;
        case "line":
          if (isActive) {
          } else {
          }
          break;
        case "list":
          if (isActive) {
            formatStack.unshift("ul");
          } else {
          }
          break;
        case "format":
          // const
          break;
      }
      return false;
    });
  },
  isEmpty: function () {
    return $(this).text() == "";
  },
});

if (typeof toast !== "function") {
  function toast(msg, color = "green", time) {
    time = time || 2000;
    var ts = $("<div>")
      .appendTo("body")
      .css({
        overflow: "hidden",
        display: "block",
        position: "fixed",
        "z-index": "8001",
        top: "20%",
        left: "4%",
        "background-color": color,
        "border-radius": "2px",
        "text-align": "center",
      })
      .attr({ id: "toast-container_" })
      .append(
        $("<div>")
          .css({
            "background-color": color,
            opacity: "1",
            float: "left",
            padding: "5px 10px",
            "text-align": "center",
            "line-height": "30px",
            "margin-top": "0px",
            "font-size": "15px",
            color: "white",
            "font-family": "'Raleway', sans-serif",
          })
          .addClass("toast_")
          .text(msg)
      );
    $(ts).animate({ top: 0 }, 2000);
    setTimeout(function () {
      $(ts).remove();
    }, time);
  }
}
