let fileDialog,
  formDialog,
  genericDialog,
  dateRangePicker,
  classType = "preview",
  clickedInside = false;
const courseObj = {},
  surveyList = [];
$(document).ready(function () {
  dateRangePicker = new DateRangePicker();
  formDialog = new FormDialog();
  genericDialog = new GenericDialog(formDialog.updateTopics.bind(formDialog));
  fileDialog = new FileDialog();
  createLoaderModal();
  $(".search-course").on("keypress", function ({ key }) {
    if (key == "Enter") {
      const searchStr = $(this).val().trim();

      if (!searchStr) {
        toast("Search is empty", "black");
        return;
      }

      if ($("#searchResults")[0]) {
        $("#searchResults").empty().append(loading());
        loadPageContent(this, "search");
      } else {
        if (!$(this).attr("data-page")) {
          $(this).attr("data-page", "search");
        }
        gotoPage.call(this);
      }
    }
  });
  $(window).on("beforeunload", function (evt) {
    if (typeof evt.originalEvent !== "undefined") {
      evt.originalEvent.returnValue = "";
    }
    return "Are you sure you want to refresh the page? You may loose unsaved work.";
  });
});

function createLoaderModal() {
  $("<div>")
    .addClass("modal fade")
    .appendTo($("body"))
    .attr({
      id: "loaderModal",
      tabindex: "-1",
      role: "dialog",
      "aria-hidden": "true",
      "data-backdrop": "static",
    })
    .append(
      $("<div>")
        .addClass("modal-dialog modal-dialog-centered")
        .attr("role", "document")
        .append(
          $("<div>")
            .addClass("spinner-border text-light")
            .css({ width: "3rem", height: "3rem" })
            .attr("role", "status")
        )
    );
}

function loadPageContent(elem, page) {
  switch (page) {
    case "dashboard": {
      $(".search-course").attr("placeholder", "Search").prop("search", "all");
      pageLoadAjax = $.get(
        `${site.process}api?option=insight`,
        function (jsonRes) {
          pageLoadAjax = null;
          console.log("jsonRes", jsonRes);
          const res = JSON.parse(jsonRes);
          parseDashboardInfo(res);
        }
      );
      break;
    }
    case "assigned_courses": {
      $(".search-course")
        .attr("placeholder", "Search in courses")
        .prop("search", "courses");
      const page = $(elem).prop("page") ?? 1,
        page_size = $(elem).prop("page_size") ?? 10,
        count = $(elem).prop("count"),
        type = $(elem).prop("type"),
        params = { page, page_size, first_time: count ? false : true, type };
      pageLoadAjax = $.get(
        `${site.process}api?option=assigned_courses`,
        params,
        function (jsonRes) {
          console.log(jsonRes);
          pageLoadAjax = null;
          const {
            0: draftCourses,
            1: availCourses,
            avail_count,
            draft_count,
          } = JSON.parse(jsonRes);
          const availContainer = $("#nav-avail"),
            draftContainer = $("#nav-draft");
          $("#course-per-page")
            .next(".dropdown-menu")
            .children()
            .removeClass("disabled");

          if (!type || type == "1") {
            params.count = count || avail_count;
            params.type = 1;
            parseAssignedCourses(availContainer, availCourses, params);
          }

          if (!type || type == "0") {
            params.count = count || draft_count;
            params.type = 0;
            parseAssignedCourses(draftContainer, draftCourses, params);
          }
        }
      );
      $("#nav-avail-tab, #nav-draft-tab").click(function () {
        if ($(this).hasClass("active")) return;

        if ($(this).attr("id") == "nav-avail-tab") {
          $("#nav-draft").removeClass("show active");
          const activeLi = $("nav ul li").filter((_, li) =>
            $(li).prop("active")
          );
          setTimeout(() => {
            if (!$(activeLi).hasClass("active")) $(activeLi).addClass("active");
          }, 100);
        } else {
          $("#nav-avail").removeClass("show active");
        }
      });
      break;
    }
    case "survey": {
      $(".search-course")
        .attr("placeholder", "Search in survey")
        .prop("search", "survey");
      const page = $(elem).prop("page") ?? 1,
        page_size = $(elem).prop("page_size") ?? 10,
        count = $(elem).prop("count"),
        course_id = $(elem).prop("course_id"),
        params = {
          page,
          page_size,
          first_time: count ? false : true,
          course_id,
        };
      pageLoadAjax = $.get(
        `${site.process}api?option=fetch_survey`,
        function (jsonRes) {
          console.log('jsonRes', jsonRes);
          pageLoadAjax = null;
          const {
            surveys,
            count: surveyCount,
            course_id,
          } = JSON.parse(jsonRes);
          params.count = count || surveyCount;

          if (course_id) params.course_id = course_id;
          surveyList.splice(0, surveyList.length, ...surveys);
          parseSurvey(surveys, params);
        }
      );
      const dropdownMenu = $("#survey-filter .dropdown-menu");

      if ($(dropdownMenu).is(":empty")) {
        $.get(`${site.process}api?option=course_list`, function (jsonRes) {
          const courses = JSON.parse(jsonRes);
          $(dropdownMenu).append(
            $("<a>").addClass("dropdown-item").text("All").click(filterSurvey)
          );

          for (const { name, id } of courses) {
            $("<a>")
              .addClass("dropdown-item text-truncate")
              .css("width", "13rem")
              .text(name)
              .prop("course_id", id)
              .appendTo(dropdownMenu)
              .click(filterSurvey);
          }
        });
      }
      break;
    }
    case "students": {
      $(".search-course")
        .attr("placeholder", "Search in students")
        .prop("search", "students");
      const page = $(elem).prop("page") ?? 1,
        page_size = $(elem).prop("page_size") ?? 10,
        count = $(elem).prop("count"),
        type = $(elem).prop("type"),
        params = { page, page_size, first_time: count ? false : true, type };
      pageLoadAjax = $.get(
        `${site.process}api?option=get_students`,
        params,
        function (jsonRes) {
          console.log("student", jsonRes);
          pageLoadAjax = null;
          const {
            0: currentStudents,
            1: pastStudents,
            current_count,
            past_count,
          } = JSON.parse(jsonRes);
          const curStudContainer = $("#current_students"),
            pastStudContainer = $("#past_students");

          if (!type || type == 0) {
            params.count = count ? count : current_count;
            params.type = params.type ? params.type : 0;
            parseStudents(currentStudents, curStudContainer, params);
          }

          if (!type || type == 1) {
            params.count = count ? count : past_count;
            params.type = params.type ? params.type : 1;
            parseStudents(pastStudents, pastStudContainer, params);
          }
        }
      );
      break;
    }
    case "payment": {
      $(".search-course")
        .attr("placeholder", "Search in payment")
        .prop("search", "payment");
      const page = $(elem).prop("page") ?? 1,
        page_size = $(elem).prop("page_size") ?? 10,
        count = $(elem).prop("count"),
        filter_time = $(elem).prop("filter_time"),
        filter_mode = $(elem).prop("filter_mode"),
        params = {
          page,
          page_size,
          filter_time,
          filter_mode,
        };
      pageLoadAjax = $.get(
        `${site.process}api?option=payments`,
        params,
        function (jsonRes) {
          console.log("payments", jsonRes);
          pageLoadAjax = null;
          const { payments, pay_count } = JSON.parse(jsonRes);
          params.count = count ? count : pay_count;
          parsePayments(payments, params);
        }
      );
      break;
    }
    case "edit_course": {
      $(".search-course")
        .attr("placeholder", "Search in courses")
        .prop("search", "courses");
      const id = $(elem).prop("course_id");
      gotoCourse(id);
      break;
    }
    case "search": {
      const search = $(".search-course").val(),
        page = $(".search-course").prop("search"),
        more = !!$(elem).prop("more"),
        page_num = $(elem).prop("page_num"),
        page_size = $(elem).prop("page_size"),
        type = $(elem).prop("type");
      pageLoadAjax = $.get(
        `${site.process}api?option=search`,
        { search, page, page_num, page_size, type },
        function (jsonRes) {
          const result = JSON.parse(jsonRes);
          console.log("result", result);
          pageLoadAjax = null;
          searchInPage(result, more);
        }
      );
      break;
    }
    case "classroom": {
      const course_id = $(elem).prop("course_id");
      pageLoadAjax = $.get(
        `${site.process}api?option=course_preview`,
        { course_id },
        function (jsonRes) {
          const courseContent = JSON.parse(jsonRes);
          console.log("courseContent", courseContent);
          pageLoadAjax = null;
          prepareClassroom(courseContent);
        }
      );
    }
  }
}

function parseDashboardInfo({ payments, assigned_courses, branch, graph, ...stats }) {
  $("#tutor_count").text(stats.tutor_count);
  $("#course_count").text(stats.course_count);
  $("#student_count").text(shortenFigure(stats.current_student_count));
  $("#graduate_count").text(shortenFigure(stats.past_student_count));
  $("#tableName")
    .text(payments ? "Last payments" : "Assigned courses")
    .next("a")
    .removeClass("disabled")
    .attr("data-page", payments ? "payment" : "assigned_courses");
  let tableEmpty = false;
  // console.log('graph', parent)
  $(".graph").each((i, parent) => {
    const id = i == 0 ? "studentChart" : "paymentChart";
    $("<div>")
      .css("min-height", "320px")
      .appendTo($(parent).empty())
      .append(
        $("<canvas>")
          .css({ width: "100%", "max-width": "600px" })
          .attr("id", id)
      );
  });
  
  if (branch) {
    $('#branch-header').append('<svg width="1em" height="1em" viewBox="-6 0 48 48" xmlns="http://www.w3.org/2000/svg"><path d="M591.943,471a3.676,3.676,0,0,1-2.38-1.207C583,462.245,574,451.5,574,441a18,18,0,1,1,36,0c0,10.5-9,21.242-15.564,28.79A3.676,3.676,0,0,1,592.057,471ZM576,441c0,10.242,9.9,21.162,15.764,27.793a1.537,1.537,0,0,1,.206.211,1.767,1.767,0,0,0,.215-.155C598.1,462.162,608,451.242,608,441a16,16,0,0,0-32,0Zm7-.005a9,9,0,1,1,9,9A9.01,9.01,0,0,1,583,440.995Zm1,0a8,8,0,1,0,8-8A8.007,8.007,0,0,0,584,440.994Z" transform="translate(-573.998 -423)" fill="#494949" /></svg>').append($('<span>').addClass('text-capitalize ml-1').css('font-weight', '500').text(branch));
  }

  if (payments) {
    const paymentLen = payments.length;

    if (paymentLen) {
      const headers = [
        "#",
        "Course",
        "Student",
        "Amount",
        "Last payment",
        "Status",
        "Invoice ref",
        "Date",
      ];
      const tr1 = $("<tr>"),
        tbody = $("<tbody>").appendTo(
          $("<table>")
            .addClass("table table-bordered table-sm")
            .appendTo($("#dashboard-table").empty())
            .append($("<thead>").append(tr1))
        );

      for (const header of headers) {
        $("<th>").appendTo(tr1).text(header);
      }

      for (let i = 0; i < paymentLen; i++) {
        let payment = payments[i];
        const tr = $("<tr>")
          .appendTo(tbody)
          .append($("<td>").text(`${i + 1}.`));

        for (const key in payment) {
          const val =
            key == "amount"
              ? Intl.NumberFormat("en-NG", {
                  currency: "NGN",
                  style: "currency",
                }).format(payment[key])
              : payment[key];
          $("<td>").appendTo(tr).text(val);
        }
      }
    } else {
      tableEmpty = true;
    }
  } else {
    if (assigned_courses.length == 0) {
      tableEmpty = true;
    } else {
      const tr = $("<tr>");
      const tbody = $("<tbody>").appendTo(
        $("<table>")
          .addClass("table table-bordered table-sm")
          .appendTo($("#dashboard-table").empty())
          .append($("<thead>").append(tr))
      );
      const headers = [
        "#",
        "Course name",
        "Status",
        "Date created",
        "Last modified",
        "",
      ];

      for (const header of headers) $("<th>").appendTo(tr).text(header);

      const courseLen = assigned_courses.length;
      let id;

      for (let i = 0; i < courseLen; i++) {
        const course = assigned_courses[i],
          itemNo = i + 1;
        const tr = $("<tr>")
          .appendTo(tbody)
          .append($("<td>").text(`${itemNo}.`));

        for (const key in course) {
          const val = course[key];

          if (key == "id") {
            id = val;
            continue;
          }
          $("<td>").appendTo(tr).text(val);
        }
        $("<td>")
          .appendTo(tr)
          .append(
            $("<a>")
              .addClass("btn btn-sm")
              .css({ "background-color": "#d4edda", color: "#155724" })
              .attr({ href: "javascript:void(0);", "data-page": "edit_course" })
              .prop("course_id", id)
              .append($("<i>").addClass("fa fa-arrow-right pe-none"))
              .click(gotoPage)
          );
      }
    }
  }

  if (tableEmpty) {
    $("#tableName").parent().remove();
    $("#dashboard-table").remove();
  }
  drawChart(graph);
}

function parseAssignedCourses(
  container,
  courses,
  { page, page_size, count, type }
) {
  $(container).empty();

  if (courses.length == 0) {
    $("<div>")
      .addClass(
        "rounded border d-flex justify-content-center align-items-center ml-xl-3"
      )
      .css("height", "12rem")
      .appendTo(container)
      .append(
        $("<h5>").text(
          `You don't have any ${
            type == 1 ? "available courses" : "drafts"
          } assigned to you`
        )
      );
  } else {
    const tr = $("<tr>");
    const tbody = $("<tbody>").appendTo(
      $("<table>")
        .addClass("table table-bordered table-sm")
        .appendTo(container)
        .append($("<thead>").append(tr))
    );
    const headers = [
      "#",
      "Course name",
      "Status",
      "Date created",
      "Last modified",
      "",
    ];

    for (const header of headers) $("<th>").appendTo(tr).text(header);

    paginateTable(
      container,
      { page, page_size, count, type },
      "assigned_courses",
      $("#course-per-page").next(".dropdown-menu").children()
    );
    const courseLen = courses.length;

    for (let i = 0; i < courseLen; i++) {
      const course = courses[i],
        itemNo = (page - 1) * page_size + i + 1;
      const tr = $("<tr>")
        .appendTo(tbody)
        .append($("<td>").text(`${itemNo}.`));
      let id;

      for (const key in course) {
        const val = course[key];

        if (key == "id") {
          id = val;
          continue;
        }
        $("<td>").appendTo(tr).text(val);
      }
      $("<td>")
        .appendTo(tr)
        .append(
          $("<a>")
            .addClass("btn btn-sm")
            .css({ "background-color": "#d4edda", color: "#155724" })
            .attr({ href: "javascript:void(0);", "data-page": "edit_course" })
            .prop("course_id", id)
            .append($("<i>").addClass("fa fa-arrow-right pe-none"))
            .click(gotoPage)
        );
    }
  }
}

function parseStudents(students, container, { page, page_size, count, type }) {
  const studentType = type == 0 ? "current" : "past";
  $("#create_student").removeClass("disabled");
  $(container).empty();

  if (students.length == 0) {
    $("<div>")
      .addClass(
        "rounded border d-flex justify-content-center align-items-center ml-xl-3"
      )
      .css("height", "12rem")
      .appendTo(container)
      .append($("<h5>").text(`No ${studentType} students were found`));
  } else {
    const toolbar =
        $(container).prev("h4").find(".btn-toolbar")[0] ||
        $("<div>")
          .addClass("btn-toolbar mb-2 mb-md-0 float-right")
          .appendTo($(container).prev("h4"))
          .append(
            $("<button>")
              .addClass("btn btn-sm text-dark dropdown-toggle shadow-0")
              .css("background-color", "#f8f9fa")
              .attr("data-toggle", "dropdown")
              .text("10 Items")
          )
          .append(
            $("<div>")
              .addClass("dropdown-menu mt-2")
              .append(
                $("<a>")
                  .addClass("dropdown-item py-0")
                  .attr("href", "javascript:void(0);")
                  .text("10 Items")
                  .prop("page_size", 10)
                  .click(({ target }) => switchStudentsCount(target))
              )
              .append(
                $("<a>")
                  .addClass("dropdown-item py-0")
                  .attr("href", "javascript:void(0);")
                  .text("25 Items")
                  .prop("page_size", 25)
                  .click(({ target }) => switchStudentsCount(target))
              )
              .append(
                $("<a>")
                  .addClass("dropdown-item py-0")
                  .attr("href", "javascript:void(0);")
                  .text("50 Items")
                  .prop("page_size", 50)
                  .click(({ target }) => switchStudentsCount(target))
              )
          ),
      dropdownToggle = $('[data-toggle="dropdown"]', toolbar).dropdown();
    const headers = [
      "#",
      "Student name",
      "Email",
      "Phone",
      "Course(s)",
      "Learning center",
      "Registered by",
      "Registered on",
    ];
    const tr = $("<tr>"),
      tbody = $("<tbody>").appendTo(
        $("<table>")
          .addClass("table table-bordered table-sm")
          .appendTo(container)
          .append($("<thead>").append(tr))
      );

    for (const header of headers) $("<th>").appendTo(tr).text(header);

    paginateTable(
      container,
      { page, page_size, count, type },
      "students",
      dropdownToggle
    );
    const studentLen = students.length,
      keys = ["name", "email", "phone", "courses", "branch", "creator", "date"];

    for (let i = 0; i < studentLen; i++) {
      const student = students[i],
        itemNo = (page - 1) * page_size + i + 1,
        id = student.id;
      const tr = $("<tr>")
        .appendTo(tbody)
        .append(
          $("<td>").append(
            student.editable
              ? $("<div>")
                  .addClass("custom-control custom-checkbox")
                  .append(
                    $("<input>")
                      .addClass("custom-control-input")
                      .attr({
                        type: "checkbox",
                        id: `${studentType}StudentCheck${i}`,
                      })
                  )
                  .change(function () {
                    // TODO
                  })
                  .append(
                    $("<label>")
                      .addClass("custom-control-label")
                      .attr("for", `${studentType}StudentCheck${i}`)
                      .css("cursor", "pointer")
                      .text(`${itemNo}.`)
                  )
              : `${itemNo}.`
          )
        );

      for (const key of keys) {
        const val = student[key];
        $("<td>").addClass("text-capitalize").appendTo(tr).text(val);
      }
      $("<td>")
        .appendTo(tr)
        .append(
          student.editable
            ? $("<a>")
                .addClass("btn btn-sm bg-light text-dark")
                .attr({
                  href: "javascript:void(0);",
                  "data-target": "#generalDialog",
                  "data-toggle": "modal",
                  "data-action": "edit_student",
                })
                .prop("student_id", id)
                .text("Edit")
                .append($("<i>").addClass("fas fa-edit ml-2"))
            : ""
        );
    }
  }
}

function parsePayments(
  payments,
  { page, page_size, filter_time, filter_mode, count }
) {
  const container = $("#pay-container").empty();
  $("#record_payment").removeClass("disabled");

  if (payments.length == 0) {
    $("<div>")
      .addClass(
        "rounded border d-flex justify-content-center align-items-center ml-xl-3"
      )
      .css("height", "12rem")
      .appendTo(container)
      .append($("<h5>").text(`No payment records were found`));
  } else {
    const headers = [
      "#",
      "Course",
      "Student",
      "Amount",
      "Debt",
      "Last payment",
      "Invoice ref",
      "Status",
      "Date",
      "",
    ];
    const tr = $("<tr>"),
      tbody = $("<tbody>").appendTo(
        $("<table>")
          .addClass("table table-bordered table-sm")
          .appendTo(container)
          .append($("<thead>").append(tr))
      );
    const caption = $("<caption>")
        .css("caption-side", "top")
        .prependTo($(tbody).closest("table")),
      payModeDropdown = $("#payModeDropdown");
    $("input", payModeDropdown).each(function () {
      if ($(this).is(":checked")) {
        const txt = $(this).next("label").text();
        $("<span>")
          .addClass("btn btn-sm text-dark bg-light rounded-pill shadow-0 mr-2")
          .css("font-size", "small")
          .text(txt)
          .appendTo(caption)
          .append($("<i>").addClass("fas fa-check text-secondary ml-1"));
      }
    });

    if (filter_time) {
      $("<span>")
        .addClass("btn btn-sm text-dark bg-light rounded-pill shadow-0 mr-2")
        .css("font-size", "small")
        .text(`Within ${filter_time.toLowerCase()}`)
        .appendTo(caption)
        .append($("<i>").addClass("fas fa-check ml-1"));
    }
    $(container)
      .prev()
      .find(".filterby, select, input")
      .removeClass("disabled")
      .prop("disabled", false);

    for (const header of headers) $("<th>").appendTo(tr).text(header);

    paginateTable(
      container,
      { page, page_size, filter_time, filter_mode, count },
      "payment",
      $(".filterby")
    );
    const paymentLen = payments.length,
      keys = [
        "course",
        "student",
        "amount",
        "balance",
        "payment_method",
        "ref",
        "completed",
        "payment_date",
      ];

    for (let i = 0; i < paymentLen; i++) {
      const payment = payments[i],
        itemNo = (page - 1) * page_size + i + 1,
        id = payment.id;
      const tr = $("<tr>")
        .appendTo(tbody)
        .append(
          $("<td>").append(
            payment.editable
              ? $("<div>")
                  .addClass("custom-control custom-checkbox")
                  .append(
                    $("<input>")
                      .addClass("custom-control-input")
                      .attr({
                        type: "checkbox",
                        id: `paymentCheck${i}`,
                      })
                  )
                  .change(function () {
                    // TODO
                  })
                  .append(
                    $("<label>")
                      .addClass("custom-control-label")
                      .attr("for", `paymentCheck${i}`)
                      .css("cursor", "pointer")
                      .text(`${itemNo}.`)
                  )
              : `${itemNo}.`
          )
        );

      for (const key of keys) {
        const val = key.match(/amount|balance/)
          ? Intl.NumberFormat("en-NG", {
              currency: "NGN",
              style: "currency",
            }).format(payment[key])
          : payment[key];
        $("<td>").addClass("text-capitalize").appendTo(tr).text(val);
      }
      $("<td>")
        .appendTo(tr)
        .append(
          payment.editable
            ? $("<a>")
                .addClass("btn btn-sm bg-light text-dark mr-3 rounded-circle")
                .attr({
                  href: "javascript:void(0);",
                  "data-target": "#generalDialog",
                  "data-toggle": "modal",
                  "data-action": "edit_payment",
                })
                .prop("payment_id", id)
                .append($("<i>").addClass("fas fa-edit ml-2"))
            : ""
        )
        .append(
          $("<a>")
            .addClass("btn btn-sm bg-light text-dark")
            .attr({
              href: "javascript:void(0);",
              "data-target": "#generalDialog",
              "data-toggle": "modal",
              "data-action": "view_payment",
            })
            .prop("invoice_ref", payment.ref)
            .text("View")
        );
    }
  }
  $("#payTimeInterval")
    .off()
    .on("keydown", function ({ key }) {
      if (key == "Enter") filterPayments($(this).prev());
    });

  $(container)
    .prev()
    .find(".btn-group")
    .off()
    .on({
      "hide.bs.dropdown": function (evt) {
        if (clickedInside) {
          clickedInside = false;
          evt.preventDefault();
        }
      },
      click: function (evt) {
        if ($(".dropdown-menu", this).has(evt.target)[0]) {
          clickedInside = true;
        }
      },
    });
}

function paginateTable(container, params, view, blurElems = $()) {
  const { page, page_size, count } = params;

  if (count > page_size) {
    const startPage = page > 3 ? page - 2 : 1,
      lastPage = Math.ceil(count / page_size),
      endPage = Math.min(startPage + 2, lastPage);
    const nav = $("<ul>")
      .addClass("pagination float-right")
      .appendTo($("<nav>").appendTo(container))
      .append(
        $("<li>")
          .addClass("page-item")
          .append(
            $("<a>")
              .addClass(`page-link${page == 1 ? " disabled" : ""}`)
              .attr({ href: "javascript:void(0);", "aria-label": "Previous" })
              .append(
                $("<span>")
                  .addClass("fa fa-chevron-left")
                  .attr("aria-hidden", "true")
              )
              .append($("<span>").addClass("sr-only").text("Previous"))
              .prop({ ...params, page: page - 1 })
              .click(function () {
                if (page <= 1) return;
                loading(container);
                blurElems.addClass("disabled");
                loadPageContent(this, view);
              })
          )
      )
      .append(startPage > 1 ? $("<li>").addClass("page-item").text("...") : "");

    let liElem = $("<li>")
      .addClass("page-item")
      .appendTo(nav)
      .append(
        $("<a>")
          .addClass(`page-link${page == lastPage ? " disabled" : ""}`)
          .attr({ href: "javascript:void(0);", "aria-label": "Next" })
          .append(
            $("<span>")
              .addClass("fa fa-chevron-right")
              .attr("aria-hidden", "true")
          )
          .append($("<span>").addClass("sr-only").text("Next"))
          .prop({ ...params, page: page + 1 })
          .click(function () {
            if (page >= lastPage) return;
            loading(container);
            blurElems.addClass("disabled");
            loadPageContent(this, view);
          })
      );

    if (lastPage > endPage) {
      const more = $("<li>")
        .addClass("page-item")
        .text("...")
        .insertBefore(liElem);
      liElem = more;
    }

    for (let i = startPage; i <= endPage; i++) {
      const innerLi = $("<li>")
        .addClass("page-item")
        .insertBefore(liElem)
        .append(
          $("<a>")
            .addClass("page-link")
            .attr("href", "javascript:void(0);")
            .text(i)
            .prop({ ...params, page: i })
            .click(function () {
              if ($(this).hasClass("active")) return;
              loading(container);
              blurElems.addClass("disabled");
              loadPageContent(this, view);
            })
        );

      if (page == i) $(innerLi).addClass("active").prop("active", true);
    }
  }
}

function switchCourseCount(elem, pageSize) {
  $("#course-per-page").text(`${pageSize} Items`);
  $(elem).prop("page_size", pageSize);
  $(elem).parent().children().addClass("disabled");
  const containers = $("#nav-avail, #nav-draft").empty();
  loading(containers);
  loadPageContent(elem, "assigned_courses");
}

function switchStudentsCount(elem) {
  const containers = $("#current_students, #past_students").empty();
  $(elem).parent().prev("[data-toggle]").text($(elem).text());
  loading(containers);
  loadPageContent(elem, "students");
}

function gotoCourse(id) {
  $("#main").find("a, button:first-of-type").addClass("disabled");
  $("#main .progress-loader").removeClass("invisible");
  pageLoadAjax = $.get(
    `${site.process}api`,
    { option: "edit_course", course_id: id },
    function (jsonRes) {
      pageLoadAjax = null;
      $("#main").find("a, button:first-of-type").removeClass("disabled");
      $("#main .progress-loader").addClass("invisible");
      console.log("jsonRes", jsonRes);
      try {
        const [course, topics] = JSON.parse(jsonRes);
        $(".sidebar .nav-link.active").removeClass("active");
        $("<span>")
          .addClass("course-title")
          .appendTo($("#courseTitle").empty());
        const courseTitleElem = $(".course-title").text(course.name).last();
        $("#courseBgImg").attr("src", course.bg_ref.replace("tbn/", ""));
        $("#course-desc").html(course.contents).prev("div").remove();
        $("#goback-courses").click(() =>
          $('.sidebar .nav-link[data-page="assigned_courses"]').click()
        );
        $("#previewCourse").prop("course_id", id).click(gotoPage);
        course.available &&
          $("<sup>")
            .addClass("fa bg-success rounded-circle ml-1")
            .insertAfter(courseTitleElem)
            .css({ width: "5px", height: "5px", cursor: "pointer" })
            .attr({
              "data-toggle": "tooltip",
              "data-placement": "top",
              title: "Available",
            })
            .tooltip();
        courseObj.id = id;
        courseObj.name = course.name;
        parseTopics(topics);
        fileDialog.updateFiles(courseObj);
        $("#courseReview, .saveCourse").click(function () {
          if ($(this).attr("id") == "courseReview") saveCourse(true);
          else saveCourse();
        });
      } catch (err) {
        console.error(err);
      }
    }
  ).fail((_, __, err) => {
    console.log("failed:", err);
    $("#main").find("a, button:first-of-type").removeClass("disabled");
    $("#main .progress-loader").addClass("invisible");
    toast("An error occurred. Try later", "red");
  });
}

function luminance(r, g, b) {
  return 0.299 * r + 0.587 * g + 0.114 * b;
}

function contrast(rgb1, rgb2) {
  const lum1 = luminance(rgb1[0], rgb1[1], rgb1[2]);
  const lum2 = luminance(rgb2[0], rgb2[1], rgb2[2]);
  const brightest = Math.max(lum1, lum2);
  const darkest = Math.min(lum1, lum2);
  return (brightest + 0.05) / (darkest + 0.05);
}

function genGraphRandColors(obj, prop) {
  const colors = [];
  let retries = 0;
  for (const entry of obj) {
    const newColor = new Array(3);
    do {
      newColor[0] = Math.floor(Math.random() * 200);
      newColor[1] = Math.floor(Math.random() * 170) + 30;
      newColor[2] = Math.floor(Math.random() * 120) + 100;
    } while (colors.some((color) => contrast(color, newColor) < 2 && retries < 100 && ++retries));
    colors.push(newColor);
    entry[prop] = `rgb(${newColor.join(',')})`;
  }
}

function drawChart(graph) {
  const { enrolData, paymentData, months, year } = graph;
  genGraphRandColors(enrolData, "backgroundColor");
  const data = {
    labels: months,
    datasets: enrolData,
  };

  const config = {
    type: "bar",
    data: data,
    options: {
      title: {
        display: true,
        text: `Admissions for ${year}`,
      },
      responsive: true,
      scales: {
        xAxes: [
          {
            stacked: true,
            barPercentage: 0.4,
          },
        ],
        yAxes: [
          {
            stacked: true,
            barPercentage: 0.4,
          },
        ],
      },
    },
  };
  new Chart("studentChart", config);

  // for (const payment of paymentData) {
  //   let color;
  //   do {
  //     color = `rgb(${160 - Math.floor(Math.random() * 80)}, ${
  //       160 - Math.floor(Math.random() * 80)
  //     }, ${160 + Math.floor(Math.random() * 60)})`;
  //   } while (colors.includes(color));
  //   colors.push(color);
  //   payment.borderColor = color;
  //   payment.fill = false;
  // }
  for (const payment of paymentData) payment.fill = false;
  genGraphRandColors(paymentData, "borderColor");

  new Chart("paymentChart", {
    type: "line",
    data: {
      labels: months,
      datasets: paymentData,
    },
    options: {
      legend: { display: false },
      title: {
        display: true,
        text: `Payments(₦) for courses for ${year}`,
      },
      tooltips: {
        callbacks: {
          label: function (tooltipItem, data) {
            var label = data.datasets[tooltipItem.datasetIndex].label || "";

            if (label) {
              label += ": ";
            }
            label += new Intl.NumberFormat("en-NG", {
              currency: "NGN",
              style: "currency",
            }).format(tooltipItem.yLabel);
            return label;
          },
        },
      },
    },
  });
}

function parseTopics(topics) {
  courseObj.topics = topics;
  $("#courseContents").empty();

  if (topics.length == 0) {
    $("<div>")
      .addClass(
        "rounded border d-flex justify-content-center align-items-center mb-4 col-11 mx-auto empty-lesson"
      )
      .css("height", "10rem")
      .appendTo($("#courseContents"))
      .append($("<h5>").text("No content created yet."));
  } else {
    let id = parseInt(Date.now().toString().slice(-10));
    const topicCount = topics.length;

    for (let i = 0; i < topicCount; i++) {
      const topic = topics[i];
      topic.id = `topic-${id}`;
      topic.index = i;
      genericDialog.returnTopicCard(topic, id);
      const collapsible = $(`#${topic.id}`).addClass("show");

      if (Array.isArray(topic.lessons)) {
        for (const lesson of topic.lessons) {
          id++;
          lesson.parent = topic;
          const lessonElem = formDialog.returnLessonCard(lesson, id);
          lessonElem.appendTo(collapsible);

          const contentContainer = $("<ul>")
            .addClass("list-group list-group-horizontal-xl mt-2")
            .appendTo($(`#lesson-${id}`, lessonElem));

          $.each(lesson.attachments, function (_, obj) {
            const url = obj.url,
              title = obj.title,
              filetype = obj.filetype,
              description = obj.description,
              iconClass =
                filetype == "youtube" || filetype == "video"
                  ? "fa fa-file-video"
                  : filetype == "image"
                  ? "fa fa-file-image"
                  : filetype == "file"
                  ? "fa fa-file"
                  : "fa fa-link";
            $("<li>")
              .data(obj)
              .addClass(
                "list-group-item d-flex justify-content-between align-items-start"
              )
              .append(
                $("<div>")
                  .addClass("ml-2 mr-auto")
                  .append($("<div>").addClass("font-weight-bold").text(title))
                  .append(description || url)
              )
              .append(
                $("<span>").addClass(iconClass).css("font-size", "x-large")
              )
              .appendTo(contentContainer);
          });

          if (lesson.type == "assignment" || lesson.type == "quiz") {
            const { dueDate, totalPoints, closeSubmitAfterDue, duration } =
              lesson.grading ?? {};

            if (lesson.type == "quiz") {
              const showQuizBtn = $("<div>").append(
                $("<button>")
                  .addClass("btn show-question-set")
                  .attr({
                    "data-toggle": "modal",
                    "data-target": "#generalDialog",
                    "data-action": "questionSet",
                  })
                  .css("background", "#17a2b8")
                  .data({
                    questions: lesson.questionSet,
                    title: lesson.title,
                  })
                  .text("View questions")
              );
              $(contentContainer).is(":empty")
                ? $(contentContainer).replaceWith(showQuizBtn)
                : $(contentContainer).before(showQuizBtn);
            }

            const itemDataElem = $("<div>")
              // .data(lesson.grading)
              .appendTo(
                $("<div>")
                  .addClass("d-block mt-3 float-right")
                  .prependTo($(`#lesson-${id}`, lessonElem))
              )
              .addClass("card-text d-flex")
              .css("height", "80px")
              .append(
                $("<div>")
                  .addClass("btn-group-vertical ")
                  .append($("<span>").addClass("mb-2").text(totalPoints))
                  .append(
                    $("<span>")
                      .addClass("font-weight-bold")
                      .text("Total points")
                  )
              )
              .append($("<div>").addClass("vr mx-3"))
              .append(
                $("<div>")
                  .addClass("btn-group-vertical")
                  .append($("<span>").addClass("mb-2").text(dueDate))
                  .append(
                    $("<span>").addClass("font-weight-bold").text("Due date")
                  )
              );

            if (duration) {
              $(itemDataElem)
                .append($("<div>").addClass("vr mx-3"))
                .append(
                  $("<div>")
                    .addClass("btn-group-vertical")
                    .append($("<span>").addClass("mb-2").text(duration))
                    .append(
                      $("<span>").addClass("font-weight-bold").text("Duration")
                    )
                );
            }

            if (closeSubmitAfterDue) {
              $(itemDataElem).after(
                $("<span>")
                  .append(
                    $("<i>")
                      .addClass("mr-2 text-success fas fa-check-square")
                      .css("font-size", "large")
                  )
                  .append(
                    $("<span>")
                      .addClass("text-muted")
                      .text("Close submissions after due date")
                  )
              );
            }
          }
        }
      }
      const lessonCount = topic.lessons?.length ?? 0;
      $(`#${topic.id}`)
        .prev()
        .text(
          `● ${lessonCount > 0 ? lessonCount : "No"} lesson${
            lessonCount > 1 ? "s" : ""
          } added`
        );
      id++;
    }
  }
  formDialog.updateTopics(topics.filter((topic) => topic.editable));
}

function shortenFigure(num) {
  if (num < 1000) return num;
  const affixes = ["", "K", "M", "B"];
  const pos = Math.floor(Math.log10(num) / 3);
  const fix = affixes[pos];
  const decimal = num / 1000 ** pos;
  if (decimal > Math.floor(decimal)) fix += "+";
  const convertedNum = `${Math.floor(decimal)}${fix}`;
  return convertedNum;
}

function saveCourse(publish = false, hideLoader = false) {
  if (courseObj.topics.length == 0) {
    return toast("You haven't created any content", "black");
  }

  if (!hideLoader) $("#loaderModal").modal("show");
  const params = { course_id: courseObj.id, topics: [] };

  for (let i = 0; i < courseObj.topics.length; i++) {
    const topicObj = courseObj.topics[i];
    const parentObj = {
      _id: topicObj._id,
      topic: topicObj.topic,
      editable: topicObj.editable,
    };

    if (!topicObj.editable) {
      params.topics.push(parentObj);
      continue;
    }
    parentObj.lessons = [];
    const lessonCount = topicObj.lessons?.length ?? 0;

    for (let j = 0; j < lessonCount; j++) {
      const lessonObj = topicObj.lessons[j],
        childObj = { ...lessonObj };
      delete childObj.parent;
      parentObj.lessons.push(childObj);
    }
    params.topics.push(parentObj);
  }

  $.post(
    `${site.process}api`,
    { option: "save_course", ...params, publish },
    function (jsonRes) {
      const res = JSON.parse(jsonRes);

      if (res.status == 1) {
        if (!hideLoader) {
          setTimeout(() => {
            $("#loaderModal").modal("hide");
            toast(
              publish ? "Course sent for review" : "Content saved!",
              "green"
            );
          }, 1000);
        }
        courseObj.topics
          .filter((topic) => !topic._id)
          .forEach((topic, i) => {
            topic._id = res.new_ids[i];
          });
      }
    }
  ).catch((err) => {
    console.error("err", err);
  });
}

function filterPayments(elem, page_size = null) {
  let filter_mode, filter_time, page;

  if (page_size) {
    $(elem).parent(".dropdown-menu").prev("button").text(`${count} Items `);
    $("#payment-header")
      .find(".filterby, select, input")
      .addClass("disabled")
      .prop("disabled", true);
    page = 1;
  }
  $('#payment-header input[type="checkbox"]').each(function () {
    if ($(this).is(":checked")) {
      const mode = $(this).attr("mode");
      filter_mode ??= "";
      filter_mode = filter_mode.concat(mode, ",");
    }
  });
  const timeUnit = $("#payment-header select").val(),
    timeInterval = $('#payment-header input[type="number"]').val().trim();

  if (timeUnit && timeInterval) {
    filter_time = `${timeInterval} ${timeUnit}`;
  }
  $(elem).prop({ filter_mode, filter_time, page, page_size });
  console.log("payment", filter_mode, filter_time);
  loading($("#pay-container"));
  loadPageContent(elem, "payment");
}

function parseSurvey(surveys, { page, page_size, count, course_id }) {
  const container = $("#survey_list").empty(),
    dropdownBtn = $("#survey-filter button").removeClass("disabled");

  if (surveys.length == 0) {
    $("<div>")
      .addClass(
        "col-11 rounded border d-flex justify-content-center align-items-center ml-3 ml-lg-4"
      )
      .css("height", "12rem")
      .appendTo(container)
      .append($("<h5>").text(`No surveys have beeen created`));
  } else {
    paginateTable(
      container,
      { page, page_size, count, course_id },
      "survey",
      dropdownBtn
    );

    for (const survey of surveys) {
      const surveyElem = formDialog.returnSurveyCard(survey);
      $(container).append(surveyElem);
      $('[data-toggle="tooltip"]', surveyElem).tooltip();
    }
  }
}

function filterSurvey() {
  loading($("#survey_list"));
  loadPageContent(this, "survey");
}

function searchInPage({ page, ...results }, more) {
  const resultContainer = more
    ? $("#searchResults")
    : $("#searchResults").empty();
  let includeMoreBtn = false;

  switch (page) {
    case "courses": {
      for (const key in results) {
        const courses = results[key];

        if (courses.length) {
          const tr = $("<tr>");
          const tbody = $("<tbody>").appendTo(
            $("<table>")
              .addClass("table table-bordered table-sm")
              .appendTo(
                $(resultContainer).append(
                  $("<h5>")
                    .addClass("mb-3")
                    .text(
                      `${
                        key == "available_courses"
                          ? "Available course"
                          : "Drafted course"
                      }${courses.length > 1 ? "s" : ""}`
                    )
                )
              )
              .append($("<thead>").append(tr))
          );
          const headers = [
            "#",
            "Course name",
            "Status",
            "Date created",
            "Last modified",
            "",
          ];

          for (const header of headers) $("<th>").appendTo(tr).text(header);
          const courseLen = courses.length;

          for (let i = 0; i < courseLen; i++) {
            const course = courses[i],
              itemNo = i + 1;
            const tr = $("<tr>")
              .appendTo(tbody)
              .append($("<td>").text(`${itemNo}.`));
            let id;

            for (const key in course) {
              const val = course[key];

              if (key == "id") {
                id = val;
                continue;
              }
              $("<td>").appendTo(tr).text(val);
            }
            $("<td>")
              .appendTo(tr)
              .append(
                $("<a>")
                  .addClass("btn btn-sm")
                  .css({ "background-color": "#d4edda", color: "#155724" })
                  .attr({
                    href: "javascript:void(0);",
                    "data-page": "edit_course",
                  })
                  .prop("course_id", id)
                  .append($("<i>").addClass("fa fa-arrow-right pe-none"))
                  .click(gotoPage)
              );
          }
        }
      }

      if (
        Object.values(results).every((courseList) => courseList.length == 0)
      ) {
        $("<div>")
          .addClass(
            "col-11 rounded border d-flex justify-content-center align-items-center ml-3 ml-lg-4 mt-3"
          )
          .css("height", "12rem")
          .appendTo(resultContainer)
          .append($("<h5>").text("No result was found"));
      }
      break;
    }
    case "payment": {
      const { payments } = results,
        paymentLen = payments.length;

      if (!more && paymentLen == 0) {
        $("<div>")
          .addClass(
            "rounded border d-flex justify-content-center align-items-center ml-xl-3 mt-4"
          )
          .css("height", "12rem")
          .appendTo(resultContainer)
          .append($("<h5>").text(`No result was found`));
      } else {
        let tbody;

        if (!more) {
          const headers = [
            "#",
            "Course",
            "Student",
            "Amount",
            "Transaction No",
            "Class",
            "Payment Mode",
            "Confirmed",
            "Date",
            "",
          ];
          const tr = $("<tr>");
          tbody = $("<tbody>").appendTo(
            $("<table>")
              .addClass("table table-bordered table-sm")
              .appendTo(resultContainer)
              .append($("<thead>").append(tr))
          );

          if (paymentLen >= 10) includeMoreBtn = true;

          for (const header of headers) $("<th>").appendTo(tr).text(header);
        } else {
          tbody = $("table tbody", resultContainer);
          if (paymentLen >= $(".moreResult").prop("page_size"))
            $(".moreResult").removeClass("disabled");
        }
        const keys = [
          "course",
          "student",
          "amount",
          "trx_no",
          "enroll_type",
          "payment_method",
          "confirmed",
          "payment_date",
        ];

        for (let i = 0; i < paymentLen; i++) {
          const payment = payments[i],
            itemNo = i + 1,
            id = payment.id;
          const tr = $("<tr>")
            .appendTo(tbody)
            .append(
              $("<td>").append(
                payment.editable
                  ? $("<div>")
                      .addClass("custom-control custom-checkbox")
                      .append(
                        $("<input>")
                          .addClass("custom-control-input")
                          .attr({
                            type: "checkbox",
                            id: `paymentCheck${i}`,
                          })
                      )
                      .change(function () {
                        // TODO
                      })
                      .append(
                        $("<label>")
                          .addClass("custom-control-label")
                          .attr("for", `paymentCheck${i}`)
                          .css("cursor", "pointer")
                          .text(`${itemNo}.`)
                      )
                  : `${itemNo}.`
              )
            );

          for (const key of keys) {
            const val = key.match(/amount|balance/)
              ? Intl.NumberFormat("en-NG", {
                  currency: "NGN",
                  style: "currency",
                }).format(payment[key])
              : payment[key];
            $("<td>").addClass("text-capitalize").appendTo(tr).text(val);
          }
          $("<td>")
            .appendTo(tr)
            .append(
              payment.editable
                ? $("<a>")
                    .addClass(
                      "btn btn-sm bg-light text-dark mr-3 rounded-circle"
                    )
                    .attr({
                      href: "javascript:void(0);",
                      "data-target": "#generalDialog",
                      "data-toggle": "modal",
                      "data-action": "edit_payment",
                    })
                    .prop("payment_id", id)
                    .append($("<i>").addClass("fas fa-edit ml-2"))
                : ""
            )
            .append(
              $("<a>")
                .addClass("btn btn-sm bg-light text-dark")
                .attr({
                  href: "javascript:void(0);",
                  "data-target": "#generalDialog",
                  "data-toggle": "modal",
                  "data-action": "view_payment",
                })
                .prop("payment_id", id)
                .text("View")
            );
        }
      }
      break;
    }
    case "survey": {
      const { surveys } = results,
        surveyLen = surveys.length;
      let container;

      if (!more && surveyLen == 0) {
        $("<div>")
          .addClass(
            "col-11 rounded border d-flex justify-content-center align-items-center ml-3 ml-lg-4"
          )
          .css("height", "12rem")
          .appendTo(resultContainer)
          .append($("<h5>").text(`No surveys have beeen created`));
      } else {
        container =
          $("div.row", resultContainer)[0] ||
          $("<div>")
            .addClass("row mt-4")
            .appendTo($(resultContainer).css("overflow-x", "hidden"));

        if (more) {
          if (surveyLen >= $(".moreResult").prop("page_size"))
            $(".moreResult").removeClass("disabled");
        } else if (surveyLen >= 10) includeMoreBtn = true;
      }

      for (const survey of surveys) {
        const surveyElem = formDialog.returnSurveyCard(survey);
        $(container).append(surveyElem);
        $('[data-toggle="tooltip"]', surveyElem).tooltip();
      }
      break;
    }
    case "students": {
      let j = 0;

      for (const type in results) {
        const students = results[type],
          studentLen = students.length;

        if (studentLen > 0) {
          let tbody;

          if (!more) {
            const headers = [
              "#",
              "Student name",
              "Email",
              "Phone",
              "Course(s)",
              "Learning center",
              "Registered by",
              "Registered on",
            ];
            const tr = $("<tr>");
            tbody = $("<tbody>").appendTo(
              $("<table>")
                .addClass("table table-bordered table-sm")
                .appendTo(
                  $(resultContainer).append(
                    $("<h5>")
                      .addClass("mb-3 mt-4")
                      .text(
                        `${
                          type == "current" ? "Current student" : "Past student"
                        }${students.length > 1 ? "s" : ""}`
                      )
                  )
                )
                .append($("<thead>").append(tr))
            );

            if (studentLen >= 10) {
              $("<div>")
                .addClass("d-flex justify-content-center mt-5")
                .append(
                  $("<button>")
                    .addClass(
                      "btn btn-sm bg-light shadow-0 border dropdown-toggle moreResult"
                    )
                    .text("More results")
                    .prop({
                      more: true,
                      page_num: 1,
                      page_size: 10,
                      type: type == "current" ? 0 : 1,
                    })
                    .click(function () {
                      $(this)
                        .prop({
                          page_num: $(this).prop("page_num") + 1,
                          page_size: $(this).prop("page_size") + 5,
                        })
                        .addClass("disabled");
                      loadPageContent(this, "search");
                    })
                );
            }

            for (const header of headers) $("<th>").appendTo(tr).text(header);
          } else {
            const moreResult = $(".moreResult")[j];
            tbody = $("table tbody", resultContainer)[j];

            if (studentLen >= $(moreResult).prop("page_size")) {
              $(moreResult).removeClass("disabled");
            }
          }
          const keys = [
            "name",
            "email",
            "phone",
            "courses",
            "branch",
            "creator",
            "date",
          ];

          for (let i = 0; i < studentLen; i++) {
            const student = students[i],
              itemNo = i + 1,
              id = student.id;
            const tr = $("<tr>")
              .appendTo(tbody)
              .append(
                $("<td>").append(
                  student.editable
                    ? $("<div>")
                        .addClass("custom-control custom-checkbox")
                        .append(
                          $("<input>")
                            .addClass("custom-control-input")
                            .attr({
                              type: "checkbox",
                              id: `${studentType}StudentCheck${i}`,
                            })
                        )
                        .change(function () {})
                        .append(
                          $("<label>")
                            .addClass("custom-control-label")
                            .attr("for", `${studentType}StudentCheck${i}`)
                            .css("cursor", "pointer")
                            .text(`${itemNo}.`)
                        )
                    : `${itemNo}.`
                )
              );

            for (const key of keys) {
              const val = student[key];
              $("<td>").addClass("text-capitalize").appendTo(tr).text(val);
            }
            $("<td>")
              .appendTo(tr)
              .append(
                student.editable
                  ? $("<a>")
                      .addClass("btn btn-sm bg-light text-dark")
                      .attr({
                        href: "javascript:void(0);",
                        "data-target": "#generalDialog",
                        "data-toggle": "modal",
                        "data-action": "edit_student",
                      })
                      .prop("student_id", id)
                      .text("Edit")
                      .append($("<i>").addClass("fas fa-edit ml-2"))
                  : ""
              );
          }
        }
        j++;
      }

      if (Object.values(results).every((students) => students.length == 0)) {
        $("<div>")
          .addClass(
            "rounded border d-flex justify-content-center align-items-center ml-xl-3"
          )
          .css("height", "12rem")
          .appendTo(resultContainer)
          .append($("<h5>").text(`No result was found`));
      }
      break;
    }
  }

  if (page != "students" && includeMoreBtn) {
    $(resultContainer).append(
      $("<div>")
        .addClass("d-flex justify-content-center mt-5")
        .append(
          $("<button>")
            .addClass(
              "btn btn-sm bg-light shadow-0 border dropdown-toggle moreResult"
            )
            .text("More results")
            .prop({ more: true, page_num: 1, page_size: 10 })
            .click(function () {
              $(this)
                .prop({
                  page_num: $(this).prop("page_num") + 1,
                  page_size: $(this).prop("page_size") + 5,
                })
                .addClass("disabled");
              loadPageContent(this, "search");
            })
        )
    );
  }
}

function downloadQuizReport(filename, data) {
  const csvContent = "data:text/csv;charset=utf-8,";

  // Convert data to CSV format
  data.forEach(function (row) {
    csvContent += row.join(",") + "\n";
  });

  // Create a link element to trigger the download
  const encodedUri = encodeURI(csvContent);
  $("<a>").attr({ href: encodedUri, download: filename })[0].click();
}
