let offsetTop, classType = 'standard', genericDialog, dateRangePicker;
$(document).ready(function () {
  genericDialog = new GenericDialog();
  dateRangePicker = new DateRangePicker();
  $(".search-course").on("keypress", function ({ key }) {
    if (key == "Enter") {
      const searchStr = $(this).val().trim();

      if (!searchStr) {
        toast("Search is empty", "black");
        return;
      }

      if ($("#searchResults")[0]) {
        $("#searchResults").empty().append(loading());
        loadPageContent(this, "search");
      } else {
        if (!$(this).attr("data-page")) {
          $(this).attr("data-page", "search");
        }
        gotoPage.call(this);
      }
    }
  });
  
  $(window).on('scroll', function () {
      const scrollAmt = $(window).scrollTop();
      console.log("offsetTop", offsetTop);

      if ($("#scrollto")[0]) {
        const navHeight = $(".navbar").innerHeight();
        offsetTop ??= $("#scrollto").offset().top;

        if (offsetTop - scrollAmt <= navHeight) {
          const h =
            navHeight +
            parseFloat(
              $("#scrollto")
                .css("margin-bottom")
                .replace(/[^0-9.]/, "")
            );
          $("#scrollto")
            .removeClass("position-relative")
            .addClass("fixed-top shadow-sm")
            .css({ top: navHeight + "px", "z-index": 1, left: "4.5rem" });
          $("#course-detail").css("margin-top", h + "px");
        } else {
          $("#scrollto")
            .removeClass("fixed-top shadow-sm")
            .addClass("position-relative")
            .css({ top: 0, left: 0, "z-index": 0 });
          $("#course-detail").css("margin-top", 0);
        }
      }
  });
});

function loadPageContent(elem, page) {
  switch (page) {
    case "dashboard": {
      $(".search-course").attr("placeholder", "Search").prop("search", "courses");
      pageLoadAjax = $.get(
        `${site.process}student?option=get_courses`,
        { is_dashboard: true },
        function (jsonRes) {
          const { enrolled_count, completed_count, courses, related_courses, wallet } =
            JSON.parse(jsonRes);
          const studyHrs = courses.reduce(
            (accum, { study_hrs }) => accum + parseInt(study_hrs),
            0
          );
          const courseContainer = $("#my-courses"),
            relCourseContainer = $("#related-courses");
          pageLoadAjax = null;
          $("#enrolled_count").text(enrolled_count);
          $("#completed_count").text(completed_count);
          $("#wallet").text(new Intl.NumberFormat("en-NG", {
            currency: "NGN",
            style: "currency",
          }).format(wallet).replace(/\.00$/, ''));
          $("#resume_last_course")
            .attr("data-page", "course_page")
            .prop("action", "last_course")
            .click(gotoPage);
          $("#goto_courses").click(() => $('[data-page="courses"]').click());
          $("#goto_all_courses").click(gotoPage);
          $("#study_length").text(studyHrs);
          parseCourses(
            courses,
            related_courses,
            courseContainer,
            relCourseContainer
          );
        }
      );
      break;
    }
    case "courses": {
      $(".search-course")
        .attr("placeholder", "Search in courses")
        .prop("search", "courses");
      const page = $(elem).prop("page") ?? 1;
      pageLoadAjax = $.get(
        `${site.process}student?option=get_courses`,
        { page, related_size: 10 },
        function (jsonRes) {
          const { courses, related_courses } = JSON.parse(jsonRes);
          const courseContainer = $("#my-courses"),
            relCourseContainer = $("#related-courses");
          pageLoadAjax = null;
          $("#resume_last_course")
            .attr("data-page", "course_page")
            .prop("action", "last_course")
            .click(gotoPage);
          $("#load_related_courses")
            .removeClass("disabled")
            .prop("page", page + 1)
            .click(fetchMoreCourses);
          parseCourses(
            courses,
            related_courses,
            courseContainer,
            relCourseContainer,
            true
          );
        }
      );
      $.get(
        `${site.process}student?option=course_category`,
        function (jsonRes) {
          const categories = JSON.parse(jsonRes);
          pageLoadAjax = null;
          parseCategoryFilter(categories);
        }
      );
      break;
    }
    case "progress": {
      $(".search-course")
        .attr("placeholder", "Search in progress")
        .prop("search", "course_progress");
      pageLoadAjax = $.get(
        `${site.process}student?option=progress`,
        function (jsonRes) {
          const {
            uncompleted_count,
            completed_count,
            uncompleted_courses,
            completed_courses,
          } = JSON.parse(jsonRes);
          pageLoadAjax = null;
          $("#uncompleted_count").text(uncompleted_count);
          $("#completed_count").text(completed_count);
          showCourseProgress(uncompleted_courses, completed_courses);
        }
      );
      break;
    }

    case "receipt": {
      $(".search-course")
        .attr("placeholder", "Search in payment & receipt")
        .prop("search", "receipt");
      pageLoadAjax = $.get(
        `${site.process}student?option=payment`,
        function (jsonRes) {
          console.log('jsonRes', jsonRes)
          const { last_payments, installments } = JSON.parse(jsonRes);
          pageLoadAjax = null;
          showLastPayment(last_payments, installments);
        }
      );
      break;
    }

    case "survey": {
      $(".search-course")
        .attr("placeholder", "Search in survey")
        .prop("search", "survey");
      pageLoadAjax = $.get(
        `${site.process}student?option=survey`,
        function (jsonRes) {
          const { completed, uncompleted } = JSON.parse(jsonRes);
          pageLoadAjax = null;
          parseStudentSurvey(uncompleted, completed);
        }
      );
      break;
    }

    case "search": {
      const search = $(".search-course").val(), page = $('.search-course').prop("search");
      pageLoadAjax = $.get(
        `${site.process}student?option=search`,
        { search, page },
        function (jsonRes) {
          const result = JSON.parse(jsonRes);
          pageLoadAjax = null;
          parseSearchResults(result);
        }
      );
      break;
    }

    case "course_page": {
      const course_id = $(elem).prop("course_id"),
        params = { course_id };
      pageLoadAjax = $.get(
        `${site.process}student?option=course_page`,
        params,
        function (jsonRes) {
          const course = JSON.parse(jsonRes);
          pageLoadAjax = null;
          $("#courseTitle").text(course.name);
          loadCoursePage(course);
        }
      );
      break;
    }

    case "classroom": {
      const course_id = $(elem).prop("course_id"),
        params = { course_id };
      pageLoadAjax = $.get(
        `${site.process}student?option=classroom`,
        params,
        function (jsonRes) {
          const courseContent = JSON.parse(jsonRes);
          pageLoadAjax = null;
          prepareClassroom(courseContent);
        }
      );
      break;
    }

    case "view_course": {
      const course_id = $(elem).prop("course_id");
      pageLoadAjax = $.get(
        `${site.process}student?option=view_course`,
        { course_id },
        function (jsonRes) {
          const courseContent = JSON.parse(jsonRes);
          pageLoadAjax = null;
          prepareCourseView(courseContent);
        }
      );
      break;
    }

    case "all_courses": {
      pageLoadAjax = $.get(
        `${site.process}student?option=all_courses`,
        function (jsonRes) {
          const courses = JSON.parse(jsonRes);
          pageLoadAjax = null;
          prepareAllCoursesView(courses);
        }
      );
      break;
    }
  }
}

function parseCourses(
  courses,
  relatedCourses,
  courseContainer,
  relCourseContainer,
  showLastStudy = false
) {
  if (showLastStudy) {
    const course = courses[0];
    $("<div>")
      .addClass("row justify-content-center ml-3")
      .appendTo($("#current-course").empty())
      .append(
        $("<div>")
          .addClass("card col-md-8")
          .css({
            "box-shadow": "none",
            background: `no-repeat top/cover url("${course.bg_ref.replace(
              "tbn/",
              ""
            )}")`,
            "max-width": "500px",
          })
      )
      .append(
        $("<div>")
          .addClass("col-md-4")
          .append(
            $("<div>")
              .addClass("card-body pt-0")
              .append($("<h5>").addClass("font-weight-bold").text(course.name))
              .append(
                $("<p>")
                  .addClass("card-text")
                  .text(`${convertToWord(course.duration)} Months `)
                  .append(
                    $("<small>")
                      .addClass("fa fa-circle mx-2 align-middle")
                      .css("font-size", "3px")
                  )
                  .append(course.course_period)
              )
              .append(
                $("<p>")
                  .addClass("card-text text-muted")
                  .text(`Last study: ${course.last_study}`)
              )
              .append(
                $("<a>")
                  .addClass("btn btn-sm text-dark mr-2 py-2 px-4 mb-4")
                  .css({ "background-color": "#FAA61A", 'font-weight': '500' })
                  .attr({
                    href: "javascript:void(0);",
                    "data-page": "course_page",
                  })
                  .prop("course_id", course.course_id)
                  .text("Resume course")
                  .click(gotoPage)
              )
          )
      );
  }
  $(courseContainer).empty();

  parseEnrolledCourses(courses, courseContainer);
  parseRelCourses(relatedCourses, relCourseContainer);
}

function parseEnrolledCourses(courses, container) {
  for (const course of courses) {
    $("<div>")
      .addClass("col-12 col-md-6 col-lg-4 mb-3")
      .css({ "max-width": "350px" })
      .appendTo(container)
      .append(
        $("<div>")
          .addClass("card w-100 h-100 mr-4 course-card")
          .css({ "max-height": "400px", position: "relative" })
          .append(
            $("<div>")
              .css({ height: "40%", overflow: "hidden" })
              .append(
                $("<img>")
                  .addClass("rounded-top card-img-top h-100")
                  .css("object-fit", "cover")
                  .attr("src", course.bg_ref.replace("tbn/", ""))
              )
          )
          .append(
            $("<div>")
              .addClass("card-body pb-5")
              .append(
                $("<h5>")
                  .addClass("font-weight-bold mb-2 mt-1")
                  .css({ "font-size": "18px", 'font-family': "'Spartan', sans-serif" })
                  .text(course.name)
              )
              .append(
                $("<p>")
                  .addClass("card-text text-truncate-two-lines")
                  .css("font-size", "smaller")
                  .html(course.description)
              )
              .append(
                $("<p>")
                  .addClass("card-text")
                  .css("font-size", "small")
                  .text(`${convertToWord(course.duration)} Months `)
                  .append(
                    $("<small>")
                      .addClass("fa fa-circle mx-1 align-middle")
                      .css("font-size", "3px")
                  )
                  .append(course.course_period)
              )
              .append(
                $("<p>")
                  .addClass("card-text text-muted mb-5")
                  .css("font-size", "small")
                  .text(`Last study: ${course.last_study}`)
              )
              .append(
                $("<a>")
                  .addClass("btn bg-transparent text-dark btn-hover resume-btn")
                  .css({ 'font-weight': '500', border: '1px solid #FAA61A', 'border-radius': '10px', 'box-shadow': 'none' })
                  .attr({
                    href: "javascript:void(0);",
                    "data-page": "course_page",
                  })
                  .prop("course_id", course.course_id)
                  .text("Resume course")
                  .click(gotoPage)
              )
          )
      );
  }
}

function parseRelCourses(relatedCourses, container) {
  $(container).empty();

  for (const relCourse of relatedCourses) {
    relCourse.description ??= relCourse.contents;
    relCourse.course_id ??= relCourse.id;
    const positions = [
        relCourse.description.indexOf("."),
        relCourse.description.indexOf("<br>"),
      ].filter((pos) => pos !== -1),
      shortDesc = positions.length
        ? relCourse.description.substring(0, Math.min(...positions))
        : relCourse.description;

    $("<div>")
      .addClass("col-12 col-xl-6 mb-3 d-flex justify-content-center")
      .appendTo(container)
      .append(
        $("<div>")
          .addClass("card")
          .css({ "max-width": "800px" })
          .append(
            $("<div>")
              .addClass("row h-100")
              .append(
                $("<div>")
                  .addClass("col-md-4 bg-light")
                  .css({
                    background: `no-repeat center/cover url("${relCourse.bg_ref.replace(
                      "tbn/",
                      ""
                    )}")`,
                  })
              )
              .append(
                $("<div>")
                  .addClass("col-md-8")
                  .append(
                    $("<div>")
                      .addClass("card-body")
                      .append(
                        $("<h5>")
                          .addClass("font-weight-bold mt-1")
                          .css({ "font-size": "18px", 'font-family': "'Spartan', sans-serif" })
                          .text(relCourse.name)
                      )
                      .append(
                        $("<p>")
                          .addClass("card-text text-truncate-two-lines mb-2")
                          .css({ "font-size": "small", height: "40px" })
                          .html(shortDesc)
                      )
                      .append(
                        $("<p>")
                          .addClass("card-text")
                          .css("font-size", "small")
                          .text(
                            `Duration: ${convertToWord(
                              relCourse.duration
                            )} Months `
                          )
                          .append(
                            $("<i>")
                              .addClass("fa fa-certificate align-text-top")
                              .attr({
                                "data-toggle": "tooltip",
                                "data-placement": "top",
                                title: "Certified course",
                                "aria-hidden": "true",
                              })
                              .css({
                                color: "rgb(3 68 134)",
                                cursor: "pointer",
                              })
                          )
                      )
                      .append(
                        $("<p>")
                          .addClass("card-text mb-0")
                          .append($('<span>').css('font-weight', '500').text("Immersive class"))
                          .append($('<i>').addClass('fa fa-circle mx-2 align-middle').css({'font-size':'3px'}))
                          .append(
                            $("<span>")
                              .css("font-size", "18px")
                              .text(
                                "₦" +
                                  parseFloat(relCourse.group_price)
                                    .toFixed(0)
                                    .replace(/\B(?=(\d{3})+(?!\d))/g, "$&,")
                              )
                          )
                      )
                      .append(
                        $("<p>")
                          .addClass("card-text")
                          .append($('<span>').css('font-weight', '500').text("Executive class"))
                          .append($('<i>').addClass('fa fa-circle mx-2 align-middle').css({'font-size':'3px'}))
                          .append(
                            $("<span>")
                              .css("font-size", "18px")
                              .text(
                                "₦" +
                                  parseFloat(relCourse.executive_price)
                                    .toFixed(0)
                                    .replace(/\B(?=(\d{3})+(?!\d))/g, "$&,")
                              )
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("")
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm text-dark mr-2 py-2")
                              .attr({
                                href: "javascript:void(0);",
                                "data-page": "view_course",
                              })
                              .prop("course_id", relCourse.course_id)
                              .css({ "background-color": "#FAA61A", 'font-weight': '500' })
                              .text("View course ")
                              .click(gotoPage)
                          )
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm px-4 py-2")
                              .css({ "background-color": "#291CF3", 'font-weight': '500' })
                              .attr({
                                href: "javascript:void(0);",
                                "data-toggle": "modal",
                                "data-target": "#generalDialog",
                                "data-action": "enrol_form",
                              })
                              .prop("course", relCourse)
                              .text("Enroll ")
                          )
                      )
                  )
              )
          )
      );
  }
  $('[data-toggle="tooltip"]', container).tooltip();
}

function fetchMoreCourses() {
  const loader = loading(),
    page = $(this).prop("page"),
    btn = this;
  console.log("page", page);
  $(this).addClass("disabled").parent().before(loader);
  pageLoadAjax = $.get(
    `${site.process}student?option=get_courses`,
    { page, related_size: 10, only_related: true },
    function (jsonRes) {
      const { related_courses } = JSON.parse(jsonRes);
      const relCourseContainer = $("#related-courses");
      pageLoadAjax = null;
      $(loader).remove();
      $(btn)
        .removeClass("disabled")
        .prop("page", related_courses ? page + 1 : page);
      parseCourses([], related_courses, null, relCourseContainer);
    }
  );
}

function submitResponse(response, survey_id, edit_response) {
  $.post(
    `${site.process}student?option=survey_response`,
    { survey_id, response, edit_response },
    function (jsonRes) {
      const res = JSON.parse(jsonRes);

      if (res.status) {
        toast(res.message, "green");
        $("#completed_surveys, #uncompleted_surveys").append(loading());
        loadPageContent(null, "survey");
      } else {
        toast(res.message, "red");
      }
    }
  );
}

function parseStudentSurvey(uncompletedSurveys, completedSurveys) {
  const completedContainer = $("#completed_surveys").empty(),
    uncompletedContainer = $("#uncompleted_surveys").empty();
  let emptyContainers = $();

  for (const survey of uncompletedSurveys) {
    $("<div>")
      .addClass("col-12 col-md-6 col-xl-4 col-xxl-3")
      .appendTo(uncompletedContainer)
      .append(
        $("<div>")
          .addClass("card")
          .css({ cursor: "pointer" })
          .append(
            $("<div>")
              .addClass("card-body")
              .css({ "min-height": "120px" })
              .append(
                $("<div>")
                  .addClass("d-flex mb-2")
                  .append(
                    $("<h6>")
                      .addClass("fs-15 mb-0 flex-grow-1 text-truncate")
                      .append(
                        $("<a>")
                          .addClass("text-body d-block stretched-link")
                          .attr({
                            href: "#generalDialog",
                            "data-toggle": "modal",
                            "data-action": "take_survey",
                          })
                          .prop("survey", survey)
                          .text(survey.title)
                      )
                  )
              )
              .append(
                $("<p>")
                  .addClass("text-muted text-truncate-two-lines")
                  .text(survey.description)
              )
          )
          .append(
            $("<div>")
              .addClass("card-footer bg-transparent")
              .css({ "border-top-style": "dashed" })
              .append(
                $("<div>")
                  .addClass("d-flex")
                  .append(
                    $("<div>")
                      .addClass("flex-grow-1")
                      .append(
                        $("<span>")
                          .addClass("text-muted mb-0")
                          .text("Closing date")
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("flex-shrink-0")
                      .append($("<span>").text(survey.close_date))
                  )
              )
          )
      );
  }

  if (uncompletedSurveys.length == 0)
    emptyContainers = $(emptyContainers).add(uncompletedContainer);

  if (completedSurveys.length) {
    const headers = ["#", "Title", "Course", "Close date", "Completed", ""];
    const tr = $("<tr>"),
      tbody = $("<tbody>").appendTo(
        $("<table>")
          .addClass("table table-bordered table-sm")
          .appendTo(completedContainer)
          .append($("<thead>").append(tr))
      );
    const keys = ["title", "course", "close_date"];
    let id;

    for (const header of headers) $("<th>").appendTo(tr).text(header);
    const surveyLen = completedSurveys.length;

    for (let i = 0; i < surveyLen; i++) {
      const survey = completedSurveys[i],
        itemNo = i + 1;
      const tr = $("<tr>")
        .appendTo(tbody)
        .append($("<td>").text(`${itemNo}.`));

      for (const key of keys) {
        const val = survey[key];

        if (key == "id") {
          id = val;
          continue;
        }
        $("<td>").appendTo(tr).text(val);
      }
      $("<td>")
        .appendTo(tr)
        .append(
          `<svg width="1.5em" height="1.5em"><use xlink:href="#tick-icon" /></svg>`
        );

      $("<td>")
        .appendTo(tr)
        .append(
          $("<a>")
            .addClass("btn btn-sm")
            .css({ "background-color": "#d4edda", color: "#155724" })
            .attr({
              href: "#generalDialog",
              "data-toggle": "modal",
              "data-action": "take_survey",
            })
            .prop({ edit: true, survey })
            .prop("survey_id", id)
            .append($("<i>").addClass("fa fa-edit"))
            .click(function () {})
        );
    }
  } else {
    emptyContainers = $(emptyContainers).add(
      completedContainer.addClass("mt-4")
    );
  }

  if ($(emptyContainers)[0]) {
    $("<div>")
      .addClass(
        "rounded border d-flex justify-content-center align-items-center col"
      )
      .css("height", "12rem")
      .appendTo(emptyContainers)
      .append($("<h5>").text(`No surveys were found`));
  }
}

function parseCategoryFilter(categories) {
  const catContainer = $("#courseCategoryFilter").addClass("py-0").empty(),
    catLen = categories.length;

  for (const category of categories) {
    $("<a>")
      .addClass("dropdown-item text-truncate")
      .attr("href", "javascript:void(0);")
      .text(category.name)
      .prop(category)
      .appendTo(catContainer);
  }
}

function showLastPayment(lastPayments, installments) {
  const paymentContainer = $("#lastPayments").empty(),
    installmentContainer = $("#installments").empty();

  for (const payment of lastPayments) {
    $("<div>")
      .addClass("col-md-6")
      .appendTo(paymentContainer)
      .append(
        $("<div>")
          .addClass("card border border-light mb-3 no-scaling")
          .css({ "max-width": "28rem", "box-shadow": "none" })
          .append($("<div>").addClass("card-header border-0").text("Payment"))
          .append(
            $("<div>")
              .addClass("card-body pb-0")
              .append(
                $("<table>")
                  .addClass("table table-borderless m-0")
                  .append(
                    $("<caption>")
                      .addClass("text-end")
                      .append(
                        $("<a>")
                          .addClass(
                            "animateBg text-decoration-none btn btn-sm bg-transparent mr-2"
                          )
                          .attr({
                            href: "#generalDialog",
                            "data-toggle": "modal",
                            "data-action": "student_payments",
                          })
                          .text("All payments")
                      )
                      .append(
                        $("<a>")
                          .addClass("btn btn-sm py-2")
                          .attr({ href: "javascript:void(0);" })
                          .css({ "background-color": "#364574" })
                          .prop({ pay_id: payment.id, course: payment.course })
                          .append(
                            `<svg class="mr-1 align-bottom" width="1.2em" height="1.2em"><use xlink:href="#download-icon"></use></svg>`
                          )
                          .append("Receipt")
                          .click(downloadReceipt)
                      )
                  )
                  .append(
                    $("<tbody>")
                      .append(
                        $("<tr>")
                          .append($("<td>").text("Course name"))
                          .append(
                            $("<th>")
                              .attr({ scope: "row" })
                              .text(payment.course)
                          )
                      )
                      .append(
                        $("<tr>")
                          .append($("<td>").text("Payer"))
                          .append(
                            $("<th>")
                              .attr({ scope: "row" })
                              .text(payment.student)
                          )
                      )
                      .append(
                        $("<tr>")
                          .append($("<td>").text("Amount"))
                          .append(
                            $("<th>")
                              .attr({ scope: "row" })
                              .text(
                                new Intl.NumberFormat("en-NG", {
                                  currency: "NGN",
                                  style: "currency",
                                }).format(payment.amount)
                              )
                          )
                      )
                      .append(
                        $("<tr>")
                          .append($("<td>").text("Mode of payment"))
                          .append(
                            $("<th>")
                              .attr({ scope: "row" })
                              .text(payment.payment_method)
                          )
                      )
                      .append(
                        $("<tr>")
                          .append($("<td>").text("Date"))
                          .append(
                            $("<th>")
                              .attr({ scope: "row" })
                              .text(payment.payment_date)
                          )
                      )
                  )
              )
          )
      );
  }

  if (installments.length) {
    const headers = [
      "#",
      "Course",
      "Fee",
      "Deposit",
      "Balance",
      "Last payment",
      "",
    ];
    const tr = $("<tr>"),
      tbody = $("<tbody>").appendTo(
        $("<table>")
          .addClass("table table-bordered table-sm")
          .appendTo(installmentContainer)
          .append($("<thead>").append(tr))
      );
    const keys = ["course", "price", "amount", "balance", "payment_date"];

    for (const header of headers) $("<th>").appendTo(tr).text(header);
    const installmentLen = installments.length;

    for (let i = 0; i < installmentLen; i++) {
      const installment = installments[i],
        itemNo = i + 1;
      const tr = $("<tr>")
        .appendTo(tbody)
        .append($("<td>").text(`${itemNo}.`));

      for (const key of keys) {
        const val = installment[key];
        $("<td>").appendTo(tr).text(val);
      }
      $("<td>")
        .appendTo(tr)
        .append(
          $("<a>")
            .addClass("btn btn-sm")
            .css({ "background-color": "#d4edda", color: "#155724" })
            .attr({
              href: "#generalDialog",
              "data-toggle": "modal",
              "data-action": "",
            })
            .prop("course_id", installment.coursed_id)
            .text("Track")
        );
    }
  } else {
    $("<div>")
      .addClass(
        "rounded border d-flex justify-content-center align-items-center col"
      )
      .css("height", "12rem")
      .appendTo(installmentContainer)
      .append($("<h5>").text(`You have not made any installments`));
  }
}

function showCourseProgress(uncompletedCourses, completedCourses) {
  const completedContainer = $("#completed_courses").empty(),
    uncompletedContainer = $("#uncompleted_courses").empty();
  console.log("completedContainer", uncompletedCourses, completedCourses);

  for (const course of uncompletedCourses) {
    $("<div>")
      .addClass("col-xl-4 col-sm-6")
      .css({ cursor: "pointer" })
      .appendTo(uncompletedContainer)
      .append(
        $("<div>")
          .addClass("card box-shadow")
          .append(
            $("<div>")
              .addClass("card-body")
              .append(
                $("<div>")
                  .addClass("d-flex flex-column h-100")
                  .append(
                    $("<div>")
                      .addClass("d-flex")
                      .append(
                        $("<div>")
                          .addClass("flex-grow-1")
                          .append(
                            $("<p>")
                              .addClass("text-muted mb-3")
                              .text(`Started: ${course.date_enrolled}`)
                          )
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("d-flex mb-2")
                      .append(
                        $("<div>")
                          .addClass("rounded mt-1 mr-3")
                          .css({
                            width: "100px",
                            height: "60px",
                          })
                          .append(
                            $("<img>")
                              .addClass("img-fluid rounded h-100 w-100")
                              .css({
                                "min-width": "60px",
                                "object-fit": "cover",
                              })
                              .attr({
                                src: `${course.bg_ref.replace("tbn/", "")}`,
                              })
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("flex-grow-1")
                          .append(
                            $("<h6>")
                              .addClass("text-capitalize mb-1")
                              .append(
                                $("<a>")
                                  .addClass(
                                    "text-dark text-decoration-none stretched-link"
                                  )
                                  .attr({
                                    href: "javascript:void(0);",
                                    "data-page": "course_page",
                                  })
                                  .text(course.name)
                                  .prop("course_id", course.course_id)
                                  .click(gotoPage)
                              )
                          )
                          .append(
                            $("<p>")
                              .addClass(
                                "text-muted text-truncate-two-lines mb-3"
                              )
                              .css({ "font-size": "smaller" })
                              .text(course.description)
                          )
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("mt-3")
                      .append(
                        $("<div>")
                          .addClass("d-flex mb-2")
                          .append(
                            $("<div>")
                              .addClass("flex-grow-1")
                              .append($("<span>").text("Progress"))
                          )
                          .append(
                            $("<div>")
                              .addClass("flex-shrink-0")
                              .append($("<span>").text(`${course.covered}%`))
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("progress progress-sm animated-progress")
                          .append(
                            $("<div>")
                              .addClass("progress-bar bg-success")
                              .attr({
                                role: "progressbar",
                                "aria-valuenow": `${course.covered}`,
                                "aria-valuemin": "0",
                                "aria-valuemax": "100",
                              })
                              .css({
                                width: `${course.covered}%`,
                                height: "5px",
                              })
                          )
                      )
                  )
              )
          )
          .append(
            $("<div>")
              .addClass("card-footer bg-transparent py-2")
              .css("border-top-style", "dashed")
              .append(
                $("<div>")
                  .addClass("d-flex align-items-center justify-content-between")
                  .append(
                    $("<div>")
                      .addClass("flex-grow-1")
                      .append(
                        $("<span>").addClass("text-muted").text("Last study")
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("flex-shrink-0")
                      .append($("<span>").text(course.last_study))
                  )
              )
          )
      );
  }

  if (completedCourses.length) {
    const headers = ["#", "Course", "Start date", "Expected end date", ""];
    const tr = $("<tr>"),
      tbody = $("<tbody>").appendTo(
        $("<table>")
          .addClass("table table-bordered table-sm")
          .appendTo(completedContainer)
          .append($("<thead>").append(tr))
      );
    const keys = ["name", "date_enrolled", "amount", "end_date"];
    let courseId;

    for (const header of headers) $("<th>").appendTo(tr).text(header);
    const courseLen = completedCourses.length;

    for (let i = 0; i < courseLen; i++) {
      const course = completedCourses[i],
        itemNo = i + 1;
      const tr = $("<tr>")
        .appendTo(tbody)
        .append($("<td>").text(`${itemNo}.`));

      for (const key of keys) {
        const val = course[key];

        if (key == "course_id") {
          courseId = val;
          continue;
        }
        $("<td>").appendTo(tr).text(val);
      }
      $("<td>")
        .appendTo(tr)
        .append(
          $("<div>")
            .addClass(
              "d-block d-md-flex d-lg-block d-xl-flex justify-content-center"
            )
            .append(
              $("<a>")
                .addClass("btn btn-sm mr-3 mb-2")
                .css({ "background-color": "#fff3cd", color: "#856404" })
                .attr({
                  href: "javascript:void(0);",
                  "data-page": "course_page",
                })
                .prop("course_id", courseId)
                .text("View")
                .click(gotoPage)
            )
            .append(
              $("<a>")
                .addClass("btn btn-sm mb-2")
                .css({ "background-color": "#d4f8d7daedda", color: "#721c24" })
                .attr({ href: "javascript:void(0);" })
                .prop("course_id", courseId)
                .text("Certficate")
                .click(downloadCertificate)
            )
        );
    }
  } else {
    $("<div>")
      .addClass(
        "rounded border d-flex justify-content-center align-items-center col"
      )
      .css("height", "12rem")
      .appendTo(completedContainer)
      .append($("<h5>").text(`You have not completed any course`));
  }
}

function prepareCourseView(course) {
  offsetTop = null;
  const courseCaption = $("#courseCaption"),
    breadCurmbs = $("#courseViewNav").find("li.breadcrumb-item");
  $(breadCurmbs).first().find("a").click(gotoPage);
  $(breadCurmbs)
    .last()
    .add($("h1", courseCaption))
    .add($("#enrollment-info .decor"))
    .text(course.name);
  const endPositions = [
    course.contents.indexOf("."),
    course.contents.indexOf("<br>"),
  ].filter((pos) => pos !== -1);
  const introStr = endPositions.length
    ? course.contents.substring(0, Math.min(...endPositions))
    : course.contents;
  $("h1", courseCaption).css({
    "animation-delay": "0.01s",
    "animation-duration": "1.5s",
  });
  $("p", courseCaption)
    .text(introStr)
    .css({ "animation-delay": "0.11s", "animation-duration": "1.5s" });
  $("img", courseCaption).attr("src", course.bg_ref.replace("tbn/", ""));
  $("a", courseCaption)
    .add($("#enrollment-info button.disabled"))
    .removeClass("disabled")
    .prop("course", course);
  $("#about-course").children().first().replaceWith(course.contents);
  $("#toggle-size")
    .removeClass("disabled")
    .click(function () {
      if ($("#about-course").css("max-height") != "none") {
        $("#about-course").css("max-height", "unset");
        $(this).text("Show less");
      } else {
        $("#about-course").css("max-height", "16rem");
        $(this).text("Show more");
      }
    });
  $(".nav-item .nav-link2").click(function () {
    if ($(this).hasClass("active")) {
      return;
    } else {
      $(this).closest("ul").find(".nav-link2.active").removeClass("active");
      $(this).addClass("active");
    }
  });
  const ulTopics = $("ul#briefContent").empty().removeClass("pl-0");

  for (const topic of course.topics) {
    $("<li>").appendTo(ulTopics).append($("<strong>").text(topic));
  }
  parseRelCourses(course.top_courses, $("#topCourses"));
  parseRelCourses(course.related_courses, $("#relatedCourses"));
  const courseLink = encodeURIComponent(
      `${site.domain}view-course/${course.id}-${course.name.replace(
        /\s+/,
        "-"
      )}`
    ),
    escCourseName = encodeURIComponent(course.name);
  $("#shareFacebook").attr(
    "href",
    `https://web.facebook.com/dialog/share?app_id=652484810127701&href=${courseLink}&display=popup&_rdc=1&_rdr`
  ).add(
    $("#shareX").attr(
      "href",
      `https://twitter.com/intent/post?url=${courseLink}&text=${escCourseName}&via=Digital%20Dreams%20ICT%20Academy`
    )
  ).add(
    $("#shareWhatsApp").attr(
      "href",
      `https://api.whatsapp.com/send/?text=${courseLink}&type=custom_url&app_absent=0`
    )
  ).add(
    $("#shareLinkedin").attr(
      "href",
      `https://www.linkedin.com/shareArticle?url=${courseLink}&title=${escCourseName}&source=Digital%20Dreams%20ICT%20Academy`
    )
  ).tooltip();
}

function prepareAllCoursesView({ top_courses, courses }) {
  console.log("courses", top_courses, courses);
  const topCourseContainer = $("#topCourses").empty(),
    coursesContainer = $("#coursesContainer").empty(),
    catDropdown = $("#filterCat")
      .next(".dropdown-menu")
      .empty()
      .removeClass("text-center")
      .append(
        $("<a>")
          .addClass("dropdown-item text-truncate text-capitalize")
          .text("All categories")
          .click(parseFilteredCourses)
      );
  const topCoursesLen = top_courses.length;

  for (let i = 0; i < topCoursesLen; i++) {
    const course = top_courses[i];
    $("<div>")
      .appendTo(topCourseContainer)
      .addClass(
        `${
          i == 0 ? "col-md-6 big" : "col-md-3"
        } posRel float-left animated bounceInUp pulse`
      )
      .css({ "margin-top": "15px" })
      .append(
        // $("<div>")
        //   .addClass("course")
        //   .append(
        //     $("<div>")
        //       .addClass("card-text")
        //       .append(
        //         $("<a>")
        //           .attr({
        //             href: "javascript:void(0);",
        //             "data-page": "all_courses",
        //           })
        //           .addClass('text-dark')
        //           .prop("course_id", course.id)
        //           .text(course.name)
        //           .click(gotoPage)
        //       )
        //       .append($('<span>').addClass('text-muted').text(course.category))
        //     // $("<div>")
        //     //   .addClass("card-overlay")
        //     //   .append(

        //     //   )
        //   )
        //   .append(
        //     $("<div>")
        //       .addClass("thumbnail")
        //       .css({
        //         "background-image": `url(${course.bg_ref})`,
        //       })
        //   )
        $("<article>")
          .addClass("css-11e0ig7")
          .attr({ role: "group" })
          .append(
            $("<div>")
              .addClass("css-uou2ha")
              .append($("<div>").addClass("css-17s7wz2"))
          )
          .append(
            $("<div>")
              .addClass("css-15q7znr")
              .append(
                $("<div>")
                  .addClass("css-2bv3l0")
                  .append($("<div>").addClass("css-hutnrx"))
                  .append(
                    $("<img>")
                      .attr({
                        alt: "",
                        loading: "lazy",
                        decoding: "async",
                        "data-nimg": "fill",
                        sizes: "442px",
                        src: course.bg_ref,
                      })
                      .css({
                        position: "absolute",
                        height: "100%",
                        width: "100%",
                        inset: "0px",
                        "object-fit": "cover",
                        color: "transparent",
                        "border-radius": "4px 4px 0px 0px",
                      })
                  )
                  .append(
                    $("<div>")
                      .addClass("css-c5zt4b")
                      .append(
                        $("<a>")
                          .addClass("chakra-heading css-1rsglaw")
                          .attr({
                            href: "/course/introduction-to-python--cd0024",
                          })
                          .text("Introduction to Python ")
                      )
                  )
              )
              .append(
                $("<div>")
                  .addClass("chakra-stack css-1a1nbps")
                  .append(
                    $("<div>")
                      .addClass("css-k008qs")
                      .append(
                        $("<p>")
                          .addClass("chakra-text css-1osqk4n")
                          .text("4 weeks")
                      )
                      .append(
                        $("<p>")
                          .addClass("chakra-text css-5ucqax")
                          .text(", Beginner")
                      )
                  )
              )
          )
      );
  }

  for (const category in courses) {
    const courseCat = courses[category];
    $("<a>")
      .addClass("dropdown-item text-truncate text-capitalize")
      .appendTo(catDropdown)
      .text(category)
      .prop("category", category)
      .click(parseFilteredCourses);
    const courseCatContainer = $("<div>")
      .addClass("row")
      .appendTo(
        $(coursesContainer).append(
          $("<h5>").addClass("mb-3 mt-4 text-capitalize").text(category)
        )
      );
    parseRelCourses(courseCat, courseCatContainer);
  }
}

function parseFilteredCourses() {
  const coursesContainer = $("#coursesContainer").empty().append(loading()),
    category = $(this).prop("category");
  $("#filterCat").text(category || $(this).text());
  pageLoadAjax = $.get(
    `${site.process}student?option=all_courses`,
    { category },
    function (jsonRes) {
      const { courses } = JSON.parse(jsonRes);
      pageLoadAjax = null;
      $(coursesContainer).empty();

      for (const category in courses) {
        const courseCat = courses[category],
          courseCatContainer = $("<div>")
            .addClass("row")
            .appendTo(
              $(coursesContainer).append(
                $("<h5>").addClass("mb-3 mt-4 text-capitalize").text(category)
              )
            );
        parseRelCourses(courseCat, courseCatContainer);
      }
    }
  );
}

function parseSearchResults({ page, ...results }) {
  const resultContainer = $("#searchResults").empty();

  switch (page) {
    case 'courses': {
      const {enrolled_courses, other_courses} = results;

      if (enrolled_courses.length) {
        const enrolledContainer = $("<div>")
          .addClass("row")
          .appendTo(
            $(resultContainer).append(
              $("<h5>")
                .addClass("mb-3 mt-4 text-capitalize")
                .text("Enrolled courses")
            )
          );
        parseEnrolledCourses(enrolled_courses, enrolledContainer);
      }
    
      if (other_courses.length) {
        const otherContainer = $("<div>")
          .addClass("row")
          .appendTo(
            $(resultContainer).append(
              $("<h5>").addClass("mb-3 mt-4 text-capitalize").text("Other courses")
            )
          );
        parseRelCourses(other_courses, otherContainer);
      }
    
      if (!enrolled_courses.length && !other_courses.length) {
        $(resultContainer).append(
          $("<div>")
            .addClass(
              "rounded border d-flex justify-content-center align-items-center col"
            )
            .css("height", "12rem")
            .append($("<h5>").text("No result was found"))
        );
      }
      break;
    }
  
    case 'progress':
      break;

    case 'payment':
      break;

    case 'survey':
      break;
  }
}

function downloadCertificate() {}

function loadCoursePage(course) {
  const percentCovered = parseInt(course.covered),
    theta = Math.round((percentCovered * 2 * Math.PI) / 100) - Math.PI,
    x3 = 43.75 * Math.cos(theta) + 50,
    y3 = 43.75 * Math.sin(theta) + 50,
    progressDiv = $("#course_covered");
  $(".spinner-border", progressDiv).replaceWith(
    $("<span>").text(`${percentCovered}%`)
  );
  $(progressDiv).prepend(
    `<svg viewBox="0 0 100 100" style="height: 140%; width: 135%;left: -17%; top: -20%; position:absolute;"><path ${
      percentCovered > 0 ? 'stroke-linecap="round"' : ""
    } stroke="#099885" stroke-width="12.5" fill="none" d="M6.25 50A43.75 43.75 0 ${
      percentCovered > 50 ? "1" : "0"
    } 1 ${x3.toFixed(2)} ${y3.toFixed(2)}"></path></svg>`
  );
  $("#goto-class").prop("course_id", course.id).click(gotoPage);
  $("#weekly_goal")
    .children()
    .each((i, div) => {
      if (i == course.schedule) {
        $(div).addClass("border border-dark");
      }
      div.onclick = function () {
        if ($(this).hasClass("border-dark")) return;
        const goalElem = $(this)
          .parent()
          .children()
          .addClass("disabled")
          .filter(".border-dark")
          .removeClass("border border-dark");
        $(this).addClass("border border-dark");
        $.post(
          `${site.process}student?option=study_goal`,
          { schedule: i, course_id: course.id },
          (jsonRes) => {
            const res = JSON.parse(jsonRes);
            $(this).parent().children().removeClass("disabled");
            if (res.status) {
              toast("Weekly goal saved.", "green");
            }
          }
        ).catch((err) => {
          console.error("Weekly goal", err);
          toast("An error occurred while saving weekly goal.", "black");
          $(this).parent().children().removeClass("disabled");
          $(this).add(goalElem).toggleClass("border border-dark");
        });
      };
    });
  $("#course-schedule")
    .find(".spinner-border")
    .replaceWith(`${course.duration} month${course.duration > 1 ? "s" : ""}`);

  const dl = $("<dl>")
      .css("border-start", "1px solid #f8f9fa")
      .appendTo($("#courseContent").empty()),
    topicLen = course.topics.length;

  for (let i = 0; i < topicLen; i++) {
    const topic = course.topics[i];
    const dl2 = $("<dl>").appendTo(
      $("<div>")
        .addClass(`ml-4 mb-3 collapse${i == 0 ? " show" : ""}`)
        .attr({ id: `course-topic${i}` })
        .appendTo(
          $(dl)
            .append(
              $("<small>")
                .addClass("rounded-circle bg-light text-center")
                .css({
                  height: "35px",
                  width: "35px",
                  "line-height": "35px",
                  position: "absolute",
                  translate: "-50%",
                  "font-size": "small",
                })
                .text(i + 1)
            )
            .append(
              $("<dt>")
                .addClass(
                  "ml-4 mb-3 pt-1 custom-toggler bg-white text-dark collapsed"
                )
                .attr({
                  "data-toggle": "collapse",
                  "data-target": `#course-topic${i}`,
                  "aria-expanded": "false",
                })
                .css({ cursor: "pointer" })
                .text(topic.title)
            )
        )
    );

    for (const lesson of topic.lessons) {
      $(dl2).append(
        $("<dd>").append($("<span>").css("font-size", "small").text(lesson))
      );
    }
  }
  $("#download_curriculum")
    .prop("course_id", course.id)
    .removeClass("disabled")
    .click(downloadCurriculum);
  $("#shareCorse")
    .prop("course", course)
    .removeClass("disabled");
}

function downloadCurriculum() {}