<?php
$data = contentCache('talent-home');
$slider = reset($data->image);

$states = $paramControl->load_sources('states');
$countries = $paramControl->load_sources('countries');
$job_status = $paramControl->load_sources('job_status');
$degrees = $paramControl->load_sources('degrees');
$disciplines = $paramControl->load_sources('disciplines');
$all_cats = $paramControl->load_sources('tech_roles');

$category = decodeFromUrl($uri->content_id);
$search = decodeFromUrl($uri->event);
$talents = !empty($category) && $category != 'all' ? talentProfiles(categories: [$category], search: $search) : talentProfiles(search: $search);
$result_count = array_reduce($talents, function ($accum, $group) {
  return $accum + count($group);
}, 0);
$data->itemprop = $data->body;
$data->title = "Find tech talents by role, location, degree, ... | Digital Dreams LTD";
?>
<!DOCTYPE html>
<html lang="en">

<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title><?=$data->title?></title>

  <?php require_once("components/seo_tags.php") ?>
  <?php require_once("components/csslinks.php") ?>
</head>

<body>
  <?php require_once("components/nav.php") ?>
  <svg xmlns="http://www.w3.org/2000/svg" class="d-none">
    <symbol id="bent-line" viewBox="0 0 180 13" fill="currentColor">
      <path d="M173.163 12.3756C97.1838 -3.8242 30.6496 5.67799 7.32494 12.2553C5.30414 12.8252 2.43544 12.6722 0.917529 11.9135C-0.600387 11.1549 -0.192718 10.0779 1.82808 9.50807C27.5427 2.25675 98.002 -7.60121 177.683 9.38783C179.881 9.85641 180.65 10.9051 179.402 11.7301C178.154 12.5552 175.361 12.8442 173.163 12.3756Z" fill="#FAA61A"></path>
    </symbol>
  </svg>
  <div class="posFix w-100" style="background-color:rgb(3, 45, 87);height:4.2rem;top:0;z-index:1;"></div>
  <section class="mx-3 mb-5" id="searchContainer" style="margin-top: <?php if (isMobile()):?>6rem<?php else:?>8rem<?php endif; ?>;">
    <div class="row">
      <div class="col-auto mb-5 mt-md-5 pr-0 mr-4" style="max-width: 20rem;">
        <h6 style="text-transform: none;">Filter by</h6>
        <div class="btn-group">
          <button class="btn bg-white text-dark btn-lg dropdown-toggle p-0" id="cat-toggle" data-toggle="dropdown" type="button" style="box-shadow: none;font-size:large;">Category</button>
          <div class="dropdown-menu p-2">
            <div class="d-flex flex-column" id="cat_filter" style="gap: 4px;width:18rem;max-height:60vh;overflow-y:auto;">
              <?php
              foreach ($all_cats as $cat) {
                $cat_id = 'cat-' . preg_replace('/\s+/', '_', $cat);
              ?>
                <div class="py-1">
                  <div class="d-inline-flex align-items-center" role="group"><input type="checkbox" class="cat-checkbox-input" id="<?= $cat_id ?>" value="<?= $cat ?>" <?php if (array_key_exists($cat, $talents)): ?>checked="" <?php endif; ?> onchange="reloadTalents()"><label for="<?= $cat_id ?>"><?= $cat ?></label></div>
                </div>
              <?php } ?>
            </div>
          </div>
        </div>
        <div class="my-4">
          <h5>Other</h5>
          <div class="d-flex">
            <div class="form-group col-lg-6 p-0 pr-1">
              <label for="loc_state">State (in Nigeria)</label>
              <select class="form-control" id="loc_state">
                <option value="" selected disabled>Select state</option>
                <?php foreach ($states as $key => $state) { ?>
                  <option value="<?= $key ?>"><?= $state ?></option>
                <?php } ?>
              </select>
            </div>
            <div class="form-group col-lg-6 p-0 pr-1">
              <label for="loc_country">Country</label>
              <select class="form-control" id="loc_country">
                <option value="" selected disabled>Select country</option>
                <?php foreach ($countries as $country) { ?>
                  <option value="<?= $country ?>"><?= $country ?></option>
                <?php } ?>
              </select>
            </div>
          </div>
          <div class="form-group my-4">
            <label for="work_type">Job status</label>
            <select class="form-control" id="job_status">
              <option value="" selected disabled>Select job status</option>
              <?php foreach ($job_status as $status) { ?>
                <option value="<?= $status ?>"><?= $status ?></option>
              <?php } ?>
            </select>
          </div>
        </div>
        <div class="form-group">
          <label for="salaryInput" class="form-label">Degree</label>
          <div class="input-group mb-3">
            <div class="input-group-prepend">
              <select class="form-control" style="width: 6.2rem;border-right:0;border-top-left-radius: 50rem;border-bottom-left-radius:50rem;outline:none;" id="degreeSelector">
                <option value="" selected disabled>Degree</option>
                <?php foreach ($degrees as $degree) { ?>
                  <option value="<?= $degree ?>"><?= $degree ?></option>
                <?php } ?>
              </select>
            </div>
            <select class="form-control" id="discipline">
              <option value="" selected disabled>Course</option>
              <?php foreach ($disciplines as $faculty => $courses) { ?>
                <option value="<?= $faculty ?>" class="font-weight-bold" disabled><?= $faculty ?></option>
                <?php foreach ($courses as $course) { ?>
                  <option value="<?= $course ?>"><?= $course ?></option>
                <?php } ?>
              <?php } ?>
            </select>
          </div>
        </div>
        <a href="javascript:void(0);" class="btn btn-sm btn-block mt-3 academy-anchor animated bounceInUp px-5" style="animation-delay: 0.21s;animation-duration: 1.5s;border-radius:50rem;background-color: #291CF3;" onclick="reloadTalents()">
          Filter talents
        </a>
      </div>
      <div class="col-md">
        <div class="card no-scaling mb-4" style="border-radius:50rem;max-width:500px;overflow:visible;box-shadow:none;">
          <div class="card-header bg-light border-0 row justify-content-between align-items-center flex-nowrap p-0" style="border-radius:50rem;">
            <div class="col d-flex align-items-center">
              <svg width="1.5em" height="1.5em" viewBox="0 0 20 20" fill="none" xmlns="http://www.w3.org/2000/svg">
                <path d="M18 18L14.1394 14.1394M14.1394 14.1394C15.4263 12.8526 16.2222 11.0748 16.2222 9.11111C16.2222 5.18375 13.0385 2 9.11111 2C5.18375 2 2 5.18375 2 9.11111C2 13.0385 5.18375 16.2222 9.11111 16.2222C11.0748 16.2222 12.8526 15.4263 14.1394 14.1394Z" stroke="#292929" />
              </svg>
              <input type="text" class="border-0 ml-3 bg-transparent" style="outline:none;flex:1;" placeholder="Title, keyword ..." id="searchTalents">
            </div>
            <div class="col-auto d-flex align-items-center pr-2">
              <a href="javascript:void(0);" class="btn btn-sm px-4 ml-3 academy-anchor" style="background-color:#291CF3;border-radius:50rem;" id="searchTalentsBtn">
                Search
              </a>
            </div>
          </div>
        </div>
        <div class="d-flex mb-3 align-items-center flex-wrap" id="search-filter" style="gap: 1rem !important;">
          <?php
          foreach ($talents as $cat => $_) {
            $cat_id = 'cat-' . preg_replace('/\s+/', '_', $cat);
          ?>
            <div class="badge bg-light d-flex align-items-center text-dark border-0 text-capitalize p-2"><?= $cat ?><button class="close ml-2" style="font-size: x-small;" data-catid="<?= $cat_id ?>" onclick="removeFilter(this, reloadTalents)"></button></div>
          <?php } ?>
          <a href="javascript:void(0);" class="btn-link text-dark font-weight-bold" style="font-size: smaller;" id="clearFilter">Clear all</a>
        </div>
        <div class="d-flex justify-content-between">
          <h2 class="text-capitalize mb-1" id="talentResCount" style="font-family: 'Roboto Condensed', sans-serif;"><?php if (!empty($search)): ?>Search for "<?= $search ?>"<?php else: ?>Talents<?php endif; ?> (<?= $result_count ?> Result<?= $result_count > 1 ? 's' : '' ?>)</h2>
        </div>
        <div class="row" id="talent-results">
          <?php
          $c = 0;
          foreach ($talents as $cat => $group) {
            $group_count = count($group);
          ?>
            <div class="d-flex justify-content-between<?= $c > 0 ? ' mt-5' : '' ?> w-100">
              <h5 class="text-capitalize w-100 mx-3" style="font-weight: 500;flex:1;"><?= $cat ?></h5>
              <h6 class="font-weight-normal text-capitalize" style="font-size:18px;">Showing (<?= $group_count ?>)</h6>
            </div>
            <?php $fit = count($group) == 1;
            foreach ($group as $talent) { ?>
              <div class="col-md-6" <?php if ($fit) { ?>style="width:fit-content;max-width: 100%;flex: none;" <?php } ?>>
                <div class="card talent-card mb-3">
                  <div class="row align-items-stretch">
                    <div class="col-md-4 p-0">
                      <img src="<?= $talent->picture_ref ?? $uri->site . 'asset/picture/photo_silhouette.png' ?>" class="img-fluid rounded-left h-100" alt="Profile picture" style="background-color:#e8f0fe;object-fit:<?php if (!empty($talent->picture_ref)) { ?>cover<?php } else { ?>contain<?php }?>;object-position:center;max-height: 220px;">
                    </div>
                    <div class="col-md-8 d-flex justify-content-center">
                      <div class="card-body" style="min-height: unset !important;max-width:100vw;">
                        <h5 class="card-title"><?= $talent->name ?></h5>
                        <h6 class="card-subtitle font-weight-normal text-capitalize mb-2" style="font-family:'Spartan', sans-serif;color:rgb(109, 122, 140);"><?= $talent->profile->title ?><?php if (!empty($talent->profile->status)) { ?><i class="fas fa-circle align-middle mx-1" style="font-size:3px;"></i> <?= $talent->profile->status ?><?php } ?></h6>
                        <p class="card-text text-truncate-lines-2" style="min-height: unset !important;"><?php
                        if (preg_match('/^.*?[.!?](?:\s|$)/', $talent->bio, $sentence) !== FALSE) echo $sentence[0];
                        ?></p><a href="<?= $uri->site ?>view-profile/<?= $talent->id . '-' . str_replace(' ', '-', $talent->name) ?>" class="btn text-dark btn-hover bg-transparent px-0 text-center" style="border-radius:10px;font-weight:600;box-shadow:none;border: 1px solid rgb(250, 166, 26);width:160px;height:40px;font-size: small;">View profile <i class="fas fa-long-arrow-alt-right ml-1 align-middle"></i></a>
                      </div>
                    </div>
                  </div>
                </div>
              </div>
          <?php }
            $c++;
          } ?>
        </div>
      </div>
    </div>
  </section>
  <?php require_once("components/footer.php") ?>
</body>

</html>