class GenericDialog {
  #editedLessonCard;
  static courseData;
  callback = (confirmed) => alert(confirmed);
  #onTopicChange = () => null;

  constructor(onTopicChange = null) {
    console.log("inside generalDialog");
    this.#createDialog();

    if (typeof onTopicChange == "function") {
      this.#onTopicChange = onTopicChange;
    }
  }

  #createDialog() {
    if (!$("#generalDialog")[0]) {
      $("<div>")
        .addClass("modal fade")
        .attr({
          id: "generalDialog",
          tabindex: "-1",
          role: "dialog",
          "data-keyboard": false,
          "aria-labelledby": "generalDialogLabel",
          "aria-hidden": "true",
        })
        .insertBefore($("body").find("script").first())
        .on({
          "show.bs.modal": this.#onShowModal.bind(this),
          "hidden.bs.modal": this.#onModalHidden.bind(this),
          "shown.bs.modal": () => {
            if ($("#topicInput")[0]) {
              $("#topicInput").focus();
            }
          },
        });
    }
    this.#reset();
  }

  #reset() {
    $("#generalDialog")
      .empty()
      .append(
        $("<div>")
          .addClass("modal-dialog modal-sm modal-dialog-centered")
          .attr("role", "document")
          .append(
            $("<div>")
              .addClass("modal-content w-100")
              .append(
                $("<div>")
                  .addClass("modal-header")
                  .append(
                    $("<h5>")
                      .addClass("modal-title")
                      .attr("id", "generalDialogLabel")
                  )
                  .append(
                    $("<button>")
                      .addClass("close")
                      .attr({
                        "data-dismiss": "modal",
                        "aria-label": "Close",
                        type: "button",
                      })
                      .append($("<span>").attr("aria-hidden", "true").text("×"))
                  )
              )
              .append($("<div>").addClass("modal-body"))
          )
      );
  }

  #onShowModal(evt) {
    const relElem = evt.relatedTarget,
      action = $(relElem).attr("data-action");

    if (action == "lesson") {
      this.#createLessonContent();
      this.#editedLessonCard = $(relElem).prop("edit")
        ? $(relElem).closest(".card")
        : null;

      if (this.#editedLessonCard) {
        const topic = $(this.#editedLessonCard)
          .find("a:only-of-type")
          .first()
          .text();
        $("#topicInput").data("oldname", topic).val(topic);
        $("#addTopic").removeClass("disabled").text("Edit");
      }
    } else if (action == "confirm") {
      this.#createConfirmContent();

      if ($(relElem).prop("callback")) {
        this.callback = $(relElem).prop("callback");
      }
    } else if (action == "questionSet") {
      this.#createQuestionSet(relElem);
    } else if (action == "settings" && GenericDialog.courseData) {
      this.#createSettingsDialog();
    } else if (action == "rearrange") {
      this.#createRearrangeDialog();
    } else if (action == "new_student") {
      this.#createStudentDialog();
    } else if (action == "edit_student") {
      this.#editStudentDialog($(relElem).prop("student_id"));
    } else if (action == "new_payment") {
      this.#createPaymentFormDialog();
    } else if (action == "view_payment") {
      this.#trackPaymentDialog($(relElem).prop("invoice_ref"));
    } else if (action == "edit_payment") {
    } else if (action == "take_quiz") {
      this.#takeQuiz($(relElem).prop("quiz"));
    } else if (action == "quiz_result") {
      this.#showQuizResult($(relElem).prop("result"));
    } else if (action == "enrol_form") {
      this.#createEnrolForm($(relElem).prop("course"));
    } else if (action == "take_survey") {
      const edit = $(relElem).prop("edit") ? true : false;
      this.#takeSurvey($(relElem).prop("survey"), edit);
    } else if (action == "student_payments") {
      this.#createStudentPaymentsDialog();
    } else if (action == "share") {
      this.#createShareDialog($(relElem).prop("course"));
    } else if (action == "quiz_stats") {
      this.#createQuizStatsDialog();
    } else if (action == "submissions") {
      this.#createSubmissionsDialog();
    } else if (action == "wallet") {
      this.#createWalletDialog();
    } else if (action == "timetable") {
      this.#createTimetableDialog();
    } else if (action == "calendar") {
      this.#createCalendarDialog();
    } else if (action == "profile") {
      this.#createProfileDialog();
    } else if (action == "notification") {
      this.#createNotificationDialog();
    } else if (action == "settings") {
      this.#createAccountSettingsDialog();
    } else if (action == "custom") {
    } else {
      evt.preventDefault();
      this.#editedLessonCard = null;
      action == "settings"
        ? toast("'courseData' is empty", "black")
        : toast(
            "'data-action' attribute is not set correctly on the caller",
            "black"
          );
    }
  }

  #onModalHidden() {
    this.#reset();
    this.#editedLessonCard = null;
  }

  #createLessonContent() {
    $("#generalDialogLabel").html(
      `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" fill="currentColor" focusable="false" width="1.2em" height="1.2em" class="align-middle rounded-circle svg-bg mr-2"><rect fill="none" height="24" width="24" /><path d="M3,5v14h18V5H3z M7,7v2H5V7H7z M5,13v-2h2v2H5z M5,15h2v2H5V15z M19,17H9v-2h10V17z M19,13H9v-2h10V13z M19,9H9V7h10V9z" /></svg> Add topic`
    );

    $("#generalDialog .modal-body")
      .append(
        $("<div>")
          .addClass("form-group create-lesson-title bg-light pt-3")
          .append(
            $("<input>")
              .addClass("form-control bg-transparent")
              .attr({
                type: "text",
                id: "topicInput",
                placeholder: "Topic",
              })
              .css({ border: "none", "font-size": "14px" })
              .on({
                beforeinput: function (evt) {
                  if ($(this).val().length > 100) evt.preventDefault();
                },
                input: function () {
                  const topic = $(this).val(),
                    topicLen = topic.length;

                  if (topic.trim()) {
                    if ($("#addTopic").hasClass("disabled"))
                      $("#addTopic").removeClass("disabled");
                  } else if (!$("#addTopic").hasClass("disabled")) {
                    $("#addTopic").addClass("disabled");
                  }
                  $(this).parent().next("div").text(`${topicLen} / 100`);
                },
                keydown: function ({ key }) {
                  if (key == "Enter" && !$("#addTopic").hasClass("disabled")) {
                    $("#addTopic").click();
                  }
                },
              })
          )
          .append($("<label>").attr("for", "topicInput").text("Topic"))
      )
      .append(
        $("<div>")
          .addClass("text-muted text-end")
          .css("font-size", "12px")
          .text("0 / 100")
      )
      .after(
        $("<div>")
          .addClass("modal-footer border-0")
          .append(
            $("<button>")
              .addClass("btn btn-sm shadow-0 text-dark")
              .attr({ type: "button", "data-dismiss": "modal" })
              .css("background", "none")
              .text("Cancel")
          )
          .append(
            $("<button>")
              .addClass("btn btn-sm bg-white shadow-0 text-dark disabled")
              .attr({ id: "addTopic", type: "button", "data-dismiss": "modal" })
              .text("Add")
              .click(this.#addTopic.bind(this))
          )
      );
  }

  #createConfirmContent() {
    $("#generalDialogLabel").text("Delete item?");

    $("#generalDialog .modal-body")
      .append($("<p>").text("Item will be deleted permanently."))
      .after(
        $("<div>")
          .addClass("modal-footer")
          .append(
            $("<button>")
              .addClass("btn btn-sm bg-light text-dark mr-2 shadow-0")
              .attr({ type: "button", "data-dismiss": "modal" })
              .text("Cancel")
          )
          .append(
            $("<button>")
              .addClass("btn btn-sm bg-danger")
              .attr({
                type: "button",
                "data-dismiss": "modal",
                id: "deleteItem",
              })
              .text("Delete")
              .click(() => this.callback(true))
          )
      );
  }

  #createQuestionSet(relElem) {
    $("#generalDialog .modal-dialog")
      .addClass("modal-lg")
      .removeClass("modal-sm");
    const questions = $(relElem).data("questions"),
      title = $(relElem).data("title"),
      type = $(relElem).closest(".card").data("type");
    $("#generalDialogLabel").html(
      `${
        type == "survey"
          ? '<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="align-middle rounded-circle svg-bg mr-2">x<line x1="18" y1="20" x2="18" y2="10"></line><line x1="12" y1="20" x2="12" y2="4"></line><line x1="6" y1="20" x2="6" y2="14"></line></svg>'
          : '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" focusable="false" width="1em" height="1em" class="align-middle rounded-circle svg-bg mr-2" fill="currentColor"><path d="M7 15h7v2H7zm0-4h10v2H7zm0-4h10v2H7z"></path><path d="M19 3h-4.18C14.4 1.84 13.3 1 12 1c-1.3 0-2.4.84-2.82 2H5c-.14 0-.27.01-.4.04a2.008 2.008 0 0 0-1.44 1.19c-.1.23-.16.49-.16.77v14c0 .27.06.54.16.78s.25.45.43.64c.27.27.62.47 1.01.55.13.02.26.03.4.03h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm-7-.25c.41 0 .75.34.75.75s-.34.75-.75.75-.75-.34-.75-.75.34-.75.75-.75zM19 19H5V5h14v14z"></path></svg>'
      } ${title}`
    );

    const qParent = $("<div>")
      .attr("id", "questionSetContainer")
      .appendTo($("#generalDialog .modal-body"));
    const questionLen = questions.length;

    for (let i = 0; i < questionLen; i++) {
      const quizQuestion = questions[i];
      const { qtype, section, question, qImage, answer, instruct, options } =
        quizQuestion;

      const headerElem =
        qtype == "section"
          ? $("<h6>").addClass("mt-3 mx-4 text-dark")
          : $("<a>")
              .addClass(
                "btn text-start bg-white w-100 shadow-0 text-dark align-top qHeader"
              )
              .attr("href", "javascript:void(0);");
      const questionElem = $("<div>")
        .addClass(`m-0${type != "survey" ? " qItem" : ""}`)
        .css("min-height", "unset")
        .appendTo(qParent)
        .append(
          $(headerElem)
            .css({
              "font-size": "medium",
              "text-transform": "none",
              cursor: "pointer",
            })
            .text(question || section)
            .append(
              qImage
                ? $("<img>")
                    .addClass("float-right mr-4")
                    .css({
                      "max-width": "60px",
                      "max-height": "60px",
                    })
                    .attr("src", qImage)
                : ""
            )
        )
        .append($("<div>").addClass(`mx-4 question${i}`));

      const qBody = $(`.question${i}`, questionElem);

      switch (qtype) {
        case "multiple":
          const blockElem = $("<div>").addClass("d-block").appendTo(qBody);
          let ansLetter;
          $.each(options, (j, { correct, optionTxt, optImage }) => {
            $("<div>")
              .appendTo(blockElem)
              .addClass(
                "d-flex justify-content-between align-items-center mb-2"
              )
              .append(
                $("<span>")
                  .addClass("qOption")
                  .append(
                    type == "survey"
                      ? $("<i>")
                          .addClass("mr-2")
                          .css("font-size", "large")
                          .text("🞅")
                      : ""
                  )
                  .append($("<span>").addClass("text-muted").text(optionTxt))
              )
              .append(
                optImage
                  ? $("<img>")
                      .css({
                        "max-width": "60px",
                        "max-height": "60px",
                      })
                      .attr("src", optImage)
                  : ""
              );

            if (correct) {
              ansLetter = String.fromCharCode(65 + j);
            }
          });

          if (type != "survey") {
            $("<p>")
              .appendTo(qBody)
              .addClass("card-text font-weight-bold")
              .text(`Answer is ${ansLetter}.`);
          }
          break;
        case "short":
          if (type == "survey") break;
        case "section":
          (qtype != "section" || section) &&
            $("<p>")
              .appendTo(qBody)
              .addClass("card-text font-weight-bold")
              .text(
                qtype == "short"
                  ? `Answer: ${answer}`
                  : `Instruction (optional): ${instruct}`
              );
          break;
      }

      if (type != "survey" && qtype != "section") {
        $("<a>")
          .addClass("btn btn-link text-secondary")
          .attr({
            "data-toggle": "collapse",
            href: `#qAnswer${i}`,
            role: "button",
            "aria-expanded": "false",
            "aria-controls": `qAnswer${i}`,
          })
          .text("Show answer ...")
          .insertBefore(
            $("p", qBody).addClass("collapse").attr("id", `qAnswer${i}`)
          )
          .click(function () {
            if ($(this).text().includes("Show"))
              $(this).text("Hide answer ...");
            else $(this).text("Show answer ...");
          });
      }
    }
  }

  #createSettingsDialog() {
    $("#generalDialog .modal-content").addClass("fullscreen");
    $("#generalDialogLabel").html("General settings");
  }

  #createRearrangeDialog() {
    $("#generalDialog .modal-dialog").removeClass("modal-sm");
    $("#generalDialogLabel").html("Rearrange topics");
    const listElem = $("<ol>").appendTo(
      $("#generalDialog .modal-body").after(
        $("<div>")
          .addClass("modal-footer py-2")
          .append(
            $("<button>")
              .addClass("btn btn-sm bg-light shadow-0 mr-3 text-dark")
              .attr("data-dismiss", "modal")
              .text("Close")
          )
          .append(
            $("<button>")
              .addClass("btn btn-sm btn-success")
              .text("Save")
              .click(this.#rearrangeTopics.bind(this))
          )
      )
    );
    const topicLen = courseObj.topics.length;
    let pos = 0;

    for (const topicObj of courseObj.topics) {
      $("<li>")
        .prop({ topic: topicObj, before: pos, after: pos })
        .appendTo(listElem)
        .append(
          $("<div>")
            .addClass("d-flex justify-content-between")
            .append($("<strong>").text(topicObj.topic))
            .append(
              $("<span>")
                .addClass("text-nowrap")
                .append(
                  $("<i>")
                    .addClass("fas fa-caret-up mr-3")
                    .css({ cursor: "pointer", "font-size": "larger" })
                    .click((evt) => this.#moveTopics(evt, "up"))
                )
                .append(
                  $("<i>")
                    .addClass("fas fa-caret-down")
                    .css({ cursor: "pointer", "font-size": "larger" })
                    .click((evt) => this.#moveTopics(evt, "down"))
                )
            )
        )
        .append(
          $("<small>")
            .addClass("text-muted")
            .css("font-size", "small")
            .text(`Topic ${pos + 1} of ${topicLen}`)
        );
      pos++;
    }
  }

  #moveTopics(evt, dir) {
    const liElem = $(evt.target).closest("li"),
      afterProp = $(liElem).prop("after"),
      topicLen = courseObj.topics.length;

    if (dir == "up") {
      const prevSib = $(liElem).prev("li")[0];

      if (prevSib) {
        $(liElem)
          .prop("after", afterProp - 1)
          .find("small")
          .text(`Topic ${afterProp} of ${topicLen}`);
        $(prevSib)
          .prop("after", afterProp)
          .before(liElem)
          .find("small")
          .text(`Topic ${afterProp + 1} of ${topicLen}`);
      }
    } else {
      const nxtSib = $(liElem).next("li")[0];

      if (nxtSib) {
        $(liElem)
          .prop("after", afterProp + 1)
          .find("small")
          .text(`Topic ${afterProp + 2} of ${topicLen}`);
        $(nxtSib)
          .prop("after", afterProp)
          .after(liElem)
          .find("small")
          .text(`Topic ${afterProp + 1} of ${topicLen}`);
      }
    }
  }

  #rearrangeTopics() {
    let changed = false;
    const liElems = $("#generalDialog .modal-body ol li"),
      topicList = courseObj.topics;
    $(liElems).each((i, elem) => {
      const before = $(elem).prop("before");

      if (before != i) {
        if (!changed) changed = true;
        const topicElems = $("#courseContents").children(),
          beforeElem = $(topicElems)[before],
          afterElem = $(topicElems)[i],
          removeObjs = topicList.splice(before, 1);

        if (before > i) {
          const liAtAfter = $(liElems).filter(
            (_, li) => $(li).prop("before") == i
          );
          $(afterElem).before(beforeElem);
          $(liAtAfter).prop("before", i + 1);
          topicList.splice(i, 0, ...removeObjs);
        } else {
          $(afterElem).after(beforeElem);
          topicList.splice(i + 1, 0, ...removeObjs);
        }
      }
    });
    $("#generalDialog").modal("hide");

    if (changed) {
      saveCourse(false);
    }
  }

  #createEnrolForm(course) {
    $("#generalDialog .modal-dialog")
      .addClass("modal-lg")
      .removeClass("modal-sm");
    $("#generalDialogLabel")
      .html(`Enrollment for ${course.name}`)
      .addClass("font-weight-bold");
    const immersivePrice = new Intl.NumberFormat("en-NG", {
      currency: "NGN",
      style: "currency",
    }).format(course.group_price);
    const executivePrice = new Intl.NumberFormat("en-NG", {
      currency: "NGN",
      style: "currency",
    }).format(course.executive_price);
    const showClassFee = function () {
      $("#courseReg").text(
        `Continue to payment portal (${$(this).data("price")})`
      );
    };
    $("#generalDialog .modal-body")
      .append($("<h4>").addClass("mb-4 mt-2").text("Course Information"))
      .append(
        $("<div>")
          .addClass("row")
          .append(
            $("<div>")
              .addClass("col-md-6")
              .append(
                $("<h6>")
                  .addClass("text-capitalize mb-3")
                  .text("Immersive class")
              )
              .append(
                $("<ul>")
                  .addClass("list-group mb-3")
                  .append(
                    $("<li>")
                      .addClass(
                        "list-group-item d-flex justify-content-between"
                      )
                      .append(
                        $("<span>")
                          .addClass("mr-4 text-nowrap")
                          .text("Course name")
                      )
                      .append(
                        $("<h6>").addClass("my-0 text-end").text(course.name)
                      )
                  )
                  .append(
                    $("<li>")
                      .addClass(
                        "list-group-item d-flex justify-content-between"
                      )
                      .append($("<div>").append($("<span>").text("Fee")))
                      .append($("<h6>").addClass("my-0").text(immersivePrice))
                  )
                  .append(
                    $("<li>")
                      .addClass(
                        "list-group-item d-flex justify-content-between"
                      )
                      .append(
                        $("<span>").addClass("text-dark").text("Promo discount")
                      )
                      .append($("<h6>").addClass("my-0").text("None"))
                  )
                  .append(
                    $("<li>")
                      .addClass(
                        "list-group-item d-flex justify-content-between"
                      )
                      .append($("<span>").text("Total (₦)"))
                      .append(
                        $("<strong>").text(
                          parseFloat(course.group_price)
                            .toFixed(2)
                            .replace(/\B(?=(\d{3})+(?!\d))/g, "$&,")
                        )
                      )
                  )
              )
          )
          .append(
            $("<div>")
              .addClass("col-md-6")
              .append(
                $("<ul>")
                  .addClass("list-group mb-3")
                  .append(
                    $("<h6>")
                      .addClass("text-capitalize mb-3")
                      .text("Executive class")
                  )
                  .append(
                    $("<li>")
                      .addClass(
                        "list-group-item d-flex justify-content-between"
                      )
                      .append(
                        $("<span>")
                          .addClass("mr-4 text-nowrap")
                          .text("Course name")
                      )
                      .append(
                        $("<h6>").addClass("my-0 text-end").text(course.name)
                      )
                  )
                  .append(
                    $("<li>")
                      .addClass(
                        "list-group-item d-flex justify-content-between"
                      )
                      .append($("<div>").append($("<span>").text("Fee")))
                      .append(
                        $("<h6>")
                          .addClass("my-0 executive_price")
                          .text(executivePrice)
                      )
                  )
                  .append(
                    $("<li>")
                      .addClass(
                        "list-group-item d-flex justify-content-between"
                      )
                      .append(
                        $("<span>").addClass("text-dark").text("Promo discount")
                      )
                      .append($("<h6>").addClass("my-0").text("None"))
                  )
                  .append(
                    $("<li>")
                      .addClass(
                        "list-group-item d-flex justify-content-between"
                      )
                      .append($("<span>").text("Total (₦)"))
                      .append(
                        $("<strong>").text(
                          parseFloat(course.executive_price)
                            .toFixed(2)
                            .replace(/\B(?=(\d{3})+(?!\d))/g, "$&,")
                        )
                      )
                  )
              )
          )
      )
      .append(
        $("<form>")
          .addClass("form-row")
          .attr({ method: "post" })
          .append(
            $("<div>")
              .addClass("d-block my-3 ml-2 mr-4")
              .append(
                $("<div>")
                  .addClass("custom-control custom-radio")
                  .append(
                    $("<input>")
                      .addClass("custom-control-input")
                      .attr({
                        id: "immerse-radio",
                        name: "price_cat",
                        type: "radio",
                        value: "immersive",
                        checked: "",
                        required: "",
                      })
                      .data("price", immersivePrice)
                      .change(showClassFee)
                  )
                  .append(
                    $("<label>")
                      .addClass("custom-control-label")
                      .attr({ for: "immerse-radio" })
                      .text("Immersive class")
                  )
              )
              .append(
                $("<div>")
                  .addClass("custom-control custom-radio")
                  .append(
                    $("<input>")
                      .addClass("custom-control-input")
                      .attr({
                        id: "exec-radio",
                        name: "price_cat",
                        type: "radio",
                        value: "executive",
                        required: "",
                      })
                      .data("price", executivePrice)
                      .change(showClassFee)
                  )
                  .append(
                    $("<label>")
                      .addClass("custom-control-label")
                      .attr({ for: "exec-radio" })
                      .text("Executive class")
                  )
              )
          )
          .append(
            $("<div>")
              .addClass("col-md-6 form-group mb-3 mt-3")
              .append(
                $("<label>")
                  .attr({ for: "coupon-code" })
                  .text("Coupon code ")
                  .append($("<span>").addClass("text-muted").text("(If any)"))
              )
              .append(
                $("<input>").addClass("form-control").attr({
                  type: "text",
                  placeholder: "Enter coupon code",
                  name: "coupon_code",
                  id: "coupon-code",
                })
              )
          )
          .append(
            $("<button>")
              .addClass("btn btn-primary btn-lg btn-block form-submitBtn")
              .attr({ id: "courseReg", type: "button" })
              .text(`Continue to payment portal (${immersivePrice})`)
              .append($("<i>").addClass("fa fa-spinner fa-pulse d-none"))
              .click(function () {
                const courseName = course.name,
                  courseId = course.course_id;
                const form = $(this)
                  .closest("form")
                  .attr("action", `${site.process}payment`);

                if (!$('input[type="radio"]', form).is(":checked")) {
                  $('input[type="radio"]', form)
                    .next("label")
                    .addClass("text-danger");
                  return;
                }
                $('input[type="radio"]', form)
                  .next("label")
                  .removeClass("text-danger");
                $("<input>")
                  .attr({
                    type: "hidden",
                    name: "process_url",
                  })
                  .val(
                    `${site.process}custom?case=process_payment&id=${courseId}&course=${courseName}`
                  )
                  .appendTo(form)
                  .after(
                    $("<input>")
                      .attr({
                        type: "hidden",
                        name: "callbefore",
                      })
                      .val("get_payment_fields")
                  )
                  .after(
                    $("<input>").attr({
                      type: "hidden",
                      value: course.course_id,
                      name: "id",
                      id: "courseId",
                    })
                  );
                let inputToDel = $("#courseName");

                if (!$("#coupon-code").val().trim())
                  inputToDel = $(inputToDel).add($("#coupon-code"));
                $(inputToDel).remove();
                $(form).submit();
              })
          )
      );
  }

  #createShareDialog(course) {
    const rawCourseLink = `${site.domain}view-course/${
        course.id
      }-${course.name.replace(/\s+/, "-")}?code=${course.referral_code}`,
      courseLink = encodeURIComponent(rawCourseLink),
      escCourseName = encodeURIComponent(course.name);
    $("#generalDialog .modal-dialog")
      .addClass("modal-lg")
      .removeClass("modal-sm");
    $("#generalDialogLabel").html("Share to relatives & friends");
    $("#generalDialog .modal-body")
      .append(
        $("<div>")
          .addClass("btn-toolbar flex-grow-1 justify-content-center")
          .attr({ role: "toolbar", "aria-label": "Toolbar with share buttons" })
          .append(
            $("<div>")
              .addClass("btn-group-vertical mr-4 align-items-center")
              .append(
                $("<a>")
                  .addClass("btn")
                  .css("background-color", "rgb(64, 81, 137)")
                  .attr({
                    href: `https://web.facebook.com/dialog/share?app_id=652484810127701&href=${courseLink}&display=popup&_rdc=1&_rdr`,
                    target: "_blank",
                  })
                  .append(
                    $("<i>")
                      .addClass("fab fa-facebook-f align-middle")
                      .css("font-size", "1.5em")
                  )
                  .css({
                    width: "50px",
                    height: "50px",
                    "border-radius": "50%",
                    "padding-top": "1rem",
                  })
              )
              .append(
                $("<h6>").addClass("text-capitalize mt-2").text("Facebook")
              )
          )
          .append(
            $("<div>")
              .addClass("btn-group-vertical mr-4 align-items-center")
              .append(
                $("<a>")
                  .addClass("btn bg-dark text-white")
                  .attr({
                    href: `https://twitter.com/intent/post?url=${courseLink}&text=${escCourseName}&via=Digital%20Dreams%20ICT%20Academy`,
                    target: "_blank",
                  })
                  .append(
                    `<svg xmlns="http://www.w3.org/2000/svg" class="align-middle" height="1.5em" height="1.5em" viewBox="0 0 512 512" fill="currentColor"><path d="M389.2 48h70.6L305.6 224.2 487 464H345L233.7 318.6 106.5 464H35.8L200.7 275.5 26.8 48H172.4L272.9 180.9 389.2 48zM364.4 421.8h39.1L151.1 88h-42L364.4 421.8z"></path></svg>`
                  )
                  .css({
                    width: "50px",
                    height: "50px",
                    "border-radius": "50%",
                    "padding-left": "1rem",
                    "padding-top": "1rem",
                  })
              )
              .append($("<h6>").addClass("text-capitalize mt-2").text("X"))
          )
          .append(
            $("<div>")
              .addClass("btn-group-vertical mr-4 align-items-center")
              .append(
                $("<a>")
                  .addClass("btn bg-success")
                  .attr({
                    href: `https://api.whatsapp.com/send/?text=${courseLink}&type=custom_url&app_absent=0`,
                    target: "_blank",
                  })
                  .append(
                    $("<i>")
                      .addClass("fab fa-whatsapp align-middle")
                      .css("font-size", "1.8em")
                  )
                  .css({
                    width: "50px",
                    height: "50px",
                    "border-radius": "50%",
                    "padding-left": "1rem",
                    "padding-top": ".8rem",
                  })
              )
              .append(
                $("<h6>").addClass("text-capitalize mt-2").text("WhatsApp")
              )
          )
          .append(
            $("<div>")
              .addClass("btn-group-vertical mr-4 align-items-center")
              .append(
                $("<a>")
                  .addClass("btn bg-info")
                  .attr({
                    href: `https://www.linkedin.com/shareArticle?url=${courseLink}&title=${escCourseName}&source=Digital%20Dreams%20ICT%20Academy`,
                    target: "_blank",
                  })
                  .append(
                    $("<i>")
                      .addClass("fab fa-linkedin-in align-middle")
                      .css("font-size", "1.5em")
                  )
                  .css({
                    width: "50px",
                    height: "50px",
                    "border-radius": "50%",
                    "padding-left": "1.1rem",
                    "padding-top": "1rem",
                  })
              )
              .append(
                $("<h6>").addClass("text-capitalize mt-2").text("LinkedIn")
              )
          )
      )
      .append(
        $("<div>")
          .addClass("form-group w-100 mx-auto")
          .css({ "max-width": "600px" })
          .append(
            $("<div>")
              .addClass("input-group mt-3")
              .append(
                $("<input>")
                  .addClass("form-control")
                  .attr({
                    type: "text",
                    placeholder: "",
                    id: "url-input-videos",
                  })
                  .css({
                    padding: "0.75rem 1rem",
                    "font-size": "0.9rem",
                    "line-height": "1.5",
                  })
                  .val(rawCourseLink)
                  .on({
                    focusin: function () {
                      this.select();
                    },
                    keydown: false,
                  })
              )
              .append(
                $("<div>")
                  .addClass("input-group-append text-center")
                  .attr({ id: "url-videos-btn" })
                  .prop("link", rawCourseLink)
                  .css({
                    "border-top-right-radius": "50%",
                    "border-bottom-right-radius": "50%",
                    width: "50px",
                    cursor: "pointer",
                    "background-color": "rgb(64, 81, 137)",
                  })
                  .append(
                    $("<span>").addClass(
                      "input-group-text text-white far fa-clone mt-3"
                    )
                  )
                  .click(function () {
                    navigator.clipboard
                      .writeText($(this).prop("link"))
                      .then(() => {
                        toast("Link copied!", "green");
                      })
                      .catch((err) => {
                        toast(
                          "Couldn't copy link. Try clicking on the link and copy it",
                          "black"
                        );
                      });
                  })
              )
          )
          .append($("<small>").addClass("invalid-feedback"))
      )
      .append(
        $("<p>")
          .addClass("card-text mx-auto my-4")
          .css("max-width", "600px")
          .text(
            "Share the link to this course with your relatives and friends. You receive a referral bonus of ₦1,000 for every person that registers with your referral link. You can redeem as cash at anytime or purchase the course of your chosen to boost your high-flying career."
          )
      );
  }

  #createQuizStatsDialog() {}
  #createSubmissionsDialog() {}
  #createWalletDialog() {}
  #createTimetableDialog() {}
  #createCalendarDialog() {}

  #editStudentDialog(student_id) {
    $("#generalDialog .modal-dialog")
      .removeClass("modal-sm")
      .addClass("modal-lg");
    $("#generalDialogLabel")
      .html("Edit student's information")
      .addClass("font-weight-bold");
    $("#generalDialog .modal-body");
    const instance = this;
    $.get(
      `${site.process}api`,
      { option: "edit_student", student_id },
      function (jsonRes) {
        const { user, enrollments, branches } = JSON.parse(jsonRes);
        const formFields = [
          {
            heading: "Student information",
            fields: [],
            formData: user,
          },
          { heading: "Learning centers for courses", fields: [] },
        ];
        const types = {
          first_name: "text",
          last_name: "text",
          email: "email",
          phone: "tel",
          password: "password",
        };

        for (const [key, type] of Object.entries(types)) {
          const feedback = key.replace("_", " ");
          formFields[0].fields.push({
            name: key,
            type,
            feedback,
            label: feedback[0].toUpperCase() + feedback.slice(1),
            id: `student_${key}`,
          });
        }

        for (let i = 0; i < enrollments.length; i++) {
          const enrollment = enrollments[i];
          formFields[1].fields.push({
            name: "branch",
            type: "select",
            feedback: "center",
            label: "Learning center",
            title: enrollment.course,
            id: `learningCenter${i}`,
            value: enrollment.branch,
            options: branches,
          });
        }
        const form = instance.#buildForm(formFields, null, "Submit", "");
        $("#submitFormBtn", form).click(function () {
          const params = { option: "edit_student", submit: true };
          const editedInfo = {};
          const selectedBranches = [];
          const submitBtn = this;
          let allValid = true;
          $(".invalid-feedback", form).removeClass("d-block");
          $("input, select", form).each(function (_, input) {
            console.log("inputs", input);
            let value = $(input).val()?.trim();

            if (!value) value = user[input.name];

            switch (input.type) {
              case "email":
                if (!input.validity.valid) {
                  $(input).next(".invalid-feedback").addClass("d-block");
                  if (allValid) allValid = false;
                } else if (value != user.email) {
                  editedInfo.email = value;
                }
                break;
              case "tel":
                if (!/^\+?(\d{11}|\d{13})$/.test(value)) {
                  $(input).next(".invalid-feedback").addClass("d-block");
                  if (allValid) allValid = false;
                } else if (value != user.phone) {
                  editedInfo.phone = value;
                }
                break;
              case "select-one":
                if (value != user[input.name]) {
                  selectedBranches.push({
                    id: $(input).attr("rel_id"),
                    branch: value,
                  });
                }
                break;
              default:
                if (value != user[input.name]) {
                  editedInfo[input.name] = value;
                }
                break;
            }
          });

          if (!Object.keys(editedInfo).length && !selectedBranches.length) {
            $("#generalDialog").modal("hide");
            return;
          }

          if (Object.keys(editedInfo).length) {
            editedInfo.student_id = user.id;
            params.student = editedInfo;
          }

          if (selectedBranches.length) params.branches = selectedBranches;
          const spinner = $("<i>")
            .addClass("fa fa-spinner fa-spin ml-1")
            .appendTo($(this).addClass("disabled"));

          $.post(
            `${site.process}api?option=edit_student`,
            params,
            function (jsonRes) {
              console.log("jsonRes", jsonRes);
              const res = JSON.parse(jsonRes);
              $(spinner).remove();
              $(submitBtn).removeClass("disabled");

              if (res.status == 1) {
                $("#generalDialog").modal("hide");
                loadPageContent(null, "students");
              } else {
                toast(res.message, "red");
              }
            }
          ).catch(function () {
            toast("An error occurred, try again later", "black");
          });
        });
      }
    );
  }

  #createStudentDialog() {
    $("#generalDialog .modal-dialog")
      .removeClass("modal-sm")
      .addClass("modal-lg");
    $("#generalDialogLabel").html("Student form").addClass("font-weight-bold");
    $("#generalDialog .modal-body").append(
      $("<div>")
        .addClass("d-flex justify-content-center align-items-center")
        .css("height", "70vh")
        .append(
          $("<div>")
            .addClass("spinner-border")
            .css({ width: "3rem", height: "3rem" })
            .attr("role", "status")
        )
    );
    const instance = this;
    $.get(
      `${site.process}api`,
      { option: "reg_form_data" },
      function (jsonRes) {
        console.log("student_info", jsonRes);
        const { available_courses, payment_modes, centers, tutors } =
          JSON.parse(jsonRes);
        const cartContent = [];
        const formFields = [
          {
            heading: "Student information",
            note: true,
            tabs: [
              {
                id: "navNewStudentTab",
                label: "New student",
                target: "navNewStudent",
                active: true,
              },
              {
                id: "navStudentExistTab",
                label: "Returning student",
                target: "navStudentExist",
              },
            ],
            fields: [
              {
                name: "first_name",
                type: "text",
                feedback: "first name",
                label: "First name",
                required: true,
                id: "studentFirstName",
                rowIndex: 0,
              },
              {
                name: "last_name",
                type: "text",
                feedback: "last name",
                label: "Last name",
                required: true,
                id: "studentLastName",
                rowIndex: 0,
              },
              {
                name: "email",
                type: "email",
                feedback: "email",
                label: "Email",
                id: "studentEmail",
                rowIndex: 0,
              },
              {
                name: "phone",
                type: "tel",
                feedback: "phone number",
                label: "Phone number",
                required: true,
                id: "studentPhone",
                rowIndex: 0,
              },
              {
                name: "password",
                type: "password",
                feedback: "password",
                label: "Password",
                id: "studentPsw",
                rowIndex: 0,
              },
              {
                name: "student",
                type: "text",
                feedback: "student",
                label: "Student's name, phone or email",
                required: true,
                id: "oldStudent",
                hasDropdown: true,
                callback: (form, exemptIds) =>
                  instance.#filterCourseList(
                    form,
                    available_courses.filter(
                      (course) => !exemptIds.includes(course.id)
                    )
                  ),
                rowIndex: 1,
              },
            ],
            clearFn: () => {
              $("<div>")
                .addClass(
                  "col-11 rounded border d-flex justify-content-center align-items-center"
                )
                .css("height", "12rem")
                .append(
                  $("<h6>")
                    .addClass("text-muted font-weight-normal")
                    .text("No course added")
                )
                .replaceAll($("#courseCart *"));
              cartContent.splice(0);
              instance.#filterCourseList(form, available_courses);
              $("#submitFormBtn").text("Register student");
            },
          },
          {
            heading: "Payment mode",
            note: true,
            fields: [
              {
                name: "payment_method",
                type: "select",
                feedback: "payment mode",
                label: "Payment mode",
                required: true,
                id: "paymentModes",
                options: payment_modes,
              },
            ],
          },
          {
            heading: "Course information",
            note: true,
            multiple: true,
            fields: [
              {
                name: "course",
                type: "select",
                feedback: "course name",
                label: "Course",
                required: true,
                id: "courseList",
                options: available_courses,
              },
              {
                name: "branch",
                type: "select",
                feedback: "center",
                label: "Learning center",
                id: "learningCenters",
                options: centers,
              },
              {
                name: "tutor_id",
                type: "select",
                feedback: "assigned tutor",
                label: "Assigned tutor",
                id: "assignedTutor",
                options: tutors,
              },
              {
                name: "enroll_type",
                type: "radio",
                label: "Class type",
                className: "col-6 col-lg-3",
                feedback: "price category",
                required: true,
                radios: [
                  {
                    id: "immersiveClass",
                    value: "immersive",
                    label: "Immersive class",
                    checked: true,
                  },
                  {
                    id: "executiveClass",
                    value: "executive",
                    label: "Executive class",
                  },
                  {
                    id: "onlineClass",
                    value: "online",
                    label: "Online class",
                  },
                ],
              },
              {
                name: "payment_type",
                type: "radio",
                label: "Payment type",
                className: "col-6 col-lg-3",
                feedback: "payment type",
                required: true,
                radios: [
                  {
                    id: "fullPayment",
                    value: "full",
                    label: "Full payment",
                    checked: true,
                  },
                  {
                    id: "byInstallment",
                    value: "installment",
                    label: "By installments",
                  },
                ],
                toggled_id: "installmentAmount",
              },
              {
                name: "discount",
                type: "number",
                feedback: "discount",
                label: "Discount",
                value: 0,
                id: "percentDiscount",
                units: ["%", "₦"],
              },
              {
                name: "amount_due",
                type: "text",
                feedback: "amount",
                label: "Amount due (₦)",
                value: "₦0",
                readonly: true,
                replaced: true,
                required: true,
                id: "amountDue",
              },
              {
                name: "amount",
                type: "number",
                feedback: "amount",
                label: "Deposit (₦)",
                required: true,
                disabled: true,
                className: "col-md-6 d-none",
                id: "installmentAmount",
              },
            ],
          },
        ];
        const form = instance.#buildForm(formFields),
          studentForm = $(form)
            .find("#nav-tabContent")
            .add($(form).children("div.row").first()),
          courseForm = $(".multiselect", form);
        instance.#addValidation(studentForm, available_courses, cartContent);
        instance.#addValidation(
          courseForm,
          available_courses,
          cartContent,
          true
        );
      }
    );
  }

  #addValidation(
    form,
    courses,
    cart,
    isMultiple = false,
    form_type = "students",
    invoices = null
  ) {
    const dialog = this;
    const submitBtn = isMultiple ? $("#addToCart") : $("#submitFormBtn");
    $(submitBtn).click(function () {
      const inputs = $("input, select", form).not(
        '[type="radio"], .disabled, #units'
      );
      const params = isMultiple ? {} : { form_type },
        elemCount = inputs.length;
      let allValid = true;
      $(".invalid-feedback", form).removeClass("d-block");
      $('input[type="radio"]', form).next("label").removeClass("text-danger");

      for (let i = 0; i < elemCount; i++) {
        const input = inputs[i];

        switch (input.type) {
          case "number":
          case "text":
          case "select-one":
            if (
              !input.value.trim().match(/\b\w+\b/) ||
              ($(input).attr("id") == "oldStudent" && !$(input).data("student"))
            ) {
              if (!$(input).is(":required")) continue;

              if (allValid) allValid = false;
              $(input).next(".invalid-feedback").addClass("d-block");
            }
            break;
          case "email":
            if (!input.validity.valid && $(input).is(":required")) {
              if (allValid) allValid = false;
              $(input).next(".invalid-feedback").addClass("d-block");
            }
            break;
          case "phone":
            if (
              !input.value.trim().match(/^\+?(\d{11}|\d{13})$/) &&
              $(input).is(":required")
            ) {
              if (allValid) allValid = false;
              $(input).next(".invalid-feedback").addClass("d-block");
            }
            break;
        }

        if (allValid) {
          if ($(input).attr("id") == "oldStudent") {
            const student = $(input).data("student");
            params.student_id = student.id;
            params.first_name = student.first_name;
            params.last_name = student.last_name;
            params.email = student.email;
            params.phone = student.phone;
          } else {
            params[input.name] = input.value;

            if ($(input).attr("id") == "courseList") {
              params.course_id = $($(input).prop("selectedOptions")).data(
                "rel_id"
              );
            }
          }
        }
      }

      if (!allValid) return;
      const checkedRadio = $("input[type='radio']", form);

      if (checkedRadio.lenth && $(checkedRadio).filter(":checked").length < 2) {
        $('input[type="radio"]', form)
          .not(":checked")
          .next("label")
          .addClass("text-danger");
        return false;
      }

      if ($(this).attr("id") == "addToCart") {
        if (
          params.amount &&
          parseInt(params.amount) >= parseInt(params.amount_due)
        ) {
          return toast(
            "First installment should be less than the course fee",
            "black"
          );
        }
        return dialog.#addToCart(form, cart, courses, params);
      }

      if (invoices) dialog.#getPayments(cart, invoices, params);

      if (!cart.length)
        return toast(
          `No ${form_type == "student" ? "course" : "payment"} has been added!`,
          "black"
        );
      form_type == "students"
        ? (params.courses = cart)
        : (params.payments = cart);
      params.new_student = $(this).prop("newStudent");
      const spinner = $("<i>")
        .addClass("fa fa-spinner fa-spin ml-1")
        .appendTo($(this).addClass("disabled"));

      $.post(
        `${site.process}api?option=register_student`,
        params,
        function (jsonRes) {
          console.log("jsonRes", jsonRes);
          const res = JSON.parse(jsonRes);
          $(spinner).remove();
          $(submitBtn).removeClass("disabled");

          if (res.status) {
            toast(
              `${
                form_type == "students" ? "Registration" : "Payment"
              } successful`,
              "green"
            );
            setTimeout(() => {
              const trx_no = Array.isArray(res.trx_no)
                ? res.trx_no
                : [res.trx_no];
              $("#generalDialog").modal("hide");
              downloadPaymentReceipt(trx_no);
              loadPageContent(null, form_type);
            }, 1000);
          } else {
            toast(res.message, "red");
          }
        }
      );
    });

    $("select#courseList, input[type=radio], input#percentDiscount", form).on(
      "input",
      function () {
        switch ($(this).attr("id")) {
          case "immersiveClass":
          case "executiveClass":
          case "onlineClass":
          case "courseList":
          case "percentDiscount":
          case "units":
            if (
              !$("input:checked", form)[0] ||
              !$("#courseList").prop("selectedIndex")
            )
              return;
            const index = $("#courseList").prop("selectedIndex") - 1;
            const course = courses[index];
            let price = $("#immersiveClass").is(":checked")
                ? course.group_price
                : $("#executiveClass").is(":checked")
                ? course.executive_price
                : course.online_price,
              discount = $("#percentDiscount").val();
            price = parseFloat(price);

            if (Number.isNaN(discount)) {
              discount = 0;
              $("#percentDiscount").val("0");
            }

            if ($("#units").val() == "%") {
              discount = Math.max(0, Math.min(discount, 100));
              price -= (price * discount) / 100;
            } else {
              discount = Math.max(0, Math.min(discount, price));
              price -= discount;
            }
            $("#amountDue")
              .val(price)
              .prev("input")
              .val(
                new Intl.NumberFormat("en-NG", {
                  currency: "NGN",
                  style: "currency",
                }).format(price)
              );
            break;
          case "fullPayment":
          case "byInstallment":
            const mode = $(this).val(),
              toggleId = $(this).prop("toggled_id");
            const inputElem = $(`#${toggleId}`);

            if (mode == "installment") {
              $(inputElem)
                .removeClass("disabled")
                .closest(".col-md-6")
                .removeClass("d-none");
            } else if (!$(inputElem).hasClass("disabled")) {
              $(inputElem)
                .val("")
                .addClass("disabled")
                .closest(".col-md-6")
                .addClass("d-none");
            }
            break;
        }
      }
    );
    $("input#percentDiscount", form)
      .attr("min", "0")
      .blur(function () {
        const discount = $(this).val();

        if (discount == "") $(this).val("0");
        const max = $("#units").val() != "%" ? Infinity : 100;
        $(this).val(Math.max(0, Math.min(discount, max)));
      });
  }

  #getPayments(newPayments, invoices, params) {
    const payments = [];
    newPayments.splice(0);

    for (const { ref, balance, course } of invoices) {
      if (params.hasOwnProperty(ref)) {
        const amt = parseInt(params[ref]);
        delete params[ref];

        if (amt > parseInt(balance)) {
          return toast("Amount is more than the balance to be paid", "black");
        }

        if (amt) {
          payments.push({ invoice_ref: ref, amount: amt, course });
        }
      }
    }

    if (payments.length) newPayments.push(...payments);
  }

  #addToCart(form, cart, courses, params = null) {
    const cartContainer = $("#courseCart"),
      addBtn = $("#addToCart"),
      exemptCourses = $("#navStudentExist.active", form)[0]
        ? $("#oldStudent").data("student").exempt_ids
        : [];

    if (params) {
      params.enroll_type = $("input[name='enroll_type']:checked", form).val();
      params.payment_type = $("input[name='payment_type']:checked", form).val();

      if ($(addBtn).prop("editIndex") != null) {
        const index = $(addBtn).prop("editIndex");
        cart[index] = params;
        $(addBtn).removeProp("editIndex");
      } else {
        cart.push(params);
      }
    } else if (!cart.length) {
      $("<div>")
        .addClass(
          "col-11 rounded border d-flex justify-content-center align-items-center"
        )
        .css("height", "12rem")
        .append(
          $("<h6>")
            .addClass("text-muted font-weight-normal")
            .text("No course added")
        )
        .replaceAll($(cartContainer).children());
      $("#submitFormBtn").text("Register student");
      return this.#filterCourseList(
        form,
        courses.filter((course) => !exemptCourses.includes(course.id))
      );
    }
    this.#filterCourseList(
      form,
      courses.filter(
        (course) =>
          !exemptCourses.includes(course.id) &&
          !cart.some((item) => item.course_id == course.id)
      )
    );

    const table = $("<table>")
      .addClass("table table-striped table-hover")
      .replaceAll($(cartContainer).children())
      .append($("<thead>").append($("<tr>").append($("<th>"))))
      .append($("<tbody>"))
      .append($("<tfoot>"));
    const keyMap = new Map([
      ["course", "Course"],
      ["enroll_type", "Class"],
      ["payment_type", "Type"],
      ["discount", "Discount"],
      ["amount_due", "Fee"],
      ["amount", "Deposit"],
    ]);

    for (const col of keyMap.values()) {
      $("thead tr th:last-child", table).before($("<th>").text(col));
    }

    for (const item of cart) {
      const tr = $("<tr>").appendTo($("tbody", table));
      for (const key of keyMap.keys()) {
        const val =
          (/^amount/.test(key) && item[key]) ||
          (key == "discount" && $("#units").val() != "%")
            ? new Intl.NumberFormat("en-NG", {
                currency: "NGN",
                style: "currency",
              }).format(item[key])
            : key == "discount" && $("#units").val() == "%"
            ? item[key] + "%"
            : item[key];
        $(tr).append(
          $("<td>")
            .addClass("text-capitalize text-wrap")
            .text(val ? val : "—")
        );
      }
      $("<td>")
        .addClass("d-flex justify-content-between align-items-center")
        .appendTo(tr)
        .append(
          $("<a>")
            .addClass("close mr-3")
            .css("font-size", "smaller")
            .append($("<i>").addClass("fas fa-pen"))
            .click(() => {
              const index = cart.lastIndexOf(item),
                keys = [...keyMap.keys(), "branch", "tutor_id"];
              $(addBtn).prop("editIndex", index);
              this.#filterCourseList(
                form,
                courses.filter(
                  (course) =>
                    item.course_id == course.id ||
                    (!exemptCourses.includes(course.id) &&
                      !cart.some((it) => it.course_id == course.id))
                )
              );

              for (const key of keys) {
                const val = item[key],
                  elem = $(`[name='${key}']`, form);

                if ($(elem)[0].type == "radio") {
                  $(elem)
                    .filter((_, radio) => $(radio).val() == val)
                    .prop("checked", true);
                } else {
                  $(elem).val(val);
                }
              }
              $("#amountDue", form)
                .prev("input")
                .val(
                  new Intl.NumberFormat("en-NG", {
                    currency: "NGN",
                    style: "currency",
                  }).format(item.amount_due)
                );

              if ($("#installmentAmount").val() != "") {
                $("#installmentAmount")
                  .removeClass("disabled")
                  .closest(".col-md-6")
                  .removeClass("d-none");
              }
              $("#units").val(item.discount > 100 ? "₦" : "%");
            })
        )
        .append(
          $("<button>")
            .addClass("close")
            .attr({ type: "button", "aria-label": "Close" })
            .append($("<span>").attr({ "aria-hidden": "true" }).text("×"))
            .click(() => {
              const index = cart.lastIndexOf(item);

              if ($(addBtn).prop("editIndex") == index) {
                $('input:not([type="radio"]), select', form).val("");
                $("#percentDiscount", form).val("0");
                $("#immersiveClass, #fullPayment", form).prop("checked", true);
                $(addBtn).removeProp("editIndex");
              }
              cart.splice(index, 1);
              this.#addToCart(form, cart, courses);
            })
        );
    }
    $('input:not([type="radio"]), select', form).val("");
    $("#units").val("%");
    $("#percentDiscount", form).val("0");
    if (!$("#installmentAmount").hasClass("disabled")) {
      $("#installmentAmount")
        .addClass("disabled")
        .closest(".col-md-6")
        .addClass("d-none");
    }
    $("#immersiveClass, #fullPayment", form).prop("checked", true);
    const price = new Intl.NumberFormat("en-NG", {
      currency: "NGN",
      style: "currency",
    }).format(
      cart.reduce(
        (accum, item) => accum + ~~(item.amount || item.amount_due),
        0
      )
    );
    $("<tr>")
      .addClass("border-secondary")
      .css("border-bottom", "2px solid")
      .appendTo($("tfoot", table))
      .append($("<th>").attr("scope", "row").text("Total:"))
      .append(
        $("<th>")
          .addClass("text-end")
          .attr({ scope: "row", colspan: 7 })
          .text(price)
      )[0]
      .scrollIntoView({
        behavior: "smooth",
        block: "center",
      });
    $("#submitFormBtn").text(`Register student (${price})`);
  }

  #filterCourseList(form, courses) {
    const selectElem = $("#courseList", form);
    $("option:nth-child(n+2)", selectElem).remove();
    console.log("filter", courses);

    for (const course of courses) {
      $("<option>")
        .text(course.name)
        .val(course.name)
        .data("rel_id", course.id)
        .appendTo(selectElem);
    }
  }

  #buildForm(
    formFields,
    endnote = null,
    submitLabel = "student",
    prefix = "Register"
  ) {
    const form = $("<form>")
      .addClass("needs-validation")
      .appendTo($("#generalDialog .modal-body").empty());

    for (const section of formFields) {
      $(form).append(
        $("<h4>").addClass("mb-3 mt-0 text-capitalize").text(section.heading)
      );

      if (section.tabs) {
        const nav = $("<nav>")
            .addClass("nav nav-tabs flex-nowrap d-flex px-0 pt-0 pb-4")
            .attr({ id: "nav-tab", role: "tablist" })
            .appendTo(form)
            .after(
              $("<div>").attr("id", "nav-tabContent").addClass("tab-content")
            ),
          tabLen = section.tabs.length;
        for (let i = 0; i < tabLen; i++) {
          const tab = section.tabs[i];
          $(nav)
            .append(
              $("<a>")
                .addClass(
                  `nav-item nav-link custom-tabs btn btn-sm${
                    tab.active ? " active" : ""
                  }`
                )
                .attr({
                  id: tab.id,
                  "data-toggle": "tab",
                  href: `#${tab.target}`,
                  role: "tab",
                  "aria-controls": `${tab.target}`,
                  "aria-selected": `${tab.target == true}`,
                })
                .text(tab.label)
                .click(function () {
                  if ($(this).hasClass("active")) return;
                  $(`.tab-pane:not(#${tab.target}) input`, form).addClass(
                    "disabled"
                  );
                  $(`#${tab.target} input`, form).removeClass("disabled");
                  $("#submitFormBtn").prop(
                    "newStudent",
                    tab.target == "navNewStudent"
                  );
                  let collapsible;

                  if (
                    (collapsible = $(".multi-collapse.show", form).has(
                      "button.close"
                    )[0])
                  ) {
                    $("button.close", collapsible).click();
                  } else {
                    section.clearFn?.call();
                  }
                })
            )
            .next("div")
            .append(
              $("<div>")
                .attr("id", tab.target)
                .addClass(`tab-pane fade${tab.active ? " show active" : ""}`)
                .append(
                  $("<div>").addClass("row").attr("id", `${tab.target}_row`)
                )
            );
        }
      }
      const row = section.tabs
        ? $("#nav-tabContent [id$=_row]")
        : $("<div>")
            .addClass(`row${section.multiple ? " multiselect" : ""}`)
            .appendTo(form);
      $(row).append(
        section.note
          ? $("<div>")
              .addClass("col-12 mb-2")
              .append(
                $("<span>")
                  .addClass("font-weight-bold text-muted")
                  .css("font-size", "12px")
                  .html(
                    'Fields marked with <sup class="text-primary">*</sup> are required.'
                  )
              )
          : ""
      );
      const formData = section.formData ?? {};

      for (const field of section.fields) {
        const label = field.title || field.label;
        const col = $("<div>")
          .addClass(`${field.className ?? "col-md-6"} mb-3`)
          .appendTo(section.tabs ? $(row)[field.rowIndex] : row)
          .append(
            label
              ? $("<label>")
                  .attr("for", field.id)
                  .html(
                    `${label}${
                      field.required
                        ? '<sup class="text-primary ml-1">*</sup>'
                        : ""
                    }`
                  )
              : ""
          );
        switch (field.type) {
          case "select":
            const options = field.options,
              isArr = Array.isArray(options);
            const selectElem = $("<select>")
              .addClass("form-control")
              .attr({
                id: field.id,
                placeholder: `Enter ${field.feedback}`,
                name: field.name,
              })
              .prop("required", field.required)
              .css({ "font-size": "small", "line-height": "1.5" })
              .appendTo(col)
              .after(
                $("<div>")
                  .addClass("invalid-feedback")
                  .text(`Select a valid ${field.feedback}.`)
              );
            $("<option>")
              .text(`Select ${field.label.toLowerCase()}`)
              .val("")
              .prop({ disabled: true, selected: true })
              .appendTo(selectElem);

            for (const key in options) {
              const val = options[key],
                isObj = typeof val == "object";
              $("<option>")
                .text(isObj ? val.name : val)
                .val(isArr ? (isObj ? val.name : val.toLowerCase()) : key)
                .data("rel_id", isObj ? val.id : null)
                .appendTo(selectElem);
            }
            break;

          case "radio":
            const radios = field.radios;
            $(col).addClass("d-block");

            for (const radio of radios) {
              $("<div>")
                .addClass("custom-control custom-radio")
                .appendTo(col)
                .append(
                  $("<input>")
                    .addClass("custom-control-input")
                    .attr({
                      name: field.name,
                      type: "radio",
                      id: radio.id,
                      value: radio.value,
                    })
                    .prop({
                      required: !!field.required,
                      checked: !!radio.checked,
                      toggled_id: field.toggled_id,
                    })
                )
                .append(
                  $("<label>")
                    .addClass("custom-control-label")
                    .attr("for", radio.id)
                    .text(radio.label)
                )
                .after(
                  $("<div>")
                    .addClass("invalid-feedback")
                    .text(`Select a valid ${field.feedback}.`)
                );
            }
            break;

          default:
            $(col)
              .append(
                field.replaced &&
                  $("<input>")
                    .addClass("form-control disabled")
                    .attr("placeholder", field.value)
                    .css({ "font-size": "small", "line-height": "1.5" })
              )
              .append(
                $("<input>")
                  .addClass(`form-control${field.disabled ? " disabled" : ""}`)
                  .attr({
                    type: field.replaced ? "hidden" : field.type,
                    id: field.id,
                    placeholder: `Enter ${field.feedback}`,
                    name: field.name,
                  })
                  .prop({
                    required: !!field.required,
                    readonly: !!field.readonly,
                  })
                  .css({ "font-size": "small", "line-height": "1.5" })
                  .val(
                    field.value != undefined
                      ? field.value
                      : formData[field.name]
                  )
              )
              .append(
                $("<div>")
                  .addClass("invalid-feedback")
                  .text(`Enter a valid ${field.feedback}.`)
              );

            if (field.type == "password") {
              $(col).append(
                $("<button>")
                  .addClass("btn bg-white posAbs text-dark p-0 m-0")
                  .css({
                    "box-shadow": "none",
                    right: "2rem",
                    bottom: "0.6rem",
                  })
                  .attr("type", "button")
                  .append($("<i>").addClass("far fa-eye-slash"))
                  .click(function () {
                    const input = $(
                      "input[type=password], input[type=text]",
                      col
                    );

                    if ($(input).attr("type") == "password") {
                      $(input).attr("type", "text");
                      $("i", this)
                        .removeClass("fa-eye-slash")
                        .addClass("fa-eye");
                    } else {
                      $(input).attr("type", "password");
                      $("i", this)
                        .removeClass("fa-eye")
                        .addClass("fa-eye-slash");
                    }
                  })
              );
            }
            break;
        }

        if (field.hasDropdown) {
          $(col)
            .toggleClass("col-md-6 col-md-10")
            .wrapInner('<div class="dropdown w-100 form-inline"></div>')
            .children(".dropdown")
            .append($("<ul>").addClass("dropdown-menu"))
            .find("input")
            .addClass("w-100 ml-3")
            .on({
              keyup: function ({ key }) {
                const str = $(this).val().trim(),
                  input = this,
                  dropdownMenu = $(this)
                    .closest(".dropdown")
                    .find(".dropdown-menu");

                if (
                  key == "ArrowDown" &&
                  $(dropdownMenu).children().length > 0
                ) {
                  $(input).blur();
                  $(dropdownMenu).children().first().focus();
                  return;
                }

                if (!str) return;

                if ($(this).data("student")) $(this).removeData("stduent");
                $.post(
                  `${site.process}api?option=find_student&keyword=${str}${
                    submitLabel == "student" ? "&filter=course" : ""
                  }`,
                  function (jsonRes) {
                    console.log("student", jsonRes);
                    const students = JSON.parse(jsonRes);
                    $(dropdownMenu).empty();

                    for (const student of students) {
                      $("<a>")
                        .addClass("dropdown-item")
                        .attr({
                          "data-toggle": "tooltip",
                          "data-html": "true",
                          title: `<div class="text-left" style="width:fit-content;">Name: <b>${
                            student.first_name
                          } ${student.last_name}</b><br>Email: <b>${
                            student.email ? student.email : "None"
                          }</b><br>Phone: <b>${
                            student.phone ? student.phone : "None"
                          }</b></div>`,
                        })
                        .text(student.matched_col)
                        .data("student", student)
                        .appendTo(dropdownMenu)
                        .on("click keypress", function ({ key }) {
                          if (key && key != "Enter") {
                            if (key == "ArrowUp") {
                              if ($(this).prev()[0]) $(this).prev().focus();
                              else $(this).parent().children().last().focus();
                            } else if (key == "ArrowDown") {
                              if ($(this).next()[0]) $(this).next().focus();
                              else $(this).parent().children().first().focus();
                            }
                            return;
                          }
                          const student = $(this).data("student"),
                            value = student.first_name.concat(
                              " ",
                              student.last_name
                            );
                          $(input).val(value).data("student", student);
                          const pillBox = $(input)
                            .closest(".multi-collapse")
                            .next()
                            .find(".badge")[0];
                          $("img", pillBox).attr(
                            "src",
                            student.picture_ref ??
                              `${site.domain}asset/picture/photo_silhouette.png`
                          );
                          pillBox.childNodes[1].textContent = value;
                          $(".multi-collapse").collapse("toggle");
                          student.exempt_ids
                            ? field.callback?.call(
                                null,
                                form,
                                student.exempt_ids
                              )
                            : field.callback?.call(null, student.id);
                        });
                    }

                    if (!students.length) {
                      $("<a>")
                        .addClass("dropdown-item disabled")
                        .text("No students match the input")
                        .appendTo(dropdownMenu);
                    } else {
                      $('[data-toggle="tooltip"]').tooltip();
                    }
                  }
                );
              },
              focusout: function () {
                if (!$(this).data("student") && $(this).val() != "")
                  $(this).val("");
              },
            })
            .each((_, input) => {
              if (section.formData) {
                const student = section.formData,
                  value = student.first_name.concat(" ", student.last_name);
                $(input).data("student", student).val(value);
                setTimeout(() => {
                  const pillBox = $(input)
                    .closest(".multi-collapse")
                    .next()
                    .find(".badge")[0];
                  $("img", pillBox).attr(
                    "src",
                    student.picture_ref ??
                      `${site.domain}asset/picture/photo_silhouette.png`
                  );
                  pillBox.childNodes[1].textContent = value;
                  $(".multi-collapse").collapse("toggle");
                  section.onData();
                }, 200);
              }
            })
            .wrap(
              $("<div>")
                .addClass("multi-collapse flex-grow-1 collapse show")
                .attr("data-toggle", "dropdown")
            )
            .parent()
            .after(
              $("<div>")
                .addClass("multi-collapse collapse")
                .append(
                  $("<div>")
                    .addClass(
                      "badge bg-white border border-secondary text-dark d-flex align-items-center pr-3 py-2 ml-3"
                    )
                    .css({
                      "border-radius": "50rem",
                      "font-weight": "500",
                      "font-size": "14px",
                      "text-transform": "none",
                    })
                    .append(
                      $("<img>")
                        .addClass("rounded-circle mr-2")
                        .attr({
                          src: `${site.domain}asset/picture/photo_silhouette.png`,
                        })
                        .css({ width: "1.5em" })
                    )
                    .append(document.createTextNode(" "))
                    .append(
                      $("<button>")
                        .addClass("close ml-3")
                        .attr({
                          type: "button",
                          "data-toggle": "collapse",
                          "data-target": ".multi-collapse",
                        })
                        .click(function () {
                          $("#oldStudent").removeData("student").val("");
                          $(this)
                            .parent()
                            .find("img")
                            .attr(
                              "src",
                              `${site.domain}asset/picture/photo_silhouette.png`
                            );
                          section.clearFn?.call();
                        })
                    )
                )
                .on("hidden.bs.collapse", function () {
                  $(".badge", this)[0].childNodes[1].textContent = " ";
                  $("#oldStudent").focus();
                })
            );
        } else if (field.units) {
          console.log("col", $("input, select, .invalid-feedback", col).length);
          $("input, .invalid-feedback", col)
            .wrapAll($("<div>").addClass("col pr-0"))
            .filter("input")
            .toggleClass("form-control w-100 rounded-pill")
            .css({
              padding: ".5rem .75rem",
              border: "1px solid #E3E3E3",
              outline: "none",
            })
            .parent()
            .wrap($("<div>").addClass("input-group"))
            .before(
              $("<div>")
                .addClass("input-group-append")
                .append(
                  $("<select>")
                    .addClass("custom-select rounded-pill")
                    .attr("id", "units")
                )
            );
          const select = $(".input-group-append select", col);

          for (const [i, unit] of Object.entries(field.units)) {
            $("<option>")
              .appendTo(select)
              .prop("selected", i == 0)
              .val(unit)
              .text(unit);
          }
        }
      }

      if (section.multiple) {
        $(form)
          .append(
            $("<div>")
              .addClass("d-flex justify-content-end")
              .append(
                $("<button>")
                  .attr({ id: "addToCart", type: "button" })
                  .addClass(
                    "btn btn-warning btn-sm d-flex align-items-center px-4 py-2 text-dark"
                  )
                  .append(
                    submitLabel == "payment"
                      ? $("<i>").addClass("fas fa-money-bill-wave mr-2")
                      : `<svg width="1.5em" height="1.5em" viewBox="0 0 96 96" xmlns="http://www.w3.org/2000/svg">
                  <defs>
                    <clipPath id="clip-cart">
                      <rect width="96" height="96"/>
                    </clipPath>
                  </defs>
                  <g id="cart" clip-path="url(#clip-cart)">
                    <g id="pills" transform="translate(0 -116)">
                    <path d="M92,132H84.619a8.361,8.361,0,0,0-7.956,5.47L63.712,174.53A8.364,8.364,0,0,1,55.755,180H21.321a8.4,8.4,0,0,1-7.773-4.994l-8.925-21C2.387,148.746,6.445,143,12.4,143H57" fill="none" stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="4"/>
                    <circle id="Ellipse_335" data-name="Ellipse 335" cx="4.5" cy="4.5" r="4.5" transform="translate(20 187)" fill="none" stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="4"/>
                    <circle id="Ellipse_336" data-name="Ellipse 336" cx="4.5" cy="4.5" r="4.5" transform="translate(49 187)" fill="none" stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="4"/>
                    </g>
                  </g>
                </svg>`
                  )
                  .append($("<span>").text("Add"))
              )
          )
          .append(
            $("<div>")
              .addClass("container mt-5")
              .append(
                $("<h5>")
                  .addClass("text-center")
                  .css({
                    "font-weight": "500",
                    "font-family": "'Roboto Condensed', sans-serif",
                  })
                  .text(
                    submitLabel == "payment"
                      ? "Payment(s) Summary"
                      : "Course(s) Summary"
                  )
              )
              .append(
                $("<div>")
                  .attr(
                    "id",
                    submitLabel == "payment" ? "paymentSummary" : "courseCart"
                  )
                  .addClass("d-flex justify-content-center")
                  .css("overflow", "visible")
                  .append(
                    $("<div>")
                      .addClass(
                        "col-11 rounded border d-flex justify-content-center align-items-center"
                      )
                      .css("height", "12rem")
                      .append(
                        $("<h6>")
                          .addClass("text-muted font-weight-normal")
                          .text(
                            submitLabel == "payment"
                              ? "No payment added"
                              : "No course added"
                          )
                      )
                  )
              )
          );
      }
      $(".tab-pane:not(.active) input", form).addClass("disabled");
    }

    if (endnote) {
      $("<p>")
        .addClass("mt-4")
        .appendTo(form)
        .append(
          $("<i>")
            .addClass("fa fa-circle align-middle text-warning mr-2")
            .css("font-size", "xx-small")
        )
        .append(endnote);
    }
    $(form).append(
      $("<button>")
        .addClass("btn btn-primary btn-lg btn-block my-4")
        .text(`${prefix} ${submitLabel}`)
        .attr({ type: "button", id: "submitFormBtn" })
        .prop("newStudent", submitLabel == "student" ? true : null)
    );
    return form;
  }

  #createStudentPaymentsDialog() {
    $("#generalDialog .modal-dialog")
      .addClass("modal-lg m-0")
      .removeClass("modal-sm modal-dialog-centered")
      .find(".modal-content")
      .css({ width: "100vw", "min-width": "100vw", height: "100vh" })
      .find(".modal-header")
      .addClass("border-bottom py-2")
      .find("button")
      .css("top", "20px");

    $("#generalDialogLabel").html(
      `<i class="far fa-credit-card align-middle rounded-circle svg-bg mr-2" style="padding-top:.8rem;padding-left:.6rem;"></i>Payments for courses`
    );
    const paymentContainer = $("<div>")
      .addClass("table-responsive")
      .appendTo($("#generalDialog .modal-body").css("overflow", "hidden auto"));
    $.get(`${site.process}student?option=all_payments`, function (jsonRes) {
      console.log("jsonRes", jsonRes);
      const payments = JSON.parse(jsonRes);
      const headers = [
        "#",
        "Course",
        "Student",
        "Amount",
        "Invoice Ref",
        "Mode of payment",
        "Date of payment",
        "",
      ];
      const tr = $("<tr>"),
        tbody = $("<tbody>").appendTo(
          $("<table>")
            .addClass("table table-bordered table-sm")
            .appendTo(paymentContainer)
            .append($("<thead>").append(tr))
        );
      const keys = [
        "course",
        "student",
        "amount",
        "invoice_ref",
        "payment_method",
        "payment_date",
      ];

      for (const header of headers) $("<th>").appendTo(tr).text(header);
      const paymentLen = payments.length;

      for (let i = 0; i < paymentLen; i++) {
        const payment = payments[i],
          itemNo = i + 1;
        const tr = $("<tr>")
          .appendTo(tbody)
          .append($("<td>").text(`${itemNo}.`));

        for (const key of keys) {
          const val =
            key == "amount"
              ? new Intl.NumberFormat("en-NG", {
                  currency: "NGN",
                  style: "currency",
                }).format(payment.amount)
              : payment[key];
          $("<td>").appendTo(tr).text(val);
        }
        $("<td>")
          .appendTo(tr)
          .append(
            $("<a>")
              .addClass("btn btn-sm")
              .attr({ href: "javascript:void(0);" })
              .css({ "background-color": "#d4edda", color: "#155724" })
              .prop({
                pay_id: payment.id,
                course: payment.course,
              })
              .text("Download receipt")
              .click(downloadReceipt)
          );
      }
    });
  }

  #createPaymentFormDialog(invoice_ref = null) {
    $("#generalDialog .modal-dialog")
      .removeClass("modal-sm")
      .addClass("modal-lg");
    $("#generalDialogLabel").html("New payment").addClass("font-weight-bold");
    $("#generalDialog .modal-body").append(
      $("<div>")
        .addClass("d-flex justify-content-center align-items-center")
        .css("height", "70vh")
        .append(
          $("<div>")
            .addClass("spinner-border")
            .css({ width: "3rem", height: "3rem" })
            .attr("role", "status")
        )
    );
    const instance = this;
    $.get(
      `${site.process}api`,
      { option: "pay_form_data", invoice_ref },
      function (jsonRes) {
        const {
          student_info,
          invoice_list = [],
          payment_modes,
        } = JSON.parse(jsonRes);
        const paymentList = [];
        const formFields = [
          {
            heading: "Student information",
            note: true,
            fields: [
              {
                name: "student",
                type: "text",
                feedback: "student",
                label: "Student's name, email or phone",
                required: true,
                id: "oldStudent",
                hasDropdown: true,
                callback: (studentId) =>
                  instance.#listInvoice(studentId, invoice_list, payment_modes),
              },
            ],
            formData: student_info,
            onData: () =>
              instance.#listInvoice(null, invoice_list, payment_modes),
            clearFn: () => {
              $("#invoiceContainer").empty();
              invoice_list.splice(0);
              $("#submitFormBtn").text("Register payment");
            },
          },
        ];
        const endnote = $("<b>")
          .addClass("text-muted")
          .css("font-size", "12px")
          .text(
            "This form is for additional payments. For first installment or full payment for a course, go to the students' page"
          );
        const form = instance.#buildForm(formFields, endnote, "payment");
        instance.#addValidation(
          form,
          null,
          paymentList,
          false,
          "payment",
          invoice_list
        );
      }
    );
  }

  #listInvoice(student_id, invoiceList, paymentModes) {
    const row = $("#oldStudent").closest(".row"),
      invoiceContainer = $(row).next("div#invoiceContainer")[0]
        ? $(row).next("div#invoiceContainer").empty()
        : $("<div>")
            .addClass("table-responsive")
            .attr("id", "invoiceContainer")
            .css("overflow", "hidden")
            .insertAfter(row);
    $(invoiceContainer)
      .append($("<h5>").text("Invoices"))
      .append(
        $("<div>")
          .addClass("mx-auto mt-3 spinner")
          .css({ width: "fit-content" })
          .append(
            $("<div>")
              .addClass("spinner-border")
              .attr({ role: "status" })
              .css({ width: "3rem", height: "3rem" })
          )
      );
    const callback = () => {
      $(".spinner", invoiceContainer).remove();
      const invoiceTable = $("<table>")
        .addClass("table table-bordered")
        .appendTo(invoiceContainer)
        .append($("<thead>").append($("<tr>")))
        .append($("<tbody>"));
      const headings = [
          "Course",
          "Invoice ref",
          "Amount due",
          "Balance",
          "Deposit",
        ],
        keys = ["course", "ref", "amount_due", "balance"];

      for (const col of headings) {
        $("<th>").text(col).appendTo($("thead tr", invoiceTable));
      }

      for (const invoice of invoiceList) {
        const tr = $("<tr>").appendTo($("tbody", invoiceTable));

        for (const key of keys) {
          const val = /(?:amount_due|balance)/.test(key)
            ? new Intl.NumberFormat("en-NG", {
                currency: "NGN",
                style: "currency",
              }).format(invoice[key])
            : invoice[key];
          $("<td>").text(val).appendTo(tr);
        }
        $("<td>")
          .appendTo(tr)
          .append(
            $("<div>")
              .addClass("input-group mb-3")
              .append(
                $("<div>")
                  .addClass("input-group-prepend pl-3")
                  .css({
                    "border-top-left-radius": "50rem",
                    "border-bottom-left-radius": "50rem",
                    border: "1px solid #e9ecef",
                    "border-right": "0",
                  })
                  .append($("<span>").addClass("input-group-text").text("₦"))
              )
              .append(
                $("<input>")
                  .addClass("form-control")
                  .attr({
                    type: "number",
                    placeholder: "Deposit amount",
                    name: invoice.ref,
                    id: invoice.ref,
                    min: 0,
                  })
                  .on("input", function () {
                    const total = Array.prototype.reduce.call(
                      $('input[type="number"]', invoiceTable),
                      (accum, input) => {
                        const amt = $(input).val();

                        if (amt) accum += parseInt(amt);
                        return accum;
                      },
                      0
                    );
                    $("#amtTotal").text(
                      new Intl.NumberFormat("en-NG", {
                        currency: "NGN",
                        style: "currency",
                      }).format(total)
                    );
                  })
              )
          );
      }

      if (!invoiceList.length) {
        $("<div>")
          .addClass("col-11 d-flex justify-content-center align-items-center")
          .css("height", "6rem")
          .appendTo(invoiceTable)
          .append(
            $("<h6>")
              .addClass("text-muted font-weight-normal")
              .text("No pending invoices")
          );
      } else {
        let options = $();

        for (const [key, mode] of Object.entries(paymentModes)) {
          options = $(options).add($("<option>").val(key).text(mode));
        }
        $("<tr>")
          .appendTo($("tbody", invoiceTable))
          .append(
            $("<td>")
              .attr("colspan", headings.length)
              .append(
                $("<div>")
                  .addClass("d-flex flex-column")
                  .append(
                    $("<h5>")
                      .addClass("d-flex justify-content-between")
                      .append($("<span>").text("Total"))
                      .append(
                        $("<span>")
                          .attr("id", "amtTotal")
                          .css("border-bottom", "4px double #000")
                          .text("₦0.00")
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("form-group row")
                      .append(
                        $("<label>")
                          .addClass("col-auto col-form-label")
                          .attr({ for: "paymentModes" })
                          .text("Payment mode")
                      )
                      .append(
                        $("<div>")
                          .addClass("col-sm-8")
                          .append(
                            $("<select>")
                              .addClass("form-control")
                              .attr({
                                id: "paymentModes",
                                name: "payment_method",
                              })
                              .prop("required", true)
                              .append(
                                $("<option>")
                                  .val("")
                                  .text("Select a payment mode")
                              )
                              .append(options)
                          )
                          .append(
                            $("<div>")
                              .addClass("invalid-feedback")
                              .text("Payment mode is required")
                          )
                      )
                  )
              )
          );
      }
    };

    if (student_id) {
      $.get(
        `${site.process}api`,
        { option: "payment_invoice", student_id },
        (jsonRes) => {
          const invoices = JSON.parse(jsonRes);
          invoiceList.splice(0, invoiceList.length, ...invoices);
          callback();
        }
      );
    } else {
      console.log("invoiceList", invoiceList);
      callback();
    }
  }

  #trackPaymentDialog(invoiceRef) {
    $("#generalDialog .modal-dialog")
      .removeClass("modal-sm")
      .addClass("modal-lg")
      .find(".modal-content")
      .css("min-height", "80vh");
    $("#generalDialogLabel").addClass("font-weight-bold").text("Track payment");
    const invoiceContainer = $("<div>")
      .addClass("table-responsive")
      .css("overflow", "unset")
      .appendTo($("#generalDialog .modal-body"))
      .append(
        $("<div>")
          .addClass("mx-auto mt-3 spinner")
          .css({ width: "fit-content" })
          .append(
            $("<div>")
              .addClass("spinner-border")
              .attr({ role: "status" })
              .css({ width: "3rem", height: "3rem" })
          )
      );

    $.get(
      `${site.process}api`,
      {
        option: "payment_invoice",
        invoice_ref: invoiceRef,
        track_payment: true,
      },
      (jsonRes) => {
        console.log("jsonRes", jsonRes);
        $(".spinner", invoiceContainer).remove();
        const invoice = JSON.parse(jsonRes)[0];
        const bigTable = $("<table>")
          .addClass("table text-capitalize accordion-generic")
          .appendTo(invoiceContainer)
          .append(
            $("<thead>")
              .addClass("bg-light")
              .append($("<tr>").append($("<th>")))
          )
          .append(
            $("<tbody>")
              .append(
                $("<tr>").append(
                  $("<td>").append(
                    $("<a>")
                      .addClass(
                        "btn btn-link collapsed px-0 border rounded-circle pl-2"
                      )
                      .attr({
                        "data-toggle": "collapse",
                        "data-target": "#paymentList",
                      })
                      .on("mouseover mouseout", function () {
                        $(this).toggleClass("bg-light");
                      })
                  )
                )
              )
              .append($("<tr>"))
          );
        const invoiceMap = new Map([
            ["student", "Student"],
            ["course", "Course"],
            ["ref", "Invoice ref"],
            ["payment_type", "Payment type"],
            ["amount_due", "Amount due"],
            ["balance", "Debt"],
          ]),
          paymentMap = new Map([
            ["trx_no", "Transaction no"],
            ["amount", "Amount"],
            ["payment_method", "Payment mode"],
            ["confirmed", "Confirmed"],
            ["desk_officer", "Received by"],
            ["payment_date", "Payment date"],
          ]);

        if (invoice.completed == 0) {
          $("<div>")
            .addClass("d-flex justify-content-end mb-3")
            .insertBefore(bigTable)
            .append(
              $("<button>")
                .addClass("btn btn-primary")
                .append($("<i>").addClass("fa fa-plus align-middle mr-2"))
                .append("New payment")
            )
            .click(() => {
              this.#reset();
              this.#createPaymentFormDialog(invoice.ref);
            });
        }

        for (const [key, col] of invoiceMap.entries()) {
          const val = /(?:amount_due|balance)/.test(key)
            ? new Intl.NumberFormat("en-NG", {
                currency: "NGN",
                style: "currency",
              }).format(invoice[key])
            : invoice[key];
          $("<th>").text(col).appendTo($("thead tr", bigTable));
          $("<td>").text(val).appendTo($("tbody tr:first-child", bigTable));
        }
        const subTable = $("<table>")
          .addClass("table table-bordered text-capitalize")
          .appendTo(
            $("<div>")
              .addClass("collapse")
              .attr("id", "paymentList")
              .appendTo(
                $("<td>")
                  .attr("colspan", invoiceMap.size + 1)
                  .appendTo($("tbody tr:last-child", bigTable))
                  .css("height", "fit-content")
              )
              .append(
                $("<h6>")
                  .addClass("text-capitalize mt-2 mb-3")
                  .text("Payments Made:")
              )
          )
          .append($("<thead>").append($("<tr>")))
          .append($("<tbody>"));

        for (const col of paymentMap.values()) {
          $("<th>").text(col).appendTo($("thead tr", subTable));
        }

        for (const payment of invoice.payments) {
          const tr = $("<tr>").appendTo($("tbody", subTable));

          for (const key of paymentMap.keys()) {
            const val =
              key == "amount"
                ? new Intl.NumberFormat("en-NG", {
                    currency: "NGN",
                    style: "currency",
                  }).format(payment[key])
                : payment[key];
            $("<td>").text(val).appendTo(tr);
          }
        }
      }
    );
  }

  #extractMonthYear(dateStr) {
    const date = new Date(dateStr);
    const month = date.toLocaleString("default", { month: "short" });
    const year = date.getFullYear();
    return `${month} ${year}`;
  }

  #calculateDuration(startStr, endStr) {
    const start = new Date(startStr);
    const end = new Date(endStr);
    const diffTime = Math.abs(end - start);
    const diffDays = Math.ceil(diffTime / (1000 * 60 * 60 * 24));
    const diffWeeks = Math.floor(diffDays / 7);
    const diffMonths = Math.floor(diffDays / 30);
    const diffYears = Math.floor(diffDays / 365);

    if (diffYears > 0) return `${diffYears} year(s)`;
    if (diffMonths > 0) return `${diffMonths} month(s)`;
    if (diffWeeks > 0) return `${diffWeeks} week(s)`;
    return `${diffDays} day(s)`;
  }

  #displayTalentProfile(array, type) {
    switch (type) {
      case "education":
        return $("<ul>")
          .addClass("list-unstyled")
          .append(
            ...array.map(function (edu) {
              return $("<li>")
                .append(
                  $("<div>")
                    .addClass("d-flex justify-content-between text-muted")
                    .css({
                      "font-family": "'Spartan', sans-serif",
                    })
                    .append($("<span>").addClass("mr-3").text(edu.degree))
                    .append(
                      $("<span>").css("font-size", "small").text(edu.period)
                    )
                )
                .append($("<p>").text(edu.description));
            })
          );
      case "skills":
        return $("<div>")
          .addClass("d-flex mb-3 align-items-center flex-wrap")
          .css("gap", "0.5rem")
          .append(
            ...array.map(function (skill) {
              return $("<div>")
                .addClass(
                  "badge d-flex align-items-center text-dark border-0 text-capitalize p-2"
                )
                .css({
                  "background-color": "rgb(225, 221, 219)",
                })
                .text(`${skill.name} — ${skill.score}%`);
            })
          );
      case "experience":
        return $("<div>").append(
          ...array.map((exp, i) =>
            $("<div>")
              .addClass(`card-body${i < array.length-1 ? ' border-bottom':''} px-0`)
              .append(
                $("<h5>")
                  .addClass("card-title")
                  .css({
                    "font-weight": "600",
                    "font-size": "100%",
                  })
                  .text(exp.role)
              )
              .append(
                $("<h6>")
                  .addClass(
                    "card-subtitle mb-2 font-weight-normal text-capitalize"
                  )
                  .css({
                    "font-family": "'Spartan', sans-serif",
                  })
                  .text(exp.firm)
              )
              .append(
                $("<p>")
                  .addClass("card-text text-muted mt-0")
                  .each((_, elem) =>
                    elem.style.setProperty("min-height", "unset", "important")
                  )
                  .append($("<span>").text(exp.period))
                  .append(
                    $("<i>")
                      .addClass("fas fa-circle align-middle mx-2")
                      .css({ "font-size": "3px" })
                  )
                  .append($("<span>").text(exp.duration))
              )
              .append(
                $("<p>")
                  .addClass("card-text mb-1")
                  .each((_, elem) =>
                    elem.style.setProperty("min-height", "unset", "important")
                  )
                  .text(exp.description)
              )
          )
        );
      case "projects":
        return $("<div>").append(
          ...array.map((proj) =>
            $("<div>")
              .css({
                "box-shadow": "none",
              })
              .append(
                $("<div>")
                  .addClass("card-body px-0 pt-0")
                  .append(
                    $("<h5>")
                      .addClass("card-title")
                      .css({
                        "font-size": "large",
                        "font-weight": "500",
                      })
                      .text(`Project: ${proj.title}`)
                  )
                  .append(
                    $("<p>")
                      .addClass("card-text")
                      .css({ "min-height": "unset !important" })
                      .text(proj.description)
                  )
                  .append(
                    $('<a>').addClass('btn btn-sm bg-white text-dark rounded-pill border border-dark').attr({
                      href: proj.link,
                      target: "_blank",
                    }).on('mouseover mouseout', function () {
                      $(this).toggleClass('bg-white bg-light');
                    }).css('box-shadow', 'none').text('Open link').append(
                      '<svg width="1em" height="1em" class="ml-1 align-middle" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg" role="img" focusable="false"><path d="M19 19H5V5h7V3H3v18h18v-9h-2v7ZM14 3v2h3.59l-9.83 9.83 1.41 1.41L19 6.41V10h2V3h-7Z" fill="currentColor"></path></svg>'
                    )
                  )
              )
          )
        );
    }
  }

  #createProfileDialog() {
    $("#generalDialog .modal-dialog")
      .addClass("modal-lg m-0")
      .removeClass("modal-sm modal-dialog-centered")
      .find(".modal-content")
      .css({ width: "100vw", "min-width": "100vw", height: "100vh" })
      .find(".modal-header")
      .addClass("border-bottom py-3")
      .find("button")
      .css("top", "20px");
    $("#generalDialogLabel").text("Profile");
    $("<div>")
      .addClass("w-100 h-100 justify-content-center")
      .appendTo($("#generalDialog .modal-body").css("overflow", "hidden auto"))
      .append(
        $("<div>")
          .addClass("mx-auto mt-3")
          .css("width", "fit-content")
          .append(
            $("<div>")
              .addClass("spinner-border")
              .attr("role", "status")
              .css({ width: "3rem", height: "3rem" })
          )
      );
    $.get(`${site.process}custom?case=account_info`, (jsonRes) => {
      console.log("jsonRes", jsonRes);
      const { user, extras } = JSON.parse(jsonRes);
      let extraFields = null;

      if (extras) {
        if (!isJson(user.other)) user.other = "{}";
        const [
          employers,
          techRoles,
          techSkills,
          states,
          countries,
          jobStatus,
          degrees,
          disciplines,
        ] = extras;
        
        const {
          location = null,
          workplace = null,
          other = null,
          title = null,
          status = null,
          open = 1,
          skills = [],
          education = [],
          experience = [],
          projects = [],
          socials = {},
        } = JSON.parse(user.other);

        const listEducation = () => {
          if (!education.length) {
            education.push({
              degree: "",
              period: "",
              description: "",
              new: true,
            });
          }
          return education
            .filter((edu) => !edu.deleted)
            .map(function (edu, i) {
              const index = edu.degree.indexOf(' ');
              const [degree, course] = [edu.degree.substring(0, index), edu.degree.substring(index + 1)];
              const [startYr, endYr] = edu.period.split(" — ");
              const currentYear = new Date().getFullYear();
              return $("<div>")
                .append(
                  $("<button>")
                    .addClass("close float-right posRel")
                    .attr("type", "button")
                    .append($("<span>").text("×"))
                    .css("z-index", 1)
                    .click(function () {
                      const parent = $(this).parent()[0];
                      const index = $(parent).parent().children().index(parent);
                      if (index >= 0) {
                        if (education[index].new) {
                          education.splice(index, 1);
                        } else {
                          const edu = education[index];
                          const degree = $("#degree_" + index).data(
                            "value"
                          ),
                          course = $("#course_" + index).data("value"),
                          startYr = $("#start_year_" + index).data(
                            "value"
                          ),
                          endYr = $("#end_year_" + index).data("value"),
                          desc = $("#edu_description_" + index).data("value");

                        edu.deleted = true;
                        edu.degree = (degree + " " + course).trim();
                        edu.period = (
                          startYr +
                          " — " +
                          endYr
                        ).trim();
                        edu.description = desc;
                        }
                      }
                      $(parent).remove();
                    })
                )
                .append(
                  $("<div>")
                    .addClass("d-flex mb-2")
                    .css("gap", "1rem")
                    .append(
                      $("<div>")
                        .addClass("form-group")
                        .append(
                          $("<label>")
                            .attr({ for: "degree_" + i })
                            .text("Degree")
                        )
                        .append(
                          $("<select>")
                            .addClass("form-control rounded-pill border-dark")
                            .attr({
                              name: "degree",
                              id: "degree_" + i,
                            })
                            .each(function () {
                              $("<option>")
                                .addClass("font-weight-bold")
                                .prop("disabled", true)
                                .val("")
                                .text("Select degree")
                                .appendTo(this);

                              for (const val of degrees) {
                                $("<option>").val(val).text(val).appendTo(this);
                              }
                            })
                            .val(degree)
                            .data("value", degree)
                            .change(function () {
                              if (!$(this).val() || !$(`#course_${i}`).val())
                                return;
                              edu.degree =
                                $(this).val() + " " + $(`#course_${i}`).val();
                            })
                        )
                        .append(
                          $("<div>")
                            .addClass("invalid-feedback")
                            .text("Degree is invalid.")
                        )
                    )
                    .append(
                      $("<div>")
                        .addClass("form-group")
                        .css("flex", "1")
                        .append(
                          $("<label>")
                            .attr({ for: "course_" + i })
                            .text("Course of study")
                        )
                        .append(
                          $("<select>")
                            .addClass("form-control rounded-pill border-dark")
                            .attr({
                              name: "course",
                              id: "course_" + i,
                            })
                            .each(function () {
                              $("<option>")
                                .val("")
                                .text("Select course")
                                .appendTo(this);

                              for (const [key, arr] of Object.entries(
                                disciplines
                              )) {
                                $("<option>")
                                  .addClass("font-weight-bold")
                                  .prop("disabled", true)
                                  .val(key)
                                  .text(key)
                                  .appendTo(this);

                                for (const val of arr) {
                                  $("<option>")
                                    .val(val)
                                    .text(val)
                                    .appendTo(this);
                                }
                              }
                            })
                            .val(course)
                            .data("value", course)
                            .change(function () {
                              if (!$(`#degree_${i}`).val() || !$(this).val())
                                return;
                              edu.degree =
                                $(`#degree_${i}`).val() + " " + $(this).val();
                            })
                        )
                        .append(
                          $("<div>")
                            .addClass("invalid-feedback")
                            .text("Course is invalid.")
                        )
                    )
                )
                .append(
                  $("<div>")
                    .addClass("d-flex align-items-center mb-2")
                    .append(
                      $("<div>")
                        .addClass("form-group")
                        .css("flex", 1)
                        .append(
                          $("<label>")
                            .attr({ for: "start_year_" + i })
                            .text("Start year")
                        )
                        .append(
                          $("<select>")
                            .addClass("form-control rounded-pill border-dark")
                            .attr({
                              name: "start_year",
                              id: "start_year_" + i,
                            })
                            .each(function () {
                              $("<option>")
                                .val("")
                                .text("Select start year")
                                .appendTo(this);

                              for (let yr = 2000; yr <= currentYear; yr++) {
                                $("<option>").val(yr).text(yr).appendTo(this);
                              }
                            })
                            .val(startYr)
                            .data("value", startYr)
                            .change(function () {
                              if (!$(this).val() && !$(`#end_year_${i}`).val())
                                return;
                              edu.period =
                                $(this).val() +
                                " — " +
                                $(`#end_year_${i}`).val();
                            })
                        )
                        .append(
                          $("<div>")
                            .addClass("invalid-feedback")
                            .text("Start year is invalid.")
                        )
                    )
                    .append($("<span>").addClass("mx-2").text("—"))
                    .append(
                      $("<div>")
                        .addClass("form-group")
                        .css("flex", 1)
                        .append(
                          $("<label>")
                            .attr({ for: "end_year_" + i })
                            .text("End year")
                        )
                        .append(
                          $("<select>")
                            .addClass("form-control rounded-pill border-dark")
                            .attr({
                              name: "end_year",
                              id: "end_year_" + i,
                            })
                            .each(function () {
                              $("<option>")
                                .val("")
                                .text("Select end year")
                                .appendTo(this)
                                .after(
                                  $("<option>").val("Present").text("Present")
                                );
                              for (let yr = 2000; yr < currentYear; yr++) {
                                $("<option>")
                                  .val(yr)
                                  .text(yr)
                                  .insertBefore($("option:last-child", this));
                              }
                            })
                            .val(endYr)
                            .data("value", endYr)
                            .change(function () {
                              if (
                                !$(this).val() &&
                                !$(`#start_year_${i}`).val()
                              )
                                return;
                              edu.period =
                                $(`#start_year_${i}`).val() +
                                " — " +
                                $(this).val();
                            })
                        )
                        .append(
                          $("<div>")
                            .addClass("invalid-feedback")
                            .text("End year is invalid.")
                        )
                    )
                )
                .append(
                  $("<div>")
                    .addClass("form-group")
                    .append(
                      $("<label>")
                        .attr({ for: "edu_description_" + i })
                        .text("Description")
                    )
                    .append(
                      $("<textarea>")
                        .addClass("form-control border-dark px-0 pt-0")
                        .attr({
                          rows: 2,
                          maxlength: 250,
                          name: "description",
                          id: "edu_description_" + i,
                        })
                        .val(edu.description)
                        .data("value", edu.description)
                        .on("input", function () {
                          edu.description = $(this).val();
                          $(this)
                            .next()
                            .text(`${$(this).val().length} / 250`);
                        })
                    )
                    .append(
                      $("<p>")
                        .addClass("form-text text-right mb-0")
                        .text(`${edu.description.length} / 250`)
                    )
                    .append(
                      $("<div>")
                        .addClass("invalid-feedback")
                        .text("Description is invalid.")
                    )
                );
            });
        };
        const listSkills = () =>
          skills
            .filter((skill) => !skill.deleted)
            .map(function (skill) {
              return $("<div>")
                .addClass(
                  "badge d-flex align-items-center text-dark border-0 text-capitalize p-2"
                )
                .css({ "background-color": "rgb(225, 221, 219)" })
                .text(`${skill.name} — ${skill.score}%`)
                .append(
                  $("<button>")
                    .addClass("close ml-2")
                    .attr("type", "button")
                    .css({ "font-size": "x-small" })
                    .click(function () {
                      const parent = $(this).parent(".badge")[0];
                      const index = $(parent).parent().children().index(parent);
                      if (index >= 0) {
                        if (skills[index].new) {
                          skills.splice(index, 1);
                        } else {
                          skills[index].deleted = true;
                        }
                      }
                      $(parent).remove();
                    })
                );
            });
        const listExperience = () => {
          const instance = this;
          if (!experience.length) {
            experience.push({
              firm: "",
              role: "",
              duration: "",
              period: "",
              description: "",
              new: true,
            });
          }
          return experience
            .filter((exp) => !exp.deleted)
            .map(function (exp, i) {
              return $("<div>")
                .append(
                  $("<button>")
                    .addClass("close float-right posRel")
                    .attr("type", "button")
                    .css("z-index", 1)
                    .append($("<span>").text("×"))
                    .click(function () {
                      const parent = $(this).parent()[0];
                      const index = $(parent).parent().children().index(parent);
                      if (index >= 0) {
                        if (experience[index].new) {
                          experience.splice(index, 1);
                        } else {
                          const exp = experience[index];
                          exp.deleted = true;
                          exp.firm = $("#firm_" + i).data(
                            "value"
                          );
                          exp.role = $("#role_" + i).data("value");
                          exp.description = $("#exp_description_" + i).data(
                            "value"
                          );
                          exp.period = $("#period_" + i).data(
                            "period"
                          );
                          exp.duration = $("#period_" + i).data(
                            "duration"
                          );
                        }
                      }
                      $(parent).remove();
                    })
                )
                .append(
                  $("<div>")
                    .addClass("form-group col-12")
                    .append(
                      $("<label>")
                        .addClass("col-form-label")
                        .attr("for", "firm_" + i)
                        .text("Firm")
                    )
                    .append(
                      $("<input>")
                        .addClass("form-control border-dark")
                        .attr({
                          list: "employers",
                          id: "firm_" + i,
                          name: "firm",
                          placeholder: "Enter name of firm",
                        })
                        .data("value", exp.firm)
                        .val(exp.firm)
                        .on("input", function () {
                          exp.firm = $(this).val();
                        })
                    )
                    .append(
                      $("<div>")
                        .addClass("invalid-feedback")
                        .text("Firm is invalid.")
                    )
                )
                .append(
                  $("<div>")
                    .addClass("d-flex mb-2")
                    .append(
                      $("<div>")
                        .addClass("form-group col-md")
                        .append(
                          $("<label>")
                            .addClass("col-form-label")
                            .attr({ for: "role_" + i })
                            .text("Job role")
                        )
                        .append(
                          $("<select>")
                            .addClass("form-control rounded-pill border-dark")
                            .attr({
                              name: "role",
                              id: "role_" + i,
                            })
                            .each(function () {
                              $("<option>")
                                .addClass("font-weight-bold")
                                .prop("disabled", true)
                                .val("")
                                .text("Select role")
                                .appendTo(this);

                              for (const val of techRoles) {
                                $("<option>").val(val).text(val).appendTo(this);
                              }
                            })
                            .val(exp.role)
                            .data("value", exp.role)
                            .change(function () {
                              exp.role = $(this).val();
                            })
                        )
                        .append(
                          $("<div>")
                            .addClass("invalid-feedback")
                            .text("Job role is invalid.")
                        )
                    )
                    .append(
                      $("<div>")
                        .addClass("form-group")
                        .append(
                          $("<label>").addClass("col-form-label").text("Period")
                        )
                        .append(
                          $("<button>")
                            .addClass(
                              "btn btn-block text-dark dropdown-toggle rounded-pill bg-light shadow-0 my-auto mr-3"
                            )
                            .attr({
                              type: "button",
                              id: "period_" + i,
                              "data-toggle": "modal",
                              "data-target": "#daterangepicker",
                            })
                            .text(exp.period ? exp.period : "Period")
                            .data({
                              period: exp.period,
                              duration: exp.duration,
                            })
                            .prop("callback", () => (daterange) => {
                              const lower = instance.#extractMonthYear(
                                daterange.start
                              );
                              const upper = instance.#extractMonthYear(
                                daterange.end
                              );
                              const period = (exp.period =
                                lower == upper
                                  ? lower
                                  : lower.concat(" — ", upper));
                              exp.duration = instance.#calculateDuration(
                                daterange.start,
                                daterange.end
                              );
                              $("#period_" + i).text(period);
                            })
                        )
                        .append(
                          $("<div>")
                            .addClass("invalid-feedback")
                            .text("Period is invalid.")
                        )
                    )
                )
                .append(
                  $("<div>")
                    .addClass("form-group col-12")
                    .append(
                      $("<label>")
                        .attr({ for: "exp_description_" + i })
                        .text("Description")
                    )
                    .append(
                      $("<textarea>")
                        .addClass("form-control pt-0 px-0")
                        .attr({
                          rows: 2,
                          maxlength: 300,
                          name: "description",
                          id: "exp_description_" + i,
                        })
                        .val(exp.description)
                        .data("value", exp.description)
                        .on("input", function () {
                          exp.description = $(this).val();
                          $(this)
                            .next()
                            .text(`${$(this).val().length} / 300`);
                        })
                    )
                    .append(
                      $("<p>")
                        .addClass("form-text text-right mb-0")
                        .text(`${exp.description.length} / 250`)
                    )
                    .append(
                      $("<div>")
                        .addClass("invalid-feedback")
                        .text("Description is invalid.")
                    )
                );
            });
        };
        const listProjects = () => {
          if (!projects.length) {
            projects.push({
              title: "",
              link: "",
              description: "",
              new: true,
            });
          }
          return projects
            .filter((proj) => !proj.deleted)
            .map(function (proj, i) {
              return $("<div>")
                .addClass("mb-3")
                .append(
                  $("<button>")
                    .addClass("close float-right posRel")
                    .attr("type", "button")
                    .css("z-index", 1)
                    .append($("<span>").text("×"))
                    .click(function () {
                      const parent = $(this).parent()[0];
                      const index = $(parent).parent().children().index(parent);
                      if (index >= 0) {
                        if (projects[index].new) {
                          projects.splice(index, 1);
                        } else {
                          const proj = projects[index];
                          proj.deleted = true;
                          proj.title = $("#title_" + i).data("value");
                          proj.link = $("#link_" + i).data("value");
                          proj.description = $(
                            "#proj_description_" + i
                          ).data("value");
                        }
                      }
                      $(parent).remove();
                    })
                )
                .append(
                  $("<div>")
                    .addClass("form-group")
                    .append(
                      $("<label>")
                        .addClass("col-form-label")
                        .attr("for", "title_" + i)
                        .text("Project title")
                    )
                    .append(
                      $("<input>")
                        .addClass("form-control border-dark")
                        .attr({
                          type: "text",
                          id: "title_" + i,
                          name: "title",
                          placeholder: "Enter project title",
                        })
                        .data("value", proj.title)
                        .val(proj.title)
                        .on("input", function () {
                          proj.title = $(this).val();
                        })
                    )
                    .append(
                      $("<div>")
                        .addClass("invalid-feedback")
                        .text("Title is invalid.")
                    )
                )
                .append(
                  $("<div>")
                    .addClass("form-group")
                    .append(
                      $("<label>")
                        .addClass("col-form-label")
                        .attr({ for: "link_" + i })
                        .text("Link")
                    )
                    .append(
                      $("<input>")
                        .addClass("form-control border-dark")
                        .attr({
                          type: "url",
                          id: "link_" + i,
                          name: "link",
                          placeholder: "Enter project link",
                        })
                        .val(proj.link)
                        .data("value", proj.link)
                        .on({
                          input: function () {
                            proj.link = $(this).val();
                          },
                          blur: function () {
                            const $this = this;
                            const feedback = $(this).next(".invalid-feedback");
                            const loader = $("<span>")
                              .addClass(
                                "spinner-border spinner-border-sm posAbs"
                              )
                              .attr({ "aria-hidden": "true" })
                              .css({
                                "box-shadow": "none",
                                right: "1.5rem",
                                bottom: "0.6rem",
                              })
                              .insertAfter(feedback);
                            $("[data-id=experience]").addClass("disabled");
                            $.post(
                              `${site.process}custom?case=scrape_url`,
                              { url: proj.link, check: true },
                              function (jsonRes) {
                                $("[data-id=experience]").addClass("disabled");
                                $(loader).remove();
                                const res = JSON.parse(jsonRes);

                                if (res.status == 1) {
                                  if ($($this).hasClass("is-invalid")) {
                                    $($this).removeClass("is-invalid");
                                  }
                                } else if (!$($this).hasClass("is-invalid")) {
                                  $($this).addClass("is-invalid");
                                }
                              }
                            );
                          },
                        })
                    )
                    .append(
                      $("<div>")
                        .addClass("invalid-feedback")
                        .text("Link is invalid or unreachable.")
                    )
                )
                .append(
                  $("<div>")
                    .addClass("form-group")
                    .append(
                      $("<label>")
                        .attr({ for: "proj_description_" + i })
                        .text("Description")
                    )
                    .append(
                      $("<textarea>")
                        .addClass("form-control pt-0 px-0")
                        .attr({
                          rows: 2,
                          maxlength: 300,
                          name: "description",
                          id: "proj_description_" + i,
                        })
                        .val(proj.description)
                        .data("value", proj.description)
                        .on("input", function () {
                          proj.description = $(this).val();
                          $(this)
                            .next()
                            .text(`${$(this).val().length} / 300`);
                        })
                    )
                    .append(
                      $("<p>")
                        .addClass("form-text text-right mb-0")
                        .text(`${proj.description.length} / 250`)
                    )
                    .append(
                      $("<div>")
                        .addClass("invalid-feedback")
                        .text("Description is invalid.")
                    )
                );
            });
        };
        const colonSvg = `<svg xmlns="http://www.w3.org/2000/svg" width="12" height="12" viewBox="0 0 16 16" transform="rotate(90)" fill="currentColor" focusable="false" class="align-middle ml-1" aria-busy="false"><path d="M2 10h4v4H2v-4zm8 4h4v-4h-4v4z"></path></svg>`;

        extraFields = $("<div>")
          .addClass(
            "d-flex justify-content-between align-items-center mx-0 mt-5 mb-4"
          )
          .append(
            $("<h2>")
              .addClass("text-capitalize mb-0")
              .css({ "font-family": "'Roboto Condensed', sans-serif" })
              .text("Talent Profile")
          )
          .append(
            $("<a>")
              .addClass(
                "btn btn-lg bg-transparent border border-dark text-dark px-4 py-2"
              )
              .attr({
                href: `${site.domain}view-profile/${user.id}-${user.first_name}-${user.last_name}`, target: '_blank'
              })
              .css({
                "font-weight": "600",
                "box-shadow": "none",
                "border-radius": "0",
              })
              .text("View Profile")
              .append(
                $("<span>").addClass("fas fa-arrow-right ml-1 align-text-top")
              )
              .on({
                mouseover: function () {
                  this.style.setProperty(
                    "background-color",
                    "rgba(0, 0, 0, 0.04)",
                    "important"
                  );
                },
                mouseout: function () {
                  this.style.setProperty(
                    "background-color",
                    "transparent",
                    "important"
                  );
                },
              })
          )
          .add(
            $("<div>")
              .addClass("row justify-content-between col-lg-11 col-xl-10")
              .append(
                $("<div>")
                  .addClass("form-text mb-3")
                  .text(
                    "The following information you provide will be used to create a talent profile for you where potential employers can come and view your profile. Make sure to keep your profile up-to-date with your progress."
                  )
              )
              .append(
                $("<div>")
                  .addClass("col-12 col-md-6 mb-4")
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-title show")
                      .append(
                        $("<div>")
                          .addClass("form-text d-flex justify-content-between")
                          .append($("<span>").text("Job title"))
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm btn-link py-0")
                              .attr({
                                href: "javascript:void(0);",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-title",
                              })
                              .append(
                                '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                              )
                          )
                      )
                      .append($("<strong>").text(title ? title : "None"))
                  )
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-title")
                      .append(
                        $("<div>")
                          .addClass("mb-2")
                          .append(
                            $("<label>")
                              .addClass("col-form-label")
                              .attr({ for: "job_title" })
                              .text("Profession")
                          )
                          .append(
                            $("<select>")
                              .addClass("form-control rounded-pill border-dark")
                              .attr({
                                id: "job_title",
                                name: "title",
                              })
                              .each(function () {
                                $("<option>")
                                  .addClass("font-weight-bold")
                                  .prop("disabled", true)
                                  .val("")
                                  .text("Select profession")
                                  .prop("selected", title == null)
                                  .appendTo(this);

                                for (const val of techRoles) {
                                  $("<option>")
                                    .val(val)
                                    .text(val)
                                    .appendTo(this);
                                }
                              })
                              .val(title)
                              .data("value", title)
                          )
                          .append(
                            $("<div>")
                              .addClass("invalid-feedback")
                              .text("Profession is invalid.")
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("d-grid d-md-block")
                          .append(
                            $("<button>")
                              .addClass(
                                "btn btn-sm rounded-pill text-dark bg-light mr-2"
                              )

                              .attr({
                                type: "button",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-title",
                              })
                              .text("Cancel")
                              .click(function () {
                                const commonParent =
                                  $(this).closest(".collapse");
                                $("select", commonParent).val(
                                  $("select", commonParent).data("value")
                                );

                                if (
                                  $("select", commonParent).hasClass(
                                    "is-invalid"
                                  )
                                ) {
                                  $("select", commonParent).removeClass(
                                    "is-invalid"
                                  );
                                }
                              })
                          )
                          .append(
                            $("<button>")
                              .addClass("btn bg-dark btn-sm rounded-pill px-3")
                              .attr({ type: "button" })
                              .text("Save")
                              .click(this.#editTalentProfile.bind(this))
                          )
                      )
                  )
              )
              .append(
                $("<div>")
                  .addClass("col-12 col-md-6 mb-4")
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-workplace show")
                      .append(
                        $("<div>")
                          .addClass("form-text d-flex justify-content-between")
                          .append($("<span>").text("Work place"))
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm btn-link py-0")
                              .attr({
                                href: "javascript:void(0);",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-workplace",
                              })
                              .append(
                                '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                              )
                          )
                      )
                      .append(
                        $("<strong>").text(
                          employers[workplace] || other || "None"
                        )
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-workplace")
                      .append(
                        $("<div>")
                          .addClass("mb-2")
                          .append(
                            $("<label>")
                              .addClass("col-form-label")
                              .attr({ for: "workplace" })
                              .text("Workplace")
                          )
                          .append(
                            $("<input>")
                              .addClass("form-control rounded-pill border-dark")
                              .attr({
                                list: "employers",
                                id: "workplace",
                                name: "workplace",
                                placeholder: "Select workplace",
                              })
                              .val(employers[workplace])
                              .data("value", employers[workplace])
                              .on("input focusout", function (evt) {
                                const enteredVal = $(this).val().trim();

                                if (!enteredVal) return;
                                const matched = Object.values(employers).some(
                                  (employer) =>
                                    employer == enteredVal ||
                                    employer.startsWith(enteredVal)
                                );
                                const otherOption = $(this)
                                  .next("datalist")
                                  .find("option:last-child");

                                if (!matched) {
                                  if (evt.type == "focusout") {
                                    $(this).val("Other");
                                    $(otherOption).val("Other").text("");
                                    $("#other_workplace").show(function () {
                                      $(this).val(enteredVal).focus();
                                    });
                                  } else {
                                    $(otherOption)
                                      .val(enteredVal)
                                      .text(enteredVal);
                                  }
                                } else if (evt.type == "focusout") {
                                  $("#other_workplace").val("").hide();
                                }
                              })
                          )
                          .append(
                            $("<datalist>")
                              .attr("id", "employers")
                              .each(function () {
                                for (const val of Object.values(employers)) {
                                  $("<option>")
                                    .val(val)
                                    .text(val)
                                    .appendTo(this);
                                }
                                $("<option>").val("Other").appendTo(this);
                              })
                          )
                          .append(
                            $("<input>")
                              .addClass("form-control mt-2")
                              .attr({
                                type: "text",
                                name: "other",
                                id: "other_workplace",
                                placeholder: "Other workplace",
                              })
                              .val(other)
                              .data("value", other)
                              .css({
                                display: workplace == "-1" ? "block" : "none",
                                border: 0,
                                "border-bottom": "1px solid #343a40",
                                "border-radius": 0,
                              })
                          )
                          .append(
                            $("<div>")
                              .addClass("invalid-feedback")
                              .text("Workplace is invalid.")
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("d-grid d-md-block")
                          .append(
                            $("<button>")
                              .addClass(
                                "btn btn-sm rounded-pill text-dark bg-light mr-2"
                              )
                              .attr({
                                type: "button",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-workplace",
                              })
                              .text("Cancel")
                              .click(function () {
                                const commonParent =
                                  $(this).closest(".collapse");
                                $("input, select", commonParent).each(
                                  function () {
                                    if ($(this).hasClass("is-invalid")) {
                                      $(this).removeClass("is-invalid");
                                    }
                                    $(this).val($(this).data("value"));
                                  }
                                );
                              })
                          )
                          .append(
                            $("<button>")
                              .addClass("btn bg-dark btn-sm rounded-pill px-3")
                              .attr({ type: "button" })
                              .text("Save")
                              .click(this.#editTalentProfile.bind(this))
                          )
                      )
                  )
              )
              .append(
                $("<div>")
                  .addClass("col-12 col-md-6 mb-4")
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-location show")
                      .append(
                        $("<div>")
                          .addClass("form-text d-flex justify-content-between")
                          .append($("<span>").text("Location"))
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm btn-link py-0")
                              .attr({
                                href: "javascript:void(0);",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-location",
                              })
                              .append(
                                '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                              )
                          )
                      )
                      .append(
                        $("<strong>")
                          .addClass("text-capitalize")
                          .text(location ? location : "None")
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-location")
                      .append(
                        $("<div>")
                          .addClass("d-flex mb-2")
                          .css("gap", "1.5rem")
                          .append(
                            $("<div>")
                              .addClass("col px-0")
                              .append(
                                $("<label>")
                                  .addClass("col-form-label")
                                  .attr({ for: "loc-state" })
                                  .text("Location (in Nigeria)")
                              )
                              .append(
                                $("<select>")
                                  .addClass(
                                    "form-control rounded-pill border-dark text-capitalize"
                                  )
                                  .attr({
                                    id: "loc-state",
                                    name: "state",
                                  })
                                  .each(function () {
                                    const locArr = location?.split(",") ?? [];
                                    const state =
                                      locArr.length > 1 ? locArr[0] : "";
                                    $("<option>")
                                      .addClass("font-weight-bold")
                                      .prop("disabled", true)
                                      .val("")
                                      .text("Select state")
                                      .appendTo(this);

                                    for (const key of Object.keys(states)) {
                                      $("<option>")
                                        .addClass("text-capitalize")
                                        .val(key)
                                        .text(key)
                                        .appendTo(this);
                                    }
                                    $(this).val(state).data("value", state);
                                  })
                                  .change(function () {
                                    if ($(this).val())
                                      $("#loc-country").val("Nigeria");
                                  })
                              )
                              .append(
                                $("<div>")
                                  .addClass("invalid-feedback")
                                  .text("State is invalid.")
                              )
                          )
                          .append(
                            $("<div>")
                              .addClass("col px-0")
                              .append(
                                $("<label>")
                                  .addClass("col-form-label")
                                  .attr({ for: "loc-country" })
                                  .text("Country of residence")
                              )
                              .append(
                                $("<input>")
                                  .addClass(
                                    "form-control rounded-pill border-dark"
                                  )
                                  .attr({
                                    id: "loc-country",
                                    name: "country",
                                    list: "country_list",
                                    placeholder: "Select country",
                                  })
                                  .each(function () {
                                    const locArr = location?.split(",") ?? [""];
                                    const country =
                                      locArr.length > 1
                                        ? locArr[1]?.trim()
                                        : locArr[0];
                                    $(this).val(country).data("value", country);
                                  })
                              )
                              .append(
                                $("<datalist>")
                                  .attr("id", "country_list")
                                  .each(function () {
                                    for (const val of Object.values(
                                      countries
                                    )) {
                                      $("<option>")
                                        .val(val)
                                        .text(val)
                                        .appendTo(this);
                                    }
                                  })
                              )
                              .append(
                                $("<div>")
                                  .addClass("invalid-feedback")
                                  .text("Country is invalid.")
                              )
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("d-grid d-md-block")
                          .append(
                            $("<button>")
                              .addClass(
                                "btn btn-sm rounded-pill text-dark bg-light mr-2"
                              )

                              .attr({
                                type: "button",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-location",
                              })
                              .text("Cancel")
                              .click(function () {
                                $(this)
                                  .closest(".collapse")
                                  .find("select")
                                  .each(function () {
                                    if ($(this).hasClass("is-invalid")) {
                                      $(this).removeClass("is-invalid");
                                    }
                                    $(this).val($(this).data("value"));
                                  });
                              })
                          )
                          .append(
                            $("<button>")
                              .addClass("btn bg-dark btn-sm rounded-pill px-3")
                              .attr({ type: "button" })
                              .text("Save")
                              .click(this.#editTalentProfile.bind(this))
                          )
                      )
                  )
              )
              .append(
                $("<div>")
                  .addClass("col-12 col-md-6 mb-4")
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-status show")
                      .append(
                        $("<div>")
                          .addClass("form-text d-flex justify-content-between")
                          .append($("<span>").text("Job status"))
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm btn-link py-0")
                              .attr({
                                href: "javascript:void(0);",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-status",
                              })
                              .append(
                                '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                              )
                          )
                      )
                      .append($("<strong>").text(status ? status : "None"))
                  )
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-status")
                      .append(
                        $("<div>")
                          .addClass("mb-2")
                          .append(
                            $("<label>")
                              .addClass("col-form-label")
                              .attr({ for: "job_status" })
                              .text("Job status")
                          )
                          .append(
                            $("<select>")
                              .addClass("form-control border-dark")
                              .attr({
                                id: "job_status",
                                name: "status",
                              })
                              .each(function () {
                                $("<option>")
                                  .addClass("font-weight-bold")
                                  .val("")
                                  .text("Select status")
                                  .prop("disabled", true)
                                  .appendTo(this);

                                for (const val of jobStatus) {
                                  $("<option>")
                                    .val(val)
                                    .text(val)
                                    .appendTo(this);
                                }
                              })
                              .val(status)
                              .data("value", status)
                          )
                          .append(
                            $("<div>")
                              .addClass("invalid-feedback")
                              .text("Job status is invalid.")
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("d-grid gap-2 d-md-block")
                          .append(
                            $("<button>")
                              .addClass(
                                "btn btn-sm rounded-pill text-dark bg-light mr-2"
                              )
                              .attr({
                                type: "button",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-status",
                              })
                              .text("Cancel")
                              .click(function () {
                                const commonParent =
                                  $(this).closest(".collapse");
                                $("select", commonParent).val(
                                  $("select", commonParent).data("value")
                                );

                                if (
                                  $("select", commonParent).hasClass(
                                    "is-invalid"
                                  )
                                ) {
                                  $("select", commonParent).removeClass(
                                    "is-invalid"
                                  );
                                }
                              })
                          )
                          .append(
                            $("<button>")
                              .addClass("btn bg-dark btn-sm rounded-pill px-3")
                              .attr({ type: "button" })
                              .text("Save")
                              .click(this.#editTalentProfile.bind(this))
                          )
                      )
                  )
              )
              .append(
                $("<div>")
                  .addClass("col-12 col-md-6 mb-4")
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-open show")
                      .append(
                        $("<div>")
                          .addClass("form-text d-flex justify-content-between")
                          .append($("<span>").text("Open to offers"))
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm btn-link py-0")
                              .attr({
                                href: "javascript:void(0);",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-open",
                              })
                              .append(
                                '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                              )
                          )
                      )
                      .append($("<strong>").text(open ? "Yes" : "No"))
                  )
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-open")
                      .append(
                        $("<div>")
                          .addClass("mb-2")
                          .append(
                            $("<label>")
                              .addClass("col-form-label")
                              .attr({ for: "open2work" })
                              .text("Open to offers")
                          )
                          .append(
                            $("<select>")
                              .addClass("form-control border-dark")
                              .attr({
                                id: "open2work",
                                name: "open",
                              })
                              .append(
                                $("<option>").val("1").text("Yes"),
                                $("<option>").val("0").text("No")
                              )
                              .val(`${open}`)
                              .data("value", `${open}`)
                          )
                          .append(
                            $("<div>")
                              .addClass("invalid-feedback")
                              .text("Selection is invalid.")
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("d-grid gap-2 d-md-block")
                          .append(
                            $("<button>")
                              .addClass(
                                "btn btn-sm rounded-pill text-dark bg-light mr-2"
                              )
                              .attr({
                                type: "button",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-open",
                              })
                              .text("Cancel")
                              .click(function () {
                                const commonParent =
                                  $(this).closest(".collapse");
                                $("select", commonParent).val(
                                  $("select", commonParent).data("value")
                                );

                                if (
                                  $("select", commonParent).hasClass(
                                    "is-invalid"
                                  )
                                ) {
                                  $("select", commonParent).removeClass(
                                    "is-invalid"
                                  );
                                }
                              })
                          )
                          .append(
                            $("<button>")
                              .addClass("btn bg-dark btn-sm rounded-pill px-3")
                              .attr({ type: "button" })
                              .text("Save")
                              .click(this.#editTalentProfile.bind(this))
                          )
                      )
                  )
              )
              .append(
                $("<div>")
                  .addClass("col-12 col-md-6 mb-4")
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-education show")
                      .append(
                        $("<div>")
                          .addClass("form-text d-flex justify-content-between")
                          .append($("<span>").text("Education"))
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm btn-link py-0")
                              .attr({
                                href: "javascript:void(0);",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-education",
                              })
                              .append(
                                '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                              )
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("card-text px-0")
                          .append(
                            !education.length &&
                              $("<div>")
                                .addClass(
                                  "d-flex justify-content-center align-items-center w-100 h-100"
                                )
                                .append(
                                  $("<span>")
                                    .addClass("text-muted")
                                    .css({
                                      height: "5rem",
                                      "line-height": "5rem",
                                    })
                                    .text("No education added")
                                )
                          )
                          .append(
                            education.length > 0 &&
                              this.#displayTalentProfile(education, "education")
                          )
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-education")
                      .append(
                        $("<div>")
                          .addClass("d-flex justify-content-between")
                          .append(
                            $("<h6>")
                              .addClass("text-capitalize")
                              .text("Education")
                              .css("font-weight", "500")
                          )
                          .append(
                            $("<button>")
                              .attr("type", "button")
                              .addClass(
                                "btn btn-sm bg-white text-dark rounded-pill border border-dark"
                              )
                              .css("box-shadow", "none")
                              .append("Add")
                              .on({
                                "mouseover mouseout": function () {
                                  $(this).toggleClass(
                                    "bg-white text-dark bg-dark text-white"
                                  );
                                },
                                click: function () {
                                  education.push({
                                    degree: "",
                                    period: "",
                                    description: "",
                                    new: true,
                                  });
                                  $("#edu-container")
                                    .empty()
                                    .append(...listEducation());
                                },
                              })
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("mb-2")
                          .attr("id", "edu-container")
                          .append(...listEducation())
                      )
                      .append(
                        $("<div>")
                          .addClass("d-grid d-md-block")
                          .append(
                            $("<button>")
                              .addClass(
                                "btn btn-sm rounded-pill text-dark bg-light mr-2"
                              )

                              .attr({
                                type: "button",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-education",
                              })
                              .text("Cancel")
                              .click(function (_, reset = true) {
                                if (reset) {
                                  for (const [i, edu] of Object.entries(
                                    education
                                  )) {
                                    if (edu.new || !$("#degree_" + i)[0]) continue;
                                    const degree = $("#degree_" + i).data(
                                        "value"
                                      ),
                                      course = $("#course_" + i).data("value"),
                                      startYr = $("#start_year_" + i).data(
                                        "value"
                                      ),
                                      endYr = $("#end_year_" + i).data("value"),
                                      desc = $("#edu_description_" + i).data("value");

                                    edu.degree = (degree + " " + course).trim();
                                    edu.period = (
                                      startYr +
                                      " — " +
                                      endYr
                                    ).trim();
                                    edu.description = desc;
                                  }
                                }

                                for (let i = 0; i < education.length; i++) {
                                  const edu = education[i];
                                  if (edu.new) {
                                    education.splice(i, 1);
                                    i--;
                                  } else if (edu.deleted) {
                                    delete edu.deleted;
                                  }
                                }
                                  $("#edu-container")
                                  .empty()
                                  .append(...listEducation());
                              })
                          )
                          .append(
                            $("<button>")
                              .addClass("btn bg-dark btn-sm rounded-pill px-3")
                              .attr({ type: "button", "data-id": "education" })
                              .text("Save")
                              .click((evt) =>
                                this.#editTalentProfile(evt, education)
                              )
                          )
                      )
                  )
              )
              .append(
                $("<div>")
                  .addClass("col-12 col-md-6 mb-4")
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-skills show")
                      .append(
                        $("<div>")
                          .addClass("form-text d-flex justify-content-between")
                          .append($("<span>").text("Skills"))
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm btn-link py-0")
                              .attr({
                                href: "javascript:void(0);",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-skills",
                              })
                              .append(
                                '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                              )
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("card-text px-0")
                          .append(
                            !skills.length &&
                              $("<div>")
                                .addClass(
                                  "d-flex justify-content-center align-items-center w-100 h-100"
                                )
                                .append(
                                  $("<span>")
                                    .addClass("text-muted")
                                    .css({
                                      height: "5rem",
                                      "line-height": "5rem",
                                    })
                                    .text("No skills added")
                                )
                          )
                          .append(
                            skills.length > 0 &&
                              this.#displayTalentProfile(skills, "skills")
                          )
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-skills")
                      .append(
                        $("<h6>")
                          .addClass("text-capitalize")
                          .text("Skills")
                          .css("font-weight", "500")
                      )
                      .append(
                        $("<div>")
                          .addClass("form-group")
                          .append(
                            $("<label>")
                              .addClass("col-form-label")
                              .attr({ for: "skill_name" })
                              .text("Skill")
                          )
                          .append(
                            $("<select>")
                              .addClass("form-control rounded-pill border-dark")
                              .attr({
                                id: "skill_name",
                                name: "skill",
                              })
                              .each(function () {
                                $("<option>")
                                  .addClass("font-weight-bold")
                                  .prop({ disabled: true, selected: true })
                                  .val("")
                                  .text("Select skill")
                                  .appendTo(this);

                                for (const [key, arr] of Object.entries(
                                  techSkills
                                )) {
                                  $("<option>")
                                    .val(key)
                                    .text(key)
                                    .prop("disabled", true)
                                    .appendTo(this);

                                  for (const val of arr) {
                                    $("<option>")
                                      .val(val)
                                      .text(val)
                                      .appendTo(this);
                                  }
                                }
                              })
                          )
                          .append(
                            $("<div>")
                              .addClass("invalid-feedback")
                              .text("Skill is invalid.")
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("d-flex align-items-end")
                          .append(
                            $("<div>")
                              .addClass("col form-group mb-0 pl-0")
                              .append(
                                $("<label>")
                                  .addClass("col-form-label")
                                  .attr({ for: "skill_score" })
                                  .text("Proficiency (%)")
                              )
                              .append(
                                $("<input>")
                                  .addClass(
                                    "form-control rounded-pill border-dark"
                                  )
                                  .attr({
                                    type: "number",
                                    id: "skill_score",
                                    name: "score",
                                    min: 1,
                                    max: 100,
                                    step: 1,
                                    placeholder: "Enter score",
                                  })
                              )
                              .append(
                                $("<div>")
                                  .addClass("invalid-feedback")
                                  .text("Proficiency score is invalid.")
                              )
                          )
                          .append(
                            $("<div>")
                              .addClass("d-flex justify-content-end")
                              .append(
                                $("<button>")
                                  .attr("type", "button")
                                  .addClass("btn bg-dark rounded-pill m-auto")
                                  .text("Add")
                                  .click(function () {
                                    const skill = $("#skill_name").val(),
                                      skillScore = $("#skill_score").val();
                                    let isValid = true;

                                    if (!skill) {
                                      $("#skill_name").addClass("is-invalid");
                                      isValid = false;
                                    } else if (
                                      $("#skill_name").hasClass("is-invalid")
                                    ) {
                                      $("#skill_name").removeClass(
                                        "is-invalid"
                                      );
                                    }

                                    if (!skillScore) {
                                      if (isValid) isValid = false;
                                      $("#skill_score").addClass("is-invalid");
                                    } else if (
                                      $("#skill_score").hasClass("is-invalid")
                                    ) {
                                      $("#skill_score").removeClass(
                                        "is-invalid"
                                      );
                                    }

                                    if (!isValid) return false;
                                    $("#skill_name, #skill_score").val("");

                                    for (const obj of skills) {
                                      if (obj.name == skill && !obj.deleted) {
                                        obj.deleted = true;
                                        break;
                                      }
                                    }
                                    skills.push({
                                      name: skill,
                                      score: skillScore,
                                      new: true,
                                    });
                                    $("#skillsContainer")
                                      .empty()
                                      .append(...listSkills());
                                  })
                              )
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass(
                            "d-flex mt-3 mb-2 align-items-center flex-wrap"
                          )
                          .css("gap", "0.5rem")
                          .attr("id", "skillsContainer")
                          .append(...listSkills())
                      )
                      .append(
                        $("<div>")
                          .addClass("d-grid d-md-block")
                          .append(
                            $("<button>")
                              .addClass(
                                "btn btn-sm rounded-pill text-dark bg-light mr-2"
                              )
                              .attr({
                                type: "button",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-skills",
                              })
                              .text("Cancel")
                              .click(function () {
                                for (let i = 0; i < skills.length; i++) {
                                  const skill = skills[i];
                                  if (skill.new) {
                                    skill.splice(i, 1);
                                    i--;
                                  } else if (skill.deleted) {
                                    delete skill.deleted;
                                  }
                                }
                                const commonParent =
                                  $(this).closest(".collapse");
                                $("select, input", commonParent).each(
                                  function () {
                                    if ($(this).hasClass("is-invalid"))
                                      $(this).removeClass("is-invalid");
                                    $(this).val("");
                                  }
                                );
                                $("#skillsContainer")
                                  .empty()
                                  .append(...listSkills());
                              })
                          )
                          .append(
                            $("<button>")
                              .addClass("btn bg-dark btn-sm rounded-pill px-3")
                              .attr({ type: "button", "data-id": "skills" })
                              .text("Save")
                              .click((evt) =>
                                this.#editTalentProfile(evt, skills)
                              )
                          )
                      )
                  )
              )
              .append(
                $("<div>")
                  .addClass("col-12 col-md-6 mb-4")
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-experience show")
                      .append(
                        $("<div>")
                          .addClass("form-text d-flex justify-content-between")
                          .append($("<span>").text("Experience"))
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm btn-link py-0")
                              .attr({
                                href: "javascript:void(0);",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-experience",
                              })
                              .append(
                                '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                              )
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("card-text")
                          .append(
                            !experience.length &&
                              $("<div>")
                                .addClass(
                                  "d-flex justify-content-center align-items-center w-100 h-100"
                                )
                                .append(
                                  $("<span>")
                                    .addClass("text-muted")
                                    .css({
                                      height: "5rem",
                                      "line-height": "5rem",
                                    })
                                    .text("No experience added")
                                )
                          )
                          .append(
                            experience.length > 0 &&
                              this.#displayTalentProfile(
                                experience,
                                "experience"
                              )
                          )
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-experience")
                      .append(
                        $("<div>")
                          .addClass("d-flex justify-content-between")
                          .append(
                            $("<h6>")
                              .addClass("text-capitalize")
                              .text("Experience")
                              .css("font-weight", "500")
                          )
                          .append(
                            $("<button>")
                              .attr("type", "button")
                              .addClass(
                                "btn btn-sm bg-white text-dark rounded-pill border border-dark"
                              )
                              .css("box-shadow", "none")
                              .append("Add")
                              .on({
                                "mouseover mouseout": function () {
                                  $(this).toggleClass(
                                    "bg-white text-dark bg-dark text-white"
                                  );
                                },
                                click: function () {
                                  experience.push({
                                    firm: "",
                                    role: "",
                                    period: "",
                                    duration: "",
                                    description: "",
                                    new: true,
                                  });
                                  $("#exp-container")
                                    .empty()
                                    .append(...listExperience());
                                },
                              })
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("mb-2")
                          .attr("id", "exp-container")
                          .append(...listExperience())
                      )
                      .append(
                        $("<div>")
                          .addClass("d-grid d-md-block")
                          .append(
                            $("<button>")
                              .addClass(
                                "btn btn-sm rounded-pill text-dark bg-light mr-2"
                              )
                              .attr({
                                type: "button",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-experience",
                              })
                              .text("Cancel")
                              .click(function (_, reset = true) {
                                if (reset) {
                                  for (const [i, exp] of Object.entries(
                                    experience
                                  )) {
                                    if (exp.new || !$("#firm_" + i)[0]) continue;
                                    exp.firm = $("#firm_" + i).data(
                                      "value"
                                    );
                                    exp.role = $("#role_" + i).data("value");
                                    exp.description = $("#exp_description_" + i).data(
                                      "value"
                                    );
                                    exp.period = $("#period_" + i).data(
                                      "period"
                                    );
                                    exp.duration = $("#period_" + i).data(
                                      "duration"
                                    );
                                  }
                                }

                                for (let i = 0; i < experience.length; i++) {
                                  const exp = experience[i];
                                  if (exp.new) {
                                    experience.splice(i, 1);
                                    i--;
                                  } else if (exp.deleted) {
                                    delete exp.deleted;
                                  }
                                }
                                $("#exp-container")
                                  .empty()
                                  .append(...listExperience());
                              })
                          )
                          .append(
                            $("<button>")
                              .addClass("btn bg-dark btn-sm rounded-pill px-3")
                              .attr({ type: "button", "data-id": "experience" })
                              .text("Save")
                              .click((evt) =>
                                this.#editTalentProfile(evt, experience)
                              )
                          )
                      )
                  )
              )
              .append(
                $("<div>")
                  .addClass("col-12 col-md-6 mb-4")
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-projects show")
                      .append(
                        $("<div>")
                          .addClass("form-text d-flex justify-content-between")
                          .append($("<span>").text("Projects"))
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm btn-link py-0")
                              .attr({
                                href: "javascript:void(0);",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-projects",
                              })
                              .append(
                                '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                              )
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("card-text")
                          .append(
                            !projects.length &&
                              $("<div>")
                                .addClass(
                                  "d-flex justify-content-center align-items-center w-100 h-100"
                                )
                                .append(
                                  $("<span>")
                                    .addClass("text-muted")
                                    .css({
                                      height: "5rem",
                                      "line-height": "5rem",
                                    })
                                    .text("No projects added")
                                )
                          )
                          .append(
                            projects.length > 0 &&
                              this.#displayTalentProfile(projects, "projects")
                          )
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-projects")
                      .append(
                        $("<div>")
                          .addClass("d-flex justify-content-between")
                          .append(
                            $("<h6>")
                              .addClass("text-capitalize")
                              .text("Projects")
                              .css("font-weight", "500")
                          )
                          .append(
                            $("<button>")
                              .attr("type", "button")
                              .addClass(
                                "btn btn-sm bg-white text-dark rounded-pill border border-dark"
                              )
                              .css("box-shadow", "none")
                              .append("Add")
                              .on({
                                "mouseover mouseout": function () {
                                  $(this).toggleClass(
                                    "bg-white text-dark bg-dark text-white"
                                  );
                                },
                                click: function () {
                                  projects.push({
                                    title: "",
                                    link: "",
                                    description: "",
                                    new: true,
                                  });
                                  $("#proj-container")
                                    .empty()
                                    .append(...listProjects());
                                },
                              })
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("mb-2")
                          .attr('id', 'proj-container')
                          .append(...listProjects())
                      )
                      .append(
                        $("<div>")
                          .addClass("d-grid d-md-block")
                          .append(
                            $("<button>")
                              .addClass(
                                "btn btn-sm rounded-pill text-dark bg-light mr-2"
                              )

                              .attr({
                                type: "button",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-projects",
                              })
                              .text("Cancel")
                              .click(function (_, reset = true) {
                                if (reset) {
                                  for (const [i, proj] of Object.entries(
                                    projects
                                  )) {
                                    if (proj.new || !$("#title_" + i)[0]) continue;
                                    proj.title = $("#title_" + i).data("value");
                                    proj.link = $("#link_" + i).data("value");
                                    proj.description = $(
                                      "#proj_description_" + i
                                    ).data("value");
                                  }
                                }

                                for (let i = 0; i < projects.length; i++) {
                                  const proj = projects[i];

                                  if (proj.new) {
                                    projects.splice(i, 1);
                                    i--;
                                  } else if (proj.deleted) {
                                    delete proj.deleted;
                                  }
                                }
                                $("#proj-container")
                                  .empty()
                                  .append(...listProjects());
                              })
                          )
                          .append(
                            $("<button>")
                              .addClass("btn bg-dark btn-sm rounded-pill px-3")
                              .attr({ type: "button", "data-id": "projects" })
                              .text("Save")
                              .click((evt) =>
                                this.#editTalentProfile(evt, projects)
                              )
                          )
                      )
                  )
              )
              .append(
                $("<div>")
                  .addClass("col-12 col-md-6 mb-4")
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-social show")
                      .append(
                        $("<div>")
                          .addClass("form-text d-flex justify-content-between")
                          .append($("<span>").text("Socials"))
                          .append(
                            $("<a>")
                              .addClass("btn btn-sm btn-link py-0")
                              .attr({
                                href: "javascript:void(0);",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-social",
                              })
                              .append(
                                '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                              )
                          )
                      )
                      .append(
                        $("<ul>")
                          .addClass("list-unstyled pl-3")
                          .append(
                            $("<li>")
                              .append(
                                $("<span>")
                                  .addClass("fab fa-facebook-f align-middle")
                                  .css("font-size", "medium")
                              )
                              .append(colonSvg)
                              .append(
                                $("<span>")
                                  .addClass("ml-2 facebook")
                                  .css("font-weight", "500")
                                  .text(socials.facebook)
                              )
                          )
                          .append(
                            $("<li>")
                              .append(
                                '<svg xmlns="http://www.w3.org/2000/svg" class="align-middle" height="1em" viewBox="0 0 512 512" fill="currentColor"><path d="M389.2 48h70.6L305.6 224.2 487 464H345L233.7 318.6 106.5 464H35.8L200.7 275.5 26.8 48H172.4L272.9 180.9 389.2 48zM364.4 421.8h39.1L151.1 88h-42L364.4 421.8z"></path></svg>'
                              )
                              .append(colonSvg)
                              .append(
                                $("<span>")
                                  .addClass("ml-2 x")
                                  .css("font-weight", "500")
                                  .text(socials.x)
                              )
                          )
                          .append(
                            $("<li>")
                              .append(
                                $("<span>")
                                  .addClass("fab fa-linkedin-in align-middle")
                                  .css("font-size", "medium")
                              )
                              .append(colonSvg)
                              .append(
                                $("<span>")
                                  .addClass("ml-2 linkedIn")
                                  .css("font-weight", "500")
                                  .text(socials.linkedIn)
                              )
                          )
                          .append(
                            $("<li>")
                              .append(
                                $("<span>")
                                  .addClass("fab fa-github align-middle")
                                  .css("font-size", "medium")
                              )
                              .append(colonSvg)
                              .append(
                                $("<span>")
                                  .addClass("ml-2 github")
                                  .css("font-weight", "500")
                                  .text(socials.github)
                              )
                          )
                      )
                  )
                  .append(
                    $("<div>")
                      .addClass("collapse multi-collapse-social")
                      .append(
                        $("<div>")
                          .addClass("mb-2")
                          .append(
                            $("<span>")
                              .addClass("fab fa-facebook-f ml-2 mb-1")
                              .css("font-size", "larger")
                          )
                          .append(
                            $("<input>")
                              .addClass("form-control rounded-pill border-dark")
                              .attr({
                                type: "url",
                                id: "facebook-url",
                                name: "facebook",
                                pattern:
                                  "https://(www.)?facebook.com/.+",
                                placeholder: "https://www.facebook.com/...",
                              })
                              .val(socials.facebook)
                              .data("value", socials.facebook)
                          )
                          .append(
                            $("<div>")
                              .addClass("invalid-feedback")
                              .text("Facebook link is invalid.")
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("mb-2")
                          .append(
                            '<svg xmlns="http://www.w3.org/2000/svg" class="ml-2" height="1.2em" viewBox="0 0 512 512" fill="currentColor"><path d="M389.2 48h70.6L305.6 224.2 487 464H345L233.7 318.6 106.5 464H35.8L200.7 275.5 26.8 48H172.4L272.9 180.9 389.2 48zM364.4 421.8h39.1L151.1 88h-42L364.4 421.8z"></path></svg>'
                          )
                          .append(
                            $("<input>")
                              .addClass("form-control rounded-pill border-dark")
                              .attr({
                                type: "url",
                                id: "x-url",
                                name: "x",
                                pattern:
                                  "https://(www.)?(x.com|twitter.com)/.+",
                                placeholder: "https://x.com/...",
                              })
                              .val(socials.x)
                              .data("value", socials.x)
                          )
                          .append(
                            $("<div>")
                              .addClass("invalid-feedback")
                              .text("X link is invalid.")
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("mb-2")
                          .append(
                            $("<span>")
                              .addClass("fab fa-linkedin-in ml-2 mb-1")
                              .css("font-size", "larger")
                          )
                          .append(
                            $("<input>")
                              .addClass("form-control rounded-pill border-dark")
                              .attr({
                                type: "url",
                                id: "linkedin-url",
                                name: "linkedIn",
                                pattern:
                                  "https?://(www.)?linkedin.com/in/.+",
                                placeholder: "https://www.linkedin.com/in/...",
                              })
                              .val(socials.linkedIn)
                              .data("value", socials.linkedIn)
                          )
                          .append(
                            $("<div>")
                              .addClass("invalid-feedback")
                              .text("LinkedIn link is invalid.")
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("mb-2")
                          .append(
                            $("<span>")
                              .addClass("fab fa-github ml-2 mb-1")
                              .css("font-size", "larger")
                          )
                          .append(
                            $("<input>")
                              .addClass("form-control rounded-pill border-dark")
                              .attr({
                                type: "url",
                                id: "github-url",
                                name: "github",
                                pattern:
                                  "https://(www.)?github.com/.+",
                                placeholder: "https://github.com/...",
                              })
                              .val(socials.github)
                              .data("value", socials.github)
                          )
                          .append(
                            $("<div>")
                              .addClass("invalid-feedback")
                              .text("Github link is invalid.")
                          )
                      )
                      .append(
                        $("<div>")
                          .addClass("d-grid gap-2 d-md-block")
                          .append(
                            $("<button>")
                              .addClass(
                                "btn btn-sm rounded-pill text-dark bg-light mr-2"
                              )
                              .attr({
                                type: "button",
                                "data-toggle": "collapse",
                                "data-target": ".multi-collapse-social",
                              })
                              .text("Cancel")
                              .click(function () {
                                const commonParent =
                                    $(this).closest(".collapse"),
                                  urlInput = $("input[type=url]", commonParent);
                                $(urlInput).each(function () {
                                  if (!$(this).val().trim()) {
                                    $(this).val($(this).data("value"));
                                  }

                                  if ($(this).hasClass("is-invalid")) {
                                    $(this).removeClass("is-invalid");
                                  }
                                });
                              })
                          )
                          .append(
                            $("<button>")
                              .addClass("btn bg-dark btn-sm rounded-pill px-3")
                              .attr({ type: "button", "data-id": "socials" })
                              .text("Save")
                              .click((evt) =>
                                this.#editTalentProfile(evt, socials)
                              )
                          )
                      )
                  )
              )
          );
      }
      $("<div>")
        .addClass("container-fluid")
        .appendTo($("#generalDialog .modal-body").empty())
        .append(
          $("<div>")
            .addClass("row justify-content-between align-items-center mt-4")
            .append(
              $("<div>")
                .addClass("col-sm d-flex align-items-end border-bottom mb-3")
                .append(
                  $("<div>")
                    .addClass("posRel mb-3")
                    .css({ width: "fit-content", "aspect-ratio": "1" })
                    .append(
                      $("<img>")
                        .addClass("rounded-circle")
                        .attr({
                          src:
                            $(".profile-pic").attr("src") ??
                            `${site.domain}asset/picture/photo_silhouette.png`,
                          alt: "Profile picture",
                          width: "120",
                          height: "120",
                        })
                        .css({
                          "object-position": "center top",
                          "object-fit": "cover",
                        })
                    )
                    .append(
                      $("<label>")
                        .addClass("bg-light btn rounded-circle posAbs")
                        .attr("for", "profile-pic")
                        .css({
                          "margin-right": "-1rem",
                          cursor: "pointer",
                          translate: "-50%",
                          bottom: 0,
                          right: 0,
                          padding: ".5rem .6rem",
                        })
                        .append(
                          '<svg width="1.5em" height="1.5em" viewBox="0 0 24 24" version="1" xmlns="http://www.w3.org/2000/svg"><path d="M1,19.5 L1,7.5 C1,6.11928813 2.11928813,5 3.5,5 L5.42963248,5 C5.93116226,5 6.39950926,4.7493483 6.67770792,4.33205029 L7.49024178,3.11324951 C7.95390623,2.41775284 8.73448455,2 9.57036752,2 L14.4296325,2 C15.2655155,2 16.0460938,2.41775284 16.5097582,3.11324951 L17.3222921,4.33205029 C17.6004907,4.7493483 18.0688377,5 18.5703675,5 L20.5,5 C21.8807119,5 23,6.11928813 23,7.5 L23,19.5 C23,20.8807119 21.8807119,22 20.5,22 L3.5,22 C2.11928813,22 1,20.8807119 1,19.5 Z M2,19.5 C2,20.3284271 2.67157288,21 3.5,21 L20.5,21 C21.3284271,21 22,20.3284271 22,19.5 L22,7.5 C22,6.67157288 21.3284271,6 20.5,6 L18.5703675,6 C17.7344845,6 16.9539062,5.58224716 16.4902418,4.88675049 L15.6777079,3.66794971 C15.3995093,3.2506517 14.9311623,3 14.4296325,3 L9.57036752,3 C9.06883774,3 8.60049074,3.2506517 8.32229208,3.66794971 L7.50975822,4.88675049 C7.04609377,5.58224716 6.26551545,6 5.42963248,6 L3.5,6 C2.67157288,6 2,6.67157288 2,7.5 L2,19.5 Z M12,19 C8.6862915,19 6,16.3137085 6,13 C6,9.6862915 8.6862915,7 12,7 C15.3137085,7 18,9.6862915 18,13 C18,16.3137085 15.3137085,19 12,19 Z M12,18 C14.7614237,18 17,15.7614237 17,13 C17,10.2385763 14.7614237,8 12,8 C9.23857625,8 7,10.2385763 7,13 C7,15.7614237 9.23857625,18 12,18 Z"/></svg>'
                        )
                    )
                    .append(
                      $("<input>")
                        .addClass("d-none")
                        .attr({
                          type: "file",
                          id: "profile-pic",
                          accept: ".jpeg,.jpg,.png",
                          name: "picture_ref",
                        })
                        .change(function () {
                          const file = this.files[0];
                          const maxSize = 2e6;
                          const fileInput = this;

                          if (file.size > maxSize) {
                            return toast(
                              "Image should be at most 2MB in size",
                              "black"
                            );
                          }
                          const spinner = $("<div>")
                            .addClass("posAbs")
                            .insertAfter(this)
                            .css({
                              translate: "-50% -50%",
                              top: "50%",
                              left: "50%",
                            })
                            .append(
                              $("<div>")
                                .addClass("spinner-border text-white")
                                .attr("role", "status")
                            );
                          const data = new FormData();
                          data.append(this.name, file);
                          $.ajax({
                            url: `${site.process}custom?case=edit_account`,
                            type: "POST",
                            processData: false,
                            data,
                            contentType: false,
                          })
                            .done(function (jsonRes) {
                              console.log("jsonRes", jsonRes);
                              const res = JSON.parse(jsonRes);
                              $(spinner).remove();
                              fileInput.value = "";

                              if (res.status == 1) {
                                $(fileInput)
                                  .parent()
                                  .find("img")
                                  .attr(
                                    "src",
                                    `${res.picture_ref}?t=${Date.now()}`
                                  );
                                $(".profile-pic").replaceWith(
                                  $("<img>")
                                    .addClass("rounded-circle profile-pic")
                                    .attr({
                                      src: `${res.picture_ref}?t=${Date.now()}`,
                                      alt: "pic",
                                      height: "24",
                                      widith: "24",
                                    })
                                );
                              } else {
                                toast(res.message, "red");
                              }
                            })
                            .fail(function (_, __, err) {
                              console.log("Response err", err);
                              toast("An error occurred", "red");
                            });
                        })
                    )
                )
                .append(
                  $("<p>")
                    .addClass("ml-3")
                    .append(
                      $("<h3>")
                        .addClass("mb-0 account-name")
                        .text(`${user.first_name} ${user.last_name}`)
                    )
                    .append(
                      $("<span>")
                        .addClass("text-muted")
                        .text(`Registered in ${user.reg_yr}`)
                    )
                )
            )
            .append(
              $("<div>")
                .addClass("col-auto")
                .append(
                  $("<a>")
                    .addClass("btn text-white rounded-pill bg-dark")
                    .css("font-size", "larger")
                    .text("Account Settings")
                    .append(
                      '<svg width="1em" height="1em" class="ml-2 align-middle" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg" role="img" focusable="false"><path d="M19 19H5V5h7V3H3v18h18v-9h-2v7ZM14 3v2h3.59l-9.83 9.83 1.41 1.41L19 6.41V10h2V3h-7Z" fill="currentColor"></path></svg>'
                    )
                    .click(() => {
                      this.#reset();
                      this.#createAccountSettingsDialog();
                    })
                )
            )
        )
        .append(
          $("<div>")
            .addClass("row justify-content-between col-lg-11 col-xl-10")
            .append(
              $("<div>")
                .addClass("col-12 col-md-6 mb-4")
                .append(
                  $("<div>")
                    .addClass("collapse multi-collapse-name show")
                    .append(
                      $("<div>")
                        .addClass("form-text d-flex justify-content-between")
                        .append($("<span>").text("Full name"))
                        .append(
                          $("<a>")
                            .addClass("btn btn-sm btn-link py-0")
                            .attr({
                              href: "javascript:void(0);",
                              "data-toggle": "collapse",
                              "data-target": ".multi-collapse-name",
                            })
                            .append(
                              '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                            )
                        )
                    )
                    .append(
                      $("<strong>").text(`${user.first_name} ${user.last_name}`)
                    )
                    .append($("<br>"))
                    .append(
                      $("<span>").text(
                        "This name will appear on your payment receipts and on your certificates."
                      )
                    )
                )
                .append(
                  $("<div>")
                    .addClass("collapse multi-collapse-name")
                    .append(
                      $("<div>")
                        .addClass("d-flex mb-3")
                        .css("gap", "1.5rem")
                        .append(
                          $("<div>")
                            .addClass("col px-0")
                            .append(
                              $("<label>")
                                .addClass("col-form-label")
                                .attr({ for: "user_first_name" })
                                .text("Full name")
                            )
                            .append(
                              $("<input>")
                                .addClass(
                                  "form-control rounded-pill border-dark"
                                )
                                .attr({
                                  type: "text",
                                  id: "user_first_name",
                                  name: "first_name",
                                })
                                .val(user.first_name)
                                .data("value", user.first_name)
                            )
                            .append(
                              $("<div>")
                                .addClass("invalid-feedback")
                                .text("First name is invalid.")
                            )
                        )
                        .append(
                          $("<div>")
                            .addClass("col px-0")
                            .append(
                              $("<label>")
                                .addClass("col-form-label")
                                .attr({ for: "user_last_name" })
                                .text("Last name")
                            )
                            .append(
                              $("<input>")
                                .addClass(
                                  "form-control rounded-pill border-dark"
                                )
                                .attr({
                                  type: "text",
                                  id: "user_last_name",
                                  name: "last_name",
                                })
                                .val(user.last_name)
                                .data("value", user.last_name)
                            )
                            .append(
                              $("<div>")
                                .addClass("invalid-feedback")
                                .text("Last name is invalid.")
                            )
                        )
                    )
                    .append(
                      $("<div>")
                        .addClass("d-grid d-md-block")
                        .append(
                          $("<button>")
                            .addClass(
                              "btn btn-sm rounded-pill text-dark bg-light mr-2"
                            )

                            .attr({
                              type: "button",
                              "data-toggle": "collapse",
                              "data-target": ".multi-collapse-name",
                            })
                            .text("Cancel")
                            .click(function () {
                              $(this)
                                .closest(".collapse")
                                .find("input")
                                .each(function (_, input) {
                                  if (!$(input).val().trim()) {
                                    $(input).val($(input).data("value"));
                                  }

                                  if (
                                    $(input)
                                      .next(".invalid-feedback")
                                      .hasClass("d-block")
                                  ) {
                                    $(input)
                                      .next(".invalid-feedback")
                                      .removeClass("d-block");
                                  }
                                });
                            })
                        )
                        .append(
                          $("<button>")
                            .addClass("btn bg-dark btn-sm rounded-pill px-3")
                            .attr({ type: "button" })
                            .text("Save")
                            .click(this.#changeProfileInfo.bind(this))
                        )
                    )
                )
            )
            .append(
              $("<div>")
                .addClass("col-12 col-md-6 mb-4")
                .append(
                  $("<div>")
                    .addClass("collapse multi-collapse-email show")
                    .append(
                      $("<div>")
                        .addClass("form-text d-flex justify-content-between")
                        .append($("<span>").text("Email"))
                        .append(
                          $("<a>")
                            .addClass("btn btn-sm btn-link py-0")
                            .attr({
                              href: "javascript:void(0);",
                              "data-toggle": "collapse",
                              "data-target": ".multi-collapse-email",
                            })
                            .append(
                              '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                            )
                        )
                    )
                    .append($("<strong>").text(user.email ?? "None"))
                )
                .append(
                  $("<div>")
                    .addClass("collapse multi-collapse-email")
                    .append(
                      $("<div>")
                        .addClass("mb-3")
                        .append(
                          $("<label>")
                            .addClass("col-form-label")
                            .attr({ for: "user-email" })
                            .text("Email")
                        )
                        .append(
                          $("<input>")
                            .addClass("form-control rounded-pill border-dark")
                            .attr({
                              type: "email",
                              id: "user-email",
                              name: "email",
                            })
                            .val(user.email)
                            .data("value", user.email)
                        )
                        .append(
                          $("<div>")
                            .addClass("invalid-feedback")
                            .text("Email is invalid.")
                        )
                    )
                    .append(
                      $("<div>")
                        .addClass("d-grid gap-2 d-md-block")
                        .append(
                          $("<button>")
                            .addClass(
                              "btn btn-sm rounded-pill text-dark bg-light mr-2"
                            )
                            .attr({
                              type: "button",
                              "data-toggle": "collapse",
                              "data-target": ".multi-collapse-email",
                            })
                            .text("Cancel")
                            .click(function () {
                              const commonParent = $(this).closest(".collapse");

                              if (!$("input", commonParent).val().trim()) {
                                $("input", commonParent).val(
                                  $("input", commonParent).data("value")
                                );
                              }

                              if (
                                $(".invalid-feedback", commonParent).hasClass(
                                  "d-block"
                                )
                              ) {
                                $(
                                  ".invalid-feedback",
                                  commonParent
                                ).removeClass("d-block");
                              }
                            })
                        )
                        .append(
                          $("<button>")
                            .addClass("btn bg-dark btn-sm rounded-pill px-3")
                            .attr({ type: "button" })
                            .text("Save")
                            .click(this.#changeProfileInfo.bind(this))
                        )
                    )
                )
            )
            .append(
              $("<div>")
                .addClass("col-12 col-md-6 mb-4")
                .append(
                  $("<div>")
                    .addClass("collapse multi-collapse-phone show")
                    .append(
                      $("<div>")
                        .addClass("form-text d-flex justify-content-between")
                        .append($("<span>").text("Phone"))
                        .append(
                          $("<a>")
                            .addClass("btn btn-sm btn-link py-0")
                            .attr({
                              href: "javascript:void(0);",
                              "data-toggle": "collapse",
                              "data-target": ".multi-collapse-phone",
                            })
                            .append(
                              '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                            )
                        )
                    )
                    .append($("<strong>").text(user.phone ?? "None"))
                )
                .append(
                  $("<div>")
                    .addClass("collapse multi-collapse-phone")
                    .append(
                      $("<div>")
                        .addClass("mb-3")
                        .append(
                          $("<label>")
                            .addClass("col-form-label")
                            .attr({ for: "user-phone" })
                            .text("Phone")
                        )
                        .append(
                          $("<input>")
                            .addClass("form-control rounded-pill border-dark")
                            .attr({
                              type: "tel",
                              id: "user-phone",
                              name: "phone",
                            })
                            .val(user.phone)
                            .data("value", user.phone)
                        )
                        .append(
                          $("<div>")
                            .addClass("invalid-feedback")
                            .text("Phone number is invalid.")
                        )
                    )
                    .append(
                      $("<div>")
                        .addClass("d-grid gap-2 d-md-block")
                        .append(
                          $("<button>")
                            .addClass(
                              "btn btn-sm rounded-pill text-dark bg-light mr-2"
                            )
                            .attr({
                              type: "button",
                              "data-toggle": "collapse",
                              "data-target": ".multi-collapse-phone",
                            })
                            .text("Cancel")
                            .click(function () {
                              const commonParent = $(this).closest(".collapse");

                              if (!$("input", commonParent).val().trim()) {
                                $("input", commonParent).val(
                                  $("input", commonParent).data("value")
                                );
                              }

                              if (
                                $(".invalid-feedback", commonParent).hasClass(
                                  "d-block"
                                )
                              ) {
                                $(
                                  ".invalid-feedback",
                                  commonParent
                                ).removeClass("d-block");
                              }
                            })
                        )
                        .append(
                          $("<button>")
                            .addClass("btn bg-dark btn-sm rounded-pill px-3")
                            .attr({ type: "button" })
                            .text("Save")
                            .click(this.#changeProfileInfo.bind(this))
                        )
                    )
                )
            )
            .append(
              $("<div>")
                .addClass("col-12 col-md-6 mb-4")
                .append(
                  $("<div>")
                    .addClass("collapse multi-collapse-bio show")
                    .append(
                      $("<div>")
                        .addClass("form-text d-flex justify-content-between")
                        .append($("<span>").text("About me"))
                        .append(
                          $("<a>")
                            .addClass("btn btn-sm btn-link py-0")
                            .attr({
                              href: "javascript:void(0);",
                              "data-toggle": "collapse",
                              "data-target": ".multi-collapse-bio",
                            })
                            .append(
                              '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg"><use xlink:href="#editpad"/></svg>'
                            )
                        )
                    )
                    .append($("<strong>").text(user.bio ?? "None"))
                )
                .append(
                  $("<div>")
                    .addClass("collapse multi-collapse-bio")
                    .append(
                      $("<div>")
                        .append(
                          $("<label>")
                            .addClass("col-form-label")
                            .attr({ for: "user-bio" })
                            .text("Short bio")
                        )
                        .append(
                          $("<textarea>")
                            .addClass("form-control rounded-pill border-dark")
                            .attr({
                              type: "text",
                              id: "user-bio",
                              name: "bio",
                              rows: "5",
                              maxlength: 500,
                            })
                            .val(user.bio)
                            .data("value", user.bio)
                            .on("input", function () {
                              $(this)
                                .next("p.form-text")
                                .text(`${$(this).val().length} / 500`);
                            })
                        )
                        .append(
                          $("<p>")
                            .addClass("form-text text-right mb-0")
                            .text("0 / 500")
                        )
                        .append(
                          $("<div>")
                            .addClass("invalid-feedback")
                            .text("Bio is invalid.")
                        )
                    )
                    .append(
                      $("<div>")
                        .addClass("d-grid gap-2 d-md-block")
                        .append(
                          $("<button>")
                            .addClass(
                              "btn btn-sm rounded-pill text-dark bg-light mr-2"
                            )
                            .attr({
                              type: "button",
                              "data-toggle": "collapse",
                              "data-target": ".multi-collapse-bio",
                            })
                            .text("Cancel")
                            .click(function () {
                              const commonParent = $(this).closest(".collapse");

                              if (!$("textarea", commonParent).val().trim()) {
                                $("textarea", commonParent).val(
                                  $("textarea", commonParent).data("value")
                                );
                              }

                              if (
                                $(".invalid-feedback", commonParent).hasClass(
                                  "d-block"
                                )
                              ) {
                                $(
                                  ".invalid-feedback",
                                  commonParent
                                ).removeClass("d-block");
                              }
                            })
                        )
                        .append(
                          $("<button>")
                            .addClass("btn bg-dark btn-sm rounded-pill px-3")
                            .attr({ type: "button" })
                            .text("Save")
                            .click(this.#changeProfileInfo.bind(this))
                        )
                    )
                )
            )
        )
        .append(extraFields);
    });
  }

  #changeProfileInfo(evt) {
    const sendBtn = evt.target,
      collapsible = $(sendBtn).closest(".collapse"),
      inputs = $(collapsible).find("input, textarea"),
      feedback = $(".invalid-feedback", collapsible);
    const profileObj = {};
    let allValid = true;
    $(inputs).each(function (_, input) {
      const value = $(input).val().trim();
      let invalid = false;

      if (!value) invalid = true;

      switch ($(input).attr("type")) {
        case "text":
          if (!value.match(/[^a-zA-Z]/)) invalid = true;
          break;
        case "email":
          if (!input[0].validity.valid) invalid = true;
          break;
        case "phone":
          if (!value.match(/^\+?(\d{11}|\d{13})$/)) invalid = true;
          break;
      }

      if (invalid) {
        if (!$(feedback).hasClass("d-block")) $(feedback).addClass("d-block");
      } else {
        profileObj[input.name] = value;
      }
      allValid = allValid && !invalid;
    });

    if (!allValid) {
      return;
    } else if (
      !Array.prototype.some.call(
        inputs,
        (input) => $(input).val() != $(input).data("value")
      )
    ) {
      $(sendBtn).prev("button").click();
    }
    $(sendBtn)
      .append(
        $("<div>")
          .addClass("spinner-border spinner-border-sm ml-1")
          .attr({ role: "status" })
      )
      .add($(sendBtn).prev("button"))
      .addClass("disabled");
    $.post(
      `${site.process}custom?case=edit_account`,
      profileObj,
      function (jsonRes) {
        console.log("jsonRes", jsonRes);
        const res = JSON.parse(jsonRes);
        $(".spinner-border", sendBtn).remove();
        $(sendBtn).add($(sendBtn).prev("button")).removeClass("disabled");

        if (res.status == 1) {
          $(inputs).each(function (_, input) {
            $(input).data("value", profileObj[input.name]);
          });
          const dispElems = $(collapsible)
            .prev(".collapse")
            .find("strong")
            .add(inputs.length > 1 ? ".account-name" : null);
          const value =
            inputs.length > 1
              ? `${inputs[0].value} ${inputs[1].value}`
              : $(inputs).val();
          $(dispElems).text(value);
          $(sendBtn).prev("button").click();
        } else {
          toast(res.message, "red");
        }
      }
    );
  }

  #editTalentProfile(evt, other) {
    const saveBtn = evt.target,
      collapsible = $(saveBtn).closest(".collapse"),
      inputs = $("input, select", collapsible);
    const dataId = $(saveBtn).data("id");
    const profileObj = { talent: true };
    let allValid = true;
    let newObj;

    if (!other) {
      $(inputs).each(function (_, input) {
        const value = $(input).val().trim();

        if (!value) {
          if (!$(input).hasClass("is-invalid")) $(input).addClass("is-invalid");

          if (allValid) allValid = false;
        } else {
          if (input.name == "state" || input.name == "country") {
            let location = profileObj["location"] ?? "";
            location += location ? `, ${value}` : value;
            profileObj["location"] = location;
          } else {
            profileObj[input.name] = value;
          }
        }
      });

      if (!allValid) {
        return;
      } else if (
        !Array.prototype.some.call(
          inputs,
          (input) => $(input).val() != $(input).data("value")
        )
      ) {
        $(saveBtn).prev("button").click();
      }
    } else {
      switch (dataId) {
        case "education":
        case "experience":
        case "projects":
          const arr = other.filter((obj) => !obj.deleted);
          for (let i = 0; i < arr.length; i++) {
            const obj = arr[i];

            for (const key in obj) {
              const val = obj[key];
              if (!val) {
                if (allValid) allValid = false;

                if (key == "degree") {
                  const degreeInput = $("#degree_" + i),
                    courseInput = $("#course_" + i);

                  if (
                    !$(degreeInput).val() &&
                    !$(degreeInput).hasClass("is-invalid")
                  ) {
                    $(degreeInput).addClass("is-invalid");
                  }

                  if (
                    !$(courseInput).val() &&
                    !$(courseInput).hasClass("is-invalid")
                  ) {
                    $(courseInput).addClass("is-invalid");
                  }
                } else if (key == "period" || key == "duration") {
                  if (dataId == "education") {
                    const feedback = $("#period_" + i).next(
                      ".invalid-feedback"
                    );

                    if (!$(feedback).hasClass("d-block")) {
                      $(feedback).addClass("d-block");
                    }
                  } else {
                    const startInput = $("#start_year_" + i),
                      endInput = $("#end_year_" + i);

                    if (
                      !$(startInput).val() &&
                      !$(startInput).hasClass("is-invalid")
                    ) {
                      $(startInput).addClass("is-invalid");
                    }

                    if (
                      !$(endInput).val() &&
                      !$(endInput).hasClass("is-invalid")
                    ) {
                      $(endInput).addClass("is-invalid");
                    }
                  }
                } else {
                  const input = $(`[id$=${key}_${i}]`, collapsible);
                  console.log(
                    "input",
                    input[0],
                    input[0]?.value,
                    key,
                    val,
                    obj
                  );

                  if (!$(input).val() && !$(input).hasClass("is-invalid")) {
                    $(input).addClass("is-invalid");
                  }
                }
              }
            }
          }

          if (!allValid) return;

        case "skills":
          if (!other.length) {
            toast(`No ${dataId} selected`, "red");
            return;
          }
          newObj = other
            .filter((obj) => !obj.deleted)
            .map((obj) => ({ ...obj, new: undefined }));
          profileObj[dataId] = newObj;
          break;
        case "socials":
          newObj = {};
          $("#facebook-url, #x-url, #linkedin-url, #github-url").each(
            function () {
              const value = $(this).val().trim();

              if (this.validity.valid) {
                if ($(this).hasClass("is-invalid"))
                  $(this).removeClass("is-invalid");
                newObj[this.name] = value;
              } else if (value) {
                if (allValid) allValid = false;

                if (!$(this).hasClass("is-invalid"))
                  $(this).addClass("is-invalid");
              }
            }
          );

          if (!allValid || !Object.keys(newObj).length) return;
          profileObj.socials = newObj;
          break;
      }
    }

    $(saveBtn)
      .append(
        $("<div>")
          .addClass("spinner-border spinner-border-sm ml-1")
          .attr({ role: "status" })
      )
      .add($(saveBtn).prev("button"))
      .addClass("disabled");
    $.post(`${site.process}custom?case=edit_account`, profileObj, (jsonRes) => {
      console.log("jsonRes", jsonRes);
      const res = JSON.parse(jsonRes);
      $(".spinner-border", saveBtn).remove();
      $(saveBtn).add($(saveBtn).prev("button")).removeClass("disabled");
      const otherCollapsible = $(collapsible).prev(".collapse");

      if (res.status == 1) {
        if (other && Array.isArray(other)) {
          other.splice(0, other.length, ...newObj);
          $(otherCollapsible).find(".card-text")
            .empty()
            .append(this.#displayTalentProfile(other, dataId));
          $(saveBtn).prev("button").trigger("click", [false]);
        } else {
          if (other) {
            for (const key in newObj) {
              const val = newObj[key];
              other[key] = val;
              $(`.${key}`, otherCollapsible).text(val);
            }
          }
          const dispElem = $(otherCollapsible).find("strong");

          if (inputs.length == 2) {
            const locArr = profileObj.location.split(","),
              len = locArr.length,
              state = len > 1 ? locArr[0] : "",
              country = len > 1 ? locArr[1] : locArr[0];
            $(inputs).each(function () {
              if (this.name == "state") $(this).data("value", state);
              else if (this.name == "country") $(this).data("value", country);
              $(dispElem).text(profileObj.location);
            });
          } else {
            $(inputs).each(function () {
              if (!other) $(dispElem).text(profileObj[this.name]);
              $(this).data("value", profileObj[this.name]);
            });
          }
          $(saveBtn).prev("button").click();
        }
      } else {
        toast(res.message, "red");
      }
    }).catch(() => {
      toast("Could not edit your talent profile", "black");
    });
  }

  #createAccountSettingsDialog() {
    $("#generalDialog .modal-dialog")
      .addClass("modal-lg m-0")
      .removeClass("modal-sm modal-dialog-centered")
      .find(".modal-content")
      .css({ width: "100vw", "min-width": "100vw", height: "100vh" })
      .find(".modal-header")
      .addClass("border-bottom py-3")
      .find("button")
      .css("top", "20px");

    $("#generalDialogLabel").text("Account settings");
  }

  #createNotificationDialog() {}

  #addTopic() {
    const topic = $("#topicInput").val().trim(),
      oldname = $("#topicInput").data("oldname");

    if (this.#editedLessonCard) {
      $(this.#editedLessonCard).find("a:only-of-type").first().text(topic);
      const topicObj = $(this.#editedLessonCard).data("topic");
      topicObj.topic = topic;
    } else {
      if ($("#courseContents").find(".empty-lesson")[0])
        $("#courseContents .empty-lesson").remove();
      const suffix = Date.now().toString().slice(-10);
      const topicObj = {
        topic,
        id: `topic-${suffix}`,
        editable: true,
        lessons: [],
        index: courseObj.topics.length,
      };
      const topicItem = this.returnTopicCard(topicObj, suffix);
      setTimeout(() => {
        $(topicItem)[0].scrollIntoView({ behavior: "smooth", block: "start" });
      }, 800);
      courseObj.topics.push(topicObj);

      if (courseObj.topics.length >= 2) {
        $("#courseContents")
          .children(".card:first-child")
          .find(".move-item")
          .removeClass("disabled text-muted");
      }
    }
    if (topic != oldname) saveCourse(false, true);
    this.#onTopicChange(courseObj.topics.filter((topic) => topic.editable));
  }

  returnTopicCard(topicObj, suffix) {
    const topicItem = $("<div>")
      .addClass("mb-5")
      .appendTo($("#courseContents"))
      .append(
        $("<div>")
          .addClass("card box-shadow no-scaling")
          .css("overflow", "visible")
          .data({ topic: topicObj, type: "topic" })
          .append(
            $("<div>")
              .addClass("card-body")
              .css("min-height", "unset")
              .append(
                $("<h5>")
                  .addClass("card-title dropdown w-100 d-flex mb-4")
                  .append(
                    $("<a>")
                      .addClass("flex-grow-1 text-decoration-none mr-4")
                      .attr({
                        href: `#topic-${suffix}`,
                        role: "button",
                        "aria-expanded": "false",
                        "aria-controls": `topic-${suffix}`,
                        "data-toggle": "collapse",
                      })
                      .css("border-bottom", "1px solid currentColor")
                      .text(topicObj.topic)
                  )
                  .append(
                    topicObj.editable
                      ? $("<i>")
                          .addClass("fa fa-ellipsis-v")
                          .attr("data-toggle", "dropdown")
                          .css("cursor", "pointer")
                      : ""
                  )
                  .append(
                    topicObj.editable
                      ? $("<div>")
                          .addClass("dropdown-menu")
                          .append(
                            $("<a>")
                              .addClass("dropdown-item")
                              .attr({
                                href: "javascript:void(0)",
                                "data-toggle": "modal",
                                "data-target": "#generalDialog",
                                "data-action": "lesson",
                              })
                              .prop("edit", true)
                              .text("Rename")
                          )
                          .append(
                            $("<a>")
                              .addClass(
                                `dropdown-item move-item${
                                  courseObj.topics.length == 0
                                    ? " disabled text-muted"
                                    : ""
                                }`
                              )
                              .attr({
                                href: "javascript:void(0)",
                                "data-toggle": "modal",
                                "data-target": "#generalDialog",
                                "data-action": "rearrange",
                                "data-type": "topic",
                              })
                              .text("Move")
                          )
                          .append(
                            $("<a>")
                              .addClass("dropdown-item")
                              .attr({
                                href: "javascript:void(0)",
                                "data-toggle": "modal",
                                "data-target": "#generalDialog",
                                "data-action": "confirm",
                              })
                              .text("Delete")
                              .prop("callback", () => async (confirmed) => {
                                if (confirmed) {
                                  const parentCard =
                                      $(topicItem).children(".card"),
                                    topicObj = $(parentCard).data("topic");
                                  const index = courseObj.topics.findIndex(
                                    (obj) => obj == topicObj
                                  );
                                  courseObj.topics.splice(index, 1);

                                  try {
                                    await new Promise((res, rej) => {
                                      if (topicObj._id) {
                                        $.post(
                                          `${site.process}api`,
                                          {
                                            option: "delete_topic",
                                            topic_id: topicObj._id,
                                          },
                                          function (jsonRes) {
                                            const response =
                                              JSON.parse(jsonRes);

                                            if (response.status == 1) res(true);
                                            else rej(response.message);
                                          }
                                        ).catch((err) => {
                                          rej(err);
                                        });
                                      } else {
                                        res(true);
                                      }
                                    });
                                    $(parentCard).parent().remove();

                                    if (
                                      $("#courseContents").children().length ==
                                      0
                                    ) {
                                      $("#courseContents").append(
                                        $("<div>")
                                          .addClass(
                                            "rounded border d-flex justify-content-center align-items-center mb-4 col-11 mx-auto empty-lesson"
                                          )
                                          .css("height", "10rem")
                                          .append(
                                            $("<h5>").text(
                                              "No content created yet"
                                            )
                                          )
                                      );
                                    } else if (courseObj.topics.length == 1) {
                                      $("#courseContents")
                                        .children(".card:only-child")
                                        .find(".move-item")
                                        .addClass("disabled text-muted");
                                    }
                                    this.#onTopicChange(
                                      courseObj.topics.filter(
                                        (topic) => topic.editable
                                      )
                                    );
                                  } catch (err) {
                                    console.error(err);
                                    toast("Deletion was unsuccessful", "red");
                                  }
                                }
                              })
                          )
                      : ""
                  )
              )
              .append(
                $("<div>")
                  .addClass("dropdown float-right")
                  .append(
                    $("<a>")
                      .addClass("btn btn-sm bg-light text-dark shadow-0")
                      .attr({
                        href: "javascript:void(0);",
                        "data-toggle": "dropdown",
                      })
                      .css("font-size", "small")
                      .append($("<i>").addClass("fa fa-plus mr-1"))
                      .append("New lesson")
                  )
                  .append(
                    $("<div>")
                      .addClass("dropdown-menu mt-2 mr-4")
                      .append(
                        $("<a>").addClass("dropdown-item").attr({
                          href: "#generalDialog",
                          "data-toggle": "modal",
                          "data-action": "lesson",
                        })
                          .append(`<svg focusable="false" width="1.2em" height="1.2em" class="align-middle">
                      <use xlink:href="#topic-icon"></use>
                    </svg> Topic`)
                      )
                      .append(
                        $("<a>").addClass("dropdown-item").attr({
                          href: "#lessonDialog",
                          "data-toggle": "modal",
                          "data-action": "material",
                        })
                          .append(`<svg focusable="false" width="1.2em" height="1.2em" class="align-middle">
                      <use xlink:href="#material-icon"></use></svg> Material`)
                      )
                      .append(
                        $("<a>").addClass("dropdown-item").attr({
                          href: "#lessonDialog",
                          "data-toggle": "modal",
                          "data-action": "assignment",
                        })
                          .append(`<svg focusable="false" width="1.2em" height="1.2em" class="align-middle">
                      <use xlink:href="#assignment-icon"></use></svg> Assignment`)
                      )
                      .append(
                        $("<a>").addClass("dropdown-item").attr({
                          href: "#lessonDialog",
                          "data-toggle": "modal",
                          "data-action": "quiz",
                        })
                          .append(`<svg focusable="false" width="1.2em" height="1.2em" class="align-middle">
                      <use xlink:href="#quiz-icon"></use></svg> Quiz`)
                      )
                  )
              )
              .append($("<p>").text("● No lessons added"))
              .append(
                $("<div>").addClass("collapse").attr("id", `topic-${suffix}`)
              )
          )
      );
    return topicItem;
  }

  #takeQuiz({ questions, grading, callback, title, instruction, key }) {
    let shouldSlide = false,
      timer,
      timeB4,
      dateTaken;
    const { totalPoints, closeSubmitAfterDue, dueDate, duration } = grading;
    const qcount = questions.reduce(
      (accum, item) => (item.qtype != "section" ? accum + 1 : accum),
      0
    );
    const mark =
      totalPoints == "Ungraded"
        ? "Ungraded"
        : parseFloat((parseInt(totalPoints) / qcount).toFixed(1));

    $("#generalDialog .modal-dialog")
      .addClass("modal-lg m-0")
      .removeClass("modal-sm modal-dialog-centered")
      .find(".modal-content")
      .css({ width: "100vw", "min-width": "100vw", height: "100vh" })
      .find(".modal-header")
      .addClass("border-bottom py-2")
      .find("button")
      .css("top", "20px");

    $("#generalDialogLabel").html(
      `<svg width="1em" height="1em" class="align-middle rounded-circle svg-bg mr-2"><use xlink:href="#quiz-icon"/></svg>${title}`
    );
    $("#generalDialog .modal-body")
      .addClass("p-0")
      .css("overflow", "hidden auto");
    const comparison = GenericDialog.compareWithDate(dueDate);

    if (
      comparison == "before" ||
      (comparison == "after" && closeSubmitAfterDue)
    ) {
      $("#generalDialog .modal-body")
        .append(
          $("<div>")
            .addClass(
              "w-100 h-100 d-flex justify-content-center align-items-center"
            )
            .append(
              $("<h5>")
                .addClass("w-50 text-justify")
                .css("min-width", "450px")
                .append($("<i>").addClass("fas fa-exclamation-triangle mr-2"))
                .append(
                  (comparison == "before"
                    ? "You can't take the quiz now as it's not time."
                    : "You can't take the quiz because the scheduled date for the quiz has passed.") +
                    " If you think this message is in error, please report the issue to your tutor."
                )
            )
        )
        .after(
          $("<div>")
            .addClass("modal-footer py-2 border-top")
            .append(
              $("<button>")
                .addClass("btn text-dark bg-light mr-3 px-4")
                .attr({ type: "button", "data-dismiss": "modal" })
                .text("Close")
            )
        );
    } else {
      $("#generalDialogLabel")
        .next("button.close")
        .replaceWith(
          $("<span>")
            .text(duration == "Not timed" ? "Not timed" : "Remaining time: ")
            .append(
              duration != "Not timed"
                ? $("<strong>")
                    .addClass("d-inline-block text-end")
                    .text(duration)
                : ""
            )
        );

      $("#generalDialog .modal-body")
        .append(
          $("<div>")
            .addClass("carousel slide")
            .attr({
              id: "quizCbt",
              "data-touch": "false",
              "data-keyboard": "false",
            })
            .append(
              $("<div>")
                .addClass("carousel-inner shadow-0")
                .append(
                  $("<div>")
                    .addClass("carousel-item mt-5 active")
                    .append(
                      $("<fieldset>")
                        .addClass("m-auto")
                        .css({ "max-width": "700px" })
                        .append($("<legend>").text("Before you start"))
                        .append(
                          $("<p>").html(
                            `${
                              instruction && instruction + "<br>"
                            }Once you've started the quiz, you won't be able to go back until you've submitted the quiz.<br>If the quiz is timed, your response will be automatically submitted once your time elapses.<br>Finally, be calm as this is only a test of what you've learnt.`
                          )
                        )
                    )
                )
            )
            .on({
              "slide.bs.carousel": function (evt) {
                if (shouldSlide) {
                  shouldSlide = false;
                } else {
                  evt.preventDefault();
                }
              },
              contextmenu: function (evt) {
                evt.preventDefault();
              },
              "slid.bs.carousel": function () {
                const items = $(".carousel-item", this),
                  activeElem = $(".carousel-item.active")[0],
                  firstElem = $(items)[1],
                  lastElem = $(items).last()[0],
                  startBtn = $("#startQuizBtn")[0],
                  prevBtn = $(
                    '#generalDialog .modal-footer button[data-slide="prev"]'
                  ),
                  nextBtn = $(
                    '#generalDialog .modal-footer button[data-slide="next"]'
                  );

                if (startBtn) {
                  $(startBtn).prev("button").remove();
                  $("<button>")
                    .addClass("btn bg-white text-primary mr-3 disabled")
                    .attr({
                      type: "button",
                      "data-target": "#quizCbt",
                      "data-slide": "prev",
                    })
                    .css("border", "2px solid #fa7a50")
                    .text("Previous")
                    .click(() => (shouldSlide = true))
                    .replaceAll(startBtn)
                    .after(
                      $("<button>")
                        .addClass("btn btn-primary px-4")
                        .attr({
                          "data-target": "#quizCbt",
                          "data-slide": "next",
                        })
                        .text("Next")
                        .click(() => (shouldSlide = true))
                    );
                } else if (firstElem == activeElem) {
                  $(prevBtn).addClass("disabled");
                } else if (lastElem == activeElem) {
                  $(nextBtn).addClass("disabled");
                  $("#submitQuiz").removeClass("d-none");
                } else {
                  if (!$("#submitQuiz").hasClass("d-none")) {
                    $("#submitQuiz").addClass("d-none");
                  }

                  if ($(prevBtn).hasClass("disabled")) {
                    $(prevBtn).removeClass("disabled");
                  } else if ($(nextBtn).hasClass("disabled")) {
                    $(nextBtn).removeClass("disabled");
                  }
                }
              },
            })
        )
        .after(
          $("<div>")
            .addClass("modal-footer py-2 border-top")
            .append(
              $("<div>")
                .addClass("d-grid gap-2 d-md-flex justify-content-md-end")
                .append(
                  $("<button>")
                    .addClass("btn bg-light shadow-0 px-4 mr-3 text-dark")
                    .attr("data-dismiss", "modal")
                    .text("Close")
                )
                .append(
                  $("<button>")
                    .addClass("btn btn-primary")
                    .attr({
                      "data-target": "#quizCbt",
                      "data-slide-to": "1",
                      id: "startQuizBtn",
                    })
                    .text("Start quiz")
                    .click(() => {
                      shouldSlide = true;

                      if (duration != "Not timed")
                        this.#startTimer(duration, timerCallbefore, onQuizDone);
                    })
                )
                .append(
                  $("<button>")
                    .addClass("btn btn-success ml-auto d-none")
                    .attr({
                      id: "submitQuiz",
                    })
                    .text("Submit")
                    .click(function () {
                      if (!$(this).prop("dismiss")) onQuizDone();
                      $("#generalDialog").modal("hide");
                    })
                )
            )
        );
      const qcontainer = $("#quizCbt .carousel-inner");
      const timerCallbefore = (id, time) => {
        timer = id;
        timeB4 = time;
        dateTaken = GenericDialog.getCurrentDateTime();
      };
      const onQuizDone = (timeElapsed = false) => {
        let answer_count = 0;
        const response = this.#copyObj(questions),
          qElems = $(".carousel-item", qcontainer);
        const score =
          mark == "Ungraded"
            ? "Ungraded"
            : questions.reduce((accum, { qtype, options, answer }, i) => {
                if (qtype != "section") {
                  const qElem = $(qElems)[i + 1];

                  if (qtype == "multiple") {
                    const selected = $("input:checked", qElem)
                      .next("label")
                      .data("option");

                    if (
                      options.some(
                        (option) =>
                          option.correct == "true" &&
                          option.optionTxt == selected
                      )
                    ) {
                      accum += mark;
                      response[i].correct = true;
                    } else {
                      response[i].correct = false;
                    }
                    response[i].selected = selected;
                  } else {
                    const shortAnwer = $("textarea", qElem)
                      .val()
                      .trim()
                      .toLowerCase();
                    answer = answer.toLowerCase();

                    if (shortAnwer == answer) {
                      accum += mark;
                      response[i].correct = true;
                    } else {
                      response[i].correct = false;
                    }
                    response[i].selected = shortAnwer;
                  }

                  if (response[i].selected != "") answer_count++;
                }
                return accum;
              }, 0);
        const record = {
          key,
          response,
          score,
          qcount,
          answer_count,
          points: totalPoints,
          duration,
          reviewed: false,
        };
        record.date_taken = dateTaken;
        record.time_taken =
          duration == "Not timed"
            ? "Not timed"
            : timeElapsed
            ? timeB4
            : this.#timeDifference(timeB4);

        if (timeElapsed) {
          $("input, textarea", qcontainer).prop("disabled", true);
          $("#submitQuiz")
            .prop("dismiss", true)
            .removeClass("d-none")
            .parent()
            .children(":not(#submitQuiz)")
            .remove();
          const lastIndex = $(qElems).index($(qElems).last()[0]);
          $("#quizCbt").carousel(lastIndex);
        } else {
          clearInterval(timer);
        }
        callback(record, score);
      };
      let qnum = 1;

      for (const {
        qtype,
        section,
        question,
        qImage,
        instruct,
        options,
      } of questions) {
        const qElem = $("<div>")
            .addClass("carousel-item mt-5 mx-5")
            .appendTo(qcontainer)
            .append(
              qtype != "section"
                ? $("<span>")
                    .addClass(
                      "float-right mr-5 px-3 py-2 bg-light rounded-circle"
                    )
                    .text(mark)
                : ""
            ),
          imgHolder = qImage
            ? $("<figure>").append(
                $("<div>")
                  .addClass("card border-0")
                  .css("max-width", "12rem")
                  .append(
                    $("<img>")
                      .attr({ alt: "", src: qImage })
                      .addClass("card-img")
                  )
              )
            : "";

        switch (qtype) {
          case "section": {
            $(qElem)
              .append(
                $("<h6>")
                  .addClass("font-weight-bold")
                  .text(section)
                  .append(imgHolder)
              )
              .append(
                instruct ? $("<p>").addClass("card-text").text(instruct) : ""
              );
            break;
          }

          case "multiple":
          case "short":
            $(qElem).append(
              $("<dl>").append(
                $("<dt>")
                  .addClass("mb-3")
                  .append(
                    $("<span>")
                      .addClass("mr-3")
                      .text(qnum + ".")
                  )
                  .append(question)
                  .append(imgHolder)
              )
            );

            if (qtype == "multiple") {
              const dd = $("<dd>").appendTo(qElem),
                optionsLen = options.length;

              for (let j = 0; j < optionsLen; j++) {
                const { optionTxt, optImage } = options[j];
                $(dd).append(
                  $("<div>")
                    .addClass(
                      "custom-control custom-radio d-flex align-items-center"
                    )
                    .append(
                      $("<input>")
                        .addClass("custom-control-input")
                        .attr({
                          type: "radio",
                          name: `que${qnum}`,
                          id: `q${qnum}-opt${j}`,
                        })
                    )
                    .append(
                      $("<label>")
                        .addClass("custom-control-label")
                        .attr({ for: `q${qnum}-opt${j}` })
                        .text(optionTxt)
                        .data("option", optionTxt)
                    )
                    .append(
                      optImage
                        ? $("<img>")
                            .addClass("img-thumbnail ml-3")
                            .css({
                              "object-fit": "contain",
                              "max-height": "120",
                              "max-width": "120",
                            })
                            .attr({ src: optImage, alt: "" })
                        : ""
                    )
                );
              }
              qnum++;
              break;
            }
            $(qElem).append(
              $("<dd>").append(
                $("<textarea>")
                  .addClass("form-control txtbox p-0 border-dark w-75")
                  .attr({
                    id: `qshortAnswer${qnum}`,
                    placeholder: "Short answer",
                  })
                  .css({ "font-size": "small", height: "2rem" })
              )
            );
            qnum++;
            break;
        }
      }
      $("<div>")
        .addClass("carousel-item mt-5 mx-5")
        .appendTo(qcontainer)
        .append(
          $("<h5>").addClass("font-weight-bold text-center").text("End of Quiz")
        );
    }
  }

  #takeSurvey(
    { title, description, id: survey_id, close_date },
    edit_response = false
  ) {
    let questionArr;
    $("#generalDialog .modal-dialog")
      .addClass("modal-lg m-0")
      .removeClass("modal-sm modal-dialog-centered")
      .find(".modal-content")
      .css({ width: "100vw", "min-width": "100vw", height: "100vh" })
      .find(".modal-header")
      .addClass("border-bottom py-2")
      .find("button")
      .css("top", "20px");

    $("#generalDialogLabel").html(
      `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="align-middle rounded-circle svg-bg mr-2">x<line x1="18" y1="20" x2="18" y2="10"></line><line x1="12" y1="20" x2="12" y2="4"></line><line x1="6" y1="20" x2="6" y2="14"></line></svg>${title}`
    );
    const container = $("<div>")
      .addClass("row")
      .appendTo(
        $("#generalDialog .modal-body")
          .addClass("bg-light p-0")
          .css("overflow", "hidden auto")
          .after(
            $("<div>")
              .addClass("modal-footer py-2 border-top")
              .append(
                $("<div>")
                  .addClass("d-grid gap-2 d-md-flex justify-content-md-end")
                  .append(
                    $("<button>")
                      .addClass("btn bg-light shadow-0 px-4 mr-3 text-dark")
                      .attr("data-dismiss", "modal")
                      .text("Close")
                  )
                  .append(
                    $("<button>")
                      .addClass("btn btn-success disabled")
                      .attr({
                        id: "submitSurvey",
                      })
                      .text("Submit response")
                      .click(() => {
                        const cols = $(
                            "#generalDialog .modal-body .row .col-12:nth-child(n+2)"
                          ),
                          colsLen = cols.length,
                          date_submitted = GenericDialog.getCurrentDateTime(),
                          response = this.#copyObj(questionArr);

                        for (let i = 0; i < colsLen; i++) {
                          const col = cols[i],
                            qtype = $(col).prop("qtype");
                          let uncompleted = false;

                          switch (qtype) {
                            case "multiple":
                              if (!$("input:checked", col)[0]) {
                                response[i].answer = $("input:checked", col)
                                  .next("label")
                                  .text();
                              } else {
                                uncompleted = true;
                              }
                              break;
                            case "short":
                              const shortAnswer = $("textarea", col)
                                .val()
                                .trim();

                              if (/\b\w+\b/.test(shortAnswer)) {
                                console.log("response", i, colsLen, response);
                                response[i].answer = shortAnswer;
                              } else {
                                uncompleted = true;
                              }
                              break;
                          }

                          if (uncompleted) {
                            toast("You have unfilled fields", "black");
                            return;
                          }
                        }
                        const responder = { date_submitted, response };
                        submitResponse(responder, survey_id, edit_response);
                        $("#generalDialog").modal("hide");
                      })
                  )
              )
          )
      );
    $("<div>")
      .addClass("col-12 d-flex justify-content-center")
      .appendTo(container)
      .append(
        $("<div>")
          .addClass("card no-scaling shadow-0 mb-3 mt-2 w-100")
          .css({ "max-width": "540px" })
          .append(
            $("<div>")
              .addClass("card-header bg-transparent")
              .css("border-top", "4px solid #1f88c0")
              .append($("<h5>").addClass("card-title").text(title))
          )
          .append(
            $("<div>")
              .addClass("card-body")
              .css({ "min-height": "unset" })
              .append($("<p>").addClass("card-text").text(description))
          )
          .append(
            $("<div>")
              .addClass(
                "card-footer bg-transparent border-0 d-flex justify-content-between"
              )
              .append($("<span>").text("Close date"))
              .append($("<span>").text(close_date))
          )
      );
    const loader = loading();
    $(container).append(loader);
    $.post(
      `${site.process}student?option=survey_form`,
      { survey_id, edit_response },
      (jsonRes) => {
        try {
          const { questions, response: oldResponse } = JSON.parse(jsonRes);
          let qnum = 0;
          questionArr = questions;
          console.log("oldResponse", oldResponse);
          $(loader).remove();
          $("#submitSurvey").removeClass("disabled");

          if (oldResponse) $("#submitSurvey").text("Edit response");

          for (const {
            qtype,
            section,
            question,
            instruct,
            options,
          } of questions) {
            const col = $("<div>")
              .prop("qtype", qtype)
              .addClass("col-12 d-flex justify-content-center")
              .appendTo(container)
              .append(
                $("<div>")
                  .addClass("card no-scaling shadow-0 mb-3 mt-2 w-100")
                  .css("max-width", "540px")
                  .append(
                    $("<div>").addClass("card-body").css("min-height", "unset")
                  )
              );

            switch (qtype) {
              case "section": {
                $(".card-body", col)
                  .append($("<h6>").addClass("font-weight-bold").text(section))
                  .append(
                    instruct
                      ? $("<p>").addClass("card-text").text(instruct)
                      : ""
                  );
                break;
              }

              case "short":
              case "multiple":
                $(".card-body", col).append(
                  $("<dl>").append($("<dd>").addClass("mb-3").append(question))
                );

                if (qtype == "short") {
                  $(".card-body", col).append(
                    $("<dd>").append(
                      $("<textarea>")
                        .addClass("form-control txtbox p-0 border-dark")
                        .attr({
                          id: `qshortAnswer${qnum}`,
                          placeholder: "Short answer",
                        })
                        .val(oldResponse ? oldResponse[qnum].answer : "")
                        .css({ "font-size": "small", height: "2rem" })
                        .on("input", this.#resize.bind(this))
                    )
                  );
                  qnum++;
                  break;
                }
                const dd = $("<dd>").appendTo($(".card-body", col)),
                  optionsLen = options.length;

                for (let j = 0; j < optionsLen; j++) {
                  const { optionTxt } = options[j];
                  $(dd).append(
                    $("<div>")
                      .addClass(
                        "custom-control custom-radio d-flex align-items-center"
                      )
                      .append(
                        $("<input>")
                          .addClass("custom-control-input")
                          .attr({
                            type: "radio",
                            name: `que${qnum}`,
                            id: `q${qnum}-opt${j}`,
                          })
                      )
                      .append(
                        $("<label>")
                          .addClass("custom-control-label")
                          .attr({ for: `q${qnum}-opt${j}` })
                          .text(optionTxt)
                          .data("option", optionTxt)
                      )
                  );

                  if (oldResponse?.answer == optionTxt) {
                    $(`#q${qnum}-opt${j}`).prop("checked", true);
                  }
                }

                qnum++;
                break;
            }
          }
        } catch {
          $("#submitSurvey").remove();

          if (jsonRes == "None" || jsonRes == "Closed") {
            $("<div>")
              .addClass("card no-scaling shadow-0 mb-3 mt-2 w-100")
              .css({ "max-width": "540px" })
              .append(
                $("<div>")
                  .addClass("card-body")
                  .css({ "min-height": "unset" })
                  .append(
                    $("<p>")
                      .addClass("w-50 text-justify")
                      .css("min-width", "450px")
                      .append(
                        $("<i>").addClass("fas fa-exclamation-triangle mr-2")
                      )
                      .append(
                        jsonRes == "None"
                          ? "The form could not be retrieved. You can report to your tutor or persons in charge of the issue."
                          : "The closing date for the survey has passed. As such, your response won't be collected. If you think this message is in error, please report the issue to your tutor or persons in charge."
                      )
                  )
              )
              .replaceAll(loader);
          } else {
            console.error(jsonRes);
          }
        }
      }
    );
  }

  #fetchResponse(survey) {}

  #resize(evt) {
    console.log("yeah");
    const input = evt.target;
    const cssHeight = parseFloat($(input).css("height").replace("px", "")),
      scrollHeight = $(input).prop("scrollHeight");
    if (!$(input).prop("minHeight")) $(input).prop("minHeight", cssHeight);
    const minHeight = $(input).prop("minHeight");

    if (cssHeight < scrollHeight) {
      $(input).css("height", scrollHeight + "px");
    } else if (cssHeight > scrollHeight && cssHeight > minHeight) {
      const newHeight = Math.max(scrollHeight, minHeight);
      $(input).css("height", newHeight + "px");
    }
  }

  #timeDifference(time) {
    // Parse time strings to extract hours, minutes, and seconds
    const [sec1, min1, hr1 = 0] = time.split(":").map(Number).reverse();
    const [sec2, min2, hr2 = 0] = $("#generalDialog .modal-header strong")
      .prop("time")
      .split(":")
      .map(Number)
      .reverse();

    // Convert each time to seconds
    const totalSec1 = hr1 * 3600 + min1 * 60 + sec1;
    const totalSec2 = hr2 * 3600 + min2 * 60 + sec2;

    // Calculate time difference in seconds
    let diffSeconds = Math.abs(totalSec1 - totalSec2);

    // Convert difference back to hours, minutes, and seconds
    const diffHours = Math.floor(diffSeconds / 3600);
    diffSeconds %= 3600;
    const diffMinutes = Math.floor(diffSeconds / 60);
    diffSeconds %= 60;

    // Format result
    var result = `${diffHours}:${diffMinutes}:${diffSeconds}`;

    return result;
  }

  #copyObj(obj) {
    if (typeof obj !== "object" || obj === null) {
      return obj;
    }
    const copy = Array.isArray(obj) ? [] : {};

    // Copy each property recursively
    for (const key in obj) {
      if (obj.hasOwnProperty(key)) {
        copy[key] = this.#copyObj(obj[key]);
      }
    }
    return copy;
  }

  static compareWithDate(datetime) {
    const refDate = new Date(datetime.replace(/(\d+)(st|nd|rd|th)/g, "$1")),
      now = new Date(),
      endDate = new Date(refDate);
    endDate.setDate(endDate.getDate() + 1);

    if (now < refDate) {
      return "before";
    } else if (now >= refDate && now < endDate) {
      return "current";
    } else {
      return "after";
    }
  }

  static getCurrentDateTime() {
    // Get current date and time
    const currentDate = new Date();

    // Format the date
    const day = currentDate.getDate();
    const month = currentDate.toLocaleString("default", { month: "short" });
    const year = currentDate.getFullYear();

    // Format the time
    let hours = currentDate.getHours();
    let minutes = currentDate.getMinutes();
    const ampm = hours >= 12 && hours < 24 ? "PM" : "AM";
    hours = hours % 12;
    hours = hours ? hours : 12;
    minutes = minutes < 10 ? "0" + minutes : minutes;

    // Construct the formatted date and time string
    const formattedDateTime =
      day +
      this.getDaySuffix(day) +
      " " +
      month +
      ", " +
      year +
      " " +
      hours +
      ":" +
      minutes +
      " " +
      ampm;

    return formattedDateTime;
  }

  static getDaySuffix(day) {
    if (day >= 11 && day <= 13) {
      return "th";
    }
    switch (day % 10) {
      case 1:
        return "st";
      case 2:
        return "nd";
      case 3:
        return "rd";
      default:
        return "th";
    }
  }

  #startTimer(duration, callbefore, callback) {
    const match = duration.match(/(?:(\d+)\s*hr\s*)?(?:(\d+)\s*min\s*)?/);
    const timeObj = {},
      hr = match[1] ? match[1] + ":" : "",
      min = match[2] ? match[2] : "0",
      time = hr.concat(hr != "" ? min.padStart(2, "0") : min, ":00");
    timeObj.sec = 0;
    timeObj.min = match[2] ? parseInt(match[2]) : 0;
    timeObj.hr = match[1] ? parseInt(match[1]) : 0;
    console.log();
    const countdownElem = $("#generalDialog .modal-header strong").text(time);
    $(countdownElem).css(
      "width",
      `calc(${$(countdownElem).css("width")} + .5rem)`
    );
    const timer = setInterval(
      (timeObj) => {
        let { sec, min, hr } = timeObj,
          time = "";

        if (sec == 0) {
          if (min > 0) {
            sec = 59;
            min--;
          } else if (hr > 0) {
            hr--;
            min = 59;
            sec = 59;
          } else {
            clearInterval(timer);
            callback(true);
          }
        } else sec--;

        timeObj.sec = sec;
        timeObj.min = min;
        timeObj.hr = hr;

        if (hr > 0) {
          time += `${hr}:`;
        }
        time +=
          (hr > 0 && min < 10 ? "0" : "") +
          min +
          ":" +
          sec.toString().padStart(2, "0");
        $("#generalDialog .modal-header strong").text(time).prop("time", time);
      },
      1000,
      timeObj
    );
    callbefore(timer, time);
  }

  #showQuizResult(result) {
    // qtype
    // section
    // question
    // qImage
    // answer
    // instruct
    // options
    // correct
    // optionTxt
    // optImage
    //
    // duration
    // qcount
    // points
    // time_taken
    // answer_count
    // score
    // date_taken
    // response
    //   totalPoints
    // closeSubmitAfterDue
    // dueDate
  }
}
